@@ -31,7 +31,7 @@
  *      The current 'active' code index is used to access the current running
  *      code. The 'staging' code index is used by the process that performs
  *      a code change operation. When a code change operation completes
- *      succesfully, the staging code index becomes the new active code index.
+ *      successfully, the staging code index becomes the new active code index.
  *
  *      The third code index is not explicitly used. It can be thought of as
  *      the "previous active" or the "next staging" index. It is needed to make
@@ -63,6 +63,20 @@ struct process;
 
 
 #define ERTS_NUM_CODE_IX 3
+
+#ifdef BEAMASM
+#define ERTS_ADDRESSV_SIZE (ERTS_NUM_CODE_IX + 1)
+#define ERTS_SAVE_CALLS_CODE_IX (ERTS_ADDRESSV_SIZE - 1)
+#else
+#define ERTS_ADDRESSV_SIZE ERTS_NUM_CODE_IX
+#endif
+
+/* This structure lets `Export` entries and `ErlFunEntry` share dispatch code,
+ * which greatly improves the performance of fun calls. */
+typedef struct ErtsDispatchable_ {
+    ErtsCodePtr addresses[ERTS_ADDRESSV_SIZE];
+} ErtsDispatchable;
+
 typedef unsigned ErtsCodeIndex;
 
 typedef struct ErtsCodeMFA_ {
