@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1996-2020. All Rights Reserved.
+ * Copyright Ericsson AB 1996-2022. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -34,6 +34,7 @@
 #include "erl_binary.h"
 #include "erl_bits.h"
 #include "dtrace-wrapper.h"
+#include "erl_global_literals.h"
 
 static void move_one_frag(Eterm** hpp, ErlHeapFragment*, ErlOffHeap*, int);
 
@@ -114,12 +115,20 @@ Uint size_object_x(Eterm obj, erts_literal_area_t *litopt)
 		ASSERT(is_header(hdr));
 		switch (hdr & _TAG_HEADER_MASK) {
 		case ARITYVAL_SUBTAG:
-		    ptr = tuple_val(obj);
 		    arity = header_arity(hdr);
-		    sum += arity + 1;
 		    if (arity == 0) { /* Empty tuple -- unusual. */
+                        ASSERT(!litopt &&
+                               erts_is_literal(obj,ptr) &&
+                               obj == ERTS_GLOBAL_LIT_EMPTY_TUPLE);
+                        /*
+                          The empty tuple is always a global literal
+                          constant so it does not take up any extra
+                          space.
+                        */
 			goto pop_next;
 		    }
+                    ptr = tuple_val(obj);
+                    sum += arity + 1;
 		    while (arity-- > 1) {
 			obj = *++ptr;
 			if (!IS_CONST(obj)) {
@@ -154,6 +163,7 @@ Uint size_object_x(Eterm obj, erts_literal_area_t *litopt)
                                 mp  = (flatmap_t*)flatmap_val(obj);
                                 ptr = (Eterm *)mp;
                                 n   = flatmap_get_size(mp) + 1;
+                                ASSERT(flatmap_get_size(mp) <= MAP_SMALL_MAP_LIMIT);
                                 sum += n + 2;
                                 ptr += 2; /* hdr + size words */
                                 while (n--) {
@@ -276,6 +286,7 @@ do {									\
 do {                                                                    \
     UWord result;                                                       \
     if (WSTK_CONCAT(s,_bitoffs) <= 0) {                                 \
+        ASSERT(WSTK_CONCAT(s,_offset) < (s.wsp - s.wstart));            \
         WSTK_CONCAT(s,_buffer) = s.wstart[WSTK_CONCAT(s,_offset)];      \
         WSTK_CONCAT(s,_offset)++;                                       \
         WSTK_CONCAT(s,_bitoffs) = 8*sizeof(UWord);                      \
@@ -364,7 +375,10 @@ Uint size_shared(Eterm obj)
 		int arity = header_arity(hdr);
 		sum += arity + 1;
 		if (arity == 0) { /* Empty tuple -- unusual. */
-		    goto pop_next;
+                    ASSERT(COUNT_OFF_HEAP &&
+                           erts_is_literal(obj,ptr) &&
+                           obj == ERTS_GLOBAL_LIT_EMPTY_TUPLE);
+                    goto pop_next;
 		}
 		while (arity-- > 0) {
 		    obj = *++ptr;
@@ -417,6 +431,7 @@ Uint size_shared(Eterm obj)
                     case MAP_HEADER_TAG_FLATMAP_HEAD : {
                         flatmap_t *mp  = (flatmap_t*)flatmap_val(obj);
                         Uint n = flatmap_get_size(mp) + 1;
+                        ASSERT(flatmap_get_size(mp) <= MAP_SMALL_MAP_LIMIT);
                         ptr  = (Eterm *)mp;
                         sum += n + 2;
                         ptr += 2; /* hdr + size words */
@@ -522,6 +537,9 @@ cleanup:
 	    case ARITYVAL_SUBTAG: {
 		int arity = header_arity(hdr);
 		if (arity == 0) { /* Empty tuple -- unusual. */
+                    ASSERT(COUNT_OFF_HEAP &&
+                           erts_is_literal(obj,ptr) &&
+                           obj == ERTS_GLOBAL_LIT_EMPTY_TUPLE);
 		    goto cleanup_next;
 		}
 		while (arity-- > 0) {
@@ -550,6 +568,7 @@ cleanup:
                     case MAP_HEADER_TAG_FLATMAP_HEAD : {
                         flatmap_t *mp = (flatmap_t *) ptr;
                         Uint n = flatmap_get_size(mp) + 1;
+                        ASSERT(flatmap_get_size(mp) <= MAP_SMALL_MAP_LIMIT);
                         ptr += 2; /* hdr + size words */
                         while (n--) {
                             obj = *ptr++;
@@ -735,6 +754,13 @@ Eterm copy_struct_x(Eterm obj, Uint sz, Eterm** hpp, ErlOffHeap* off_heap,
 		{
 		    int const_flag = 1; /* assume constant tuple */
 		    i = arityval(hdr);
+                    if (i == 0) {
+                        ASSERT(!litopt &&
+                               erts_is_literal(obj,objp) &&
+                               obj == ERTS_GLOBAL_LIT_EMPTY_TUPLE);
+                        *argp = ERTS_GLOBAL_LIT_EMPTY_TUPLE;
+                        break;
+                    }
 		    *argp = make_tuple(htop);
 		    tp = htop;	/* tp is pointer to new arity value */
 		    *htop++ = *objp++; /* copy arity value */
@@ -848,9 +874,15 @@ Eterm copy_struct_x(Eterm obj, Uint sz, Eterm** hpp, ErlOffHeap* off_heap,
 			*htop++ = *objp++;
 		    }
 		    funp = (ErlFunThing *) tp;
-		    funp->next = off_heap->first;
-		    off_heap->first = (struct erl_off_heap_header*) funp;
-		    erts_refc_inc(&funp->fe->refc, 2);
+
+                    if (is_local_fun(funp)) {
+                        funp->next = off_heap->first;
+                        off_heap->first = (struct erl_off_heap_header*) funp;
+                        erts_refc_inc(&funp->entry.fun->refc, 2);
+                    } else {
+                        ASSERT(is_external_fun(funp) && funp->next == NULL);
+                    }
+
 		    *argp = make_fun(tp);
 		}
 		break;
@@ -887,6 +919,7 @@ Eterm copy_struct_x(Eterm obj, Uint sz, Eterm** hpp, ErlOffHeap* off_heap,
 		switch (MAP_HEADER_TYPE(hdr)) {
 		    case MAP_HEADER_TAG_FLATMAP_HEAD :
                         i = flatmap_get_size(objp) + 3;
+                        ASSERT(flatmap_get_size(objp) <= MAP_SMALL_MAP_LIMIT);
                         *argp = make_flatmap(htop);
                         while (i--) {
                             *htop++ = *objp++;
@@ -1036,11 +1069,17 @@ do {								\
     /* no WSTK_CONCAT(s,_offset), write-only */		\
     UWord WSTK_CONCAT(s,_buffer) = 0
 
+#ifdef DEBUG
+# define DEBUG_COND(D,E) D
+#else
+# define DEBUG_COND(D,E) E
+#endif
+
 #define DECLARE_BITSTORE_FROM_INFO(s, info)		\
     /* no WSTK_DEF_STACK(s), read-only */		\
     ErtsWStack s = {					\
         info->bitstore_start,  /* wstart */		\
-        NULL,                  /* wsp,  read-only */	\
+        DEBUG_COND(info->bitstore_stop, NULL), /* wsp,  read-only */ \
         NULL,                  /* wend, read-only */	\
         NULL,                  /* wdef, read-only */	\
         info->bitstore_alloc_type /* alloc_type */	\
@@ -1087,7 +1126,9 @@ Uint copy_shared_calculate(Eterm obj, erts_shcopy_t *info)
 #ifdef DEBUG
     Eterm mypid = erts_get_current_pid();
 #endif
-
+    const Eterm empty_tuple_literal =
+        ERTS_GLOBAL_LIT_EMPTY_TUPLE;
+    
     DECLARE_EQUEUE_INIT_INFO(s, info);
     DECLARE_BITSTORE_INIT_INFO(b, info);
     DECLARE_SHTABLE_INIT_INFO(t, info);
@@ -1177,10 +1218,12 @@ Uint copy_shared_calculate(Eterm obj, erts_shcopy_t *info)
 	case TAG_PRIMARY_BOXED: {
 	    Eterm hdr;
 	    ptr = boxed_val(obj);
-	    /* off heap pointers to boxes are copied verbatim */
+	    /* off heap pointers to boxes (except pointers to the
+               empty tuple) are copied verbatim */
 	    if (erts_is_literal(obj,ptr)) {
 		VERBOSE(DEBUG_SHCOPY, ("[pid=%T] bypassed copying %p is %T\n", mypid, ptr, obj));
-                if (copy_literals || in_literal_purge_area(ptr))
+                if (obj != empty_tuple_literal &&
+                    (copy_literals || in_literal_purge_area(ptr)))
                     info->literal_size += size_object(obj);
 		goto pop_next;
 	    }
@@ -1204,10 +1247,10 @@ Uint copy_shared_calculate(Eterm obj, erts_shcopy_t *info)
 	    switch (hdr & _TAG_HEADER_MASK) {
 	    case ARITYVAL_SUBTAG: {
 		int arity = header_arity(hdr);
+                /* arity cannot be 0 as the empty tuple is always a
+                   global constant literal which is handled above */
+                ASSERT(arity != 0);
 		sum += arity + 1;
-		if (arity == 0) { /* Empty tuple -- unusual. */
-		    goto pop_next;
-		}
 		while (arity-- > 0) {
 		    obj = *++ptr;
 		    if (!IS_CONST(obj)) {
@@ -1279,6 +1322,7 @@ Uint copy_shared_calculate(Eterm obj, erts_shcopy_t *info)
                     case MAP_HEADER_TAG_FLATMAP_HEAD : {
                         flatmap_t *mp = (flatmap_t *) ptr;
                         Uint n = flatmap_get_size(mp) + 1;
+                        ASSERT(flatmap_get_size(mp) <= MAP_SMALL_MAP_LIMIT);
                         sum += n + 2;
                         ptr += 2; /* hdr + size words */
                         while (n--) {
@@ -1330,6 +1374,9 @@ Uint copy_shared_calculate(Eterm obj, erts_shcopy_t *info)
                 info->queue_end = s.end;
                 info->queue_alloc_type = s.alloc_type;
                 info->bitstore_start = b.wstart;
+#ifdef DEBUG
+                info->bitstore_stop = b.wsp;
+#endif
                 info->bitstore_alloc_type = b.alloc_type;
                 info->shtable_start = t.start;
                 info->shtable_alloc_type = t.alloc_type;
@@ -1372,7 +1419,8 @@ Uint copy_shared_perform_x(Eterm obj, Uint size, erts_shcopy_t *info,
     Eterm mypid = erts_get_current_pid();
     Eterm saved_obj = obj;
 #endif
-
+    const Eterm empty_tuple_literal =
+        ERTS_GLOBAL_LIT_EMPTY_TUPLE;
     DECLARE_EQUEUE_FROM_INFO(s, info);
     DECLARE_BITSTORE_FROM_INFO(b, info);
     DECLARE_SHTABLE_FROM_INFO(t, info);
@@ -1489,7 +1537,8 @@ Uint copy_shared_perform_x(Eterm obj, Uint size, erts_shcopy_t *info,
 	    ptr = boxed_val(obj);
 	    /* off heap pointers to boxes are copied verbatim */
 	    if (erts_is_literal(obj,ptr)) {
-                if (!(copy_literals || in_literal_purge_area(ptr))) {
+                if (obj == empty_tuple_literal ||
+                    !(copy_literals || in_literal_purge_area(ptr))) {
                     *resp = obj;
                 } else {
                     Uint bsz = 0;
@@ -1567,9 +1616,15 @@ Uint copy_shared_perform_x(Eterm obj, Uint size, erts_shcopy_t *info,
 			*hp++ = HEAP_ELEM_TO_BE_FILLED;
 		    }
 		}
-		funp->next = off_heap->first;
-		off_heap->first = (struct erl_off_heap_header*) funp;
-		erts_refc_inc(&funp->fe->refc, 2);
+
+                if (is_local_fun(funp)) {
+                    funp->next = off_heap->first;
+                    off_heap->first = (struct erl_off_heap_header*) funp;
+                    erts_refc_inc(&funp->entry.fun->refc, 2);
+                } else {
+                    ASSERT(is_external_fun(funp) && funp->next == NULL);
+                }
+
 		goto cleanup_next;
 	    }
 	    case MAP_SUBTAG:
@@ -1579,6 +1634,7 @@ Uint copy_shared_perform_x(Eterm obj, Uint size, erts_shcopy_t *info,
                     case MAP_HEADER_TAG_FLATMAP_HEAD : {
                         flatmap_t *mp = (flatmap_t *) ptr;
                         Uint n = flatmap_get_size(mp) + 1;
+                        ASSERT(flatmap_get_size(mp) <= MAP_SMALL_MAP_LIMIT);
                         *hp++  = *++ptr; /* keys */
                         while (n--) {
                             obj = *++ptr;
@@ -1733,7 +1789,7 @@ Uint copy_shared_perform_x(Eterm obj, Uint size, erts_shcopy_t *info,
 		goto cleanup_next;
 	    }
 	    case REF_SUBTAG:
-		if (is_magic_ref_thing(ptr)) {
+		if (is_magic_ref_thing_with_hdr(ptr,hdr)) {
 		    ErtsMRefThing *mreft = (ErtsMRefThing *) ptr;
 		    erts_refc_inc(&mreft->mb->intern.refc, 2);
 		    goto off_heap_node_container_common;
@@ -1782,6 +1838,7 @@ Uint copy_shared_perform_x(Eterm obj, Uint size, erts_shcopy_t *info,
                                 case MAP_HEADER_TAG_FLATMAP_HEAD : {
                                     flatmap_t *mp = (flatmap_t *) hscan;
                                     remaining = flatmap_get_size(mp) + 1;
+                                    ASSERT(flatmap_get_size(mp) <= MAP_SMALL_MAP_LIMIT);
                                     hscan += 2;
                                     break;
                                 }
@@ -1889,7 +1946,8 @@ copy_shallow_x(Eterm* ERTS_RESTRICT ptr, Uint sz, Eterm** hpp, ErlOffHeap* off_h
     Eterm* hp = *hpp;
     const Eterm res = make_tuple(hp);
     const Sint offs = (hp - tp) * sizeof(Eterm);
-
+    const Eterm empty_tuple_literal =
+        ERTS_GLOBAL_LIT_EMPTY_TUPLE;
     while (sz--) {
 	Eterm val = *tp++;
 
@@ -1899,7 +1957,11 @@ copy_shallow_x(Eterm* ERTS_RESTRICT ptr, Uint sz, Eterm** hpp, ErlOffHeap* off_h
 	    break;
 	case TAG_PRIMARY_LIST:
 	case TAG_PRIMARY_BOXED:
-	    *hp++ = byte_offset_ptr(val, offs);
+            if (val == empty_tuple_literal) {
+                *hp++ = empty_tuple_literal;
+            } else {
+                *hp++ = byte_offset_ptr(val, offs);
+            }
 	    break;
 	case TAG_PRIMARY_HEADER:
 	    *hp++ = val;
@@ -1914,12 +1976,18 @@ copy_shallow_x(Eterm* ERTS_RESTRICT ptr, Uint sz, Eterm** hpp, ErlOffHeap* off_h
 		}
 		goto off_heap_common;
 
-	    case FUN_SUBTAG:
-		{
-		    ErlFunThing* funp = (ErlFunThing *) (tp-1);
-		    erts_refc_inc(&funp->fe->refc, 2);
-		}
-		goto off_heap_common;
+            case FUN_SUBTAG:
+                {
+                    ErlFunThing* funp = (ErlFunThing *) (tp-1);
+
+                    if (is_local_fun(funp)) {
+                        erts_refc_inc(&funp->entry.fun->refc, 2);
+                        goto off_heap_common;
+                    } else {
+                        ASSERT(is_external_fun(funp) && funp->next == NULL);
+                        goto default_copy;
+                    }
+                }
 	    case EXTERNAL_PID_SUBTAG:
 	    case EXTERNAL_PORT_SUBTAG:
 	    case EXTERNAL_REF_SUBTAG:
@@ -1953,6 +2021,7 @@ copy_shallow_x(Eterm* ERTS_RESTRICT ptr, Uint sz, Eterm** hpp, ErlOffHeap* off_h
 		}
 		/* Fall through... */
 	    }
+            default_copy:
 	    default:
 		{
 		    int tari = header_arity(val);
@@ -2044,34 +2113,49 @@ move_one_frag(Eterm** hpp, ErlHeapFragment* frag, ErlOffHeap* off_heap, int lite
     Eterm* hp = *hpp;
 
     while (ptr != end) {
-	Eterm val;
-	ASSERT(ptr < end);
-	val = *ptr;
-	ASSERT(val != ERTS_HOLE_MARKER);
-	if (is_header(val)) {
-	    struct erl_off_heap_header* hdr = (struct erl_off_heap_header*)hp;
-	    ASSERT(ptr + header_arity(val) < end);
-	    ptr = move_boxed(ptr, val, &hp, &dummy_ref);
-	    switch (val & _HEADER_SUBTAG_MASK) {
-	    case REF_SUBTAG:
-		if (is_ordinary_ref_thing(hdr))
-		    break;
-	    case REFC_BINARY_SUBTAG:
-	    case FUN_SUBTAG:
-	    case EXTERNAL_PID_SUBTAG:
-	    case EXTERNAL_PORT_SUBTAG:
-	    case EXTERNAL_REF_SUBTAG:
-		hdr->next = off_heap->first;
-		off_heap->first = hdr;
-		break;
-	    }
-	}
-	else { /* must be a cons cell */
-	    ASSERT(ptr+1 < end);
-	    move_cons(ptr, val, &hp, &dummy_ref);
-	    ptr += 2;
-	}
+        Eterm val;
+        ASSERT(ptr < end);
+        val = *ptr;
+        ASSERT(val != ERTS_HOLE_MARKER);
+
+        if (is_header(val)) {
+            struct erl_off_heap_header* hdr = (struct erl_off_heap_header*)hp;
+
+            ASSERT(ptr + header_arity(val) < end);
+            ptr = move_boxed(ptr, val, &hp, &dummy_ref);
+
+            switch (val & _HEADER_SUBTAG_MASK) {
+            case REF_SUBTAG:
+                if (!is_magic_ref_thing(hdr)) {
+                    break;
+                }
+            case REFC_BINARY_SUBTAG:
+            case EXTERNAL_PID_SUBTAG:
+            case EXTERNAL_PORT_SUBTAG:
+            case EXTERNAL_REF_SUBTAG:
+                hdr->next = off_heap->first;
+                off_heap->first = hdr;
+                break;
+            case FUN_SUBTAG:
+                {
+                    ErlFunThing *funp = (ErlFunThing*)hdr;
+
+                    if (is_local_fun(funp)) {
+                        hdr->next = off_heap->first;
+                        off_heap->first = hdr;
+                    } else {
+                        ASSERT(is_external_fun(funp) && funp->next == NULL);
+                    }
+                }
+                break;
+            }
+        } else { /* must be a cons cell */
+            ASSERT(ptr+1 < end);
+            move_cons(ptr, val, &hp, &dummy_ref);
+            ptr += 2;
+        }
     }
+
     *hpp = hp;
     OH_OVERHEAD(off_heap, frag->off_heap.overhead);
     frag->off_heap.first = NULL;
