@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2018-2020. All Rights Reserved.
+ * Copyright Ericsson AB 2018-2022. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -136,7 +136,7 @@ typedef struct {
     HashTable* old_table;
     HashTable* new_table;
     Uint entry_index;
-    Eterm old_term;
+    Eterm old_bucket;
     HashTable* tmp_table;
     int must_shrink;
     ErtsPersistentTermCpyTableCtx cpy_ctx;
@@ -147,7 +147,7 @@ typedef struct {
  */
 
 static HashTable* create_initial_table(void);
-static Uint lookup(HashTable* hash_table, Eterm key);
+static Uint lookup(HashTable* hash_table, Eterm key, Eterm *bucket);
 static int is_erasable(HashTable* hash_table, Uint idx);
 static HashTable* copy_table(ErtsPersistentTermCpyTableCtx* ctx);
 static int try_seize_update_permission(Process* c_p);
@@ -307,10 +307,11 @@ BIF_RETTYPE persistent_term_put_2(BIF_ALIST_2)
     static const Uint ITERATIONS_PER_RED = 32;
     ErtsPersistentTermPut2Context* ctx;
     Eterm state_mref = THE_NON_VALUE;
+    Eterm old_bucket;
     long iterations_until_trap;
     long max_iterations;
 #define PUT_TRAP_CODE                                                   \
-    BIF_TRAP2(&bif_trap_export[BIF_persistent_term_put_2], BIF_P, state_mref, BIF_ARG_2)
+    BIF_TRAP2(BIF_TRAP_EXPORT(BIF_persistent_term_put_2), BIF_P, state_mref, BIF_ARG_2)
 #define TRAPPING_COPY_TABLE_PUT(TABLE_DEST, OLD_TABLE, NEW_SIZE, COPY_TYPE, LOC_NAME) \
     TRAPPING_COPY_TABLE(TABLE_DEST, OLD_TABLE, NEW_SIZE, COPY_TYPE, LOC_NAME, PUT_TRAP_CODE)
 
@@ -351,7 +352,7 @@ BIF_RETTYPE persistent_term_put_2(BIF_ALIST_2)
 
 
     if (!try_seize_update_permission(BIF_P)) {
-	ERTS_BIF_YIELD2(&bif_trap_export[BIF_persistent_term_put_2],
+	ERTS_BIF_YIELD2(BIF_TRAP_EXPORT(BIF_persistent_term_put_2),
                         BIF_P, BIF_ARG_1, BIF_ARG_2);
     }
     ctx->hash_table = (HashTable *) erts_atomic_read_nob(&the_hash_table);
@@ -359,14 +360,14 @@ BIF_RETTYPE persistent_term_put_2(BIF_ALIST_2)
     ctx->key = BIF_ARG_1;
     ctx->term = BIF_ARG_2;
 
-    ctx->entry_index = lookup(ctx->hash_table, ctx->key);
+    ctx->entry_index = lookup(ctx->hash_table, ctx->key, &old_bucket);
 
     ctx->heap[0] = make_arityval(2);
     ctx->heap[1] = ctx->key;
     ctx->heap[2] = ctx->term;
     ctx->tuple = make_tuple(ctx->heap);
 
-    if (is_nil(get_bucket(ctx->hash_table, ctx->entry_index))) {
+    if (is_nil(old_bucket)) {
         if (MUST_GROW(ctx->hash_table)) {
             Uint new_size = ctx->hash_table->allocated * 2;
             TRAPPING_COPY_TABLE_PUT(ctx->hash_table,
@@ -374,15 +375,17 @@ BIF_RETTYPE persistent_term_put_2(BIF_ALIST_2)
                                     new_size,
                                     ERTS_PERSISTENT_TERM_CPY_NO_REHASH,
                                     PUT2_TRAP_LOCATION_NEW_KEY);
-            ctx->entry_index = lookup(ctx->hash_table, ctx->key);
+            ctx->entry_index = lookup(ctx->hash_table,
+                                      ctx->key,
+                                      &old_bucket);
         }
         ctx->hash_table->num_entries++;
     } else {
-        Eterm tuple = get_bucket(ctx->hash_table, ctx->entry_index);
         Eterm old_term;
 
-        ASSERT(is_tuple_arity(tuple, 2));
-        old_term = boxed_val(tuple)[2];
+        ASSERT(is_tuple_arity(old_bucket, 2));
+        old_term = boxed_val(old_bucket)[2];
+
         if (EQ(ctx->term, old_term)) {
             /* Same value. No need to update anything. */
             release_update_permission(0);
@@ -439,7 +442,7 @@ BIF_RETTYPE persistent_term_put_2(BIF_ALIST_2)
 
 BIF_RETTYPE persistent_term_get_0(BIF_ALIST_0)
 {
-    HashTable* hash_table = (HashTable *) erts_atomic_read_nob(&the_hash_table);
+    HashTable* hash_table;
     TrapData* trap_data;
     Eterm res = NIL;
     Eterm magic_ref;
@@ -447,9 +450,11 @@ BIF_RETTYPE persistent_term_get_0(BIF_ALIST_0)
 
     /* Prevent concurrent updates to get a consistent view */
     if (!try_seize_update_permission(BIF_P)) {
-        ERTS_BIF_YIELD0(&bif_trap_export[BIF_persistent_term_get_0], BIF_P);
+        ERTS_BIF_YIELD0(BIF_TRAP_EXPORT(BIF_persistent_term_get_0), BIF_P);
     }
 
+    hash_table = (HashTable *) erts_atomic_read_nob(&the_hash_table);
+
     magic_ref = alloc_trap_data(BIF_P);
     mbp = erts_magic_ref2bin(magic_ref);
     trap_data = ERTS_MAGIC_BIN_DATA(mbp);
@@ -469,36 +474,45 @@ BIF_RETTYPE persistent_term_get_0(BIF_ALIST_0)
     }
 }
 
+static ERTS_INLINE Eterm
+persistent_term_get(Eterm key)
+{
+    HashTable* hash_table = (HashTable *) erts_atomic_read_nob(&the_hash_table);
+    Eterm bucket;
+
+    (void)lookup(hash_table, key, &bucket);
+
+    if (is_boxed(bucket)) {
+        ASSERT(is_tuple_arity(bucket, 2));
+        return tuple_val(bucket)[2];
+    }
+
+    return THE_NON_VALUE;
+}
+
+Eterm
+erts_persistent_term_get(Eterm key)
+{
+    return persistent_term_get(key);
+}
+
 BIF_RETTYPE persistent_term_get_1(BIF_ALIST_1)
 {
-    Eterm key = BIF_ARG_1;
-    HashTable* hash_table = (HashTable *) erts_atomic_read_nob(&the_hash_table);
-    Uint entry_index;
-    Eterm term;
-
-    entry_index = lookup(hash_table, key);
-    term = get_bucket(hash_table, entry_index);
-    if (is_boxed(term)) {
-        ASSERT(is_tuple_arity(term, 2));
-        BIF_RET(tuple_val(term)[2]);
+    Eterm result = persistent_term_get(BIF_ARG_1);
+    if (is_non_value(result)) {
+        BIF_ERROR(BIF_P, BADARG);
     }
-    BIF_ERROR(BIF_P, BADARG);
+
+    BIF_RET(result);
 }
 
 BIF_RETTYPE persistent_term_get_2(BIF_ALIST_2)
 {
-    Eterm key = BIF_ARG_1;
-    Eterm result = BIF_ARG_2;
-    HashTable* hash_table = (HashTable *) erts_atomic_read_nob(&the_hash_table);
-    Uint entry_index;
-    Eterm term;
-
-    entry_index = lookup(hash_table, key);
-    term = get_bucket(hash_table, entry_index);
-    if (is_boxed(term)) {
-        ASSERT(is_tuple_arity(term, 2));
-        result = tuple_val(term)[2];
+    Eterm result = persistent_term_get(BIF_ARG_1);
+    if (is_non_value(result)) {
+        result = BIF_ARG_2;
     }
+
     BIF_RET(result);
 }
 
@@ -535,7 +549,7 @@ BIF_RETTYPE persistent_term_erase_1(BIF_ALIST_1)
             ITERATIONS_PER_RED * ERTS_BIF_REDS_LEFT(BIF_P);
 #endif
 #define ERASE_TRAP_CODE                                                 \
-        BIF_TRAP1(&bif_trap_export[BIF_persistent_term_erase_1], BIF_P, state_mref);
+        BIF_TRAP1(BIF_TRAP_EXPORT(BIF_persistent_term_erase_1), BIF_P, state_mref);
 #define TRAPPING_COPY_TABLE_ERASE(TABLE_DEST, OLD_TABLE, NEW_SIZE, REHASH, LOC_NAME) \
         TRAPPING_COPY_TABLE(TABLE_DEST, OLD_TABLE, NEW_SIZE, REHASH, LOC_NAME, ERASE_TRAP_CODE)
     if (is_internal_magic_ref(BIF_ARG_1) &&
@@ -570,15 +584,15 @@ BIF_RETTYPE persistent_term_erase_1(BIF_ALIST_1)
         ctx->tmp_table = NULL;
     }
     if (!try_seize_update_permission(BIF_P)) {
-	ERTS_BIF_YIELD1(&bif_trap_export[BIF_persistent_term_erase_1],
+	ERTS_BIF_YIELD1(BIF_TRAP_EXPORT(BIF_persistent_term_erase_1),
                         BIF_P, BIF_ARG_1);
     }
 
     ctx->key = BIF_ARG_1;
     ctx->old_table = (HashTable *) erts_atomic_read_nob(&the_hash_table);
-    ctx->entry_index = lookup(ctx->old_table, ctx->key);
-    ctx->old_term = get_bucket(ctx->old_table, ctx->entry_index);
-    if (is_boxed(ctx->old_term)) {
+    ctx->entry_index = lookup(ctx->old_table, ctx->key, &ctx->old_bucket);
+
+    if (is_boxed(ctx->old_bucket)) {
         ctx->must_shrink = MUST_SHRINK(ctx->old_table);
         if (!ctx->must_shrink && is_erasable(ctx->old_table, ctx->entry_index)) {
             /*
@@ -605,7 +619,7 @@ BIF_RETTYPE persistent_term_erase_1(BIF_ALIST_1)
              * temporary table copy of the same size as the old one.
              */
 
-            ASSERT(is_tuple_arity(ctx->old_term, 2));
+            ASSERT(is_tuple_arity(ctx->old_bucket, 2));
             TRAPPING_COPY_TABLE_ERASE(ctx->tmp_table,
                                       ctx->old_table,
                                       ctx->old_table->allocated,
@@ -648,7 +662,7 @@ BIF_RETTYPE persistent_term_erase_1(BIF_ALIST_1)
      * Key is not present. Nothing to do.
      */
 
-    ASSERT(is_nil(ctx->old_term));
+    ASSERT(is_nil(ctx->old_bucket));
     release_update_permission(0);
     BIF_RET(am_false);
 }
@@ -659,7 +673,7 @@ BIF_RETTYPE erts_internal_erase_persistent_terms_0(BIF_ALIST_0)
     HashTable* new_table;
 
     if (!try_seize_update_permission(BIF_P)) {
-	ERTS_BIF_YIELD0(&bif_trap_export[BIF_erts_internal_erase_persistent_terms_0],
+	ERTS_BIF_YIELD0(BIF_TRAP_EXPORT(BIF_erts_internal_erase_persistent_terms_0),
                         BIF_P);
     }
     old_table = (HashTable *) erts_atomic_read_nob(&the_hash_table);
@@ -673,7 +687,7 @@ BIF_RETTYPE erts_internal_erase_persistent_terms_0(BIF_ALIST_0)
 
 BIF_RETTYPE persistent_term_info_0(BIF_ALIST_0)
 {
-    HashTable* hash_table = (HashTable *) erts_atomic_read_nob(&the_hash_table);
+    HashTable* hash_table;
     TrapData* trap_data;
     Eterm res = NIL;
     Eterm magic_ref;
@@ -681,9 +695,11 @@ BIF_RETTYPE persistent_term_info_0(BIF_ALIST_0)
 
     /* Prevent concurrent updates to get a consistent view */
     if (!try_seize_update_permission(BIF_P)) {
-        ERTS_BIF_YIELD0(&bif_trap_export[BIF_persistent_term_info_0], BIF_P);
+        ERTS_BIF_YIELD0(BIF_TRAP_EXPORT(BIF_persistent_term_info_0), BIF_P);
     }
 
+    hash_table = (HashTable *) erts_atomic_read_nob(&the_hash_table);
+
     magic_ref = alloc_trap_data(BIF_P);
     mbp = erts_magic_ref2bin(magic_ref);
     trap_data = ERTS_MAGIC_BIN_DATA(mbp);
@@ -719,10 +735,10 @@ erts_init_persistent_dumping(void)
     erts_persistent_areas = (ErtsLiteralArea **) hash_table->term;
     area_p = erts_persistent_areas;
     for (i = 0; i < hash_table->allocated; i++) {
-        Eterm term = get_bucket(hash_table, i);
+        Eterm bucket = get_bucket(hash_table, i);
 
-        if (is_boxed(term)) {
-            *area_p++ = term_to_area(term);
+        if (is_boxed(bucket)) {
+            *area_p++ = term_to_area(bucket);
         }
     }
     erts_num_persistent_areas = area_p - erts_persistent_areas;
@@ -808,15 +824,15 @@ do_get_all(Process* c_p, TrapData* trap_data, Eterm res)
     i = 0;
     heap_size = (2 + 3) * remaining;
     while (remaining != 0) {
-        Eterm term;
+        Eterm bucket;
         ASSERT(idx < hash_table->allocated);
-        term = get_bucket(hash_table, idx);
-        if (is_tuple(term)) {
+        bucket = get_bucket(hash_table, idx);
+        if (is_tuple(bucket)) {
             Uint key_size;
             Eterm* tup_val;
 
-            ASSERT(is_tuple_arity(term, 2));
-            tup_val = tuple_val(term);
+            ASSERT(is_tuple_arity(bucket, 2));
+            tup_val = tuple_val(bucket);
             key_size = size_object(tup_val[1]);
             copy_data[i].key_size = key_size;
             copy_data[i].tuple_ptr = tup_val;
@@ -893,13 +909,17 @@ do_info(Process* c_p, TrapData* trap_data)
     remaining = trap_data->remaining < max_iter ? trap_data->remaining : max_iter;
     trap_data->remaining -= remaining;
     while (remaining != 0) {
-        if (is_boxed(get_bucket(hash_table, idx))) {
-            ErtsLiteralArea* area;
-            area = term_to_area(get_bucket(hash_table, idx));
+        Eterm bucket = get_bucket(hash_table, idx);
+
+        if (is_boxed(bucket)) {
+            ErtsLiteralArea* area = term_to_area(bucket);
+
             trap_data->memory += sizeof(ErtsLiteralArea) +
                 sizeof(Eterm) * (area->end - area->start - 1);
+
             remaining--;
         }
+
         idx++;
     }
     trap_data->idx = idx;
@@ -949,7 +969,7 @@ cleanup_trap_data(Binary *bp)
 }
 
 static Uint
-lookup(HashTable* hash_table, Eterm key)
+lookup(HashTable* hash_table, Eterm key, Eterm *bucket)
 {
     Uint mask = hash_table->mask;
     Uint32 idx = make_internal_hash(key, 0);
@@ -957,11 +977,15 @@ lookup(HashTable* hash_table, Eterm key)
 
     while (1) {
         term = get_bucket(hash_table, idx & mask);
-        if (is_nil(term) || EQ(key, (tuple_val(term))[1]))
-            break;
+
+        if (is_nil(term) || EQ(key, (tuple_val(term))[1])) {
+            *bucket = term;
+
+            return idx & mask;
+        }
+
         idx++;
     }
-    return idx & mask;
 }
 
 static int
@@ -1044,12 +1068,19 @@ copy_table(ErtsPersistentTermCpyTableCtx* ctx)
              i < MIN(ctx->iterations_done + ctx->max_iterations,
                      old_size);
              i++) {
-            Eterm term = get_bucket(ctx->old_table, i);
-            if (is_tuple(term)) {
-                Eterm key = tuple_val(term)[1];
-                Uint entry_index = lookup(ctx->new_table, key);
-                ASSERT(is_nil(get_bucket(ctx->new_table, entry_index)));
-                set_bucket(ctx->new_table, entry_index, term);
+            Eterm old_bucket = get_bucket(ctx->old_table, i);
+
+            if (is_tuple(old_bucket)) {
+                Eterm key, assert_empty_bucket;
+                Uint entry_index;
+
+                key = tuple_val(old_bucket)[1];
+                entry_index = lookup(ctx->new_table, key, &assert_empty_bucket);
+
+                ASSERT(is_nil(assert_empty_bucket));
+                (void)assert_empty_bucket;
+
+                set_bucket(ctx->new_table, entry_index, old_bucket);
             }
         }
         ctx->total_iterations_done += (i - ctx->iterations_done);
@@ -1114,7 +1145,7 @@ table_updater(void* data)
     old_table = (HashTable *) erts_atomic_read_nob(&the_hash_table);
     new_table = (HashTable *) data;
     if (new_table == old_table) {
-        Eterm old_term = get_bucket(old_table, fast_update_index);
+        Eterm old_bucket = get_bucket(old_table, fast_update_index);
         ASSERT(is_value(fast_update_term));
         ASSERT(fast_update_index < old_table->allocated);
         set_bucket(old_table, fast_update_index, fast_update_term);
@@ -1122,11 +1153,11 @@ table_updater(void* data)
         fast_update_term = THE_NON_VALUE;
 #endif
 
-        if (is_not_nil(old_term))  {
+        if (is_not_nil(old_bucket))  {
             OldLiteral *olp = alloc_old_literal();
-            ASSERT(is_tuple_arity(old_term,2));
-            olp->term = old_term;
-            olp->area = term_to_area(old_term);
+            ASSERT(is_tuple_arity(old_bucket,2));
+            olp->term = old_bucket;
+            olp->area = term_to_area(old_bucket);
             olp->delete_op.type = DELETE_OP_TUPLE;
             olp->delete_op.is_scheduled = 1;
             append_to_delete_queue(&olp->delete_op);
@@ -1407,9 +1438,9 @@ static void debug_table_foreach_off_heap(HashTable *tbl,
     int i;
     
     for (i = 0; i < tbl->allocated; i++) {
-        Eterm term = get_bucket(tbl, i);
-        if (is_tuple_arity(term, 2)) {
-            debug_area_off_heap(term_to_area(term), func, arg);
+        Eterm bucket = get_bucket(tbl, i);
+        if (is_tuple_arity(bucket, 2)) {
+            debug_area_off_heap(term_to_area(bucket), func, arg);
         }
     }
 }
@@ -1481,3 +1512,17 @@ erts_debug_foreach_persistent_term_off_heap(void (*func)(ErlOffHeap *, void *),
     accessed_literal_areas = NULL;
 }
 
+Eterm erts_debug_persistent_term_xtra_info(Process* c_p)
+{
+    HashTable* hash_table = (HashTable *) erts_atomic_read_nob(&the_hash_table);
+    Uint hsz = MAP_SZ(1);
+    Eterm *hp;
+    Eterm buckets, res;
+
+    (void) erts_bld_uint(NULL, &hsz, hash_table->allocated);
+    hp = HAlloc(c_p, hsz);
+    buckets = erts_bld_uint(&hp, NULL, hash_table->allocated);
+    res = MAP1(hp, am_table, buckets);
+    BIF_RET(res);
+}
+
