@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2001-2018. All Rights Reserved.
+ * Copyright Ericsson AB 2001-2021. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,6 +23,7 @@
 
 typedef struct dist_entry_ DistEntry;
 typedef struct ErtsDistOutputBuf_ ErtsDistOutputBuf;
+typedef struct ErtsDistOutputBufsContainer_ ErtsDistOutputBufsContainer;
 void erts_ref_dist_entry(DistEntry *dep);
 void erts_deref_dist_entry(DistEntry *dep);
 
@@ -95,26 +96,21 @@ enum dist_entry_state {
 struct ErtsDistOutputBuf_ {
 #ifdef DEBUG
     Uint dbg_pattern;
-    byte *ext_startp;
-    byte *alloc_endp;
 #endif
     ErtsDistOutputBuf *next;
     Binary *bin;
-    /* Pointers to the distribution header,
-       if NULL the distr header is in the extp */
-    byte *hdrp;
-    byte *hdr_endp;
-    /* Pointers to the ctl + payload */
-    byte *extp;
-    byte *ext_endp;
-    /* Start of payload and hopefull_flags, used by transcode */
-    Uint hopefull_flags;
-    byte *msg_start;
-    /* start of the ext buffer, this is not always the same as extp
-       as the atom cache handling can use less then the allotted buffer.
-       This value is needed to calculate the size of this output buffer.*/
-    byte *ext_start;
+    /*
+     * iov[0] reserved for driver
+     * iov[1] reserved for distribution header
+     * iov[2 ... vsize-1] data
+     */
+    ErlIOVec *eiov;
+};
 
+struct ErtsDistOutputBufsContainer_ {
+    Sint fragments;
+    byte *extp;
+    ErtsDistOutputBuf obuf[1]; /* longer if fragmented... */
 };
 
 typedef struct {
@@ -147,10 +143,11 @@ struct dist_entry_ {
                                    NIL == free */
     Uint32 connection_id;	/* Connection id incremented on connect */
     enum dist_entry_state state;
-    Uint32 flags;		/* Distribution flags, like hidden, 
+    int pending_nodedown;
+    Process* suspended_nodeup;
+    Uint64 dflags;		/* Distribution flags, like hidden,
 				   atom cache etc. */
     Uint32 opts;
-    unsigned long version;	/* Protocol version */
 
     ErtsMonLnkDist *mld;        /* Monitors and links */
 
@@ -173,8 +170,6 @@ struct dist_entry_ {
 
     ErtsThrPrgrLaterOp later_op;
 
-    struct transcode_context* transcode_ctx;
-
     struct dist_sequences *sequences; /* Ongoing distribution sequences */
 };
 
@@ -205,7 +200,7 @@ Accs = lists:foldl(fun({V,<<"ERL_NODE_INC">>,_},M) -> Val = maps:get(V,M,0), M#{
 lists:usort(lists:filter(fun({V,N}) -> N /= 0 end, maps:to_list(Accs))).
 
  * There are bound to be bugs in the the instrumentation code, but
- * atleast this is a place to start when hunting refc bugs.
+ * at least this is a place to start when hunting refc bugs.
  *
  */
 #ifdef ERL_NODE_BOOKKEEP
@@ -261,10 +256,10 @@ Uint erts_dist_table_size(void);
 void erts_dist_table_info(fmtfn_t, void *);
 void erts_set_dist_entry_not_connected(DistEntry *);
 void erts_set_dist_entry_pending(DistEntry *);
-void erts_set_dist_entry_connected(DistEntry *, Eterm, Uint);
+void erts_set_dist_entry_connected(DistEntry *, Eterm, Uint64);
 ErlNode *erts_find_or_insert_node(Eterm, Uint32, Eterm);
 void erts_schedule_delete_node(ErlNode *);
-void erts_set_this_node(Eterm, Uint);
+void erts_set_this_node(Eterm, Uint32);
 Uint erts_node_table_size(void);
 void erts_init_node_tables(int);
 void erts_node_table_info(fmtfn_t, void *);
