@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2002-2021. All Rights Reserved.
+ * Copyright Ericsson AB 2002-2022. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -327,7 +327,7 @@ struct ErtsMemMapper_ {
      * Super unaligned area is located above super aligned
      * area. That is, `sa.bot` is beginning of the super
      * carrier, `sua.top` is the end of the super carrier,
-     * and sa.top and sua.bot moves towards eachother.
+     * and sa.top and sua.bot moves towards each other.
      */
     struct {
 	char *top;
@@ -1897,7 +1897,7 @@ erts_mremap(ErtsMemMapper* mm,
 #endif
 #ifdef ERTS_HAVE_OS_MREMAP
 	if (superaligned) {
-	    return remap_move(mm, flags, new_ptr, old_size, sizep);
+	    return remap_move(mm, flags, ptr, old_size, sizep);
 	} else {
 	    new_ptr = os_mremap(ptr, old_size, asize, 0);
 	    if (!new_ptr)
@@ -2587,10 +2587,10 @@ static void print_tree(enum SortOrder order, RBTNode*);
 
 /*
  * Checks that the order between parent and children are correct,
- * and that the Red-Black Tree properies are satisfied. if size > 0,
+ * and that the Red-Black Tree properties are satisfied. if size > 0,
  * check_tree() returns the node that satisfies "address order first fit"
  *
- * The Red-Black Tree properies are:
+ * The Red-Black Tree properties are:
  *   1. Every node is either red or black.
  *   2. Every leaf (NIL) is black.
  *   3. If a node is red, then both its children are black.
