@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1997-2020. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2022. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -183,7 +183,7 @@ get_xcomp_flag(Flag, Flags) ->
 get_xcomp_flag(Flag, Tag, Flags) ->
     case proplists:get_value(Flag,Flags) of
 	undefined -> "";
-	"guess" -> [" --",Tag,"=",os:cmd("$ERL_TOP/erts/autoconf/config.guess")];
+	"guess" -> [" --",Tag,"=",os:cmd("$ERL_TOP/make/autoconf/config.guess")];
 	HostVal -> [" --",Tag,"=",HostVal]
     end.
 
@@ -339,8 +339,8 @@ to_upper(String) ->
 word_size() ->
     case {erlang:system_info({wordsize,external}),
 	  erlang:system_info({wordsize,internal})} of
-	{4,4} -> "";
-	{8,8} -> "/64";
+	{4,4} -> "/32-bit";
+	{8,8} -> "/64-bit";
 	{8,4} -> "/Halfword"
     end.
 
