@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 2007-2020. All Rights Reserved.
+%% Copyright Ericsson AB 2007-2022. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -112,6 +112,8 @@ setelement(Config) when is_list(Config) ->
     {a,b,[1,2,3]} = id(setelement(3, {a,b,c}, New)),
     {a,b,[1,2,3]} = id(setelement(3, {a,X,c}, New)),
 
+    {{d,c,b,a,x}, {z,c,b,a,x}} = setelement_cover(erlang:make_tuple(5, x)),
+
     {'EXIT',{badarg,_}} = (catch setelement_crash({a,b,c,d,e,f})),
     error = setelement_crash_2({a,b,c,d,e,f}, <<42>>),
 
@@ -120,6 +122,14 @@ setelement(Config) when is_list(Config) ->
 
     ok.
 
+setelement_cover(T0) ->
+    T1 = setelement(4, T0, a),
+    T2 = setelement(3, T1, b),
+    T3 = setelement(2, T2, c),
+    T4 = setelement(1, T3, d),
+    T5 = setelement(1, T3, z),
+    {T4,T5}.
+
 setelement_crash(Tuple) ->
     %% Used to crash the compiler because sys_core_dsetel did not notice that
     %% X1 was used in bit syntax construction.
@@ -404,7 +414,7 @@ unused_multiple_values_error(Config) when is_list(Config) ->
     Core = filename:join(Dir, "unused_multiple_values_error"),
     Opts = [no_copt,clint,ssalint,return,from_core,{outdir,PrivDir}
 	   |test_lib:opt_opts(?MODULE)],
-    {error,[{unused_multiple_values_error,
+    {error,[{"unused_multiple_values_error",
 	     [{none,core_lint,{return_mismatch,{hello,1}}}]}],
      []} = c:c(Core, Opts),
     ok.
@@ -556,7 +566,7 @@ configuration(_Config) ->
     ok.
 
 configuration() ->
-    [forgotten || Components <- enemy, is_tuple(fun art/0)].
+    [forgotten || _Components <- enemy, is_tuple(fun art/0)].
 
 art() ->
  creating.
@@ -663,8 +673,25 @@ cover_letrec_effect(_Config) ->
         Any ->
             #{k := {{tag,42},<<42:16>>}} = Any
     end,
+
+    _ = catch cover_letrec_effect_1(),
+
     ok.
 
+cover_letrec_effect_1() ->
+    try
+        _ = catch ""
+    after
+        case any_atom of
+            31 when any_atom, force ->
+                true
+        end,
+        case "RG" of
+            1 when car, cdr, 3; 3, 4 ->
+                false
+        end
+    end.
+
 receive_effect(_Config) ->
     self() ! whatever,
     {} = do_receive_effect(),
