@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2003-2021. All Rights Reserved.
+%% Copyright Ericsson AB 2003-2022. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -73,17 +73,17 @@ errors(Config) when is_list(Config) ->
     Foo = #foo{a=1,b=2,c=3,d=4},
     #foo{a=19,b=42,c=3,d=4} = update_foo(Foo, 19, 42),
 
-    {'EXIT',{{badrecord,bar},_}} = (catch update_foo_bar(Foo, 19)),
-    {'EXIT',{{badrecord,bar},_}} = (catch update_foo_bar(Foo, 19, 35)),
-    {'EXIT',{{badrecord,bar},_}} = (catch update_foo_bar(Foo, 19, 35, 17)),
-    {'EXIT',{{badrecord,bar},_}} = (catch update_foo_bar(Foo, 19, 35, 17, 42)),
+    {'EXIT',{{badrecord,Foo},_}} = (catch update_foo_bar(Foo, 19)),
+    {'EXIT',{{badrecord,Foo},_}} = (catch update_foo_bar(Foo, 19, 35)),
+    {'EXIT',{{badrecord,Foo},_}} = (catch update_foo_bar(Foo, 19, 35, 17)),
+    {'EXIT',{{badrecord,Foo},_}} = (catch update_foo_bar(Foo, 19, 35, 17, 42)),
 
-    {'EXIT',{{badrecord,barf},_}} = (catch update_foo_barf(Foo, 19)),
-    {'EXIT',{{badrecord,barf},_}} = (catch update_foo_barf(Foo, 19, 35)),
-    {'EXIT',{{badrecord,barf},_}} = (catch update_foo_barf(Foo, 19, 35, 17)),
-    {'EXIT',{{badrecord,barf},_}} = (catch update_foo_barf(Foo, 19, 35, 17, 42)),
-    {'EXIT',{{badrecord,barf},_}} = (catch update_foo_barf(Foo, 19,
-								 35, 17, 42, -2)),
+    {'EXIT',{{badrecord,Foo},_}} = (catch update_foo_barf(Foo, 19)),
+    {'EXIT',{{badrecord,Foo},_}} = (catch update_foo_barf(Foo, 19, 35)),
+    {'EXIT',{{badrecord,Foo},_}} = (catch update_foo_barf(Foo, 19, 35, 17)),
+    {'EXIT',{{badrecord,Foo},_}} = (catch update_foo_barf(Foo, 19, 35, 17, 42)),
+    {'EXIT',{{badrecord,Foo},_}} = (catch update_foo_barf(Foo, 19,
+                                                          35, 17, 42, -2)),
 
     ok.
 
