@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 2003-2020. All Rights Reserved.
+%% Copyright Ericsson AB 2003-2022. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -21,14 +21,16 @@
 
 -export([all/0, suite/0,groups/0,init_per_suite/1, end_per_suite/1, 
 	 init_per_group/2,end_per_group/2,basic/1,lean_throw/1,
-	 try_of/1,try_after/1,%after_bind/1,
+	 try_of/1,try_after/1,
 	 catch_oops/1,after_oops/1,eclectic/1,rethrow/1,
 	 nested_of/1,nested_catch/1,nested_after/1,
 	 nested_horrid/1,last_call_optimization/1,bool/1,
-	 plain_catch_coverage/1,andalso_orelse/1,get_in_try/1,
+	 andalso_orelse/1,get_in_try/1,
 	 hockey/1,handle_info/1,catch_in_catch/1,grab_bag/1,
          stacktrace/1,nested_stacktrace/1,raise/1,
          no_return_in_try_block/1,
+         expression_export/1,
+         throw_opt_crash/1,
          coverage/1]).
 
 -include_lib("common_test/include/ct.hrl").
@@ -43,10 +45,12 @@ groups() ->
       [basic,lean_throw,try_of,try_after,catch_oops,
        after_oops,eclectic,rethrow,nested_of,nested_catch,
        nested_after,nested_horrid,last_call_optimization,
-       bool,plain_catch_coverage,andalso_orelse,get_in_try,
+       bool,andalso_orelse,get_in_try,
        hockey,handle_info,catch_in_catch,grab_bag,
        stacktrace,nested_stacktrace,raise,
-       no_return_in_try_block,coverage]}].
+       no_return_in_try_block,expression_export,
+       throw_opt_crash,
+       coverage]}].
 
 
 init_per_suite(Config) ->
@@ -117,7 +121,7 @@ basic(Conf) when is_list(Conf) ->
     %% Try/of
     ok = try V of
 	     {a,variable} -> ok
-	 catch nisse -> erro
+	 catch nisse -> error
 	 end,
 
     %% Unmatchable clauses.
@@ -203,84 +207,125 @@ try_of_1(X) ->
              {caught,{Class,Reason}}
     end.
 
-
-
 try_after(Conf) when is_list(Conf) ->
+    try_after_1(fun try_after_basic/2),
+    try_after_1(fun try_after_catch/2),
+    try_after_1(fun try_after_complex/2),
+    try_after_1(fun try_after_fun/2),
+    try_after_1(fun try_after_letrec/2),
+    try_after_1(fun try_after_protect/2),
+    try_after_1(fun try_after_receive/2),
+    try_after_1(fun try_after_receive_timeout/2),
+    try_after_1(fun try_after_try/2),
+    ok.
+
+try_after_1(TestFun) ->
     {{ok,[some,value],undefined},finalized} =
-	try_after_1({value,{ok,[some,value]}},finalized),
+        TestFun({value,{ok,[some,value]}},finalized),
     {{error,badarith,undefined},finalized} =
-	try_after_1({'div',{1,0}},finalized),
+        TestFun({'div',{1,0}},finalized),
     {{error,badarith,undefined},finalized} =
-	try_after_1({'add',{1,a}},finalized),
+        TestFun({'add',{1,a}},finalized),
     {{error,badarg,undefined},finalized} =
-	try_after_1({'abs',a},finalized),
+        TestFun({'abs',a},finalized),
     {{error,[the,{reason}],undefined},finalized} =
-	try_after_1({error,[the,{reason}]},finalized),
+        TestFun({error,[the,{reason}]},finalized),
     {{throw,{thrown,[reason]},undefined},finalized} =
-	try_after_1({throw,{thrown,[reason]}},finalized),
+        TestFun({throw,{thrown,[reason]}},finalized),
     {{exit,{exited,{reason}},undefined},finalized} =
-	try_after_1({exit,{exited,{reason}}},finalized),
+        TestFun({exit,{exited,{reason}}},finalized),
     {{error,function_clause,undefined},finalized} =
-	try_after_1(function_clause,finalized),
+        TestFun(function_clause,finalized),
     ok =
-	try try_after_1({'add',{1,1}}, finalized)
+        try
+            TestFun({'add',{1,1}}, finalized)
         catch
             error:{try_clause,2} -> ok
-	end,
+        end,
     finalized = erase(try_after),
     ok =
-        try try foo({exit,[reaso,{n}]})
-            after put(try_after, finalized)
+        try
+            try
+                foo({exit,[reaso,{n}]})
+            after
+                put(try_after, finalized)
             end
         catch
             exit:[reaso,{n}] -> ok
         end,
     ok.
 
-try_after_1(X, Y) ->
+-define(TRY_AFTER_TESTCASE(Block),
     erase(try_after),
     Try =
         try foo(X) of
-	    {ok,Value} -> {ok,Value,get(try_after)}
+            {ok,Value} -> {ok,Value,get(try_after)}
         catch
-	    Reason -> {throw,Reason,get(try_after)};
-	    error:Reason -> {error,Reason,get(try_after)};
-	    exit:Reason ->  {exit,Reason,get(try_after)}
+            Reason -> {throw,Reason,get(try_after)};
+            error:Reason -> {error,Reason,get(try_after)};
+            exit:Reason ->  {exit,Reason,get(try_after)}
         after
-	    put(try_after, Y)
+            Block,
+            put(try_after, Y)
         end,
-    {Try,erase(try_after)}.
+    {Try,erase(try_after)}).
 
+try_after_basic(X, Y) ->
+    ?TRY_AFTER_TESTCASE(ok).
 
+try_after_catch(X, Y) ->
+    ?TRY_AFTER_TESTCASE((catch put(try_after, Y))).
 
--ifdef(begone).
+try_after_complex(X, Y) ->
+    %% Large 'after' block, going above the threshold for wrapper functions.
+    ?TRY_AFTER_TESTCASE(case get(try_after) of
+                            unreachable_0 -> dummy:unreachable_0();
+                            unreachable_1 -> dummy:unreachable_1();
+                            unreachable_2 -> dummy:unreachable_2();
+                            unreachable_3 -> dummy:unreachable_3();
+                            unreachable_4 -> dummy:unreachable_4();
+                            unreachable_5 -> dummy:unreachable_5();
+                            unreachable_6 -> dummy:unreachable_6();
+                            unreachable_7 -> dummy:unreachable_7();
+                            unreachable_8 -> dummy:unreachable_8();
+                            unreachable_9 -> dummy:unreachable_9();
+                            _ -> put(try_after, Y)
+                        end).
 
-after_bind(Conf) when is_list(Conf) ->
-    V = [make_ref(),self()|value],
-    {value,{value,V}} =
-	after_bind_1({value,V}, V, {value,V}),
-    ok.
+try_after_fun(X, Y) ->
+    ?TRY_AFTER_TESTCASE((fun() -> ok end)()).
 
-after_bind_1(X, V, Y) ->
-    try
-        Try =
-            try foo(X) of
-                V -> value
-            catch
-                C1:V -> {caught,C1}
-            after
-                After = foo(Y)
-	    end,
-        {Try,After} 
-    of
-        V -> {value,V}
-    catch
-        C:D -> {caught,{C,D}}
-    end.
+try_after_letrec(X, Y) ->
+    List = lists:duplicate(100, ok),
+    ?TRY_AFTER_TESTCASE([L || L <- List]).
 
--endif.
+try_after_protect(X, Y) ->
+    ?TRY_AFTER_TESTCASE(case get(try_after) of
+                            N when element(52, N) < 32 -> ok;
+                            _ -> ok
+                        end).
 
+try_after_receive(X, Y) ->
+    Ref = make_ref(),
+    self() ! Ref,
+    ?TRY_AFTER_TESTCASE(receive
+                            Ref -> Ref
+                        end).
 
+try_after_receive_timeout(X, Y) ->
+    Ref = make_ref(),
+    self() ! Ref,
+    ?TRY_AFTER_TESTCASE(receive
+                            Ref -> Ref
+                        after 1000 -> ok
+                        end).
+
+try_after_try(X, Y) ->
+    ?TRY_AFTER_TESTCASE(try
+                            put(try_after, Y)
+                        catch
+                            _ -> ok
+                        end).
 
 catch_oops(Conf) when is_list(Conf) ->
     V = {v,[a,l|u],{e},self()},
@@ -308,11 +353,19 @@ catch_oops_1(X) ->
 
 after_oops(Conf) when is_list(Conf) ->
     V = {self(),make_ref()},
+
     {{value,V},V} = after_oops_1({value,V}, {value,V}),
     {{exit,V},V} = after_oops_1({exit,V}, {value,V}),
     {{error,V},undefined} = after_oops_1({value,V}, {error,V}),
     {{error,function_clause},undefined} =
-	after_oops_1({exit,V}, function_clause),
+        after_oops_1({exit,V}, function_clause),
+
+    {{value,V},V} = after_oops_2({value,V}, {value,V}),
+    {{exit,V},V} = after_oops_2({exit,V}, {value,V}),
+    {{error,V},undefined} = after_oops_2({value,V}, {error,V}),
+    {{error,function_clause},undefined} =
+        after_oops_2({exit,V}, function_clause),
+
     ok.
 
 after_oops_1(X, Y) ->
@@ -328,7 +381,25 @@ after_oops_1(X, Y) ->
         end,
     {Try,erase(after_oops)}.
 
-
+after_oops_2(X, Y) ->
+    %% GH-4859: `raw_raise` never got an edge to its catch block, making
+    %% try/catch optimization unsafe.
+    erase(after_oops),
+    Try =
+        try
+            try
+                foo(X)
+            catch E:R:S ->
+                erlang:raise(E, R, S)
+            after
+                put(after_oops, foo(Y))
+            end
+        of
+            V -> {value,V}
+        catch
+            C:D -> {C,D}
+        end,
+    {Try,erase(after_oops)}.
 
 eclectic(Conf) when is_list(Conf) ->
     V = {make_ref(),3.1415926535,[[]|{}]},
@@ -904,18 +975,10 @@ do_bool(A0, B) ->
 	    error
     end.
 
-plain_catch_coverage(Config) when is_list(Config) ->
-    %% Cover some code in beam_block:alloc_may_pass/1.
-    {a,[42]} = do_plain_catch_list(42).
-
-do_plain_catch_list(X) ->
-    B = [X],
-    catch id({a,B}).
-
 andalso_orelse(Config) when is_list(Config) ->
     {2,{a,42}} = andalso_orelse_1(true, {a,42}),
     {b,{b}} = andalso_orelse_1(false, {b}),
-    {catched,no_tuple} = andalso_orelse_1(false, no_tuple),
+    {caught,no_tuple} = andalso_orelse_1(false, no_tuple),
 
     ok = andalso_orelse_2({type,[a]}),
     also_ok = andalso_orelse_2({type,[]}),
@@ -931,7 +994,7 @@ andalso_orelse_1(A, B) ->
 		 element(1, B)
 	 end
      catch error:_ ->
-	     catched
+	     caught
      end,B}.
 
 andalso_orelse_2({Type,Keyval}) ->
@@ -1141,7 +1204,6 @@ stacktrace(_Config) ->
         error:{badmatch,_}:Stk2 ->
             [{?MODULE,stacktrace_2,0,_},
              {?MODULE,stacktrace,1,_}|_] = Stk2,
-            [] = erlang:get_stacktrace(),
             ok
     end,
 
@@ -1149,7 +1211,6 @@ stacktrace(_Config) ->
         stacktrace_3(a, b)
     catch
         error:function_clause:Stk3 ->
-            [] = erlang:get_stacktrace(),
             case lists:module_info(native) of
                 false ->
                     [{lists,prefix,[a,b],_}|_] = Stk3;
@@ -1161,7 +1222,7 @@ stacktrace(_Config) ->
     try
         throw(x)
     catch
-        throw:x:IntentionallyUnused ->
+        throw:x:_IntentionallyUnused ->
             ok
     end.
 
@@ -1170,7 +1231,6 @@ stacktrace_1(X, C1, Y) ->
             C1 -> value1
         catch
             C1:D1:Stk1 ->
-                [] = erlang:get_stacktrace(),
                 {caught1,D1,Stk1}
         after
             foo(Y)
@@ -1178,7 +1238,6 @@ stacktrace_1(X, C1, Y) ->
         V2 -> {value2,V2}
     catch
         C2:D2:Stk2 ->
-            [] = erlang:get_stacktrace(),
             {caught2,{C2,D2},Stk2}
     end.
 
@@ -1346,7 +1405,9 @@ test_raise_4(Expr) ->
     try
         do_test_raise_4(Expr)
     catch
-        exit:{exception,C,E,Stk}:Stk ->
+        exit:{exception,C,E,StkTerm}:Stk ->
+            %% it's not allowed to do the matching directly in the clause head
+            true = (Stk =:= StkTerm),
             try
                 Expr()
             catch
@@ -1387,8 +1448,106 @@ no_return_in_try_block_1(H) ->
 
 no_return() -> throw(no_return).
 
-coverage(_Config) ->
+expression_export(_Config) ->
+    42 = expr_export_1(),
+    42 = expr_export_2(),
+
+    42 = expr_export_3(fun() -> bar end),
+    beer = expr_export_3(fun() -> pub end),
+    {error,failed} = expr_export_3(fun() -> error(failed) end),
+    is_42 = expr_export_3(fun() -> 42 end),
+    no_good = expr_export_3(fun() -> bad end),
+
+    <<>> = expr_export_4(<<1:32>>),
+    <<"abcd">> = expr_export_4(<<2:32,"abcd">>),
+    no_match = expr_export_4(<<0:32>>),
+    no_match = expr_export_4(<<777:32>>),
+
+    {1,2,3} = expr_export_5(),
+    ok.
+
+expr_export_1() ->
+    try Bar = 42 of
+        _ -> Bar
+    after
+        ok
+    end.
+
+expr_export_2() ->
+    try Bar = 42 of
+        _ -> Bar
+    catch
+        _:_ ->
+            error
+    end.
+
+expr_export_3(F) ->
+    try
+        Bar = 42,
+        F()
+    of
+        bar -> Bar;
+        pub -> beer;
+        Bar -> is_42;
+        _ -> no_good
+    catch
+        error:Reason ->
+            {error,Reason}
+    end.
+
+expr_export_4(Bin) ->
+    try
+        SzSz = id(32),
+        Bin
+    of
+        <<Sz:SzSz,Tail:(4*Sz-4)/binary>> -> Tail;
+        <<_/binary>> -> no_match
+    after
+        ok
+    end.
+
+expr_export_5() ->
+    try
+        X = 1,
+        Z = 3,
+        Y = 2
+    of
+        2 -> {X,Y,Z}
+    after
+        ok
+    end.
+
+%% GH-4953: Type inference in throw optimization could crash in rare
+%% circumstances when a thrown type conflicted with one that was matched in
+%% a catch clause.
+throw_opt_crash(_Config) ->
+    try
+        throw_opt_crash_1(id(false), {pass, id(b), id(c)}),
+        throw_opt_crash_1(id(false), {crash, id(b)}),
+        ok
+    catch
+        throw:{pass, B, C} ->
+            {error, gurka, {B, C}};
+        throw:{beta, B, C} ->
+            {error, gaffel, {B, C}};
+        throw:{gamma, B, C} ->
+            {error, grammofon, {B, C}}
+    end.
+
+throw_opt_crash_1(true, {_, _ ,_}=Term) ->
+    throw(Term);
+throw_opt_crash_1(true, {_, _}=Term) ->
+    throw(Term);
+throw_opt_crash_1(false, _Term) ->
+    ok.
+
+coverage(Config) ->
     {'EXIT',{{badfun,true},[_|_]}} = (catch coverage_1()),
+    ok = coverage_ssa_throw(),
+    error = coverage_pre_codegen(),
+    {a,[42]} = do_plain_catch_list(42),
+    cover_raise(Config),
+
     ok.
 
 %% Cover some code in beam_trim.
@@ -1406,5 +1565,111 @@ coverage_1() ->
             true
     end.
 
+%% Cover some code in beam_ssa_throw.
+coverage_ssa_throw() ->
+    cst_trivial(),
+    cst_raw(),
+    cst_stacktrace(),
+    cst_types(),
+
+    ok.
+
+cst_trivial() ->
+    %% never inspects stacktrace
+    try
+        cst_trivial_1()
+    catch
+        _C:_R:_S ->
+            ok
+    end.
+
+cst_trivial_1() -> throw(id(gurka)).
+
+cst_types() ->
+    %% type tests
+    try
+        cst_types_1()
+    catch
+        throw:Val when is_atom(Val);
+                       is_bitstring(Val);
+                       is_binary(Val);
+                       is_float(Val);
+                       is_integer(Val);
+                       is_list(Val);
+                       is_map(Val);
+                       is_number(Val);
+                       is_tuple(Val) ->
+            ok;
+        throw:[_|_]=Cons when hd(Cons) =/= gurka;
+                              tl(Cons) =/= gaffel ->
+            %% is_nonempty_list, get_hd, get_tl
+            ok;
+        throw:Tuple when tuple_size(Tuple) < 5 ->
+            %% tuple_size
+            ok
+    end.
+
+cst_types_1() -> throw(id(gurka)).
+
+cst_stacktrace() ->
+    %% build_stacktrace
+    try
+        cst_stacktrace_1()
+    catch
+        throw:gurka ->
+            ok;
+        _C:_R:Stack ->
+            id(Stack),
+            ok
+    end.
+
+cst_stacktrace_1() -> throw(id(gurka)).
+
+cst_raw() ->
+    %% raw_raise
+    try
+        cst_raw_1()
+    catch
+        throw:gurka ->
+            ok;
+        _C:_R:Stack ->
+            erlang:raise(error, dummy, Stack)
+    end.
+
+cst_raw_1() -> throw(id(gurka)).
+
+%% Cover some code in beam_ssa_pre_codegen.
+coverage_pre_codegen() ->
+    try not (catch 22) of
+        true ->
+            ok
+    catch
+        _:_ ->
+            error
+    end.
+
+%% Cover some code in beam_block:alloc_may_pass/1.
+do_plain_catch_list(X) ->
+    B = [X],
+    catch id({a,B}).
+
+cover_raise(Config) ->
+    UncertainClass = uncertain_class(Config),
+    badarg = erlang:raise(UncertainClass, reason, []),
+    BadClass = bad_class(Config),
+    badarg = erlang:raise(BadClass, reason, []),
+    ok.
+
+uncertain_class(Config) ->
+    case Config of
+        [never_ever] ->  error;
+        _ -> undefined_class
+    end.
+
+bad_class(Config) ->
+    case Config of
+        [never_ever] -> bad_class;
+        _ -> also_bad
+    end.
 
 id(I) -> I.
