@@ -242,7 +242,6 @@ create_window(Wx, #options{init_plts = InitPltFiles} = DialyzerOptions) ->
 	      {?WARN_OPAQUE, ?menuID_WARN_OPAQUE},
 	      {?WARN_FAILING_CALL, ?menuID_WARN_FAIL_FUN_CALLS},
 	      {?WARN_CALLGRAPH, ?menuID_WARN_UNEXPORTED_FUN},
-              {?WARN_RACE_CONDITION, ?menuID_WARN_RACE_CONDITIONS},
 	      %% For contracts.
 	      {?WARN_CONTRACT_TYPES,?menuID_WARN_WRONG_CONTRACTS},
 	      {?WARN_CONTRACT_SYNTAX, ?menuID_WARN_CONTRACT_SYNTAX}
@@ -313,8 +312,6 @@ createWarningsMenu() ->
 		 "Functions of no return"),
   addCheckedItem(WarningsMenu, ?menuID_WARN_UNEXPORTED_FUN,
 		 "Call to unexported function"),
-  _ = wxMenu:appendCheckItem(WarningsMenu, ?menuID_WARN_RACE_CONDITIONS,
-			     "Possible race conditions"),
   addCheckedItem(WarningsMenu, ?menuID_WARN_WRONG_CONTRACTS, "Wrong contracts"),
   addCheckedItem(WarningsMenu, ?menuID_WARN_CONTRACT_SYNTAX,
 		 "Wrong contract syntax"),
@@ -466,28 +463,33 @@ gui_loop(#gui_state{backend_pid = BackendPid, doc_plt = DocPlt,
       gui_loop(State);
     %% ----- Analysis -----
     {BackendPid, ext_calls, ExtCalls} ->
+      ExtCalls1 = lists:usort([MFA || {MFA, _FileLocation} <- ExtCalls]),
+      Map = fun({M,F,A}) -> io_lib:format("\t~tp:~tp/~p",[M,F,A]) end,
+      ExtCallString = lists:join("\n", lists:map(Map, ExtCalls1)),
       Msg = io_lib:format("The following functions are called "
 			  "but type information about them is not available.\n"
 			  "The analysis might get more precise by including "
-			  "the modules containing these functions:\n\n\t~tp\n",
-			  [ExtCalls]),
+			  "the modules containing these functions:\n\n~ts\n",
+			  [ExtCallString]),
       free_editor(State,"Analysis Done",  Msg),
       gui_loop(State);
     {BackendPid, ext_types, ExtTypes} ->
-      Map = fun({M,F,A}) -> io_lib:format("~tp:~tp/~p",[M,F,A]) end,
-      ExtTypeString = string:join(lists:map(Map, ExtTypes), "\n"),
+      ExtTypes1 = lists:usort([MFA || {MFA, _FileLocation} <- ExtTypes]),
+      Map = fun({M,F,A}) -> io_lib:format("\t~tp:~tp/~p",[M,F,A]) end,
+      ExtTypeString = lists:join("\n", lists:map(Map, ExtTypes1)),
       Msg = io_lib:format("The following remote types are being used "
 			  "but information about them is not available.\n"
 			  "The analysis might get more precise by including "
 			  "the modules containing these types and making sure "
-			  "that they are exported:\n~ts\n", [ExtTypeString]),
+			  "that they are exported:\n\n~ts\n",
+                          [ExtTypeString]),
       free_editor(State, "Analysis done", Msg),
       gui_loop(State);
     {BackendPid, log, LogMsg} ->
       update_editor(Log, LogMsg),
       gui_loop(State);
     {BackendPid, warnings, Warns} ->
-      SortedWarns = lists:keysort(2, Warns),  %% Sort on file/line
+      SortedWarns = lists:keysort(2, Warns),  %% Sort on file/location
       NewState = add_warnings(State, SortedWarns),
       gui_loop(NewState);
     {BackendPid, cserver, CServer, Plt} ->
@@ -498,10 +500,10 @@ gui_loop(#gui_state{backend_pid = BackendPid, doc_plt = DocPlt,
 	end,
       ExplanationPid = spawn_link(Fun),
       gui_loop(State#gui_state{expl_pid = ExplanationPid});
-    {BackendPid, done, NewPlt, NewDocPlt} ->
+    {BackendPid, done, _NewPlt, NewDocPlt} ->
       message(State, "Analysis done"),
-      dialyzer_plt:delete(NewPlt),
       config_gui_stop(State),
+      dialyzer_plt:delete(State#gui_state.doc_plt),
       gui_loop(State#gui_state{doc_plt = NewDocPlt});
     {'EXIT', BackendPid, {error, Reason}} ->
       free_editor(State, ?DIALYZER_ERROR_TITLE, Reason),
@@ -638,7 +640,7 @@ output_sms(#gui_state{frame = Frame}, Title, Message, Type) ->
 
 free_editor(#gui_state{gui = Wx, frame = Frame}, Title, Contents0) ->
   Contents = lists:flatten(Contents0),
-  Tokens = string:tokens(Contents, "\n"),
+  Tokens = string:lexemes(Contents, "\n"),
   NofLines = length(Tokens),
   LongestLine = lists:max([length(X) || X <- Tokens]),
   Height0 = NofLines * 25 + 80,
@@ -1093,7 +1095,7 @@ macro_loop(Options, Win, Box, MacroText, TermText, Frame) ->
 	    Fun = 
 	      fun(X) ->
 		  Val = wxControlWithItems:getString(Box,X),
-		  [MacroName|_] = re:split(Val, " ", [{return, list}]),
+		  [MacroName|_] = re:split(Val, " ", [{return, list}, unicode]),
 		  list_to_atom(MacroName)
 	      end,
 	    Delete = [Fun(X) || X <- List],
@@ -1135,7 +1137,10 @@ handle_help(State, Title, Txt) ->
 add_warnings(#gui_state{warnings_box = WarnBox,
 			rawWarnings = RawWarns} = State, Warnings) ->
   NewRawWarns = RawWarns ++ Warnings,
-  WarnList = [dialyzer:format_warning(W) -- "\n" || W <- NewRawWarns],
+  %% The indentation cannot be turned off.
+  %% The column numbers of locations are always displayed.
+  WarnList = [string:trim(dialyzer:format_warning(W), trailing) ||
+               W <- NewRawWarns],
   wxListBox:set(WarnBox, WarnList),
   State#gui_state{rawWarnings = NewRawWarns}.
   
