@@ -13,6 +13,7 @@
 
 -define(suite_suffix, "_SUITE").
 -define(data_folder, "_data").
+-define(excludes, ["dialyzer_cl", "plt"]). % _SUITE_data dirs that we shouldn't automatically make suites for
 -define(suite_data, ?suite_suffix ++ ?data_folder).
 -define(erlang_extension, ".erl").
 -define(output_file_mode, write).
@@ -20,7 +21,6 @@
 -define(input_files_directory, "src").
 -define(result_files_directory, "results").
 -define(plt_filename,"dialyzer_plt").
--define(home_plt_filename,".dialyzer_plt").
 -define(plt_lockfile,"plt_lock").
 -define(required_modules, [erts, kernel, stdlib]).
 
@@ -45,7 +45,7 @@ check_plt(OutDir) ->
 	{error, _ } ->
 	    io:format("No plt found in test run directory!"),
 	    PltLockFile = filename:join(OutDir, ?plt_lockfile),
-	    case file:read_file_info(PltLockFile) of 
+	    case file:read_file_info(PltLockFile) of
 		{ok, _} ->
 		    explain_fail_with_lock(),
 		    fail;
@@ -89,14 +89,13 @@ explain_fail_with_lock() ->
 
 obtain_plt(PltFilename) ->
     io:format("Obtaining plt:"),
-    HomeDir = os:getenv("HOME"),
-    HomePlt = filename:join(HomeDir, ?home_plt_filename),
+    InitPlt = dialyzer_plt:get_default_plt(),
     io:format("Will try to use ~s as a starting point and add otp apps ~w.",
-	      [HomePlt, ?required_modules]),
+	      [InitPlt, ?required_modules]),
     try dialyzer:run([{analysis_type, plt_add},
 		      {apps, ?required_modules},
 		      {output_plt, PltFilename},
-		      {init_plt, HomePlt}]) of
+		      {init_plt, InitPlt}]) of
 	[] ->
 	    io:format("Successfully added everything!"),
 	    ok
@@ -178,7 +177,7 @@ fix_options([{pa, Path} | Rest], Dir, Acc) ->
 	true       -> fix_options(Rest, Dir, Acc);
 	{error, _} -> erlang:error("Bad directory for pa: " ++ Path)
     end;
-fix_options([{DirOption, RelativeDirs} | Rest], Dir, Acc) 
+fix_options([{DirOption, RelativeDirs} | Rest], Dir, Acc)
   when DirOption =:= include_dirs ;
        DirOption =:= files_rec ;
        DirOption =:= files ->
@@ -215,15 +214,21 @@ get_suites(Dir) ->
 	{error, _} -> [];
 	{ok, Filenames} ->
 	    FullFilenames = [filename:join(Dir, F) || F <-Filenames ],
-	    Dirs = [suffix(filename:basename(F), ?suite_data) ||
+	    Dirs = [is_suite_data(filename:basename(F), ?suite_data) ||
 		       F <- FullFilenames,
 		       file_utils:file_type(F) =:= {ok, 'directory'}],
 	    [S || {yes, S} <- Dirs]
     end.
 
-suffix(String, Suffix) ->
+is_suite_data(String, Suffix) ->
     case string:split(String, Suffix, trailing) of
-	[Prefix,[]] -> {yes, Prefix};
+        [Prefix,[]] ->
+            case lists:member(Prefix, ?excludes) of
+                true ->
+                   no;
+                false ->
+                   {yes, Prefix}
+            end;
         _ -> no
     end.
 
