@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  * 
- * Copyright Ericsson AB 2004-2016. All Rights Reserved.
+ * Copyright Ericsson AB 2004-2021. All Rights Reserved.
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -35,9 +35,3 @@ ei_set_compat_rel(unsigned rel)
 {
     compat_rel = rel;
 }
-
-int ei_internal_use_21_bitstr_expfun(void)
-{
-    return compat_rel < 22;
-}
-
