@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1997-2017. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2022. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -24,7 +24,7 @@
 	 lookup_mime_default/3, reason_phrase/1, message/3, rfc1123_date/0,
 	 rfc1123_date/1, day/1, month/1,
 	 flatlength/1, split_path/1, split_script_path/1, 
-	 suffix/1, split/3, uniq/1,
+	 suffix/1, strip_extension_dot/1, split/3, uniq/1,
 	 make_name/2,make_name/3,make_name/4,strip/1,
 	 hexlist_to_integer/1,integer_to_hexlist/1,
 	 convert_request_date/1,create_etag/1,create_etag/2,
@@ -33,6 +33,14 @@
 	 dir_validate/2, file_validate/2, mime_type_validate/1, 
 	 mime_types_validate/1, custom_date/0, error_log/2]).
 
+-deprecated({flatlength, 1, "use erlang:iolist_size/1 instead"}).
+-deprecated({hexlist_to_integer, 1, "use erlang:list_to_integer/2 with base 16 instead"}).
+-deprecated({integer_to_hexlist, 1, "use erlang:integer_to_list/2 with base 16 instead"}).
+-deprecated({strip, 1, "use string:trim/1 instead"}).
+-deprecated({suffix, 1, "use filename:extension/1 and string:trim/2 instead"}).
+-deprecated({decode_hex, 1, "use uri_string:unquote function instead"}).
+-deprecated({encode_hex, 1, "use uri_string:quote function instead"}).
+
 -compile({nowarn_deprecated_function, [{http_uri, encode, 1}]}).
 -compile({nowarn_deprecated_function, [{http_uri, decode, 1}]}).
 -export([encode_hex/1, decode_hex/1]).
@@ -121,6 +129,7 @@ reason_phrase(304) ->   "Not Modified" ;
 reason_phrase(305) ->   "Use Proxy" ;
 reason_phrase(306) ->   "(unused)" ;
 reason_phrase(307) ->   "Temporary Redirect" ;
+reason_phrase(308) ->   "Permanent Redirect" ;
 reason_phrase(400) ->   "Bad Request";
 reason_phrase(401) ->   "Unauthorized";
 reason_phrase(402) ->   "Payment Required";
@@ -254,7 +263,7 @@ convert_rfc850_date(DateStr) ->
 
 convert_rfc850_date([D1,D2,_,
 		     M,O,N,_,
-		     Y1,Y2|_Rest],[H1,H2,_Col,M1,M2,_Col,S1,S2|_Rest2])->    
+		     Y1,Y2|_Rest],[H1,H2,Col,M1,M2,Col,S1,S2|_Rest2])->    
     Year=list_to_integer([50,48,Y1,Y2]),
     Day=list_to_integer([D1,D2]),
     Month = http_util:convert_month([M,O,N]),
@@ -263,12 +272,12 @@ convert_rfc850_date([D1,D2,_,
     Sec=list_to_integer([S1,S2]),
     {ok,{{Year,Month,Day},{Hour,Min,Sec}}}.
 
-convert_ascii_date([_D,_A,_Y,_SP,
-		    M,O,N,_SP,
-		    D1,D2,_SP,
-		    H1,H2,_Col,
-		    M1,M2,_Col,
-		    S1,S2,_SP,
+convert_ascii_date([_D,_A,_Y,SP,
+		    M,O,N,SP,
+		    D1,D2,SP,
+		    H1,H2,Col,
+		    M1,M2,Col,
+		    S1,S2,SP,
 		    Y1,Y2,Y3,Y4| _Rest])->
     Year=list_to_integer([Y1,Y2,Y3,Y4]),
     Day=case D1 of 
@@ -283,12 +292,12 @@ convert_ascii_date([_D,_A,_Y,_SP,
     Sec=list_to_integer([S1,S2]),
     {ok,{{Year,Month,Day},{Hour,Min,Sec}}}.
 
-convert_rfc1123_date([_D,_A,_Y,_C,_SP,
-		      D1,D2,_SP,
-		      M,O,N,_SP,
-		      Y1,Y2,Y3,Y4,_SP,
-		      H1,H2,_Col,
-		      M1,M2,_Col,
+convert_rfc1123_date([_D,_A,_Y,_C,SP,
+		      D1,D2,SP,
+		      M,O,N,SP,
+		      Y1,Y2,Y3,Y4,SP,
+		      H1,H2,Col,
+		      M1,M2,Col,
 		      S1,S2|_Rest]) -> 
     Year=list_to_integer([Y1,Y2,Y3,Y4]),
     Day=list_to_integer([D1,D2]),
@@ -390,10 +399,11 @@ month(12) -> "Dec".
 %% decode_hex
 
 decode_hex(URI) ->
-    http_uri:decode(URI).
+    uri_string:unquote(URI).
 
 encode_hex(URI) ->
-    http_uri:encode(URI).
+    SafeChars = "!$()*", %% characters not encoded by deprecated http_uri:encode/1
+    uri_string:quote(URI, SafeChars).
 
 %% flatlength
 flatlength(List) ->
@@ -421,27 +431,12 @@ split_path(URI) ->
           query := Query} ->
             {Path, Query};
         #{path := Path} ->            
-            split_path(Path, [])
+            {Path, []}
     end.
 
 add_hashmark(Query, Fragment) ->
     Query ++ "#" ++ Fragment.
    
-split_path([],SoFar) ->
-    {lists:reverse(SoFar),[]};
-split_path([$/|Rest],SoFar) ->
-    Path=lists:reverse(SoFar),
-    case file:read_file_info(Path) of
-	{ok,FileInfo} when FileInfo#file_info.type =:= regular ->
-	    {Path,[$/|Rest]};
-	{ok, _FileInfo} ->
-	    split_path(Rest,[$/|SoFar]);
-	{error, _Reason} ->
-	    split_path(Rest,[$/|SoFar])
-    end;
-split_path([C|Rest],SoFar) ->
-    split_path(Rest,[C|SoFar]).
-
 %% split_script_path, URI has been decoded once when validate
 %% and should only be decoded once(RFC3986, 2.4).
 
@@ -454,10 +449,9 @@ split_script_path(URI) ->
             not_a_script;
         #{path := Path,
           query := Query} ->
-            {Script, PathInfo} = split_path(Path, []),
-            {Script, {PathInfo, Query}};
+            {Path, {[], Query}};
         #{path := Path} ->            
-            split_path(Path, [])
+            {Path, []}
     end.
 
 %% suffix
@@ -470,6 +464,14 @@ suffix(Path) ->
 	    tl(Extension)
     end.
 
+%% strip_extension_dot
+strip_extension_dot(Path) ->
+    case filename:extension(Path) of
+	[] ->
+	    [];
+	Extension ->
+	    tl(Extension)
+    end.
 
 %% strip
 strip(Value)->
@@ -550,7 +552,7 @@ search_and_replace(S,A,B) ->
 
 
 %%----------------------------------------------------------------------
-%% Converts  a string that constists of 0-9,A-F,a-f to a 
+%% Converts  a string that consists of 0-9,A-F,a-f to a 
 %% integer
 %%----------------------------------------------------------------------
 
