@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2004-2018. All Rights Reserved.
+%% Copyright Ericsson AB 2004-2021. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -34,7 +34,9 @@
 -define(NEW_PID_EXT,         $X).
 -define(NEW_PORT_EXT,        $Y).
 -define(NEWER_REFERENCE_EXT, $Z).
+-define(V4_PORT_EXT,        $x).
 
+-define(OLD_MAX_PIDS_PORTS, ((1 bsl 28) - 1)).
 
 -export([all/0, suite/0,groups/0,init_per_group/2,end_per_group/2,
 	 init_per_suite/1,
@@ -123,8 +125,6 @@ end_per_testcase(_Case, Config) ->
 %% Test cases
 %%
 
-pid_roundtrip(doc) -> [];
-pid_roundtrip(suite) -> [];
 pid_roundtrip(Config) when is_list(Config)->
     ThisNode = {node(), erlang:system_info(creation)},
     RemPids = [mk_pid({gurka@sallad, Cr}, Num, Ser)
@@ -136,8 +136,6 @@ pid_roundtrip(Config) when is_list(Config)->
 	     | RemPids],
 	    Config).
 
-fun_roundtrip(doc) -> [];
-fun_roundtrip(suite) -> [];
 fun_roundtrip(Config) when is_list(Config)->
     do_echo([fun(A, B) -> A + B end,
 	     fun(A) -> lists:reverse(A) end,
@@ -146,28 +144,30 @@ fun_roundtrip(Config) when is_list(Config)->
              fun ?MODULE:fun_roundtrip/1],
 	    Config).
 
-port_roundtrip(doc) -> [];
-port_roundtrip(suite) -> [];
 port_roundtrip(Config) when is_list(Config)->
     ThisNode = {node(), erlang:system_info(creation)},
     RemPorts = [mk_port({gurka@sallad, Cr}, Num)
 		|| Cr <- [1,2,3,4,16#adec0ded],
-		   Num <- [4711, 268435455]],
+		   Num <- [4711, 268435455]]
+    %% V4 ports
+        ++ [mk_port({gurka@sallad, Cr}, Num)
+            || Cr <- [17, 4711, 16#adec0ded],
+               Num <- [(1 bsl 47) bor (1 bsl 25),
+                       (1 bsl 57) bor (1 bsl 23)]],
     do_echo([hd(erlang:ports()),
 	     mk_port(ThisNode, 4711),
 	     mk_port(ThisNode, 268435455)
 	     | RemPorts],
 	    Config).
 
-ref_roundtrip(doc) -> [];
-ref_roundtrip(suite) -> [];
 ref_roundtrip(Config) when is_list(Config)->
     ThisNode = {node(), erlang:system_info(creation)},
     RemRefs = [mk_ref({gurka@sallad, Cr}, Words)
 	       || Cr <- [1,2,3,4,16#adec0ded],
 		  Words <- [[4711],
 			    [4711, 4711, 4711],
-			    [262143, 4294967295, 4294967295]]],
+			    [262143, 4294967295, 4294967295],
+                            [262143, 4294967295, 4294967294, 4294967293, 4294967292]]],
     do_echo([make_ref(),
 	     mk_ref(ThisNode, [4711]),
 	     mk_ref(ThisNode, [4711, 4711, 4711]),
@@ -175,8 +175,6 @@ ref_roundtrip(Config) when is_list(Config)->
 	     | RemRefs],
 	    Config).
 
-new_float(doc) -> [];
-new_float(suite) -> [];
 new_float(Config) when is_list(Config)->
     Two16 = float(1 bsl 16),
     X = math:sqrt(2),
@@ -185,8 +183,6 @@ new_float(Config) when is_list(Config)->
     io:format("~w", [Floats]),
     do_echo(Floats, Config).
 
-old_stuff(doc) -> [];
-old_stuff(suite) -> [];
 old_stuff(Config) when is_list(Config)->
     Terms = [0.0,math:sqrt(2)],
     OutTrans =
@@ -199,8 +195,6 @@ old_stuff(Config) when is_list(Config)->
 	end,
     do_echo(Terms, Config, OutTrans, InTrans).
 
-binary_roundtrip(doc) -> [];
-binary_roundtrip(suite) -> [];
 binary_roundtrip(Config) when is_list(Config) ->
     do_echo([<<17>>,
 	     <<1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17>>,
@@ -209,8 +203,6 @@ binary_roundtrip(Config) when is_list(Config) ->
 	     <<>>],
 	    Config).
 
-decompress_roundtrip(doc) -> [];
-decompress_roundtrip(suite) -> [];
 decompress_roundtrip(Config) when is_list(Config) ->
 	RandomBin = erlang:term_to_binary(lists:seq(1, 5 * 1024 * 1024)), % roughly 26MB
 	<<RandomBin1k:1024/binary,_/binary>> = RandomBin,
@@ -239,8 +231,6 @@ decompress_roundtrip(Config) when is_list(Config) ->
 	end,
     do_echo(Terms, Config, OutTrans, InTrans).
 
-compress_roundtrip(doc) -> [];
-compress_roundtrip(suite) -> [];
 compress_roundtrip(Config) when is_list(Config) ->
 	RandomBin = erlang:term_to_binary(lists:seq(1, 5 * 1024 * 1024)), % roughly 26MB
 	<<RandomBin1k:1024/binary,_/binary>> = RandomBin,
@@ -271,8 +261,6 @@ compress_roundtrip(Config) when is_list(Config) ->
 
 
 
-integer_roundtrip(doc) -> [];
-integer_roundtrip(suite) -> [];
 integer_roundtrip(Config) when is_list(Config) ->
     Xs = [1 bsl X || X <- [26,27,28,29,30,31,32,33,
 			   62,63,64,65,
@@ -311,8 +299,6 @@ bitlength(V) when is_integer(V) ->
 
 
 
-lists_roundtrip(doc) -> [];
-lists_roundtrip(suite) -> [];
 lists_roundtrip(Config) when is_list(Config) ->
     Ls = [lists:seq(1,10),
 	  lists:seq(11,17)++last_tail,
@@ -324,8 +310,6 @@ lists_roundtrip(Config) when is_list(Config) ->
 
 
 
-lists_roundtrip_2(doc) -> [];
-lists_roundtrip_2(suite) -> [];
 lists_roundtrip_2(Config) when is_list(Config) ->
     Ls = [{[a,b],tail},
 	  {[c,d,e],tail},
@@ -377,8 +361,6 @@ lists_roundtrip_2(Config) when is_list(Config) ->
 
 
 
-lists_iterator(doc) -> [];
-lists_iterator(suite) -> [];
 lists_iterator(Config) when is_list(Config) ->
     Ls = [["able ","was ","I ","ere ","I ","saw ","elba"]],
     do_echo(Ls, Config,
@@ -390,8 +372,6 @@ lists_iterator(Config) when is_list(Config) ->
 
 
 
-unicode(doc) -> [];
-unicode(suite) -> [];
 unicode(Config) when is_list(Config) ->
     S1 = "plain ascii",
     S2 = "iso-latin åäö ñ",
@@ -409,19 +389,19 @@ unicode(Config) when is_list(Config) ->
 		    case L of
 			X -> ok;
 			_ ->
-			    ?t:fail({mismatch,Out,In})
+			    ct:fail({mismatch,Out,In})
 		    end;
 		(Echoer, {L,Tag}=Out, {Echoer,X,Tag}=In) ->
 		    case unicode:characters_to_binary(L, utf8) of
 			X -> ok;
 			_ ->
-			    ?t:fail({mismatch,Out,In})
+			    ct:fail({mismatch,Out,In})
 		    end;
 		(Echoer, {L}=Out, {Echoer,X}=In) ->
 		    case L of
 			X -> ok;
 			_ ->
-			    ?t:fail({mismatch,Out,In})
+			    ct:fail({mismatch,Out,In})
 		    end
 	    end, ["unicode"]).
 
@@ -439,8 +419,6 @@ unicode_cp_gen(Cont) when is_function(Cont, 0) ->
 
 
 
-unicode_list_to_string(doc) -> [];
-unicode_list_to_string(suite) -> [];
 unicode_list_to_string(Config) when is_list(Config) ->
     do_echo(cp_gen(73), Config,
 	    fun ({L,_}) -> {self(),L,to_string_neg_int_list} end,
@@ -448,21 +426,19 @@ unicode_list_to_string(Config) when is_list(Config) ->
 		    case L of
 			X -> ok;
 			_ ->
-			    ?t:fail({mismatch,Out,In})
+			    ct:fail({mismatch,Out,In})
 		    end;
 		(Echoer, {L,valid}=Out, {Echoer,X,_}=In) ->
 		    B = unicode:characters_to_binary(L, unicode, {utf16,big}),
 		    case [-D || <<D:16/big>> <= B] of
 			X -> ok;
 			_ ->
-			    ?t:fail({mismatch,Out,In})
+			    ct:fail({mismatch,Out,In})
 		    end
 	    end).
 
 
 
-unicode_string_to_list(doc) -> [];
-unicode_string_to_list(suite) -> [];
 unicode_string_to_list(Config) when is_list(Config) ->
     do_echo(cp_gen(79), Config,
 	    fun ({L,_}) -> {self(),L,to_neg_int_list} end,
@@ -470,13 +446,13 @@ unicode_string_to_list(Config) when is_list(Config) ->
 		    case L of
 			X -> ok;
 			_ ->
-			    ?t:fail({mismatch,Out,In})
+			    ct:fail({mismatch,Out,In})
 		    end;
 		(Echoer, {L,valid}=Out, {Echoer,X,_}=In) ->
 		    case [-C || C <- L] of
 			X -> ok;
 			_ ->
-			    ?t:fail({mismatch,Out,In})
+			    ct:fail({mismatch,Out,In})
 		    end
 	    end, ["unicode"]).
 
@@ -584,11 +560,8 @@ cp_validity(UnicodeCP) ->
 
 
 
-connect(doc) -> [];
-connect(suite) -> [];
 connect(Config) when is_list(Config) ->
-    WD = filename:dirname(code:which(?MODULE)),
-    {ok,Other} = ?t:start_node(make_name(), slave, [{args,"-pa "++WD}]),
+    {ok,Peer,Other} = ?CT_PEER(),
     Action =
 	fun (Pid) ->
 		JName = node(Pid),
@@ -598,7 +571,7 @@ connect(Config) when is_list(Config) ->
 		    {Pid,Other,true} ->
 			ok;
 		    Unexpected1 ->
-			?t:fail({result,Unexpected1})
+			ct:fail({result,Unexpected1})
 		end,
 		Hidden = erlang:nodes(hidden),
 		Hidden = rpc:call(Other, erlang, nodes, [hidden]),
@@ -616,11 +589,12 @@ connect(Config) when is_list(Config) ->
 		    {Pid,Other,true} ->
 			ok;
 		    Unexpected2->
-			?t:fail({result,Unexpected2})
+			ct:fail({result,Unexpected2})
 		end,
 		Hidden = rpc:call(Other, erlang, nodes, [hidden])
 	end,
-    run_server(connection_server, Config, Action, []).
+    run_server(connection_server, Config, Action, []),
+    peer:stop(Peer).
 
 
 
@@ -678,7 +652,7 @@ echo_loop(Cont, Echoer, OutTrans, InTrans, TermAcc)
 	Other ->
 	    io:format("echo_server_terms unexpected ~p: receive ~P~n",
 		      [self(),Other,10]),
-	    ?t:fail({unexpected, Other})
+	    ct:fail({unexpected, Other})
     end,
     echo_loop(Cont(), Echoer, OutTrans, InTrans, []).
 
@@ -693,7 +667,7 @@ check_terms(Echoer, [Term | Rest]) ->
 	Other ->
 	    io:format("check_terms unexpected ~p: receive ~P~n",
 		      [self(),Other,10]),
-	    ?t:fail({unexpected, Other})
+	    ct:fail({unexpected, Other})
     end;
 check_terms(_, []) ->
     ok.
@@ -719,9 +693,9 @@ run_server(Server, Config, Action, ExtraArgs) ->
 	       end),
     receive
 	{Server, JName, Pid} ->
-	    ?t:format("~w: ~p (~p)~n",
+	    test_server:format("~w: ~p (~p)~n",
 		      [Server, Pid, node(Pid)]),
-	    ?t:format("nodes(hidden): ~p~n",
+	    test_server:format("nodes(hidden): ~p~n",
 		      [nodes(hidden)]),
 	    Action(Pid),
 	    Pid ! bye,
@@ -730,7 +704,7 @@ run_server(Server, Config, Action, ExtraArgs) ->
 		    ok
 	    end;
 	Other ->
-	    ?t:fail({unexpected,Other})
+	    ct:fail({unexpected,Other})
     end.
 
 %%
@@ -744,6 +718,18 @@ make_name() ->
 		 ++ "-" ++ integer_to_list(B)
 		 ++ "-" ++ integer_to_list(C)).
 
+uint64_be(Uint) when is_integer(Uint), 0 =< Uint, Uint < 1 bsl 64 ->
+    [(Uint bsr 56) band 16#ff,
+     (Uint bsr 48) band 16#ff,
+     (Uint bsr 40) band 16#ff,
+     (Uint bsr 32) band 16#ff,
+     (Uint bsr 24) band 16#ff,
+     (Uint bsr 16) band 16#ff,
+     (Uint bsr 8) band 16#ff,
+     Uint band 16#ff];
+uint64_be(Uint) ->
+    exit({badarg, uint64_be, [Uint]}).
+
 uint32_be(Uint) when is_integer(Uint), 0 =< Uint, Uint < 1 bsl 32 ->
     [(Uint bsr 24) band 16#ff,
      (Uint bsr 16) band 16#ff,
@@ -789,17 +775,24 @@ mk_pid({NodeNameExt, Creation}, Number, Serial) ->
 	    exit({unexpected_binary_to_term_result, Other})
     end.
 
-port_tag(Creation) when Creation =< 3 -> ?PORT_EXT;
-port_tag(_Creation) -> ?NEW_PORT_EXT.
+port_tag(Num, Creation) when 0 =< Num, Num =< ?OLD_MAX_PIDS_PORTS, Creation =< 3 ->
+    ?PORT_EXT;
+port_tag(Num, _Creation) when 0 =< Num, Num =< ?OLD_MAX_PIDS_PORTS ->
+    ?NEW_PORT_EXT;
+port_tag(_Num, _Creation) ->
+    ?V4_PORT_EXT.
 
 mk_port({NodeName, Creation}, Number) when is_atom(NodeName) ->
     <<?VERSION_MAGIC, NodeNameExt/binary>> = term_to_binary(NodeName),
     mk_port({NodeNameExt, Creation}, Number);
 mk_port({NodeNameExt, Creation}, Number) ->
     case catch binary_to_term(list_to_binary([?VERSION_MAGIC,
-					      port_tag(Creation),
+					      port_tag(Number, Creation),
 					      NodeNameExt,
-					      uint32_be(Number),
+					      case Number > ?OLD_MAX_PIDS_PORTS of
+						  true -> uint64_be(Number);
+						  false -> uint32_be(Number)
+					      end,
 					      enc_creation(Creation)])) of
 	Port when is_port(Port) ->
 	    Port;
