@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1996-2017. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2021. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -23,10 +23,10 @@
 -export([start/1, 
 	 load_application/1, unload_application/1, 
 	 start_application/2, start_boot_application/2, stop_application/1,
-	 control_application/1,
+	 control_application/1, is_running/1,
 	 change_application_data/2, prep_config_change/0, config_change/1,
 	 which_applications/0, which_applications/1,
-	 loaded_applications/0, info/0,
+	 loaded_applications/0, info/0, set_env/2,
 	 get_pid_env/2, get_env/2, get_pid_all_env/1, get_all_env/1,
 	 get_pid_key/2, get_key/2, get_pid_all_key/1, get_all_key/1,
 	 get_master/1, get_application/1, get_application_module/1,
@@ -37,6 +37,9 @@
 -export([handle_call/3, handle_cast/2, handle_info/2, terminate/2, 
 	 code_change/3, init_starter/4, get_loaded/1]).
 
+%% logger callback
+-export([format_log/1, format_log/2]).
+
 %% Test exports, only to be used from the test suites
 -export([test_change_apps/2]).
 
@@ -44,6 +47,7 @@
 		keyfind/3, keydelete/3, keyreplace/4]).
 
 -include("application_master.hrl").
+-include("logger.hrl").
 
 -define(AC, ?MODULE). % Name of process
 
@@ -154,7 +158,7 @@
 %% Env         = [{Key, Value}]
 %%-----------------------------------------------------------------
 
--record(appl, {name, appl_data, descr, id, vsn, restart_type, inc_apps, apps}).
+-record(appl, {name, appl_data, descr, id, vsn, restart_type, inc_apps, opt_apps, apps}).
 
 %%-----------------------------------------------------------------
 %% Func: start/1
@@ -233,6 +237,17 @@ unload_application(AppName) ->
 start_application(AppName, RestartType) ->
     gen_server:call(?AC, {start_application, AppName, RestartType}, infinity).
 
+%%-----------------------------------------------------------------
+%% Func: is_running/1
+%% Args: Application = atom()
+%% Purpose: Checks if an application is running.
+%%          This is used by application to avoid traversing an
+%%          application's child in case it is already running.
+%% Returns: boolean
+%%-----------------------------------------------------------------
+is_running(AppName) when is_atom(AppName) ->
+    gen_server:call(?AC, {is_running, AppName}, infinity).
+
 %%-----------------------------------------------------------------
 %% Func: start_boot_application/2
 %% The same as start_application/2 expect that this function is
@@ -267,13 +282,12 @@ which_applications(Timeout) ->
     gen_server:call(?AC, which_applications, Timeout).
 
 loaded_applications() ->
-    ets:filter(ac_tab,
-	       fun([{{loaded, AppName}, #appl{descr = Descr, vsn = Vsn}}]) ->
-		       {true, {AppName, Descr, Vsn}};
-		  (_) ->
-		       false
-	       end,
-	       []).
+    ets:select(ac_tab,
+               [{
+                  {{loaded, '$1'}, #appl{descr = '$2', vsn = '$3', _ = '_'}},
+                  [],
+                  [{{'$1', '$2', '$3'}}]
+                }]).
 
 %% Returns some debug info
 info() ->
@@ -344,9 +358,6 @@ get_all_env(AppName) ->
     map(fun([Key, Val]) -> {Key, Val} end,
 	ets:match(ac_tab, {{env, AppName, '$1'}, '$2'})).
 
-
-
-
 get_pid_key(Master, Key) ->
     case ets:match(ac_tab, {{application_master, '$1'}, Master}) of
 	[[AppName]] -> get_key(AppName, Key);
@@ -373,6 +384,8 @@ get_key(AppName, Key) ->
 		    {ok, (Appl#appl.appl_data)#appl_data.regs};
 		included_applications ->
 		    {ok, Appl#appl.inc_apps};
+                optional_applications ->
+                    {ok, Appl#appl.opt_apps};
 		applications ->
 		    {ok, Appl#appl.apps};
 		env ->
@@ -404,6 +417,7 @@ get_all_key(AppName) ->
 		  {maxT, (Appl#appl.appl_data)#appl_data.maxT},
 		  {registered, (Appl#appl.appl_data)#appl_data.regs},
 		  {included_applications, Appl#appl.inc_apps},
+                  {optional_applications, Appl#appl.opt_apps},
 		  {applications, Appl#appl.apps},
 		  {env, get_all_env(AppName)},
 		  {mod, (Appl#appl.appl_data)#appl_data.mod},
@@ -460,6 +474,15 @@ permit_application(ApplName, Flag) ->
 		    {permit_application, ApplName, Flag},
 		    infinity).
 
+set_env(Config, Opts) ->
+    case check_conf_data(Config) of
+	ok ->
+	    Timeout = proplists:get_value(timeout, Opts, 5000),
+	    gen_server:call(?AC, {set_env, Config, Opts}, Timeout);
+
+	{error, _} = Error ->
+	    Error
+    end.
 
 set_env(AppName, Key, Val) ->
     gen_server:call(?AC, {set_env, AppName, Key, Val, []}).
@@ -486,7 +509,7 @@ init(Init, Kernel) ->
 	{ok, ConfData} ->
 	    %% Actually, we don't need this info in an ets table anymore.
 	    %% This table was introduced because starting applications
-	    %% should be able to get som info from AC (e.g. loaded_apps).
+	    %% should be able to get some info from AC (e.g. loaded_apps).
 	    %% The new implementation makes sure the AC process can be
 	    %% called during start-up of any app.
 	    case check_conf_data(ConfData) of
@@ -515,6 +538,12 @@ init(Init, Kernel) ->
 		lists:flatten(io_lib:format("error in config file "
 					    "~tp (~w): ~ts",
 					    [File, Line, Str])),
+	    Init ! {ack, self(), {error, to_string(ReasonStr)}};
+        {error, {file_descriptor, FDString, Line, Str}} ->
+	    ReasonStr =
+		lists:flatten(io_lib:format("error in config read from file descriptor "
+					    "~tp (~w): ~ts",
+					    [FDString, Line, Str])),
 	    Init ! {ack, self(), {error, to_string(ReasonStr)}}
     end.
 
@@ -527,19 +556,15 @@ check_conf_data([]) ->
 check_conf_data(ConfData) when is_list(ConfData) ->
     [Application | ConfDataRem] = ConfData,
     case Application of
-	{kernel, List} when is_list(List) ->
-	    case check_para_kernel(List) of
-		ok ->
-		    check_conf_data(ConfDataRem);
-		Error1 ->
-		    Error1
-	    end;
 	{AppName, List} when is_atom(AppName), is_list(List) ->
-	    case check_para(List, atom_to_list(AppName)) of
-		ok ->
-		    check_conf_data(ConfDataRem);
-		Error2 ->
-		    Error2
+	    case lists:keymember(AppName, 1, ConfDataRem) of
+		true ->
+		    {error, "duplicate application config: " ++ atom_to_list(AppName)};
+		false ->
+		    case check_para(List, AppName) of
+			ok -> check_conf_data(ConfDataRem);
+			Error -> Error
+		    end
 	    end;
 	{AppName, List} when is_list(List)  ->
 	    ErrMsg = "application: "
@@ -552,36 +577,39 @@ check_conf_data(ConfData) when is_list(ConfData) ->
 		++ "; parameters must be a list",
 	    {error, ErrMsg};
 	Else ->
-	    ErrMsg = "invalid application name: " ++ 
-		lists:flatten(io_lib:format(" ~tp",[Else])),
+	    ErrMsg = "invalid application config: "
+		++ lists:flatten(io_lib:format("~tp",[Else])),
 	    {error, ErrMsg}
     end;
 check_conf_data(_ConfData) ->
-    {error, 'configuration must be a list ended by <dot><whitespace>'}.
-    
+    {error, "configuration must be a list ended by <dot><whitespace>"}.
+
+
+check_para([], _AppName) ->
+    ok;
+check_para([{Para, Val} | ParaList], AppName) when is_atom(Para) ->
+    case lists:keymember(Para, 1, ParaList) of
+	true ->
+	    ErrMsg =  "application: " ++ atom_to_list(AppName)
+		++ "; duplicate parameter: " ++ atom_to_list(Para),
+	    {error, ErrMsg};
+	false ->
+	    case check_para_value(Para, Val, AppName) of
+		ok -> check_para(ParaList, AppName);
+		{error, _} = Error -> Error
+	    end
+    end;
+check_para([{Para, _Val} | _ParaList], AppName) ->
+    {error, "application: " ++ atom_to_list(AppName) ++ "; invalid parameter name: " ++
+     lists:flatten(io_lib:format("~tp",[Para]))};
+check_para([Else | _ParaList], AppName) ->
+    {error, "application: " ++ atom_to_list(AppName) ++ "; invalid parameter: " ++
+     lists:flatten(io_lib:format("~tp",[Else]))}.
+
+check_para_value(distributed, Apps, kernel) -> check_distributed(Apps);
+check_para_value(_Para, _Val, _AppName) -> ok.
 
 %% Special check of distributed parameter for kernel
-check_para_kernel([]) ->
-    ok;
-check_para_kernel([{distributed, Apps} | ParaList]) when is_list(Apps) ->
-    case check_distributed(Apps) of
-	{error, _ErrorMsg} = Error ->
-	    Error;
-	_ ->
-	    check_para_kernel(ParaList)
-    end;
-check_para_kernel([{distributed, _Apps} | _ParaList]) ->
-    {error, "application: kernel; erroneous parameter: distributed"};
-check_para_kernel([{Para, _Val} | ParaList]) when is_atom(Para) ->
-    check_para_kernel(ParaList);
-check_para_kernel([{Para, _Val} | _ParaList]) ->
-    {error, "application: kernel; invalid parameter: " ++ 
-     lists:flatten(io_lib:format("~tp",[Para]))};
-check_para_kernel(Else) ->
-    {error, "application: kernel; invalid parameter list: " ++ 
-     lists:flatten(io_lib:format("~tp",[Else]))}.
-    
-
 check_distributed([]) ->
     ok;
 check_distributed([{App, List} | Apps]) when is_atom(App), is_list(List) ->
@@ -594,18 +622,6 @@ check_distributed(_Else) ->
     {error, "application: kernel; erroneous parameter: distributed"}.
 
 
-check_para([], _AppName) ->
-    ok;
-check_para([{Para, _Val} | ParaList], AppName) when is_atom(Para) ->
-    check_para(ParaList, AppName);
-check_para([{Para, _Val} | _ParaList], AppName) ->
-    {error, "application: " ++ AppName ++ "; invalid parameter: " ++ 
-     lists:flatten(io_lib:format("~tp",[Para]))};
-check_para([Else | _ParaList], AppName) ->
-    {error, "application: " ++ AppName ++ "; invalid parameter: " ++ 
-     lists:flatten(io_lib:format("~tp",[Else]))}.
-
-
 -type calls() :: 'info' | 'prep_config_change' | 'which_applications'
                | {'config_change' | 'control_application' |
 		  'load_application' | 'start_type' | 'stop_application' |
@@ -620,7 +636,7 @@ check_para([Else | _ParaList], AppName) ->
         {'noreply', state()} | {'reply', term(), state()}.
 
 handle_call({load_application, Application}, From, S) ->
-    case catch do_load_application(Application, S) of
+    case catch maybe_load_application(Application, S) of
 	{ok, NewS} ->
 	    AppName = get_appl_name(Application),
 	    case cntrl(AppName, S, {ac_load_application_req, AppName}) of
@@ -654,7 +670,7 @@ handle_call({start_application, AppName, RestartType}, From, S) ->
     #state{running = Running, starting = Starting, start_p_false = SPF, 
 	   started = Started, start_req = Start_req} = S,
     %% Check if the commandline environment variables are OK.
-    %% Incase of erroneous variables do not start the application,
+    %% In case of erroneous variables do not start the application,
     %% if the application is permanent crash the node.
     %% Check if the application is already starting.
     case lists:keyfind(AppName, 1, Start_req) of
@@ -694,6 +710,10 @@ handle_call({start_application, AppName, RestartType}, From, S) ->
 	    {noreply, SS}
     end;
 
+handle_call({is_running, AppName}, _From, S) ->
+    #state{running = Running} = S,
+    {reply, lists:keymember(AppName, 1, Running), S};
+
 handle_call({permit_application, AppName, Bool}, From, S) ->
     Control = S#state.control,
     Starting = S#state.starting,
@@ -819,13 +839,11 @@ handle_call({stop_application, AppName}, _From, S) ->
     end;
 
 handle_call({change_application_data, Applications, Config}, _From, S) ->
-    OldAppls = ets:filter(ac_tab,
-			  fun([{{loaded, _AppName}, Appl}]) ->
-				  {true, Appl};
-			     (_) ->
-				  false
-			  end,
-			  []),
+    OldAppls = ets:foldl(fun({{loaded, _AppName}, Appl}, Acc) ->
+                                 [Appl|Acc];
+                            (_, Acc) ->
+                                 Acc
+                         end, [], ac_tab),
     case catch do_change_apps(Applications, Config, OldAppls) of
 	{error, _} = Error ->
 	    {reply, Error, S};
@@ -862,6 +880,16 @@ handle_call(which_applications, _From, S) ->
 	       end, S#state.running),
     {reply, Reply, S};
 
+handle_call({set_env, Config, Opts}, _From, S) ->
+    _ = [add_env(AppName, Env) || {AppName, Env} <- Config],
+
+    case proplists:get_value(persistent, Opts, false) of
+	true ->
+	    {reply, ok, S#state{conf_data = merge_env(S#state.conf_data, Config)}};
+	false ->
+	    {reply, ok, S}
+    end;
+
 handle_call({set_env, AppName, Key, Val, Opts}, _From, S) ->
     ets:insert(ac_tab, {{env, AppName, Key}, Val}),
     case proplists:get_value(persistent, Opts, false) of
@@ -921,7 +949,7 @@ handle_application_started(AppName, Res, S) ->
     #state{starting = Starting, running = Running, started = Started, 
 	   start_req = Start_req} = S,
     Start_reqN = reply_to_requester(AppName, Start_req, Res),
-    {_AppName, RestartType, _Type, _From} = lists:keyfind(AppName, 1, Starting),
+    {AppName, RestartType, _Type, _From} = lists:keyfind(AppName, 1, Starting),
     case Res of
 	{ok, Id} ->
 	    case AppName of
@@ -947,8 +975,8 @@ handle_application_started(AppName, Res, S) ->
 			true ->
 			    #state{running = StopRunning, started = StopStarted} = NewS,
 			    case lists:keyfind(AppName, 1, StopRunning) of
-				{_AppName, Id} ->
-				    {_AppName2, Type} =
+				{AppName, Id} ->
+				    {AppName, Type} =
 					lists:keyfind(AppName, 1, StopStarted),
 				    stop_appl(AppName, Id, Type),
 				    NStopRunning = keydelete(AppName, 1, StopRunning),
@@ -1251,31 +1279,32 @@ get_loaded(App) ->
 	[{_Key, Appl}] -> {true, Appl};
 	_  -> false
     end.
-    
-do_load_application(Application, S) ->
+
+maybe_load_application(Application, S) ->
     case get_loaded(Application) of
 	{true, _} ->
 	    throw({error, {already_loaded, Application}});
 	false ->
-	    case make_appl(Application) of
-		{ok, Appl} -> load(S, Appl);
-		Error -> Error
-	    end
+	    do_load_application(Application, S)
+    end.
+
+do_load_application(Application, S) ->
+    case make_appl(Application) of
+	{ok, Appl} -> load(S, Appl);
+	Error -> Error
     end.
 
 %% Recursively load the application and its included apps.
 %load(S, {ApplData, ApplEnv, IncApps, Descr, Vsn, Apps}) ->
-load(S, {ApplData, ApplEnv, IncApps, Descr, Id, Vsn, Apps}) ->
+load(S, {ApplData, ApplEnv, IncApps, OptApps, Descr, Id, Vsn, Apps}) ->
     Name = ApplData#appl_data.name,
     ConfEnv = get_env_i(Name, S),
     NewEnv = merge_app_env(ApplEnv, ConfEnv),
     CmdLineEnv = get_cmd_env(Name),
     NewEnv2 = merge_app_env(NewEnv, CmdLineEnv),
-    NewEnv3 = keyreplaceadd(included_applications, 1, NewEnv2,
-			    {included_applications, IncApps}),
-    add_env(Name, NewEnv3),
-    Appl = #appl{name = Name, descr = Descr, id = Id, vsn = Vsn, 
-		 appl_data = ApplData, inc_apps = IncApps, apps = Apps},
+    add_env(Name, NewEnv2),
+    Appl = #appl{name = Name, descr = Descr, id = Id, vsn = Vsn, apps = Apps,
+		 appl_data = ApplData, inc_apps = IncApps, opt_apps = OptApps},
     ets:insert(ac_tab, {{loaded, Name}, Appl}),
     NewS =
 	foldl(fun(App, S1) ->
@@ -1291,7 +1320,7 @@ load(S, {ApplData, ApplEnv, IncApps, Descr, Id, Vsn, Apps}) ->
     {ok, NewS}.
 
 unload(AppName, S) ->
-    {ok, IncApps} = get_env(AppName, included_applications),
+    {ok, IncApps} = get_key(AppName, included_applications),
     del_env(AppName),
     ets:delete(ac_tab, {loaded, AppName}),
     foldl(fun(App, S1) ->
@@ -1312,13 +1341,13 @@ check_start_cond(AppName, RestartType, Started, Running) ->
 		    {error, {already_started, AppName}};
 		false ->
 		    foreach(
-		      fun(AppName2) ->
-			      case lists:keymember(AppName2, 1, Started) of
-				  true -> ok;
-				  false ->
-				      throw({error, {not_started, AppName2}})
-			      end
-		      end, Appl#appl.apps),
+			fun(AppName2) ->
+			    case lists:keymember(AppName2, 1, Started) orelse
+				     lists:member(AppName2, Appl#appl.opt_apps) of
+				true -> ok;
+				false -> throw({error, {not_started, AppName2}})
+			    end
+		    end, Appl#appl.apps),
 		    {ok, Appl}
 	    end;
 	false ->
@@ -1375,14 +1404,13 @@ start_appl(Appl, S, Type) ->
 	    %% Name = ApplData#appl_data.name,
 	    Running = S#state.running,
 	    foreach(
-	      fun(AppName) ->
-		      case lists:keymember(AppName, 1, Running) of
-			  true ->
-			      ok;
-			  false ->
-			      throw({info, {not_running, AppName}})
-		      end
-	      end, Appl#appl.apps),
+		fun(AppName) ->
+		    case lists:keymember(AppName, 1, Running) orelse
+			     lists:member(AppName, Appl#appl.opt_apps) of
+			true -> ok;
+			false -> throw({info, {not_running, AppName}})
+		    end
+		end, Appl#appl.apps),
 	    case application_master:start_link(ApplData, Type) of
 		{ok, _Pid} = Ok ->
 		    Ok;
@@ -1516,9 +1544,10 @@ make_appl_i({application, Name, Opts}) when is_atom(Name), is_list(Opts) ->
     MaxP = get_opt(maxP, Opts, infinity),
     MaxT = get_opt(maxT, Opts, infinity),
     IncApps = get_opt(included_applications, Opts, []),
+    OptApps = get_opt(optional_applications, Opts, []),
     {#appl_data{name = Name, regs = Regs, mod = Mod, phases = Phases,
-		mods = Mods, inc_apps = IncApps, maxP = MaxP, maxT = MaxT},
-     Env, IncApps, Descr, Id, Vsn, Apps};
+		mods = Mods, maxP = MaxP, maxT = MaxT},
+     Env, IncApps, OptApps, Descr, Id, Vsn, Apps};
 make_appl_i({application, Name, Opts}) when is_list(Opts) ->
     throw({error,{invalid_name,Name}});
 make_appl_i({application, _Name, Opts}) ->
@@ -1546,9 +1575,8 @@ do_change_apps(Applications, Config, OldAppls) ->
     %% Report errors, but do not terminate
     %% (backwards compatible behaviour)
     lists:foreach(fun({error, {SysFName, Line, Str}}) ->
-			  Str2 = lists:flatten(io_lib:format("~tp: ~w: ~ts~n",
-							     [SysFName, Line, Str])),
-			  error_logger:format(Str2, [])
+			  ?LOG_ERROR("~tp: ~w: ~ts~n",[SysFName, Line, Str],
+                                     #{error_logger=>#{tag=>error}})
 		  end,
 		  Errors),
 
@@ -1571,7 +1599,7 @@ is_loaded_app(AppName, [{application, AppName, App} | _]) ->
 is_loaded_app(AppName, [_ | T]) -> is_loaded_app(AppName, T);
 is_loaded_app(_AppName, []) -> false.
 
-do_change_appl({ok, {ApplData, Env, IncApps, Descr, Id, Vsn, Apps}},
+do_change_appl({ok, {ApplData, Env, IncApps, OptApps, Descr, Id, Vsn, Apps}},
 	       OldAppl, Config) ->
     AppName = OldAppl#appl.name,
 
@@ -1583,19 +1611,16 @@ do_change_appl({ok, {ApplData, Env, IncApps, Descr, Id, Vsn, Apps}},
     CmdLineEnv = get_cmd_env(AppName),
     NewEnv2 = merge_app_env(NewEnv1, CmdLineEnv),
 
-    %% included_apps is made into an env parameter as well
-    NewEnv3 = keyreplaceadd(included_applications, 1, NewEnv2,
-			    {included_applications, IncApps}),
-
     %% Update ets table with new application env
     del_env(AppName),
-    add_env(AppName, NewEnv3),
+    add_env(AppName, NewEnv2),
 
     OldAppl#appl{appl_data=ApplData,
 		 descr=Descr,
 		 id=Id,
 		 vsn=Vsn,
 		 inc_apps=IncApps,
+                 opt_apps=OptApps,
 		 apps=Apps};
 do_change_appl({error, _R} = Error, _Appl, _ConfData) ->
     throw(Error).
@@ -1631,8 +1656,9 @@ make_term(Str) ->
     end.
 
 handle_make_term_error(Mod, Reason, Str) ->
-    error_logger:format("application_controller: ~ts: ~ts~n",
-        [Mod:format_error(Reason), Str]),
+    ?LOG_ERROR("application_controller: ~ts: ~ts~n",
+               [Mod:format_error(Reason), Str],
+               #{error_logger=>#{tag=>error}}),
     throw({error, {bad_environment_value, Str}}).
 
 get_env_i(Name, #state{conf_data = ConfData}) when is_list(ConfData) ->
@@ -1705,7 +1731,7 @@ check_user() ->
 
 
 %%-----------------------------------------------------------------
-%% Prepare for a release upgrade by reading all the evironment variables.
+%% Prepare for a release upgrade by reading all the environment variables.
 %%-----------------------------------------------------------------
 do_prep_config_change(Apps) ->
     do_prep_config_change(Apps, []).
@@ -1799,63 +1825,136 @@ do_config_diff([{Env, Value} | AppEnvNow], AppEnvBefore, {Changed, New}) ->
     end.
 
 
+conf_param_to_conf({config, FileName}) ->
+    BFName = filename:basename(FileName,".config"),
+    FName = filename:join(filename:dirname(FileName),
+                          BFName ++ ".config"),
+    case load_file(FName) of
+        {ok, NewEnv} ->
+            %% OTP-4867 sys.config may now contain names of other
+            %% .config files as well as configuration parameters.
+            %% Therefore read and merge contents.
+            if
+                BFName =:= "sys" ->
+                    DName = filename:dirname(FName),
+                    {ok, SysEnv, Errors} =
+                        check_conf_sys(NewEnv, [], [], DName),
+                    %% Report first error, if any, and terminate
+                    %% (backwards compatible behaviour)
+                    case Errors of
+                        [] ->
+                            SysEnv;
+                        [{error, {SysFName, Line, Str}}|_] ->
+                            throw({error, {SysFName, Line, Str}})
+                    end;
+                true ->
+                    NewEnv
+            end;
+        {error, {Line, _Mod, Str}} ->
+            throw({error, {FName, Line, Str}})
+    end;
+conf_param_to_conf({configfd, FileDescStrP}) ->
+    FileDescStr = unicode:characters_to_nfc_list(FileDescStrP),
+    IsDigit = fun(C) -> lists:member(C, "0123456789") end,
+    {FdString, FileDescType} = lists:splitwith(IsDigit, FileDescStr),
+    FileDesc =
+        try list_to_integer(FdString)
+        catch
+            error:badarg ->
+                throw({error, {file_descriptor,
+                               FileDescStr,
+                               invalid_file_desc,
+                               "The given file descriptor has incorrect format. "
+                               "The format should be \"FileDescId[.FileType]\". "
+                               "Examples: 3 or 3.config"}})
+        end,
+    case lists:member(FileDescType, [".config", ""])  of
+        false ->
+            throw({error, {file_descriptor,
+                           FileDescStr,
+                           invalid_file_desc,
+                           io_lib:format("Cannot parse file descriptor of type: ~ts",
+                                         [FileDescType])}});
+        true -> ok
+    end,
+    case load_file_descriptor(FileDesc) of
+        {ok, NewEnv} ->
+            %% Load config parameters from included config files
+            BootScript = init:script_name(),
+            DName = filename:dirname(BootScript),
+            {ok, SysEnv, Errors} =
+                check_conf_sys(NewEnv, [], [], DName),
+            case Errors of
+                [] ->
+                    SysEnv;
+                [{error, {SysFName, Line, Str}}|_] ->
+                    throw({error, {SysFName, Line, Str}})
+            end;
+        {error, {Line, _Mod, Str}} ->
+            throw({error, {file_descriptor,
+                           FileDescStr,
+                           Line,
+                           Str}})
+    end.
+
+config_param_to_list({Type, [First | _] = ConfigVals})
+  when
+      is_list(First),
+      Type =:= config orelse Type =:= configfd ->
+    [{Type, Val} || Val <- ConfigVals];
+config_param_to_list({config, Val}) ->
+    [{config, Val}];
+config_param_to_list({configfd, Val}) ->
+    [{configfd, Val}];
+config_param_to_list(_) ->
+    [].
+
+
+
 %%-----------------------------------------------------------------
 %% Read the .config files.
 %%-----------------------------------------------------------------
 check_conf() ->
-    case init:get_argument(config) of
-	{ok, Files} ->
-	    {ok, lists:foldl(
-		   fun([File], Env) ->
-			   BFName = filename:basename(File,".config"),
-			   FName = filename:join(filename:dirname(File),
-						 BFName ++ ".config"),
-			   case load_file(FName) of
-			       {ok, NewEnv} ->
-				   %% OTP-4867
-				   %% sys.config may now contain names of
-				   %% other .config files as well as
-				   %% configuration parameters.
-				   %% Therefore read and merge contents.
-				   if
-				       BFName =:= "sys" ->
-					   {ok, SysEnv, Errors} =
-					       check_conf_sys(NewEnv),
+    ConfigParameters =
+        lists:flatmap(
+          fun config_param_to_list/1,
+          init:get_arguments()),
+    MergedConf =
+        lists:foldl(fun (ConfigParameter, Env) ->
+                            NewEnv = conf_param_to_conf(ConfigParameter),
+                            merge_env(Env, NewEnv)
+                    end,
+                    [],
+                    ConfigParameters),
+    {ok, MergedConf}.
 
-					   %% Report first error, if any, and
-					   %% terminate
-					   %% (backwards compatible behaviour)
-					   case Errors of
-					       [] ->
-						   merge_env(Env, SysEnv);
-					       [{error, {SysFName, Line, Str}}|_] ->
-						   throw({error, {SysFName, Line, Str}})
-					   end;
-				       true ->
-					   merge_env(Env, NewEnv)
-				   end;
-			       {error, {Line, _Mod, Str}} ->
-				   throw({error, {FName, Line, Str}})
-			   end
-		   end, [], Files)};
-	_ -> {ok, []}
-    end.
 
 check_conf_sys(Env) ->
-    check_conf_sys(Env, [], []).
+    check_conf_sys(Env, [], [], []).
 
-check_conf_sys([File|T], SysEnv, Errors) when is_list(File) ->
+check_conf_sys([File|T], SysEnv, Errors, DName) when is_list(File),is_list(DName) ->
     BFName = filename:basename(File, ".config"),
     FName = filename:join(filename:dirname(File), BFName ++ ".config"),
-    case load_file(FName) of
+    LName = case filename:pathtype(FName) of
+               relative when (DName =/= []) ->
+                  % Check if relative to sys.config dir otherwise use legacy mode,
+                  % i.e relative to cwd.
+                  RName = filename:join(DName, FName),
+                  case erl_prim_loader:read_file_info(RName) of
+                     {ok, _} -> RName ;
+                     error   -> FName
+                  end;
+		_          -> FName
+	    end,
+    case load_file(LName) of
 	{ok, NewEnv} ->
-	    check_conf_sys(T, merge_env(SysEnv, NewEnv), Errors);
+	    check_conf_sys(T, merge_env(SysEnv, NewEnv), Errors, DName);
 	{error, {Line, _Mod, Str}} ->
-	    check_conf_sys(T, SysEnv, [{error, {FName, Line, Str}}|Errors])
+	    check_conf_sys(T, SysEnv, [{error, {LName, Line, Str}}|Errors], DName)
     end;
-check_conf_sys([Tuple|T], SysEnv, Errors) ->
-    check_conf_sys(T, merge_env(SysEnv, [Tuple]), Errors);
-check_conf_sys([], SysEnv, Errors) ->
+check_conf_sys([Tuple|T], SysEnv, Errors, DName) ->
+    check_conf_sys(T, merge_env(SysEnv, [Tuple]), Errors, DName);
+check_conf_sys([], SysEnv, Errors, _) ->
     {ok, SysEnv, lists:reverse(Errors)}.
 
 load_file(File) ->
@@ -1874,6 +1973,40 @@ load_file(File) ->
 	    {error, {none, open_file, "configuration file not found"}}
     end.
 
+load_file_descriptor(FileDescriptorId) ->
+    %% We do not want to read from the same file descriptor again if
+    %% init:restart is called so we use the old config obtained from
+    %% the file descriptor
+    case init:get_configfd(FileDescriptorId) of
+        none ->
+            WarningIntervalMs = 20000, % 20 seconds
+            MaxConfSizeBytes = 1024*1024*128, % 134 MB
+            case read_fd_until_end_and_close(FileDescriptorId,
+                                             WarningIntervalMs,
+                                             MaxConfSizeBytes) of
+                {ok, Bin} ->
+                    %% Make sure that there is some whitespace at the end of the string
+                    %% (so that reading a file with no NL following the "." will work).
+                    case file_binary_to_list(Bin) of
+                        {ok, String} ->
+                            case scan_file(String ++ " ") of
+                                {ok, Config} ->
+                                    init:set_configfd(FileDescriptorId, Config),
+                                    {ok, Config};
+                                Error ->
+                                    Error
+                            end;
+                        error ->
+                            {error, {none, scan_file, "bad encoding"}}
+                    end;
+                {error, Reason} ->
+                    {error, {none,
+                             read_from_file_descriptor,
+                             io_lib:format("Could not read from file descriptor: ~s", [Reason])}}
+            end;
+        Config -> {ok, Config}
+    end.
+
 scan_file(Str) ->
     case erl_scan:tokens([], Str, 1) of
 	{done, {ok, Tokens, _}, Left} ->
@@ -1898,6 +2031,61 @@ scan_file(Str) ->
 	    {error, {none, load_file, "no ending <dot> found"}}
     end.
 
+read_fd_until_end_and_close(FileDescriptorId,
+                            TimeBetweenWarningsMilliseconds,
+                            MaxSizeBytes) ->
+    ReadLimit = 1024,
+    Read =
+        fun Read(Ref, _Fd, _Data, DataSize) when DataSize > MaxSizeBytes ->
+                Reason = io_lib:format("Max size ~w bytes exceeded",
+                                       [MaxSizeBytes]),
+                {Ref, error, Reason};
+            Read(Ref, Fd, Data, DataSize) ->
+                case file:read(Fd, ReadLimit) of
+                    eof ->
+                        {Ref, ok, erlang:iolist_to_binary(Data)};
+                    {ok, Bin} ->
+                        Read(Ref, Fd, [Data, Bin], DataSize + byte_size(Bin));
+                    {error, Reason} ->
+                        {Ref, error, Reason}
+                end
+        end,
+    Receiver = self(),
+    Ref = make_ref(),
+    Reader =
+        spawn(
+          fun() ->
+                  case prim_file:file_desc_to_ref(FileDescriptorId, [read]) of
+                      {ok, Fd} ->
+                          Res = Read(Ref, Fd, [], 0),
+                          prim_file:close(Fd),
+                          Receiver ! Res;
+                      {error, _} ->
+                          Receiver ! {Ref,
+                                      error,
+                                      "Invalid file descriptor"}
+                  end
+          end),
+    Reader ! {reader_ref, Ref},
+    (fun GetResult() ->
+            receive
+                {Ref, ok, Data} ->
+                    {ok, erlang:iolist_to_binary(Data)};
+                {Ref, error, Reason} ->
+                    {error, Reason}
+            after TimeBetweenWarningsMilliseconds ->
+                    Msg =
+                        io_lib:format(
+                          "Slow -configfd file descriptor ~p. "
+                          "The system will continue to read from "
+                          "the file descriptor and will be blocked "
+                          "until end of file is received.",
+                          [FileDescriptorId]),
+                    ?LOG_WARNING(Msg),
+                    GetResult()
+            end
+    end)().
+
 only_ws([C|Cs]) when C =< $\s -> only_ws(Cs);
 only_ws([$%|Cs]) -> only_ws(strip_comment(Cs));   % handle comment
 only_ws([_|_]) -> false;
@@ -1913,19 +2101,160 @@ config_error() ->
       "configuration file must contain ONE list ended by <dot>"}}.
 
 %%-----------------------------------------------------------------
-%% Info messages sent to error_logger
+%% Info messages sent to logger
 %%-----------------------------------------------------------------
 info_started(Name, Node) ->
-    Rep = [{application, Name},
-	   {started_at, Node}],
-    error_logger:info_report(progress, Rep).
+    ?LOG_INFO(#{label=>{application_controller,progress},
+                report=>[{application, Name},
+                         {started_at, Node}]},
+              #{domain=>[otp,sasl],
+                report_cb=>fun application_controller:format_log/2,
+                logger_formatter=>#{title=>"PROGRESS REPORT"},
+                error_logger=>#{tag=>info_report,
+                                type=>progress,
+                                report_cb=>
+                                    fun application_controller:format_log/1}}).
 
 info_exited(Name, Reason, Type) ->
-    Rep = [{application, Name},
-	   {exited, Reason},
-	   {type, Type}],
-    error_logger:info_report(Rep).
+    ?LOG_NOTICE(#{label=>{application_controller,exit},
+                  report=>[{application, Name},
+                           {exited, Reason},
+                           {type, Type}]},
+                #{domain=>[otp],
+                  report_cb=>fun application_controller:format_log/2,
+                error_logger=>#{tag=>info_report,
+                                type=>std_info,
+                                report_cb=>
+                                    fun application_controller:format_log/1}}).
 
+%% format_log/1 is the report callback used by Logger handler
+%% error_logger only. It is kept for backwards compatibility with
+%% legacy error_logger event handlers. This function must always
+%% return {Format,Args} compatible with the arguments in this module's
+%% calls to error_logger prior to OTP-21.0.
+format_log(LogReport) ->
+    Depth = error_logger:get_format_depth(),
+    FormatOpts = #{chars_limit => unlimited,
+                   depth => Depth,
+                   single_line => false,
+                   encoding => utf8},
+    format_log_multi(limit_report(LogReport, Depth), FormatOpts).
+
+limit_report(LogReport, unlimited) ->
+    LogReport;
+limit_report(#{label:={application_controller,progress},
+               report:=[{application,_}=Application,
+                        {started_at,Node}]}=LogReport,
+             Depth) ->
+    LogReport#{report=>[Application,
+                        {started_at,io_lib:limit_term(Node, Depth)}]};
+limit_report(#{label:={application_controller,exit},
+               report:=[{application,_}=Application,
+                        {exited,Reason},{type,Type}]}=LogReport,
+             Depth) ->
+    LogReport#{report=>[Application,
+                        {exited,io_lib:limit_term(Reason, Depth)},
+                        {type,io_lib:limit_term(Type, Depth)}]}.
+
+%% format_log/2 is the report callback for any Logger handler, except
+%% error_logger.
+format_log(Report, FormatOpts0) ->
+    Default = #{chars_limit => unlimited,
+                depth => unlimited,
+                single_line => false,
+                encoding => utf8},
+    FormatOpts = maps:merge(Default, FormatOpts0),
+    IoOpts =
+        case FormatOpts of
+            #{chars_limit:=unlimited} ->
+                [];
+            #{chars_limit:=Limit} ->
+                [{chars_limit,Limit}]
+        end,
+    {Format,Args} = format_log_single(Report, FormatOpts),
+    io_lib:format(Format, Args, IoOpts).
+
+format_log_single(#{label:={application_controller,progress},
+                    report:=[{application,Name},{started_at,Node}]},
+                  #{single_line:=true,depth:=Depth}=FormatOpts) ->
+    P = p(FormatOpts),
+    Format = "Application: "++P++". Started at: "++P++".",
+    Args =
+        case Depth of
+            unlimited ->
+                [Name,Node];
+            _ ->
+                [Name,Depth,Node,Depth]
+        end,
+    {Format,Args};
+format_log_single(#{label:={application_controller,exit},
+                    report:=[{application,Name},
+                             {exited,Reason},
+                             {type,Type}]},
+                  #{single_line:=true,depth:=Depth}=FormatOpts) ->
+    P = p(FormatOpts),
+    Format = lists:append(["Application: ",P,". Exited: ",P,
+                            ". Type: ",P,"."]),
+    Args =
+        case Depth of
+            unlimited ->
+                [Name,Reason,Type];
+            _ ->
+                [Name,Depth,Reason,Depth,Type,Depth]
+        end,
+    {Format,Args};
+format_log_single(Report,FormatOpts) ->
+    format_log_multi(Report,FormatOpts).
+
+format_log_multi(#{label:={application_controller,progress},
+                   report:=[{application,Name},
+                            {started_at,Node}]},
+                 #{depth:=Depth}=FormatOpts) ->
+    P = p(FormatOpts),
+    Format =
+        lists:append(
+          ["    application: ",P,"~n",
+           "    started_at: ",P,"~n"]),
+    Args =
+        case Depth of
+            unlimited ->
+                [Name,Node];
+            _ ->
+                [Name,Depth,Node,Depth]
+        end,
+    {Format,Args};
+format_log_multi(#{label:={application_controller,exit},
+                   report:=[{application,Name},
+                            {exited,Reason},
+                            {type,Type}]},
+                 #{depth:=Depth}=FormatOpts) ->
+    P = p(FormatOpts),
+    Format =
+        lists:append(
+          ["    application: ",P,"~n",
+           "    exited: ",P,"~n",
+           "    type: ",P,"~n"]),
+    Args =
+        case Depth of
+            unlimited ->
+                [Name,Reason,Type];
+            _ ->
+                [Name,Depth,Reason,Depth,Type,Depth]
+        end,
+    {Format,Args}.
+
+p(#{single_line:=Single,depth:=Depth,encoding:=Enc}) ->
+    "~"++single(Single)++mod(Enc)++p(Depth);
+p(unlimited) ->
+    "p";
+p(_Depth) ->
+    "P".
+
+single(true) -> "0";
+single(false) -> "".
+
+mod(latin1) -> "";
+mod(_) -> "t".
 
 %%-----------------------------------------------------------------
 %% Reply to all processes waiting this application to be started.  
@@ -2012,5 +2341,5 @@ to_string(Term) ->
 	true ->
 	    Term;
 	false ->
-	    lists:flatten(io_lib:format("~134217728p", [Term]))
+	    lists:flatten(io_lib:format("~0p", [Term]))
     end.
