@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1997-2020. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2022. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -31,7 +31,9 @@
 	 getif/1, getif/0, getiflist/0, getiflist/1,
 	 ifget/3, ifget/2, ifset/3, ifset/2,
 	 getstat/1, getstat/2,
-	 ip/1, stats/0, options/0, 
+         info/1, socket_to_list/1,
+	 ip/1, is_ipv4_address/1, is_ipv6_address/1, is_ip_address/1,
+	 stats/0, options/0, 
 	 pushf/3, popf/1, close/1, gethostname/0, gethostname/1, 
 	 parse_ipv4_address/1, parse_ipv6_address/1, parse_ipv4strict_address/1,
 	 parse_ipv6strict_address/1, parse_address/1, parse_strict_address/1,
@@ -39,11 +41,11 @@
 
 -export([connect_options/2, listen_options/2, udp_options/2, sctp_options/2]).
 -export([udp_module/1, tcp_module/1, tcp_module/2, sctp_module/1]).
--export([gen_tcp_module/1]).
+-export([gen_tcp_module/1, gen_udp_module/1]).
 
 -export([i/0, i/1, i/2]).
 
--export([getll/1, getfd/1, open/8, fdopen/6]).
+-export([getll/1, getfd/1, open/8, open_bind/8, fdopen/6]).
 
 -export([tcp_controlling_process/2, udp_controlling_process/2,
 	 tcp_close/1, udp_close/1]).
@@ -74,16 +76,23 @@
 %% timer interface
 -export([start_timer/1, timeout/1, timeout/2, stop_timer/1]).
 
+%% Socket monitoring
+-export([monitor/1, cancel_monitor/1]).
+
+%% Socket utility functions
+-export([ensure_sockaddr/1]).
+
 -export_type([address_family/0, socket_protocol/0, hostent/0, hostname/0, ip4_address/0,
               ip6_address/0, ip_address/0, port_number/0,
 	      family_address/0, local_address/0,
               socket_address/0, returned_non_ip_address/0,
-	      socket_setopt/0, socket_getopt/0, ancillary_data/0,
-	      posix/0, socket/0, stat_option/0]).
+	      socket_setopt/0, socket_getopt/0, socket_optval/0,
+              ancillary_data/0,
+	      posix/0, socket/0, inet_backend/0, stat_option/0]).
 %% imports
 -import(lists, [append/1, duplicate/2, filter/2, foldl/3]).
 
--define(DEFAULT_KERNEL_INET_BACKEND, inet). % inet | socket
+-define(DEFAULT_KERNEL_INET_BACKEND, inet). % inet_backend()
 
 %% Record Signature
 -define(RS(Record),
@@ -93,6 +102,9 @@
 	element(1, Record) =:= element(1, RS),
 	tuple_size(Record) =:= element(2, RS)).
 
+%% -define(DBG(T), erlang:display({{self(), ?MODULE, ?LINE, ?FUNCTION_NAME}, T})).
+
+
 %%% ---------------------------------
 %%% Contract type definitions
 
@@ -129,13 +141,20 @@
         'ewouldblock' |
         'exbadport' | 'exbadseq' | file:posix().
 -type module_socket() :: {'$inet', Handler :: module(), Handle :: term()}.
+-define(module_socket(Handler, Handle), {'$inet', (Handler), (Handle)}).
 -type socket() :: port() | module_socket().
+-type inet_backend() :: {'inet_backend', 'inet' | 'socket'}.
 
 -type socket_setopt() ::
         gen_sctp:option() | gen_tcp:option() | gen_udp:option().
 
+-type socket_optval() ::
+        gen_sctp:option_value() | gen_tcp:option() | gen_udp:option() |
+        gen_tcp:pktoptions_value().
+
 -type socket_getopt() ::
         gen_sctp:option_name() | gen_tcp:option_name() | gen_udp:option_name().
+
 -type ether_address() :: [0..255].
 
 -type if_setopt() ::
@@ -196,6 +215,8 @@ get_rc() ->
 -spec close(Socket) -> 'ok' when
       Socket :: socket().
 
+close(?module_socket(GenSocketMod, _) = Socket) when is_atom(GenSocketMod) ->
+    GenSocketMod:?FUNCTION_NAME(Socket);
 close(Socket) ->
     prim_inet:close(Socket),
     receive
@@ -206,13 +227,60 @@ close(Socket) ->
     end.
 
 
+%% -- Socket monitor
+
+-spec monitor(Socket) -> reference() when
+      Socket :: socket().
+
+monitor({'$inet', GenSocketMod, _} = Socket) when is_atom(GenSocketMod) ->
+    MRef = GenSocketMod:?FUNCTION_NAME(Socket),
+    case inet_db:put_socket_type(MRef, {socket, GenSocketMod}) of
+        ok ->
+            MRef;
+	error ->
+	    GenSocketMod:cancel_monitor(MRef),
+	    erlang:error({invalid, Socket})
+    end;
+monitor(Socket) when is_port(Socket) ->
+    MRef = erlang:monitor(port, Socket),
+    case inet_db:put_socket_type(MRef, port) of
+	ok ->
+	    MRef;
+	error ->
+	    erlang:demonitor(MRef, [flush]),
+	    erlang:error({invalid, Socket})
+    end;
+monitor(Socket) ->
+    erlang:error(badarg, [Socket]).
+
+
+%% -- Cancel socket monitor
+
+-spec cancel_monitor(MRef) -> boolean() when
+      MRef :: reference().
+
+cancel_monitor(MRef) when is_reference(MRef) ->
+    case inet_db:take_socket_type(MRef) of
+	{ok, port} ->
+	    erlang:demonitor(MRef, [info]);
+	{ok, {socket, GenSocketMod}} ->
+	    GenSocketMod:?FUNCTION_NAME(MRef);
+	error -> % Assume it has the monitor has already been cancel'ed
+	    false
+    end;
+cancel_monitor(MRef) ->
+    erlang:error(badarg, [MRef]).
+
+
+%% -- Socket peername
+
 -spec peername(Socket :: socket()) ->
 		      {ok,
 		       {ip_address(), port_number()} |
 		       returned_non_ip_address()} |
 		      {error, posix()}.
 
-peername({'$inet', GenSocketMod, _} = Socket) when is_atom(GenSocketMod) ->
+peername(?module_socket(GenSocketMod, _) = Socket) when is_atom(GenSocketMod) ->
     GenSocketMod:?FUNCTION_NAME(Socket);
 peername(Socket) -> 
     prim_inet:peername(Socket).
@@ -256,7 +324,7 @@ peernames(Socket, Assoc) ->
 		       returned_non_ip_address()} |
 		      {error, posix()}.
 
-sockname({'$inet', GenSocketMod, _} = Socket) when is_atom(GenSocketMod) ->
+sockname(?module_socket(GenSocketMod, _) = Socket) when is_atom(GenSocketMod) ->
     GenSocketMod:?FUNCTION_NAME(Socket);
 sockname(Socket) -> 
     prim_inet:sockname(Socket).
@@ -280,6 +348,8 @@ setsockname(Socket, undefined) ->
 			 returned_non_ip_address()]} |
 		       {error, posix()}.
 
+socknames(?module_socket(GenSocketMod, _) = Socket) when is_atom(GenSocketMod) ->
+    GenSocketMod:?FUNCTION_NAME(Socket);
 socknames(Socket) ->
     prim_inet:socknames(Socket).
 
@@ -298,7 +368,7 @@ socknames(Socket, Assoc) ->
       Socket :: socket(),
       Port :: port_number().
 
-port({'$inet', GenSocketMod, _} = Socket) when is_atom(GenSocketMod) ->
+port(?module_socket(GenSocketMod, _) = Socket) when is_atom(GenSocketMod) ->
     case GenSocketMod:sockname(Socket) of
         {ok, {_, Port}} -> {ok, Port};
         {error, _} = Error -> Error
@@ -319,7 +389,7 @@ send(Socket, Packet) ->
       Socket :: socket(),
       Options :: [socket_setopt()].
 
-setopts({'$inet', GenSocketMod, _} = Socket, Opts) when is_atom(GenSocketMod) ->
+setopts(?module_socket(GenSocketMod, _) = Socket, Opts) when is_atom(GenSocketMod) ->
     GenSocketMod:?FUNCTION_NAME(Socket, Opts);
 setopts(Socket, Opts) -> 
     SocketOpts =
@@ -335,9 +405,9 @@ setopts(Socket, Opts) ->
 	{'ok', OptionValues} | {'error', posix()} when
       Socket :: socket(),
       Options :: [socket_getopt()],
-      OptionValues :: [socket_setopt() | gen_tcp:pktoptions_value()].
+      OptionValues :: [socket_optval()].
 
-getopts({'$inet', GenSocketMod, _} = Socket, Opts)
+getopts(?module_socket(GenSocketMod, _) = Socket, Opts)
   when is_atom(GenSocketMod) ->
     GenSocketMod:?FUNCTION_NAME(Socket, Opts);
 getopts(Socket, Opts) ->
@@ -530,7 +600,7 @@ getstat(Socket) ->
       Options :: [stat_option()],
       OptionValues :: [{stat_option(), integer()}].
 
-getstat({'$inet', GenSocketMod, _} = Socket, What)
+getstat(?module_socket(GenSocketMod, _) = Socket, What)
   when is_atom(GenSocketMod) ->
     GenSocketMod:?FUNCTION_NAME(Socket, What);
 getstat(Socket, What) ->
@@ -602,6 +672,83 @@ gethostbyaddr(Address,Timeout) ->
 gethostbyaddr_tm(Address,Timer) ->
     gethostbyaddr_tm(Address, Timer, inet_db:res_option(lookup)).
 
+
+-spec socket_to_list(Socket) -> list() when
+      Socket :: socket().
+
+socket_to_list({'$inet', GenSocketMod, _} = Socket)
+  when is_atom(GenSocketMod) ->
+    GenSocketMod:?FUNCTION_NAME(Socket);
+socket_to_list(Socket) when is_port(Socket) ->
+    erlang:port_to_list(Socket).
+
+
+
+-spec info(Socket) -> Info when
+      Socket :: socket(),
+      Info :: term().
+
+info({'$inet', GenSocketMod, _} = Socket)
+  when is_atom(GenSocketMod) ->
+    GenSocketMod:?FUNCTION_NAME(Socket);
+info(Socket) when is_port(Socket) ->
+    case port_info(Socket) of
+	#{states := _} = PortInfo ->
+            case inet:getopts(Socket, [active]) of
+                {ok, [{active, Active}]} ->
+                    PortInfo#{active => Active};
+                _ ->
+                    PortInfo
+            end;
+	PortInfo0 ->
+	    %% Its actually possible to call this function for non-socket ports,
+	    %% but in that case we have no status or statistics.
+	    PortInfo1 =
+		case prim_inet:getstatus(Socket) of
+		    {ok, State} ->
+			PortInfo0#{states => State};
+		    _ ->
+			PortInfo0
+		end,
+	    case getstat(Socket) of
+		{ok, Stats0} ->
+		    PortInfo1#{counters => maps:from_list(Stats0)};
+		_ ->
+		    PortInfo1
+	    end
+    end.
+
+port_info(P) when is_port(P) ->
+    case erlang:port_info(P) of
+	PI0 when is_list(PI0) ->
+	    PI1 = port_info(PI0, [connected, links, input, output]) ++
+		[erlang:port_info(P, memory), erlang:port_info(P, monitors)],
+	    PI2 = pi_replace([{connected, owner}], PI1),
+	    maps:from_list(PI2);
+	_ ->
+	    #{states => [closed]}
+    end.
+
+port_info(PI, Items) when is_list(PI) ->
+    port_info(PI, Items, []).
+
+port_info(_PI, [], Acc) ->
+    Acc;
+port_info(PI, [Item | Items], Acc) ->
+    Val = proplists:get_value(Item, PI),
+    port_info(PI, Items, [{Item, Val} | Acc]).
+
+pi_replace([], Items) ->
+    Items;
+pi_replace([{Key1, Key2}|Keys], Items) ->
+    case lists:keysearch(Key1, 1, Items) of
+        {value, {Key1, Value}} ->
+            Items2 = lists:keyreplace(Key1, 1, Items, {Key2, Value}),
+            pi_replace(Keys, Items2);
+        false ->
+            pi_replace(Keys, Items)
+    end.
+
 -spec ip(Ip :: ip_address() | string() | atom()) ->
 	{'ok', ip_address()} | {'error', posix()}.
 
@@ -614,6 +761,25 @@ ip(Name) ->
 	Error -> Error
     end.
 
+-spec is_ipv4_address(IPv4Address) -> boolean() when
+      IPv4Address :: ip4_address() | term().
+is_ipv4_address({A,B,C,D}) when ?ip(A,B,C,D) ->
+    true;
+is_ipv4_address(_) ->
+    false.
+
+-spec is_ipv6_address(IPv6Address) -> boolean() when
+      IPv6Address :: ip6_address() | term().
+is_ipv6_address({A,B,C,D,E,F,G,H}) when ?ip6(A,B,C,D,E,F,G,H) ->
+    true;
+is_ipv6_address(_) ->
+    false.
+
+-spec is_ip_address(IPAddress) -> boolean() when
+      IPAddress :: ip_address() | term().
+is_ip_address(Address) ->
+    is_ipv4_address(Address) orelse is_ipv6_address(Address).
+
 %% This function returns the erlang port used (with inet_drv)
 
 -spec getll(Socket :: socket()) -> {'ok', socket()}.
@@ -714,28 +880,28 @@ ntoa(Addr) ->
 -spec parse_ipv4_address(Address) ->
 	{ok, IPv4Address} | {error, einval} when
       Address :: string(),
-      IPv4Address :: ip_address().
+      IPv4Address :: ip4_address().
 parse_ipv4_address(Addr) ->
     inet_parse:ipv4_address(Addr).
 
 -spec parse_ipv6_address(Address) ->
 	{ok, IPv6Address} | {error, einval} when
       Address :: string(),
-      IPv6Address :: ip_address().
+      IPv6Address :: ip6_address().
 parse_ipv6_address(Addr) ->
     inet_parse:ipv6_address(Addr).
 
 -spec parse_ipv4strict_address(Address) ->
 	{ok, IPv4Address} | {error, einval} when
       Address :: string(),
-      IPv4Address :: ip_address().
+      IPv4Address :: ip4_address().
 parse_ipv4strict_address(Addr) ->
     inet_parse:ipv4strict_address(Addr).
 
 -spec parse_ipv6strict_address(Address) ->
 	{ok, IPv6Address} | {error, einval} when
       Address :: string(),
-      IPv6Address :: ip_address().
+      IPv6Address :: ip6_address().
 parse_ipv6strict_address(Addr) ->
     inet_parse:ipv6strict_address(Addr).
 
@@ -781,11 +947,14 @@ stats() ->
     [recv_oct, recv_cnt, recv_max, recv_avg, recv_dvi,
      send_oct, send_cnt, send_max, send_avg, send_pend].
 
+
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% Available options for tcp:connect
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 connect_options() ->
-    [tos, tclass, priority, reuseaddr, keepalive, linger, sndbuf, recbuf, nodelay,
+    [debug,
+     tos, tclass, priority, reuseaddr, keepalive, linger, nodelay,
+     sndbuf, recbuf,
      recvtos, recvtclass, ttl, recvttl,
      header, active, packet, packet_size, buffer, mode, deliver, line_delimiter,
      exit_on_close, high_watermark, low_watermark, high_msgq_watermark,
@@ -795,15 +964,15 @@ connect_options() ->
 connect_options(Opts, Mod) ->
     BaseOpts = 
 	case application:get_env(kernel, inet_default_connect_options) of
-	    {ok,List} when is_list(List) ->
+	    {ok, List} when is_list(List) ->
 		NList = [{active, true} | lists:keydelete(active,1,List)],     
-		#connect_opts{ opts = NList};
-	    {ok,{active,_Bool}} -> 
-		#connect_opts{ opts = [{active,true}]};
-	    {ok,Option} -> 
-		#connect_opts{ opts = [{active,true}, Option]};
+		#connect_opts{opts = NList};
+	    {ok, {active,_Bool}} -> 
+		#connect_opts{opts = [{active,true}]};
+	    {ok, Option} -> 
+		#connect_opts{opts = [{active,true}, Option]};
 	    _ ->
-		#connect_opts{ opts = [{active,true}]}
+		#connect_opts{opts = [{active,true}]}
 	end,
     case con_opt(Opts, BaseOpts, connect_options()) of
 	{ok, R} ->
@@ -816,14 +985,29 @@ connect_options(Opts, Mod) ->
 
 con_opt([{raw,A,B,C}|Opts],#connect_opts{} = R,As) ->
     con_opt([{raw,{A,B,C}}|Opts],R,As);
-con_opt([Opt | Opts], #connect_opts{} = R, As) ->
+con_opt([Opt | Opts], #connect_opts{ifaddr = IfAddr} = R, As) ->
     case Opt of
-	{ip,IP}     -> con_opt(Opts, R#connect_opts { ifaddr = IP }, As);
-	{ifaddr,IP} -> con_opt(Opts, R#connect_opts { ifaddr = IP }, As);
-	{port,P}    -> con_opt(Opts, R#connect_opts { port = P }, As);
-	{fd,Fd}     -> con_opt(Opts, R#connect_opts { fd = Fd }, As);
+	{ifaddr, Addr} when is_map(Addr) ->
+            con_opt(Opts, R#connect_opts{ ifaddr = ensure_sockaddr(Addr) }, As);
+	{ifaddr, Addr} ->
+            con_opt(Opts, R#connect_opts{ ifaddr = Addr }, As);
+
+        %% This is when a previous value of ifaddr was a sockaddr_in6()
+	{ip,IP} when is_map(IfAddr) ->
+            con_opt(Opts, R#connect_opts{ ifaddr = IfAddr#{addr => IP} }, As);
+	{ip,IP}     -> con_opt(Opts, R#connect_opts{ ifaddr = IP }, As);
+
+        %% This is when a previous value of ifaddr was a sockaddr_in6()
+	{port,P} when is_map(IfAddr) ->
+            con_opt(Opts, R#connect_opts{ ifaddr = IfAddr#{port => P} }, As);
+	{port,P}    -> con_opt(Opts, R#connect_opts{ port = P }, As);
+
+	{fd,Fd}     -> con_opt(Opts, R#connect_opts{ fd = Fd }, As);
+
 	binary      -> con_add(mode, binary, R, Opts, As);
+
 	list        -> con_add(mode, list, R, Opts, As);
+
 	{netns,NS} ->
 	    BinNS = filename2binary(NS),
 	    case prim_inet:is_sockopt_val(netns, BinNS) of
@@ -832,12 +1016,16 @@ con_opt([Opt | Opts], #connect_opts{} = R, As) ->
 		false ->
 		    {error, badarg}
 	    end;
+
         {active,N} when is_integer(N), N < 32768, N >= -32768 ->
             NOpts = lists:keydelete(active, 1, R#connect_opts.opts),
             con_opt(Opts, R#connect_opts { opts = [{active,N}|NOpts] }, As);
+
 	{line_delimiter,C} when is_integer(C), C >= 0, C =< 255 ->
 	    con_add(line_delimiter, C, R, Opts, As);
+
 	{Name,Val} when is_atom(Name) -> con_add(Name, Val, R, Opts, As);
+
 	_ -> {error, badarg}
     end;
 con_opt([], #connect_opts{} = R, _) ->
@@ -854,7 +1042,9 @@ con_add(Name, Val, #connect_opts{} = R, Opts, AllOpts) ->
 %% Available options for tcp:listen
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 listen_options() ->
-    [tos, tclass, priority, reuseaddr, keepalive, linger, sndbuf, recbuf, nodelay,
+    [debug,
+     tos, tclass,
+     priority, reuseaddr, keepalive, linger, sndbuf, recbuf, nodelay,
      recvtos, recvtclass, ttl, recvttl,
      header, active, packet, buffer, mode, deliver, backlog, ipv6_v6only,
      exit_on_close, high_watermark, low_watermark, high_msgq_watermark,
@@ -885,11 +1075,23 @@ listen_options(Opts, Mod) ->
 	
 list_opt([{raw,A,B,C}|Opts], #listen_opts{} = R, As) ->
     list_opt([{raw,{A,B,C}}|Opts], R, As);
-list_opt([Opt | Opts], #listen_opts{} = R, As) ->
+list_opt([Opt | Opts], #listen_opts{ifaddr = IfAddr} = R, As) ->
     case Opt of
+	{ifaddr, Addr} when is_map(Addr) ->
+            list_opt(Opts, R#listen_opts{ ifaddr = ensure_sockaddr(Addr) }, As);
+	{ifaddr, Addr} ->
+            list_opt(Opts, R#listen_opts{ ifaddr = Addr }, As);
+
+        %% This is when a previous value of ifaddr was a sockaddr_in6()
+	{ip,IP} when is_map(IfAddr) ->
+            list_opt(Opts, R#listen_opts{ ifaddr = IfAddr#{addr => IP} }, As);
 	{ip,IP}      ->  list_opt(Opts, R#listen_opts { ifaddr = IP }, As);
-	{ifaddr,IP}  ->  list_opt(Opts, R#listen_opts { ifaddr = IP }, As);
+
+        %% This is when a previous value of ifaddr was a sockaddr_in6()
+	{port,P} when is_map(IfAddr) ->
+            list_opt(Opts, R#listen_opts{ ifaddr = IfAddr#{port => P} }, As);
 	{port,P}     ->  list_opt(Opts, R#listen_opts { port = P }, As);
+
 	{fd,Fd}      ->  list_opt(Opts, R#listen_opts { fd = Fd }, As);
 	{backlog,BL} ->  list_opt(Opts, R#listen_opts { backlog = BL }, As);
 	binary       ->  list_add(mode, binary, R, Opts, As);
@@ -944,15 +1146,20 @@ gen_tcp_module(Opts, inet) ->
 gen_tcp_module(Opts, socket) ->
     {gen_tcp_socket, Opts}.
 
+
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% Available options for udp:open
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 udp_options() ->
-    [tos, tclass, priority, reuseaddr, sndbuf, recbuf, header, active, buffer, mode,
+    [
+     debug,
+     tos, tclass,
+     priority, reuseaddr, sndbuf, recbuf, header, active, buffer, mode,
      recvtos, recvtclass, ttl, recvttl, deliver, ipv6_v6only,
      broadcast, dontroute, multicast_if, multicast_ttl, multicast_loop,
-     add_membership, drop_membership, read_packets,raw,
-     high_msgq_watermark, low_msgq_watermark, bind_to_device].
+     add_membership, drop_membership, read_packets, raw,
+     high_msgq_watermark, low_msgq_watermark, bind_to_device
+    ].
 
 
 udp_options(Opts, Mod) ->
@@ -967,11 +1174,23 @@ udp_options(Opts, Mod) ->
 
 udp_opt([{raw,A,B,C}|Opts], #udp_opts{} = R, As) ->
     udp_opt([{raw,{A,B,C}}|Opts], R, As);
-udp_opt([Opt | Opts], #udp_opts{} = R, As) ->
+udp_opt([Opt | Opts], #udp_opts{ifaddr = IfAddr} = R, As) ->
     case Opt of
-	{ip,IP}     ->  udp_opt(Opts, R#udp_opts { ifaddr = IP }, As);
-	{ifaddr,IP} ->  udp_opt(Opts, R#udp_opts { ifaddr = IP }, As);
-	{port,P}    ->  udp_opt(Opts, R#udp_opts { port = P }, As);
+	{ifaddr, Addr} when is_map(Addr) ->
+            udp_opt(Opts, R#udp_opts { ifaddr = ensure_sockaddr(Addr) }, As);
+	{ifaddr, Addr} ->
+            udp_opt(Opts, R#udp_opts { ifaddr = Addr }, As);
+
+	{ip, IP} when is_map(IfAddr) ->
+            udp_opt(Opts, R#udp_opts { ifaddr = IfAddr#{addr => IP} }, As);
+	{ip, IP}                     ->
+            udp_opt(Opts, R#udp_opts { ifaddr = IP }, As);
+
+	{port, P} when is_map(IfAddr) ->
+            udp_opt(Opts, R#udp_opts { ifaddr = IfAddr#{port => P} }, As);
+	{port, P}                     ->
+            udp_opt(Opts, R#udp_opts { port = P }, As);
+
 	{fd,Fd}     ->  udp_opt(Opts, R#udp_opts { fd = Fd }, As);
 	binary      ->  udp_add(mode, binary, R, Opts, As);
 	list        ->  udp_add(mode, list, R, Opts, As);
@@ -984,9 +1203,20 @@ udp_opt([Opt | Opts], #udp_opts{} = R, As) ->
 		    {error, badarg}
 	    end;
         {active,N} when is_integer(N), N < 32768, N >= -32768 ->
-            NOpts = lists:keydelete(active, 1, R#udp_opts.opts),
-            udp_opt(Opts, R#udp_opts { opts = [{active,N}|NOpts] }, As);
+            POpts = lists:keydelete(active, 1, R#udp_opts.opts),
+            udp_opt(Opts, R#udp_opts { opts = [{active,N}|POpts] }, As);
+
+        {Membership, {MAddr, If}}
+          when ((Membership =:= add_membership) orelse
+                (Membership =:= drop_membership)) andalso
+               (tuple_size(MAddr) =:= 4) andalso
+               ((If =:= any) orelse (tuple_size(If) =:= 4)) ->
+            MembershipOpt = {Membership, {MAddr, If, 0}},
+            POpts         = R#udp_opts.opts,
+            udp_opt(Opts, R#udp_opts{opts = [MembershipOpt|POpts]}, As);
+
 	{Name,Val} when is_atom(Name) -> udp_add(Name, Val, R, Opts, As);
+
 	_ -> {error, badarg}
     end;
 udp_opt([], #udp_opts{} = R, _SockOpts) ->
@@ -1004,6 +1234,20 @@ udp_module(Opts) ->
       Opts, udp_module, undefined,
       #{inet => inet_udp, inet6 => inet6_udp, local => local_udp}).
 
+gen_udp_module([{inet_backend, Flag}|Opts]) ->
+    gen_udp_module(Opts, Flag);
+gen_udp_module(Opts) ->
+    gen_udp_module(
+      Opts,
+      persistent_term:get(
+        {kernel, inet_backend}, ?DEFAULT_KERNEL_INET_BACKEND)).
+%%
+gen_udp_module(Opts, inet) ->
+    {gen_udp, Opts};
+gen_udp_module(Opts, socket) ->
+    {gen_udp_socket, Opts}.
+
+
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% Available options for sctp:open
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1019,6 +1263,7 @@ udp_module(Opts) ->
 %  (*) passing of open FDs ("fdopen") is not supported.
 sctp_options() ->
 [   % The following are generic inet options supported for SCTP sockets:
+    debug,
     mode, active, buffer, tos, tclass, ttl,
     priority, dontroute, reuseaddr, linger,
     recvtos, recvtclass, recvttl,
@@ -1037,32 +1282,44 @@ sctp_options() ->
 
 sctp_options(Opts, Mod)  ->
     case sctp_opt(Opts, Mod, #sctp_opts{}, sctp_options()) of
-	{ok,#sctp_opts{ifaddr=undefined}=SO} -> 
-	    {ok,
-	     SO#sctp_opts{
-	       opts=lists:reverse(SO#sctp_opts.opts),
-	       ifaddr=Mod:translate_ip(?SCTP_DEF_IFADDR)}};
 	{ok,SO} ->
 	    {ok,SO#sctp_opts{opts=lists:reverse(SO#sctp_opts.opts)}};
 	Error -> Error
     end.
 
-sctp_opt([Opt|Opts], Mod, #sctp_opts{} = R, As) ->
+sctp_opt([Opt|Opts], Mod, #sctp_opts{ifaddr = IfAddr} = R, As) ->
     case Opt of
-	{ip,IP} ->
+        %% what if IfAddr is already a map (=sockaddr)?
+        %% Shall we allow ifaddr as a list of sockaddr?
+	{ifaddr, Addr} when is_map(Addr) ->
+            sctp_opt(Opts, Mod, R#sctp_opts{ifaddr = ensure_sockaddr(Addr)}, As);
+	{ifaddr, IP} ->
 	    sctp_opt_ifaddr(Opts, Mod, R, As, IP);
-	{ifaddr,IP} ->
+
+	{ip, IP} when is_map(IfAddr) ->
+            IP2 = Mod:translate_ip(IP),
+            sctp_opt(Opts, Mod, R#sctp_opts{ifaddr = IfAddr#{addr => IP2}}, As);
+	{ip, IP} ->
 	    sctp_opt_ifaddr(Opts, Mod, R, As, IP);
-	{port,Port} ->
+
+	{port, Port} ->
 	    case Mod:getserv(Port) of
-		{ok,P} ->
-		    sctp_opt(Opts, Mod, R#sctp_opts{port=P}, As);
-		Error -> Error
+		{ok, P} when is_map(IfAddr) ->
+		    sctp_opt(Opts,
+                             Mod,
+                             R#sctp_opts{ifaddr = IfAddr#{port => P}}, As);
+		{ok, P} ->
+		    sctp_opt(Opts, Mod, R#sctp_opts{port = P}, As);
+		Error ->
+                    Error
 	    end;
-	{type,Type} when Type =:= seqpacket; Type =:= stream ->
-	    sctp_opt(Opts, Mod, R#sctp_opts{type=Type}, As);
+
+	{type, Type} when Type =:= seqpacket; Type =:= stream ->
+            sctp_opt(Opts, Mod, R#sctp_opts{type = Type}, As);
+
 	binary		-> sctp_opt (Opts, Mod, R, As, mode, binary);
 	list		-> sctp_opt (Opts, Mod, R, As, mode, list);
+
 	{netns,NS} ->
 	    BinNS = filename2binary(NS),
 	    case prim_inet:is_sockopt_val(netns, BinNS) of
@@ -1074,11 +1331,16 @@ sctp_opt([Opt|Opts], Mod, #sctp_opts{} = R, As) ->
 		false ->
 		    {error, badarg}
 	    end;
+
         {active,N} when is_integer(N), N < 32768, N >= -32768 ->
             NOpts = lists:keydelete(active, 1, R#sctp_opts.opts),
             sctp_opt(Opts, Mod, R#sctp_opts { opts = [{active,N}|NOpts] }, As);
-	{Name,Val}	-> sctp_opt (Opts, Mod, R, As, Name, Val);
-	_ -> {error,badarg}
+
+	{Name,Val}	->
+            sctp_opt(Opts, Mod, R, As, Name, Val);
+
+	_ ->
+            {error, badarg}
     end;
 sctp_opt([], _Mod, #sctp_opts{ifaddr=IfAddr}=R, _SockOpts) ->
     if is_list(IfAddr) ->
@@ -1109,6 +1371,7 @@ sctp_module(Opts) ->
       Opts, sctp_module, undefined,
       #{inet => inet_sctp, inet6 => inet6_sctp}).
 
+
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% Util to check and insert option in option list
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1160,11 +1423,14 @@ binary2filename(Bin) ->
 	    Bin
     end.
 
-translate_ip(any,      inet) -> {0,0,0,0};
-translate_ip(loopback, inet) -> {127,0,0,1};
+%% Protocol independent, i.e common code for all
+%% inet_* and inet6_* modules
+%%
+translate_ip(any,      inet)  -> {0,0,0,0};
+translate_ip(loopback, inet)  -> {127,0,0,1};
 translate_ip(any,      inet6) -> {0,0,0,0,0,0,0,0};
 translate_ip(loopback, inet6) -> {0,0,0,0,0,0,0,1};
-translate_ip(IP, _) -> IP.
+translate_ip(IP, _)           -> IP.  % undefined goes here
 
 mod(Opts, Tag, Address, Map) ->
     mod(Opts, Tag, Address, Map, undefined, []).
@@ -1202,6 +1468,16 @@ mod([], Tag, Address, Map, undefined, Acc) ->
 		 Mod ->
 		     Mod
 	     end;
+	 {_, #{family := inet6}} ->
+	     #{inet := IPv4Mod} = Map,
+	     %% Get the mod, but IPv6 address overrides default IPv4
+	     case inet_db:Tag() of
+		 IPv4Mod ->
+		     #{inet6 := IPv6Mod} = Map,
+		     IPv6Mod;
+		 Mod ->
+		     Mod
+	     end;
 	 _ ->
 	     inet_db:Tag()
      end, lists:reverse(Acc)};
@@ -1416,8 +1692,10 @@ gethostbyaddr_tm_native(Addr, Timer, Opts) ->
 	Result -> Result
     end.
 
+
 -spec open(Fd_or_OpenOpts :: integer() | list(),
-	   Addr ::
+	   BAddr ::
+             socket:sockaddr_in6() |
 	     socket_address() |
 	     {ip_address() | 'any' | 'loopback', % Unofficial
 	      port_number()} |
@@ -1428,7 +1706,7 @@ gethostbyaddr_tm_native(Addr, Timer, Opts) ->
 	      {ip6_address() | 'any' | 'loopback',
 	       port_number()}} |
 	     undefined, % Internal - no bind()
-	   Port :: port_number(),
+	   BPort :: port_number(),
 	   Opts :: [socket_setopt()],
 	   Protocol :: socket_protocol(),
 	   Family :: address_family(),
@@ -1436,42 +1714,157 @@ gethostbyaddr_tm_native(Addr, Timer, Opts) ->
 	   Module :: atom()) ->
 	{'ok', port()} | {'error', posix()}.
 
-open(FdO, Addr, Port, Opts, Protocol, Family, Type, Module)
-  when is_integer(FdO), FdO < 0;
-       is_list(FdO) ->
+open(Fd, BAddr, BPort, Opts, Protocol, Family, Type, Module)
+  when is_integer(Fd), 0 =< Fd ->
+    open_fd(Fd, BAddr, BPort, Opts, Protocol, Family, Type, Module);
+open(Fd_or_OpenOpts, BAddr, BPort, Opts, Protocol, Family, Type, Module) ->
+    open_opts(
+      Fd_or_OpenOpts,
+      if
+          BAddr =:= undefined, BPort =/= 0 ->
+              translate_ip(any, Family);
+          true ->
+              BAddr
+      end, BPort, Opts, Protocol, Family, Type, Module).
+
+%% The only difference between open/8 and open_bind/8 is that
+%% if Fd_or_OpenOpts is not a FileDescriptor :: non_neg_integer()
+%% i.e option {fd,Fd} has not been used hence we are not handling
+%% an already open socket handle, and also if no bind address
+%% has been specified (BAddr =:= undefined).
+%%
+%% Then open_bind/8 will bind to the wildcard address and the
+%% specified port (BPort, 0 = wildcard port per default),
+%% which is the legacy behaviour by this module when opening a socket.
+%%
+%% In the same situation, open/8 will bind to the wildcard address
+%% and the specified port, only if BPort is not 0, i.e a bind port
+%% has been specified to not be the wildcard port.
+%%
+%% So open/8 per default does not bind to an address, which
+%% is used by TCP connect to let the OS automatically bind to
+%% an address later, during TCP connect operation.  This
+%% gives the OS more freedom in choosing the originating port and
+%% therefore makes far more effective use of the port range.
+
+-spec open_bind(Fd_or_OpenOpts :: integer() | list(),
+                BAddr ::
+                  socket:sockaddr_in6() |
+                  socket_address() |
+                  {ip_address() | 'any' | 'loopback', % Unofficial
+                   port_number()} |
+                  {inet, % Unofficial
+                   {ip4_address() | 'any' | 'loopback',
+                    port_number()}} |
+                  {inet6, % Unofficial
+                   {ip6_address() | 'any' | 'loopback',
+                    port_number()}} |
+                  undefined, % Internal - translated to 'any'
+                BPort :: port_number(),
+                Opts :: [socket_setopt()],
+                Protocol :: socket_protocol(),
+                Family :: address_family(),
+                Type :: socket_type(),
+                Module :: atom()) ->
+                       {'ok', port()} | {'error', posix()}.
+
+open_bind(Fd, BAddr, BPort, Opts, Protocol, Family, Type, Module)
+  when is_integer(Fd), 0 =< Fd ->
+    %% ?DBG([{fd, Fd},
+    %%       {baddr, BAddr}, {bport, BPort},
+    %%       {opts, Opts}, {proto, Protocol}, {fam, Family},
+    %%       {type, Type}, {mod, Module}]),
+    open_fd(Fd, BAddr, BPort, Opts, Protocol, Family, Type, Module);
+open_bind(
+  Fd_or_OpenOpts, BAddr, BPort, Opts, Protocol, Family, Type, Module) ->
+    %% ?DBG([{fd_or_openopts, Fd_or_OpenOpts},
+    %%       {baddr, BAddr}, {bport, BPort},
+    %%       {opts, Opts}, {proto, Protocol}, {fam, Family},
+    %%       {type, Type}, {mod, Module}]),
+    open_opts(
+      Fd_or_OpenOpts,
+      if
+          BAddr =:= undefined ->
+              translate_ip(any, Family);
+          true ->
+              BAddr
+      end, BPort, Opts, Protocol, Family, Type, Module).
+
+
+open_fd(Fd, BAddr, BPort, Opts, Protocol, Family, Type, Module) ->
+    DoNotBind =
+	%% We do not do any binding if no port+addr options
+	%% were given, in order to keep backwards compatibility
+	%% with pre Erlang/OTP 17
+        BAddr =:= undefined, % Presumably already bound
+    if
+        DoNotBind ->
+            0 = BPort, ok; % Assertion
+        true ->
+            ok
+    end,
+    case prim_inet:fdopen(Protocol, Family, Type, Fd, DoNotBind) of
+	{ok, S} ->
+            open_setopts(S, BAddr, BPort, Opts, Module);
+        Error ->
+            Error
+    end.
+
+open_opts(Fd_or_OpenOpts, BAddr, BPort, Opts, Protocol, Family, Type, Module) ->
+    %% ?DBG([{fd_or_openopts, Fd_or_OpenOpts},
+    %%       {baddr, BAddr}, {bport, BPort},
+    %%       {opts, Opts}, {proto, Protocol}, {fam, Family},
+    %%       {type, Type}, {mod, Module}]),
     OpenOpts =
-	if  is_list(FdO) -> FdO;
+	if
+            is_list(Fd_or_OpenOpts) -> Fd_or_OpenOpts;
 	    true -> []
 	end,
     case prim_inet:open(Protocol, Family, Type, OpenOpts) of
 	{ok,S} ->
-	    case prim_inet:setopts(S, Opts) of
-		ok when Addr =:= undefined ->
-		    inet_db:register_socket(S, Module),
-		    {ok,S};
-		ok ->
-		    case bind(S, Addr, Port) of
-			{ok, _} ->
-			    inet_db:register_socket(S, Module),
-			    {ok,S};
-			Error  ->
-			    prim_inet:close(S),
-			    Error
-		    end;
-		Error  ->
-		    prim_inet:close(S),
-		    Error
-	    end;
-	Error ->
-	    Error
-    end;
-open(Fd, Addr, Port, Opts, Protocol, Family, Type, Module)
-  when is_integer(Fd) ->
-    fdopen(Fd, Addr, Port, Opts, Protocol, Family, Type, Module).
+            open_setopts(S, BAddr, BPort, Opts, Module);
+        Error ->
+            Error
+    end.
+
+%% If BAddr is undefined - do not bind to an address
+%%
+open_setopts(S, BAddr, BPort, Opts, Module) ->
+    %% ?DBG([{s, S}, {baddr, BAddr}, {bport, BPort}, {opts, Opts}, {mod, Module}]),
+    case prim_inet:setopts(S, Opts) of
+        ok when BAddr =:= undefined ->
+            %% ?DBG("register socket"),
+            inet_db:register_socket(S, Module),
+            {ok,S};
+        ok ->
+            %% ?DBG("try bind"),
+            try bind(S, BAddr, BPort) of
+                {ok, _} ->
+                    %% ?DBG("bound"),
+                    inet_db:register_socket(S, Module),
+                    {ok,S};
+                Error  ->
+                    %% ?DBG(["bind error", {error, Error}]),
+                    prim_inet:close(S),
+                    Error
+            catch
+                BC:BE:BS ->
+                    %% ?DBG(["bind failed", {class, BC}, {error, BE}, {stack, BS}]),
+                    prim_inet:close(S),
+                    erlang:raise(BC, BE, BS)
+            end;
+        Error  ->
+            %% ?DBG(["setopts error", {error, Error}]),
+            prim_inet:close(S),
+            Error
+    end.
+
+
 
 bind(S, Addr, Port) when is_list(Addr) ->
     bindx(S, Addr, Port);
 bind(S, Addr, Port) ->
+    %% ?DBG([{s, S}, {addr, Addr}, {port, Port}]),
     prim_inet:bind(S, Addr, Port).
 
 bindx(S, [Addr], Port0) ->
@@ -1510,43 +1903,9 @@ change_bindx_0_port({_IP, _Port}=Addr, _AssignedPort) ->
 	     Module :: atom()) ->
 	{'ok', socket()} | {'error', posix()}.
 
-fdopen(Fd, Opts, Protocol, Family, Type, Module) ->
-    fdopen(Fd, any, 0, Opts, Protocol, Family, Type, Module).
-
-fdopen(Fd, Addr, Port, Opts, Protocol, Family, Type, Module) ->
-    Bound =
-	%% We do not do any binding if default port+addr options
-	%% were given, in order to keep backwards compatability
-	%% with pre Erlang/OTP 17
-	case Addr of
-	    {0,0,0,0} when Port =:= 0 -> true;
-	    {0,0,0,0,0,0,0,0} when Port =:= 0 -> true;
-	    any when Port =:= 0 -> true;
-	    _ -> false
-	end,
-    case prim_inet:fdopen(Protocol, Family, Type, Fd, Bound) of
-	{ok, S} ->
-	    case prim_inet:setopts(S, Opts) of
-		ok
-		  when Addr =:= undefined;
-		       Bound ->
-		    inet_db:register_socket(S, Module),
-		    {ok, S};
-		ok ->
-		    case bind(S, Addr, Port) of
-			{ok, _} ->
-			    inet_db:register_socket(S, Module),
-			    {ok, S};
-			Error  ->
-			    prim_inet:close(S),
-			    Error
-                    end;
-		Error ->
-		    prim_inet:close(S),
-		    Error
-	    end;
-	Error -> Error
-    end.
+fdopen(Fd, Opts, Protocol, Family, Type, Module)
+  when is_integer(Fd), 0 =< Fd ->
+    open_fd(Fd, undefined, 0, Opts, Protocol, Family, Type, Module).
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %%  socket stat
@@ -1599,6 +1958,59 @@ upper(C) when C >= $a, C =< $z -> (C-$a) + $A;
 upper(C) -> C.
 
     
+info({'$inet', GenSocketMod, _} = S, F, Proto) when is_atom(GenSocketMod) ->
+    case F of
+	owner ->
+	    case GenSocketMod:info(S) of
+		#{owner := Owner} when is_pid(Owner) -> pid_to_list(Owner);
+		_ -> " "
+	    end;
+	port ->
+	    case GenSocketMod:getopts(S, [fd]) of
+		{ok, [{fd, FD}]} ->
+		    "esock[" ++ integer_to_list(FD) ++ "]";
+		_ ->
+		    "esock"
+	    end;
+	sent ->
+	    case GenSocketMod:getstat(S, [send_oct]) of
+		{ok, [{send_oct, N}]} -> integer_to_list(N);
+		_ -> " "
+	    end;
+	recv ->
+	    case GenSocketMod:getstat(S, [recv_oct]) of
+		{ok, [{recv_oct, N}]} -> integer_to_list(N);
+		_ -> " "
+	    end;
+	local_address ->
+	    fmt_addr(GenSocketMod:sockname(S), Proto);
+	foreign_address ->
+	    fmt_addr(GenSocketMod:peername(S), Proto);
+	state ->
+	    case GenSocketMod:info(S) of
+		#{rstates := RStates,
+		  wstates := WStates} -> fmt_compat_status(RStates, WStates);
+		_ -> " "
+	    end;
+	packet ->
+	    case GenSocketMod:which_packet_type(S) of
+		{ok, Type} -> atom_to_list(Type);
+		_ -> " "
+	    end;
+	type ->
+	    case GenSocketMod:info(S) of
+		#{type := stream} -> "STREAM";
+		_ -> " "
+	    end;
+	%% Why do we have this here? Its never called (see i/2 calling i/2).
+	fd ->
+	    case GenSocketMod:getopts(S, [fd]) of
+		{ok, [{fd, Fd}]} -> integer_to_list(Fd);
+		_ -> " "
+	    end;
+	module ->
+	    atom_to_list(GenSocketMod)
+    end;
 info(S, F, Proto) ->
     case F of
 	owner ->
@@ -1643,6 +2055,7 @@ info(S, F, Proto) ->
 		{ok,{_,seqpacket}} -> "SEQPACKET";
 		_ -> " "
 	    end;
+	%% Why do we have this here? Its never called (see i/2 calling i/2).
 	fd ->
 	    case prim_inet:getfd(S) of
 		{ok, Fd} -> integer_to_list(Fd);
@@ -1654,6 +2067,7 @@ info(S, F, Proto) ->
 		_ -> "prim_inet"
 	    end
     end.
+
 %% Possible flags: (sorted)
 %% [accepting,bound,busy,connected,connecting,listen,listening,open]
 %% Actually, we no longer gets listening...
@@ -1672,6 +2086,19 @@ fmt_status(Flags) ->
 	Sorted                        -> fmt_status2(Sorted)
     end.
 
+fmt_compat_status(RFlags, WFlags) ->
+    fmt_status(fmt_compat_status_merge(RFlags, WFlags)).
+
+fmt_compat_status_merge(RFlags, WFlags) ->
+    fmt_compat_status_merge(RFlags, WFlags, []).
+    
+fmt_compat_status_merge([], WFlags, Merged) ->
+    Merged ++ WFlags;
+fmt_compat_status_merge([RFlag|RFlags], WFlags, Merged) ->
+    fmt_compat_status_merge(RFlags,
+			    lists:delete(RFlag, WFlags),
+			    [RFlag|Merged]).
+
 fmt_status2([H]) ->
     fmt_status3(H);
 fmt_status2([H|T]) ->
@@ -1693,6 +2120,8 @@ fmt_status3(listening) ->
     "LG";
 fmt_status3(open) ->
     "O";
+fmt_status3(selected) ->
+    "SD";
 fmt_status3(X) when is_atom(X) ->
     string:uppercase(atom_to_list(X)).
 
@@ -1717,8 +2146,8 @@ fmt_port(N, Proto) ->
     end.
 
 %% Return a list of all tcp sockets
-tcp_sockets() -> port_list("tcp_inet").
-udp_sockets() -> port_list("udp_inet").
+tcp_sockets()  -> port_list("tcp_inet") ++ gen_tcp_socket:which_sockets().
+udp_sockets()  -> port_list("udp_inet") ++ gen_udp_socket:which_sockets().
 sctp_sockets() -> port_list("sctp_inet").
 
 %% Return all ports having the name 'Name'
@@ -1731,6 +2160,7 @@ port_list(Name) ->
 	      end
       end, erlang:ports()).
 
+
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %%  utils
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1898,3 +2328,11 @@ lock_socket(S,Val) ->
 	_ ->
 	    prim_inet:ignorefd(S,Val)
     end.
+
+
+ensure_sockaddr(SockAddr) ->
+    try prim_socket:enc_sockaddr(SockAddr)
+    catch
+        throw : {invalid, _} = Invalid : Stacktrace ->
+            erlang:raise(error, Invalid, Stacktrace)
+    end.
