@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1997-2017. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2021. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -45,6 +45,7 @@
 -import(lists, [reverse/1]).
 
 -include_lib("kernel/include/file.hrl").
+-include("inet_int.hrl").
 
 %% --------------------------------------------------------------------------
 %% Parse services internet style
@@ -95,7 +96,7 @@ hosts(Fname,File) ->
 		 %% interface with a %if suffix. These kind of
 		 %% addresses maybe need to be gracefully handled
 		 %% throughout inet* and inet_drv.
-		 case string:tokens(Address, "%") of
+		 case string:lexemes(Address, "%") of
 		     [Addr,_] ->
 			 {ok,_} = address(Addr),
 			 skip;
@@ -407,7 +408,7 @@ is_dom1([C | Cs]) when C >= $a, C =< $z -> is_dom_ldh(Cs);
 is_dom1([C | Cs]) when C >= $A, C =< $Z -> is_dom_ldh(Cs);
 is_dom1([C | Cs]) when C >= $0, C =< $9 -> 
     case is_dom_ldh(Cs) of
-	true  -> is_dom2(string:tokens([C | Cs],"."));
+	true  -> is_dom2(string:lexemes([C | Cs],"."));
 	false -> false
     end;
 is_dom1(_) -> false.
@@ -759,7 +760,7 @@ dup(N, E, L) when is_integer(N), N >= 1 ->
 
 %% Convert IPv4 address to ascii
 %% Convert IPv6 / IPV4 address to ascii (plain format)
-ntoa({A,B,C,D}) when (A band B band C band D band (bnot 16#ff)) =:= 0 ->
+ntoa({A,B,C,D}) when ?ip(A,B,C,D) ->
     integer_to_list(A) ++ "." ++ integer_to_list(B) ++ "." ++ 
 	integer_to_list(C) ++ "." ++ integer_to_list(D);
 %% ANY
@@ -767,14 +768,12 @@ ntoa({0,0,0,0,0,0,0,0}) -> "::";
 %% LOOPBACK
 ntoa({0,0,0,0,0,0,0,1}) -> "::1";
 %% IPV4 ipv6 host address
-ntoa({0,0,0,0,0,0,A,B}) when (A band B band (bnot 16#ffff)) =:= 0 ->
+ntoa({0,0,0,0,0,0,A,B}) when ?ip6(0,0,0,0,0,0,A,B) ->
     "::" ++ dig_to_dec(A) ++ "." ++ dig_to_dec(B);
 %% IPV4 non ipv6 host address
-ntoa({0,0,0,0,0,16#ffff,A,B}) when (A band B band (bnot 16#ffff)) =:= 0 ->
+ntoa({0,0,0,0,0,16#ffff=X,A,B}) when ?ip6(0,0,0,0,0,X,A,B) ->
     "::ffff:" ++ dig_to_dec(A) ++ "." ++ dig_to_dec(B);
-ntoa({A,B,C,D,E,F,G,H})
-  when (A band B band C band D band E band F band G band H band
-            (bnot 16#ffff)) =:= 0 ->
+ntoa({A,B,C,D,E,F,G,H}) when ?ip6(A,B,C,D,E,F,G,H) ->
     if
         A =:= 16#fe80, B =/= 0;
         A =:= 16#ff02, B =/= 0 ->
