@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1999-2017. All Rights Reserved.
+%% Copyright Ericsson AB 1999-2022. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -26,17 +26,9 @@
 	 init_per_group/2,end_per_group/2]).
 
 -export([init_per_testcase/2, end_per_testcase/2, whitebox/1, 
-	 switch_options/1, missing_compulsory_dflags/1]).
+	 switch_options/1, missing_compulsory_dflags/1,
+         dflag_mandatory_25/1]).
 
-%% 1)
-%%
-%% Connections are now always set up symmetrically with respect to
-%% publication. If connecting node doesn't send DFLAG_PUBLISHED
-%% the other node wont send DFLAG_PUBLISHED. If the connecting
-%% node send DFLAG_PUBLISHED but the other node doesn't send
-%% DFLAG_PUBLISHED, the connecting node should consider its
-%% DFLAG_PUBLISHED as dropped, i.e the connecting node wont be
-%% published on the other node.
 
 -define(to_port(Socket, Data),
 	case inet_tcp:send(Socket, Data) of
@@ -47,25 +39,42 @@
 	        R
         end).
 
--define(DFLAG_PUBLISHED,1).
--define(DFLAG_ATOM_CACHE,2).
--define(DFLAG_EXTENDED_REFERENCES,4).
--define(DFLAG_DIST_MONITOR,8).
--define(DFLAG_FUN_TAGS,16#10).
--define(DFLAG_DIST_MONITOR_NAME,16#20).
--define(DFLAG_HIDDEN_ATOM_CACHE,16#40).
--define(DFLAG_NEW_FUN_TAGS,16#80).
--define(DFLAG_EXTENDED_PIDS_PORTS,16#100).
--define(DFLAG_UTF8_ATOMS, 16#10000).
+-define(DIST_VER_HIGH, 6).
+-define(DIST_VER_LOW, 6).
 
-%% From R9 and forward extended references is compulsory
-%% From R10 and forward extended pids and ports are compulsory
-%% From R20 and forward UTF8 atoms are compulsory
-%% From R21 and forward NEW_FUN_TAGS is compulsory (no more tuple fallback {fun, ...})
--define(COMPULSORY_DFLAGS, (?DFLAG_EXTENDED_REFERENCES bor
-                            ?DFLAG_EXTENDED_PIDS_PORTS bor
-                            ?DFLAG_UTF8_ATOMS bor
-                            ?DFLAG_NEW_FUN_TAGS)).
+-define(DFLAG_PUBLISHED,                16#01).
+-define(DFLAG_ATOM_CACHE,               16#02).
+-define(DFLAG_EXTENDED_REFERENCES,      16#04).
+-define(DFLAG_DIST_MONITOR,             16#08).
+-define(DFLAG_FUN_TAGS,                 16#10).
+-define(DFLAG_NEW_FUN_TAGS,             16#80).
+-define(DFLAG_EXTENDED_PIDS_PORTS,     16#100).
+-define(DFLAG_EXPORT_PTR_TAG,          16#200).
+-define(DFLAG_BIT_BINARIES,            16#400).
+-define(DFLAG_NEW_FLOATS,              16#800).
+-define(DFLAG_UTF8_ATOMS,            16#10000).
+-define(DFLAG_MAP_TAG,               16#20000).
+-define(DFLAG_BIG_CREATION,          16#40000).
+-define(DFLAG_HANDSHAKE_23,        16#1000000).
+-define(DFLAG_MANDATORY_25_DIGEST, 16#4000000).
+
+%% From OTP R9 extended references are compulsory.
+%% From OTP R10 extended pids and ports are compulsory.
+%% From OTP 20 UTF8 atoms are compulsory.
+%% From OTP 21 NEW_FUN_TAGS is compulsory (no more tuple fallback {fun, ...}).
+%% From OTP 23 BIG_CREATION is compulsory.
+%% From OTP 25 NEW_FLOATS, MAP_TAG, EXPORT_PTR_TAG, and BIT_BINARIES are compulsory.
+-define(COMPULSORY_DFLAGS,
+        (?DFLAG_EXTENDED_REFERENCES bor
+             ?DFLAG_FUN_TAGS bor
+             ?DFLAG_EXTENDED_PIDS_PORTS bor
+             ?DFLAG_UTF8_ATOMS bor
+             ?DFLAG_NEW_FUN_TAGS bor
+             ?DFLAG_BIG_CREATION bor
+             ?DFLAG_NEW_FLOATS bor
+             ?DFLAG_MAP_TAG bor
+             ?DFLAG_EXPORT_PTR_TAG bor
+             ?DFLAG_BIT_BINARIES)).
 
 -define(PASS_THROUGH, $p).
 
@@ -91,7 +100,7 @@ suite() ->
      {timetrap,{minutes,1}}].
 
 all() -> 
-    [whitebox, switch_options, missing_compulsory_dflags].
+    [whitebox, switch_options, missing_compulsory_dflags, dflag_mandatory_25].
 
 groups() -> 
     [].
@@ -128,13 +137,13 @@ switch_options(Config) when is_list(Config) ->
 
 %% Whitebox testing of distribution handshakes.
 whitebox(Config) when is_list(Config) ->
-    {ok, Node} = start_node(?MODULE,""),
+    {ok, Peer, Node} = ?CT_PEER(),
     Cookie = erlang:get_cookie(),
     {_,Host} = split(node()),
     ok = pending_up_md5(Node, join(ccc,Host), Cookie),
     ok = simultaneous_md5(Node, join('A',Host), Cookie),
     ok = simultaneous_md5(Node, join(zzzzzzzzzzzzzz,Host), Cookie),
-    stop_node(Node),
+    peer:stop(Peer),
     ok.
 
 %%
@@ -202,15 +211,15 @@ test_switch_active_and_packet() ->
 %%
 %% Handshake tests
 %%
-pending_up_md5(Node,OurName,Cookie) ->
+pending_up_md5(Node, OurName, Cookie) ->
     {NA,NB} = split(Node),
-    {port,PortNo,_} = erl_epmd:port_please(NA,NB),
+    {port,PortNo,?DIST_VER_HIGH} = erl_epmd:port_please(NA,NB),
     {ok, SocketA} = gen_tcp:connect(atom_to_list(NB),PortNo,
 				    [{active,false},
 				     {packet,2}]),
-    send_name(SocketA,OurName,5),
+    send_name(SocketA,OurName),
     ok = recv_status(SocketA),
-    {hidden,Node,5,HisChallengeA} = recv_challenge(SocketA), % See 1)
+    {Node,HisChallengeA} = recv_challenge(SocketA),
     OurChallengeA = gen_challenge(),
     OurDigestA = gen_digest(HisChallengeA, Cookie),
     send_challenge_reply(SocketA, OurChallengeA, OurDigestA),
@@ -224,11 +233,11 @@ pending_up_md5(Node,OurName,Cookie) ->
     {ok, SocketB} = gen_tcp:connect(atom_to_list(NB),PortNo,
 				    [{active,false},
 				     {packet,2}]),
-    send_name(SocketB,OurName,5),
+    send_name(SocketB,OurName),
     alive = recv_status(SocketB),
     send_status(SocketB, true),
     gen_tcp:close(SocketA),
-    {hidden,Node,5,HisChallengeB} = recv_challenge(SocketB), % See 1)
+    {Node,HisChallengeB} = recv_challenge(SocketB),
     OurChallengeB = gen_challenge(),
     OurDigestB = gen_digest(HisChallengeB, Cookie),
     send_challenge_reply(SocketB, OurChallengeB, OurDigestB),
@@ -254,15 +263,15 @@ simultaneous_md5(Node, OurName, Cookie) when OurName < Node ->
 		  Else ->
 		      exit(Else)
 	      end,
-    EpmdSocket = register(OurName, LSocket, 1, 5),
+    EpmdSocket = register_node(OurName, LSocket),
     {NA, NB} = split(Node),
     rpc:cast(Node, net_adm, ping, [OurName]),
     receive after 1000 -> ok end,
-    {port, PortNo, _} = erl_epmd:port_please(NA,NB),
+    {port, PortNo, ?DIST_VER_HIGH} = erl_epmd:port_please(NA,NB),
     {ok, SocketA} = gen_tcp:connect(atom_to_list(NB),PortNo,
 				    [{active,false},
 				     {packet,2}]),
-    send_name(SocketA,OurName,5),
+    send_name(SocketA,OurName),
     %% We are still not marked up on the other side, as our first message 
     %% is not sent.
     SocketB = case gen_tcp:accept(LSocket) of
@@ -275,11 +284,12 @@ simultaneous_md5(Node, OurName, Cookie) when OurName < Node ->
     %% Now we are expected to close A
     gen_tcp:close(SocketA),
     %% But still Socket B will continue
-    {normal,Node,5} = recv_name(SocketB),  % See 1)
+    {Node,GotFlags} = recv_name(SocketB),
+    true = (GotFlags band ?DFLAG_HANDSHAKE_23) =/= 0,
     send_status(SocketB, ok_simultaneous),
     MyChallengeB = gen_challenge(),
-    send_challenge(SocketB, OurName, MyChallengeB,5),
-    HisChallengeB = recv_challenge_reply(SocketB, MyChallengeB, Cookie),
+    send_challenge(SocketB, OurName, MyChallengeB, GotFlags),
+    {ok,HisChallengeB} = recv_challenge_reply(SocketB, MyChallengeB, Cookie),
     DigestB = gen_digest(HisChallengeB,Cookie),
     send_challenge_ack(SocketB, DigestB),
     inet:setopts(SocketB, [{active, false},
@@ -301,11 +311,11 @@ simultaneous_md5(Node, OurName, Cookie) when OurName > Node ->
 		  Else ->
 		      exit(Else)
 	      end,
-    EpmdSocket = register(OurName, LSocket, 1, 5),
+    EpmdSocket = register_node(OurName, LSocket),
     {NA, NB} = split(Node),
     rpc:cast(Node, net_adm, ping, [OurName]),
     receive after 1000 -> ok end,
-    {port, PortNo, _} = erl_epmd:port_please(NA,NB),
+    {port, PortNo, ?DIST_VER_HIGH} = erl_epmd:port_please(NA,NB),
     {ok, SocketA} = gen_tcp:connect(atom_to_list(NB),PortNo,
 				    [{active,false},
 				     {packet,2}]),
@@ -315,16 +325,16 @@ simultaneous_md5(Node, OurName, Cookie) when OurName > Node ->
 		  Else2 ->
 		      exit(Else2)
 	      end,
-    send_name(SocketA,OurName,5),
+    send_name(SocketA,OurName),
     ok_simultaneous = recv_status(SocketA),
     %% Socket B should die during this
     case catch begin
-		   {normal,Node,5} = recv_name(SocketB),  % See 1)
+		   {Node,GotFlagsB} = recv_name(SocketB),
+                   true = (GotFlagsB band ?DFLAG_HANDSHAKE_23) =/= 0,
 		   send_status(SocketB, ok_simultaneous),
 		   MyChallengeB = gen_challenge(),
-		   send_challenge(SocketB, OurName, MyChallengeB,
-				  5),
-		   HisChallengeB = recv_challenge_reply(
+		   send_challenge(SocketB, OurName, MyChallengeB, GotFlagsB),
+		   {ok,HisChallengeB} = recv_challenge_reply(
 				     SocketB,
 				     MyChallengeB,
 				     Cookie),
@@ -346,7 +356,7 @@ simultaneous_md5(Node, OurName, Cookie) when OurName > Node ->
     end,
     gen_tcp:close(SocketB),
     %% But still Socket A will continue
-    {hidden,Node,5,HisChallengeA} = recv_challenge(SocketA), % See 1)
+    {Node,HisChallengeA} = recv_challenge(SocketA),
     OurChallengeA = gen_challenge(),
     OurDigestA = gen_digest(HisChallengeA, Cookie),
     send_challenge_reply(SocketA, OurChallengeA, OurDigestA),
@@ -364,18 +374,34 @@ simultaneous_md5(Node, OurName, Cookie) when OurName > Node ->
     ok.
 
 missing_compulsory_dflags(Config) when is_list(Config) ->
-    [Name1, Name2] = get_nodenames(2, missing_compulsory_dflags),
-    {ok, Node} = start_node(Name1,""),
+    {ok, Peer, Node} = ?CT_PEER(),
     {NA,NB} = split(Node),
     {port,PortNo,_} = erl_epmd:port_please(NA,NB),
     {ok, SocketA} = gen_tcp:connect(atom_to_list(NB),PortNo,
-				    [{active,false},
-				     {packet,2}]),
-    BadNode = list_to_atom(atom_to_list(Name2)++"@"++atom_to_list(NB)),
-    send_name(SocketA,BadNode,5,0),
+                                    [{active,false},
+                                     {packet,2}]),
+    BadNode = list_to_atom(?CT_PEER_NAME()++"@"++atom_to_list(NB)),
+    send_name(SocketA,BadNode, 0),
     not_allowed = recv_status(SocketA),
     gen_tcp:close(SocketA),
-    stop_node(Node),
+    peer:stop(Peer),
+    ok.
+
+%% Test that instead of passing all compulsory flags, we can instead
+%% pass only ?DFLAG_MANDATORY_25_DIGEST to ensure that we will be able to communicate
+%% with a future release where ?DFLAG_MANDATORY_25_DIGEST is mandatory.
+dflag_mandatory_25(_Config) ->
+    {ok, Peer, Node} = ?CT_PEER(),
+    {NA,NB} = split(Node),
+    {port,PortNo,_} = erl_epmd:port_please(NA, NB),
+    {ok, SocketA} = gen_tcp:connect(atom_to_list(NB),
+                                    PortNo,
+                                    [{active,false},{packet,2}]),
+    OtherNode = list_to_atom(?CT_PEER_NAME()++"@"++atom_to_list(NB)),
+    send_name(SocketA, OtherNode, ?DFLAG_MANDATORY_25_DIGEST),
+    ok = recv_status(SocketA),
+    gen_tcp:close(SocketA),
+    peer:stop(Peer),
     ok.
 
 %%
@@ -466,7 +492,7 @@ gen_digest(Challenge, Cookie) when is_integer(Challenge), is_atom(Cookie) ->
 
 
 %%
-%% The differrent stages of the MD5 handshake
+%% The different stages of the MD5 handshake
 %%
 
 send_status(Socket, Stat) ->
@@ -486,46 +512,67 @@ recv_status(Socket) ->
 	    exit(Bad)
     end.
 
-send_challenge(Socket, Node, Challenge, Version) ->
-    send_challenge(Socket, Node, Challenge, Version, ?COMPULSORY_DFLAGS).
-send_challenge(Socket, Node, Challenge, Version, Flags) ->
+send_challenge(Socket, Node, Challenge, GotFlags) ->
+    Flags = ?COMPULSORY_DFLAGS bor ?DFLAG_MANDATORY_25_DIGEST,
+    send_challenge(Socket, Node, Challenge, GotFlags, Flags).
+
+send_challenge(Socket, Node, Challenge, GotFlags, Flags) ->
+    true = (GotFlags band ?DFLAG_HANDSHAKE_23) =/= 0,
     {ok, {{_Ip1,_Ip2,_Ip3,_Ip4}, _}} = inet:sockname(Socket),
-    ?to_port(Socket, [$n,?int16(Version),?int32(Flags),
-		      ?int32(Challenge), atom_to_list(Node)]).
+    NodeName = atom_to_list(Node),
+    Nlen = length(NodeName),
+    Creation = erts_internal:get_creation(),
+    ?to_port(Socket, [$N, <<(Flags bor ?DFLAG_HANDSHAKE_23):64>>,
+                      <<Challenge:32>>, <<Creation:32>>,
+                      <<Nlen:16>>, NodeName
+                      ]).
 
 recv_challenge(Socket) ->
-    case gen_tcp:recv(Socket, 0) of
-	{ok,[$n,V1,V0,Fl1,Fl2,Fl3,Fl4,CA3,CA2,CA1,CA0 | Ns]} ->
-	    Flags = ?u32(Fl1,Fl2,Fl3,Fl4),
-	    Type = case Flags band ?DFLAG_PUBLISHED of
-		       0 ->
-			   hidden;
-		       _ ->
-			   normal
-		   end,
-	    Node =list_to_atom(Ns),
-	    Version = ?u16(V1,V0),
+    {ok, Msg} = gen_tcp:recv(Socket, 0),
+    %%io:format("recv_challenge Msg=~p\n", [Msg]),
+    case Msg of
+        [$N, F7,F6,F5,F4,F3,F2,F1,F0, CA3,CA2,CA1,CA0,
+         Cr3,Cr2,Cr1,Cr0, NL1,NL0 | Ns] ->
+	    <<Flags:64>> = <<F7,F6,F5,F4,F3,F2,F1,F0>>,
+            verify_flags(Flags),
+            <<Creation:32>> = <<Cr3,Cr2,Cr1,Cr0>>,
+            true = (Creation =/= 0),
+            <<NameLen:16>> = <<NL1,NL0>>,
+            NameLen = length(Ns),
+	    Node = list_to_atom(Ns),
 	    Challenge = ?u32(CA3,CA2,CA1,CA0),
-	    {Type,Node,Version,Challenge};
+	    {Node, Challenge};
+
 	_ ->
 	    ?shutdown(no_node)	    
     end.
 
+verify_flags(Flags) ->
+    RequiredFlags = ?COMPULSORY_DFLAGS bor ?DFLAG_MANDATORY_25_DIGEST,
+    if
+        Flags band RequiredFlags =:= RequiredFlags ->
+            ok;
+        true ->
+            io:format("Given flags:    ~.16.0B\n", [Flags]),
+            io:format("Required flags: ~.16.0B\n", [RequiredFlags]),
+            ct:fail(missing_dflags)
+    end.
+
 send_challenge_reply(Socket, Challenge, Digest) ->
     ?to_port(Socket, [$r,?int32(Challenge),Digest]).
 
 recv_challenge_reply(Socket, ChallengeA, Cookie) ->
     case gen_tcp:recv(Socket, 0) of
-	{ok,[$r,CB3,CB2,CB1,CB0 | SumB]} when length(SumB) == 16 ->
+	{ok,[$r,CB3,CB2,CB1,CB0 | SumB]=Data} when length(SumB) == 16 ->
 	    SumA = gen_digest(ChallengeA, Cookie),
 	    ChallengeB = ?u32(CB3,CB2,CB1,CB0),
 	    if SumB == SumA ->
-		    ChallengeB;
+		    {ok,ChallengeB};
 	       true ->
-		    ?shutdown(bad_challenge_reply)
+		    {error,Data}
 	    end;
-	_ ->
-	    ?shutdown(no_node)
+	Err ->
+            {error,Err}
     end.
 
 send_challenge_ack(Socket, Digest) ->
@@ -539,20 +586,20 @@ recv_challenge_ack(Socket, ChallengeB, CookieA) ->
 		    ok;
 	       true ->
 		    ?shutdown(bad_challenge_ack)
-	    end;
-	_ ->
-	    ?shutdown(bad_challenge_ack)
+	    end
     end.
 
-send_name(Socket, MyNode0, Version) ->
-    send_name(Socket, MyNode0, Version, ?COMPULSORY_DFLAGS).
-send_name(Socket, MyNode0, Version, Flags) ->
+send_name(Socket, MyNode0) ->
+    Flags = ?COMPULSORY_DFLAGS bor ?DFLAG_MANDATORY_25_DIGEST,
+    send_name(Socket, MyNode0, Flags).
+
+send_name(Socket, MyNode0, Flags) ->
     MyNode = atom_to_list(MyNode0),
-    ok = ?to_port(Socket, [<<$n,Version:16,Flags:32>>|MyNode]).
+    Creation = erts_internal:get_creation(),
+    NameLen = length(MyNode),
+    ok = ?to_port(Socket, [<<$N, (Flags bor ?DFLAG_HANDSHAKE_23):64,
+                             Creation:32,NameLen:16>>|MyNode]).
 
-%%
-%% recv_name is common for both old and new handshake.
-%%
 recv_name(Socket) ->
     case gen_tcp:recv(Socket, 0) of
 	{ok,Data} ->
@@ -561,19 +608,13 @@ recv_name(Socket) ->
 	    ?shutdown({no_node,Res})
     end.
 
-get_name([$m,VersionA,VersionB,_Ip1,_Ip2,_Ip3,_Ip4|OtherNode]) ->
-    {normal, list_to_atom(OtherNode), ?u16(VersionA,VersionB)};
-get_name([$h,VersionA,VersionB,_Ip1,_Ip2,_Ip3,_Ip4|OtherNode]) ->
-    {hidden, list_to_atom(OtherNode), ?u16(VersionA,VersionB)};
-get_name([$n,VersionA, VersionB, Flag1, Flag2, Flag3, Flag4 | OtherNode]) ->
-    Type = case ?u32(Flag1, Flag2, Flag3, Flag4) band ?DFLAG_PUBLISHED of
-	       0 ->
-		   hidden;
-	       _ -> 
-		   normal
-	   end,
-    {Type, list_to_atom(OtherNode), 
-     ?u16(VersionA,VersionB)};
+get_name([$N, F7,F6,F5,F4,F3,F2,F1,F0,
+          _C3,_C2,_C1,_C0, NLen1,NLen2 | OtherNode]) ->
+    <<Flags:64>> = <<F7,F6,F5,F4,F3,F2,F1,F0>>,
+    true = (Flags band ?DFLAG_HANDSHAKE_23) =/= 0,
+    <<NameLen:16>> = <<NLen1,NLen2>>,
+    NameLen = length(OtherNode),
+    {list_to_atom(OtherNode), Flags};
 get_name(Data) ->
     ?shutdown(Data).
 
@@ -588,7 +629,7 @@ get_epmd_port() ->
             4369 % Default epmd port
     end.
 
-do_register_node(NodeName, TcpPort, VLow, VHigh) ->    
+do_register_node(NodeName, TcpPort) ->
     case gen_tcp:connect({127,0,0,1}, get_epmd_port(), []) of
 	{ok, Socket} ->
 	    {N0,_} = split(NodeName),
@@ -600,8 +641,8 @@ do_register_node(NodeName, TcpPort, VLow, VHigh) ->
 				  ?int16(TcpPort),
 				  $M,
 				  0,
-				  ?int16(VHigh),
-				  ?int16(VLow),
+				  ?int16(?DIST_VER_HIGH),
+				  ?int16(?DIST_VER_LOW),
 				  ?int16(length(Name)),
 				  Name,
 				  ?int16(Elen),
@@ -620,6 +661,13 @@ wait_for_reg_reply(Socket, SoFar) ->
     receive
 	{tcp, Socket, Data0} ->
 	    case SoFar ++ Data0 of
+		[$v, Result, A, B, C, D] ->
+		    case Result of
+			0 ->
+			    {alive, Socket, ?u32(A, B, C, D)};
+			_ ->
+			    {error, duplicate_name}
+		    end;
 		[$y, Result, A, B] ->
 		    case Result of
 			0 ->
@@ -640,9 +688,9 @@ wait_for_reg_reply(Socket, SoFar) ->
     end.
 
 
-register(NodeName, ListenSocket, VLow, VHigh) ->
+register_node(NodeName, ListenSocket) ->
     {ok,{_,TcpPort}} = inet:sockname(ListenSocket),
-    case do_register_node(NodeName, TcpPort, VLow, VHigh) of
+    case do_register_node(NodeName, TcpPort) of
 	{alive, Socket, _Creation} ->
 	    Socket;
 	Other ->
@@ -666,13 +714,13 @@ split(Atom) ->
 
 %% Build a distribution message that will make rex answer
 build_rex_message(Cookie,OurName) ->
-    [$?,term_to_binary({6,self(),Cookie,rex}),
+    [?PASS_THROUGH,term_to_binary({6,self(),Cookie,rex}),
      term_to_binary({'$gen_cast',
 		     {cast,
-		      rpc,
-		      cast,
-		      [OurName, hello, world, []],
-		      self()} })].
+		      erlang,
+		      send,
+		      [{regname, OurName}, "hello world"],
+                      self()}})].
 
 %% Receive a distribution message    
 recv_message(Socket) ->
@@ -698,24 +746,3 @@ recv_message(Socket) ->
 %% Build a nodename
 join(Name,Host) ->
     list_to_atom(atom_to_list(Name) ++ "@" ++ atom_to_list(Host)).
-
-%% start/stop slave.
-start_node(Name, Param) ->
-    test_server:start_node(Name, slave, [{args, Param}]).
-
-stop_node(Node) ->
-    test_server:stop_node(Node).
-
-
-get_nodenames(N, T) ->
-    get_nodenames(N, T, []).
-
-get_nodenames(0, _, Acc) ->
-    Acc;
-get_nodenames(N, T, Acc) ->
-    U = erlang:unique_integer([positive]),
-    get_nodenames(N-1, T, [list_to_atom(?MODULE_STRING
-					++ "-"
-					++ atom_to_list(T)
-					++ "-"
-					++ integer_to_list(U)) | Acc]).
