@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2009-2020. All Rights Reserved.
+%% Copyright Ericsson AB 2009-2022. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -27,13 +27,15 @@
 -include("kernel_test_lib.hrl").
 
 
--export([all/0, suite/0,groups/0,init_per_suite/1, end_per_suite/1, 
-	 init_per_group/2,end_per_group/2,
+-export([all/0, suite/0, groups/0, init_per_suite/1, end_per_suite/1,
+	 init_per_group/2, end_per_group/2,
 	 init_per_testcase/2, end_per_testcase/2
         ]).
--export([basic/1, resolve/1, edns0/1, txt_record/1, files_monitor/1,
-	 last_ms_answer/1, intermediate_error/1,
-         servfail_retry_timeout_default/1, servfail_retry_timeout_1000/1
+-export([basic/1, name_addr_and_cached/1, resolve/1,
+         edns0/1, txt_record/1, files_monitor/1,
+	 nxdomain_reply/1, last_ms_answer/1, intermediate_error/1,
+         servfail_retry_timeout_default/1, servfail_retry_timeout_1000/1,
+         label_compression_limit/1
         ]).
 -export([
 	 gethostbyaddr/0, gethostbyaddr/1,
@@ -46,9 +48,10 @@
 	 host_and_addr/0, host_and_addr/1
 	]).
 
--define(RUN_NAMED, "run-named").
+-define(RUN_NS, "run-ns").
+-define(LOG_FILE, "ns.log").
 
-%% This test suite use a script ?RUN_NAMED that tries to start
+%% This test suite use a script ?RUN_NS that tries to start
 %% a temporary local nameserver BIND 8 or 9 that must be installed
 %% on your machine.
 %%
@@ -69,9 +72,12 @@ suite() ->
      {timetrap,{minutes,1}}].
 
 all() -> 
-    [basic, resolve, edns0, txt_record, files_monitor,
-     last_ms_answer,
-     intermediate_error, servfail_retry_timeout_default, servfail_retry_timeout_1000,
+    [basic, resolve, name_addr_and_cached,
+     edns0, txt_record, files_monitor,
+     nxdomain_reply, last_ms_answer,
+     intermediate_error,
+     servfail_retry_timeout_default, servfail_retry_timeout_1000,
+     label_compression_limit,
      gethostbyaddr, gethostbyaddr_v6, gethostbyname,
      gethostbyname_v6, getaddr, getaddr_v6, ipv4_to_ipv6,
      host_and_addr].
@@ -119,12 +125,14 @@ end_per_group(_GroupName, Config) ->
 
 zone_dir(TC) ->
     case TC of
-	basic              -> otptest;
-	resolve            -> otptest;
-	edns0              -> otptest;
-	files_monitor      -> otptest;
-	last_ms_answer     -> otptest;
-        intermediate_error ->
+	basic                -> otptest;
+	name_addr_and_cached -> otptest;
+	resolve              -> otptest;
+	edns0                -> otptest;
+	files_monitor        -> otptest;
+	nxdomain_reply       -> otptest;
+	last_ms_answer       -> otptest;
+        intermediate_error   ->
             {internal,
              #{rcode => ?REFUSED}};
         servfail_retry_timeout_default ->
@@ -166,7 +174,7 @@ init_per_testcase(Func, Config) ->
 	    [{nameserver, NsSpec}, {res_lookup, Lookup} | Config]
     catch
 	SkipReason ->
-            ?P("init_per_testcase -> catched:"
+            ?P("init_per_testcase -> caught:"
                "~n      SkipReason: ~p", [SkipReason]),
 	    {skip, SkipReason}
     end.
@@ -215,7 +223,7 @@ ns_init(ZoneDir, PrivDir, DataDir) ->
 			      PNum
 		      end,
             ?P("ns_init -> use port number ~p", [PortNum]),
-	    RunNamed = filename:join(DataDir, ?RUN_NAMED),
+	    RunNamed = filename:join(DataDir, ?RUN_NS),
             ?P("ns_init -> use named ~p", [RunNamed]),
 	    NS = {{127,0,0,1},PortNum},
             ?P("ns_init -> try open port (exec)"),
@@ -247,7 +255,7 @@ ns_start(ZoneDir, PrivDir, NS, P) ->
 	"Error: "++Error ->
             ?P("ns_start -> error: "
                "~n      ~p", [Error]),
-	    ns_printlog(filename:join([PrivDir,ZoneDir,"named.log"])),
+	    ns_printlog(filename:join([PrivDir,ZoneDir,?LOG_FILE])),
 	    throw(Error);
 	_X ->
             ?P("ns_start -> retry"),
@@ -289,7 +297,7 @@ ns_end(undefined, _PrivDir) -> undefined;
 ns_end({ZoneDir,_NS,P}, PrivDir) when is_port(P) ->
     port_command(P, ["quit",io_lib:nl()]),
     ns_stop(P),
-    ns_printlog(filename:join([PrivDir,ZoneDir,"named.log"])),
+    ns_printlog(filename:join([PrivDir,ZoneDir,"ns.log"])),
     ok;
 ns_end({Tag,_NS,P}, _PrivDir) when is_pid(P) ->
     Mref = erlang:monitor(process, P),
@@ -506,66 +514,353 @@ basic(Config) when is_list(Config) ->
     NS = ns(Config),
     Name = "ns.otptest",
     NameC = caseflip(Name),
-    IP = {127,0,0,254},
+    NameD = NameC ++ ".",
+    IP1 = {127,0,0,253},
+    IP2 = {127,0,0,254},
     %%
     %% nslookup
     {ok,Msg1} = inet_res:nslookup(Name, in, a, [NS]),
     ?P("nslookup with ~p: ~n      ~p", [Name, Msg1]),
-    [RR1] = inet_dns:msg(Msg1, anlist),
-    IP = inet_dns:rr(RR1, data),
+    [RR1, RR2] = lists:sort(inet_dns:msg(Msg1, anlist)),
+    IP1 = inet_dns:rr(RR1, data),
+    IP2 = inet_dns:rr(RR2, data),
     Bin1 = inet_dns:encode(Msg1),
     %%io:format("Bin1 = ~w~n", [Bin1]),
     {ok,Msg1} = inet_dns:decode(Bin1),
     %% Now with scrambled case
     {ok,Msg1b} = inet_res:nslookup(NameC, in, a, [NS]),
     ?P("nslookup with ~p: ~n      ~p", [NameC, Msg1b]),
-    [RR1b] = inet_dns:msg(Msg1b, anlist),
-    IP = inet_dns:rr(RR1b, data),
+    [RR1b, RR2b] = lists:sort(inet_dns:msg(Msg1b, anlist)),
+    IP1 = inet_dns:rr(RR1b, data),
+    IP2 = inet_dns:rr(RR2b, data),
     Bin1b = inet_dns:encode(Msg1b),
     %%io:format("Bin1b = ~w~n", [Bin1b]),
     {ok,Msg1b} = inet_dns:decode(Bin1b),
     true =
 	(tolower(inet_dns:rr(RR1, domain))
 	 =:= tolower(inet_dns:rr(RR1b, domain))),
+    true =
+	(tolower(inet_dns:rr(RR2, domain))
+	 =:= tolower(inet_dns:rr(RR2b, domain))),
     %%
     %% resolve
     {ok,Msg2} = inet_res:resolve(Name, in, a, [{nameservers,[NS]},verbose]),
     ?P("resolve with ~p: ~n      ~p", [Name, Msg2]),
-    [RR2] = inet_dns:msg(Msg2, anlist),
-    IP = inet_dns:rr(RR2, data),
+    [RR1c, RR2c] = lists:sort(inet_dns:msg(Msg2, anlist)),
+    IP1 = inet_dns:rr(RR1c, data),
+    IP2 = inet_dns:rr(RR2c, data),
     Bin2 = inet_dns:encode(Msg2),
     %%io:format("Bin2 = ~w~n", [Bin2]),
     {ok,Msg2} = inet_dns:decode(Bin2),
     %% Now with scrambled case
     {ok,Msg2b} = inet_res:resolve(NameC, in, a, [{nameservers,[NS]},verbose]),
     ?P("resolve with ~p: ~n      ~p", [NameC, Msg2b]),
-    [RR2b] = inet_dns:msg(Msg2b, anlist),
-    IP = inet_dns:rr(RR2b, data),
+    [RR1d, RR2d] = lists:sort(inet_dns:msg(Msg2b, anlist)),
+    IP1 = inet_dns:rr(RR1d, data),
+    IP2 = inet_dns:rr(RR2d, data),
     Bin2b = inet_dns:encode(Msg2b),
     %%io:format("Bin2b = ~w~n", [Bin2b]),
     {ok,Msg2b} = inet_dns:decode(Bin2b),
     true =
-	(tolower(inet_dns:rr(RR2, domain))
-	  =:= tolower(inet_dns:rr(RR2b, domain))),
+	(tolower(inet_dns:rr(RR1c, domain))
+	  =:= tolower(inet_dns:rr(RR1d, domain))),
+    true =
+	(tolower(inet_dns:rr(RR2c, domain))
+	  =:= tolower(inet_dns:rr(RR2d, domain))),
+    ?P("resolve \"127.0.0.1\"~n", []),
+    {ok, Msg3} =
+        inet_res:resolve("127.0.0.1", in, a, [{nameservers,[NS]},verbose]),
+    [] = inet_dns:msg(Msg3, anlist),
+    {ok, Msg4} =
+        inet_res:resolve("127.0.0.1", in, ptr, [{nameservers,[NS]},verbose]),
+    [RR4] = inet_dns:msg(Msg4, anlist),
+    "1.0.0.127.in-addr.arpa" = inet_dns:rr(RR4, domain),
+    "test1-78901234567890123456789012345678.otptest" =
+        inet_dns:rr(RR4, data),
     %%
     %% lookup
     ?P("lookup"),
-    [IP] = inet_res:lookup(Name, in, a, [{nameservers,[NS]},verbose]),
-    [IP] = inet_res:lookup(NameC, in, a, [{nameservers,[NS]},verbose]),
+    [IP1, IP2] =
+        lists:sort(
+          inet_res:lookup(Name, in, a, [{nameservers,[NS]},verbose])),
+    [IP1, IP2] =
+        lists:sort(
+          inet_res:lookup(NameC, in, a, [{nameservers,[NS]},verbose])),
+    [IP1, IP2] =
+        lists:sort(
+          inet_res:lookup(NameD, in, a, [{nameservers,[NS]},verbose])),
     %%
     %% gethostbyname
     ?P("gethostbyname"),
-    {ok,#hostent{h_addr_list=[IP]}} = inet_res:gethostbyname(Name),
-    {ok,#hostent{h_addr_list=[IP]}} = inet_res:gethostbyname(NameC),
+    {ok,#hostent{h_addr_list=IPs1}} = inet_res:gethostbyname(Name),
+    [IP1, IP2] = lists:sort(IPs1),
+    {ok,#hostent{h_addr_list=IPs2}} = inet_res:gethostbyname(NameC),
+    [IP1, IP2] = lists:sort(IPs2),
     %%
     %% getbyname
     ?P("getbyname"),
-    {ok,#hostent{h_addr_list=[IP]}} = inet_res:getbyname(Name, a),
-    {ok,#hostent{h_addr_list=[IP]}} = inet_res:getbyname(NameC, a),
+    {ok,#hostent{h_addr_list=IPs3}} = inet_res:getbyname(Name, a),
+    [IP1, IP2] = lists:sort(IPs3),
+    {ok,#hostent{h_addr_list=IPs4}} = inet_res:getbyname(NameC, a),
+    [IP1, IP2] = lists:sort(IPs4),
     ?P("end"),
     ok.
 
 
+%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+%% Check that lookup of names and addresses works the same
+%% also when cached, by simply repeating the lookups
+
+name_addr_and_cached(Config) when is_list(Config) ->
+    ?P("begin"),
+
+%%%    dbg:tracer(),
+%%%    dbg:p(all, c),
+%%%    dbg:tpl(inet_res, do_query, cx),
+
+    NS = ns(Config),
+    Domain = "otptest",
+    Options =
+        [{lookup, [dns]},
+         {resolv_conf, []},
+         {hosts_file, []},
+         {domain, Domain},
+         {nameservers, [NS]},
+         {search, [Domain]},
+         {alt_nameservers, []},
+         {inet6, false},
+         {usevc, false},
+         {edns, 0}],
+    SavedOptions =
+        [{Option, inet_db:res_option(Option)}
+         || {Option, _Value} <- Options],
+    [inet_db:res_option(Option, Value)
+     || {Option, Value} <- Options],
+    try
+        ?P("first pass"),
+        %% Flip character case randomly
+        name_addr(Domain, fun caseflip/1),
+        %%
+        ?P("second pass"),
+        %% Use only character upper case,
+        %% should get identical results from the cache
+        name_addr(Domain, fun toupper/1)
+    after
+        [ok = inet_db:res_option(Option, Value)
+         || {Option, Value} <- SavedOptions]
+    end,
+    ?P("done"),
+    ok.
+
+name_addr(Domain, CFlip) ->
+%%    RDomain4 = "0.0.127.in-addr.arpa",
+%%    RDomain6 = "0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.ip6.arpa",
+    H4 =
+        #hostent{
+           h_addrtype = inet,
+           h_length = 4},
+    H6 =
+        #hostent{
+           h_addrtype = inet6,
+           h_length = 16},
+    H_mx              = CFlip("mx"),
+    H_mx_             = CFlip("mx."),
+    IP4_mx            = {127,0,0,10},
+    %%
+    H_resolve         = CFlip("resolve"),
+    H_resolve_        = CFlip("resolve."),
+    H_cname_resolve   = CFlip("cname.resolve"),
+    H_cname_resolve_  = CFlip("cname.resolve."),
+    %%
+    IP4_resolve       = {127,0,0,28},
+    IP6_resolve       = {0,0,0,0,0,0,127 bsl 8,28},
+    IP64_resolve      = {0,0,0,0,0,16#ffff,127 bsl 8,28},
+    %%
+    H_ns              = CFlip("ns"),
+    H_ns_             = CFlip("ns."),
+    IP4_ns_1          = {127,0,0,253},
+    IP4_ns_2          = {127,0,0,254},
+    IP6_ns_1          = {0,0,0,0,0,0,127 bsl 8,253},
+    IP6_ns_2          = {0,0,0,0,0,0,127 bsl 8,254},
+    IP64_ns_1         = {0,0,0,0,0,16#ffff,127 bsl 8,253},
+    IP64_ns_2         = {0,0,0,0,0,16#ffff,127 bsl 8,254},
+    Lookups =
+        %% The search list is [Domain] so a lookup of a short
+        %% name should return the fully qualified name with
+        %% Domain appended.
+        %%
+        %% Lookup results should have the same character case
+        %% as the query.  Both for short (search) lookups and
+        %% for full name lookups.
+        %%
+        %% Lookup via a CNAME record should return the
+        %% result as stored in DNS but the CNAME should be
+        %% returned as an alias with the query character case
+        %% preserved.
+        %%
+        %% Address lookups (reverse/PTR lookups) can only
+        %% return the one address we looked up and the
+        %% fully qualified name as in DNS, and no aliases.
+        %%
+        %% IPv6 address lookups in ::ffff.A.B.C.D
+        %% (IPv4-compatible IPv6 addresses) should be
+        %% internally done as IPv4 lookups by the client,
+        %% without the caller noticing.
+        %%
+        [{{H_mx, inet}, fun norm/1, ?LINE,
+          H4#hostent{
+            h_name = H_mx++[$.|Domain],
+            h_addr_list = [IP4_mx]}},
+         {{H_mx_++Domain, inet}, fun norm/1, ?LINE,
+          H4#hostent{
+            h_name = H_mx_++Domain,
+            h_addr_list = [IP4_mx]}},
+         {{H_resolve, inet}, fun norm/1, ?LINE,
+          H4#hostent{
+            h_name = H_resolve++[$.|Domain],
+            h_addr_list = [IP4_resolve]}},
+         {{H_resolve_++Domain, inet}, fun norm/1, ?LINE,
+          H4#hostent{
+            h_name = H_resolve_++Domain,
+            h_addr_list = [IP4_resolve]}},
+         {{H_cname_resolve, inet}, fun lower_h_name/1, ?LINE,
+          H4#hostent{
+            h_name = "resolve."++Domain,
+            h_aliases = [H_cname_resolve++[$.|Domain]],
+            h_addr_list = [IP4_resolve]}},
+         {{H_cname_resolve_++Domain, inet}, fun lower_h_name/1, ?LINE,
+          H4#hostent{
+            h_name = "resolve."++Domain,
+            h_aliases = [H_cname_resolve_++Domain],
+            h_addr_list = [IP4_resolve]}},
+         %%
+         {{H_ns, inet}, fun norm/1, ?LINE,
+          H4#hostent{
+            h_name = H_ns++[$.|Domain],
+            h_addr_list = [IP4_ns_1,IP4_ns_2]}},
+         {{H_ns_++Domain, inet}, fun norm/1, ?LINE,
+          H4#hostent{
+            h_name = H_ns_++Domain,
+            h_addr_list = [IP4_ns_1,IP4_ns_2]}},
+         %%
+         {IP4_ns_2, fun norm/1, ?LINE,
+          H4#hostent{
+            h_name = "ns."++Domain,
+            h_addr_list = [IP4_ns_2]}},
+         {IP4_ns_1, fun norm/1, ?LINE,
+          H4#hostent{
+            h_name = "ns."++Domain,
+            h_addr_list = [IP4_ns_1]}},
+         {IP4_mx, fun norm/1, ?LINE,
+          H4#hostent{
+            h_name = "mx."++Domain,
+            h_addr_list = [IP4_mx]}},
+         {IP4_mx, fun norm/1, ?LINE,
+          H4#hostent{
+            h_name = "mx."++Domain,
+            h_addr_list = [IP4_mx]}},
+         %%
+         %%
+         %%
+         {{H_resolve_++Domain, inet6}, fun norm/1, ?LINE,
+          H6#hostent{
+            h_name = H_resolve_++Domain,
+            h_addr_list = [IP6_resolve]}},
+         {{H_resolve, inet6}, fun norm/1, ?LINE,
+          H6#hostent{
+            h_name = H_resolve++[$.|Domain],
+            h_addr_list = [IP6_resolve]}},
+         {{H_cname_resolve, inet6}, fun lower_h_name/1, ?LINE,
+          H6#hostent{
+            h_name = "resolve."++Domain,
+            h_aliases = [H_cname_resolve++[$.|Domain]],
+            h_addr_list = [IP6_resolve]}},
+         {{H_cname_resolve_++Domain, inet6}, fun lower_h_name/1, ?LINE,
+          H6#hostent{
+            h_name = "resolve."++Domain,
+            h_aliases = [H_cname_resolve_++Domain],
+            h_addr_list = [IP6_resolve]}},
+         {IP6_resolve, fun norm/1, ?LINE,
+          H6#hostent{
+            h_name = "resolve."++Domain,
+            h_addr_list = [IP6_resolve]}},
+         {IP64_resolve, fun norm/1, ?LINE,
+          H6#hostent{
+            h_name = "resolve."++Domain,
+            h_addr_list = [IP64_resolve]}},
+         %%
+         {{H_ns, inet6}, fun norm/1, ?LINE,
+          H6#hostent{
+            h_name = H_ns++[$.|Domain],
+            h_addr_list =
+                [IP6_ns_1,
+                 IP6_ns_2]}},
+         {{H_ns_++Domain, inet6}, fun norm/1, ?LINE,
+          H6#hostent{
+            h_name = H_ns_++Domain,
+            h_addr_list =
+                [IP6_ns_1,
+                 IP6_ns_2]}},
+         {IP6_ns_1, fun norm/1, ?LINE,
+          H6#hostent{
+            h_name = "ns."++Domain,
+            h_addr_list = [IP6_ns_1]}},
+         {IP6_ns_2, fun norm/1, ?LINE,
+          H6#hostent{
+            h_name = "ns."++Domain,
+            h_addr_list = [IP6_ns_2]}},
+         {IP64_ns_1, fun norm/1, ?LINE,
+          H6#hostent{
+            h_name = "ns."++Domain,
+            h_addr_list = [IP64_ns_1]}},
+         {IP64_ns_2, fun norm/1, ?LINE,
+          H6#hostent{
+            h_name = "ns."++Domain,
+            h_addr_list = [IP64_ns_2]}}
+         %%
+         %%
+         %%
+        ],
+    Results =
+        [case Target of
+             {H, T} ->
+                 inet_res:gethostbyname(H, T);
+             Addr ->
+                 inet_res:gethostbyaddr(Addr)
+         end || {Target, _TFun, _Line, _HE} <- Lookups],
+    ?P("inet_cache: ~p~n", [ets:tab2list(inet_cache)]),
+    [] = merge_results(Lookups, Results),
+    ok.
+
+lower_h_name(#hostent{h_name = HName} = HE) ->
+    norm(HE#hostent{h_name = tolower(HName)}).
+
+%%%lower_h_aliases(#hostent{h_aliases = HAliases} = HE) ->
+%%%    HE#hostent{
+%%%      h_aliases = [tolower(HAlias) || HAlias <- HAliases]}.
+
+norm(#hostent{h_addr_list = Addrs} = HE) ->
+    HE#hostent{h_addr_list = lists:sort(Addrs)}.
+
+merge_results([], []) ->
+    [];
+merge_results([{H_T, TFun, Line, HE} | Lookups], [Result | Results]) ->
+    case
+        case Result of
+            {ok, ResultHE} ->
+                {ok, TFun(ResultHE)};
+            _ ->
+                Result
+        end
+    of
+        {ok, HE} ->
+            merge_results(Lookups, Results);
+        Other ->
+            [{{Line, H_T, HE}, Other} | merge_results(Lookups, Results)]
+    end.
+
+
 %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 %% Lookup different records using resolve/2..4.
@@ -582,7 +877,9 @@ resolve(Config) when is_list(Config) ->
 	 {cname,"cname."++Name,[{cname,Name}],undefined},
 	 {a,"cname."++Name,[{cname,Name},{a,{127,0,0,28}}],undefined},
 	 {ns,"ns."++Name,[],[{ns,Name}]},
-	 {soa,Domain,[],[{soa,{"ns.otptest","lsa.otptest",1,60,10,300,30}}]},
+	 {soa,Domain,
+          undefined,
+          [{soa,{"ns.otptest","lsa\\.soa.otptest",1,60,10,300,30}}]},
 	 %% WKS: protocol TCP (6), services (bits) TELNET (23) and SMTP (25)
 	 {wks,"wks."++Name,[{wks,{{127,0,0,28},6,<<0,0,1,64>>}}],undefined},
 	 {ptr,"28."++RDomain4,[{ptr,Name}],undefined},
@@ -597,16 +894,20 @@ resolve(Config) when is_list(Config) ->
 	  [{txt,["Hej ","du ","glade "]},{txt,["ta ","en ","spade!"]}],
 	  undefined},
 	 {mb,"mb."++Name,[{mb,"mx."++Name}],undefined},
-	 {mg,"mg."++Name,[{mg,"Lsa."++Domain}],undefined},
-	 {mr,"mr."++Name,[{mr,"LSA."++Domain}],undefined},
+	 {mg,"mg."++Name,[{mg,"lsa\\.mg."++Domain}],undefined},
+	 {mr,"mr."++Name,[{mr,"lsa\\.mr."++Domain}],undefined},
 	 {minfo,"minfo."++Name,
-	  [{minfo,{"minfo-OWNER."++Name,"MinfoBounce."++Name}}],
+	  [{minfo,{"minfo-owner."++Name,"minfo-bounce."++Name}}],
 	  undefined},
+         {uri,"uri."++Name,[{uri,{10,3,"http://erlang.org"}}],undefined},
+         {caa,"caa."++Name,
+          [{caa,{1,"iodef","http://iodef.erlang.org"}}],
+          undefined},
 	 {any,"cname."++Name,[{cname,Name}],undefined},
 	 {any,Name,
-	  [{a,{127,0,0,28}},
-	   {aaaa,{0,0,0,0,0,0,32512,28}},
-	   {hinfo,{"BEAM","Erlang/OTP"}}],
+	  #{ {a,{127,0,0,28}} => [],
+             {aaaa,{0,0,0,0,0,0,32512,28}} => [],
+             {hinfo,{"BEAM","Erlang/OTP"}} => [] },
 	  undefined}
 	],
     ?P("resolve -> with edns 0"),
@@ -635,82 +936,107 @@ resolve(Class, Opts, [{Type,Nm,Answers,Authority}=Q|Qs]) ->
 	    _ ->
 		{caseflip(Nm),Nm}
 	end,
-    AnList =
-	if
-	    Answers =/= undefined ->
-		normalize_answers(Answers);
-	    true ->
-		undefined
-	end,
-    NsList =
-	if
-	    Authority =/= undefined ->
-		normalize_answers(Authority);
-	    true ->
-		undefined
-	end,
+    NormAnswers = normalize_rrs(Answers),
+    NormNSs = normalize_rrs(Authority),
     ?P("resolve -> resolve with ~p", [Name]),
     {ok,Msg} = inet_res:resolve(Name, Class, Type, Opts),
-    check_msg(Class, Type, Msg, AnList, NsList),
+    check_msg(Class, Type, Msg, NormAnswers, NormNSs),
     ?P("resolve -> resolve with ~p", [NameC]),
     {ok,MsgC} = inet_res:resolve(NameC, Class, Type, Opts),
-    check_msg(Class, Type, MsgC, AnList, NsList),
+    check_msg(Class, Type, MsgC, NormAnswers, NormNSs),
     ?P("resolve -> next"),
     resolve(Class, Opts, Qs).
 
 
 
-normalize_answers(AnList) ->
-    lists:sort([normalize_answer(Answer) || Answer <- AnList]).
+normalize_rrs(undefined = RRs) -> RRs;
+normalize_rrs(RRList) when is_list(RRList) ->
+    lists:sort([normalize_rr(RR) || RR <- RRList]);
+normalize_rrs(RRs) when is_map(RRs) ->
+    maps:fold(
+      fun (RR, V, NormRRs) ->
+              NormRRs#{(normalize_rr(RR)) => V}
+      end, #{}, RRs).
 
-normalize_answer({soa,{NS,HM,Ser,Ref,Ret,Exp,Min}}) ->
+normalize_rr({soa,{NS,HM,Ser,Ref,Ret,Exp,Min}}) ->
     {tolower(NS),tolower_email(HM),Ser,Ref,Ret,Exp,Min};
-normalize_answer({mx,{Prio,DN}}) ->
+normalize_rr({mx,{Prio,DN}}) ->
     {Prio,tolower(DN)};
-normalize_answer({srv,{Prio,Weight,Port,DN}}) ->
+normalize_rr({srv,{Prio,Weight,Port,DN}}) ->
     {Prio,Weight,Port,tolower(DN)};
-normalize_answer({naptr,{Order,Pref,Flags,Service,RE,Repl}}) ->
+normalize_rr({naptr,{Order,Pref,Flags,Service,RE,Repl}}) ->
     {Order,Pref,Flags,Service,RE,tolower(Repl)};
-normalize_answer({minfo,{RespM,ErrM}}) ->
+normalize_rr({minfo,{RespM,ErrM}}) ->
     {tolower_email(RespM),tolower_email(ErrM)};
-normalize_answer({T,MN}) when T =:= mg; T =:= mr ->
+normalize_rr({T,MN}) when T =:= mg; T =:= mr ->
     tolower_email(MN);
-normalize_answer({T,DN}) when T =:= cname; T =:= ns; T =:= ptr; T =:= mb ->
+normalize_rr({T,DN}) when T =:= cname; T =:= ns; T =:= ptr; T =:= mb ->
     tolower(DN);
-normalize_answer(Answer) ->
-    Answer.
+normalize_rr(RR) ->
+    RR.
 
-check_msg(Class, Type, Msg, AnList, NsList) ->
+check_msg(Class, Type, Msg, ExpectedAnswers, ExpectedNSs) ->
     ?P("check_msg ->"
        "~n      Type: ~p"
        "~n      Msg:  ~p", [Type,Msg]),
-    case {normalize_answers(
-	    [begin
-		 Class = inet_dns:rr(RR, class),
-		 {inet_dns:rr(RR, type),inet_dns:rr(RR, data)}
-	     end || RR <- inet_dns:msg(Msg, anlist)]),
-	  normalize_answers(
-	    [begin
-		 Class = inet_dns:rr(RR, class),
-		 {inet_dns:rr(RR, type),inet_dns:rr(RR, data)}
-	     end || RR <- inet_dns:msg(Msg, nslist)])} of
-	{AnList,NsList} ->
-	    ok;
-	{NsList,AnList} when Type =:= ns ->
-	    %% This whole case statement is kind of inside out just
-	    %% to accept this case when some legacy DNS resolvers
-	    %% return the answer to a NS query in the answer section
-	    %% instead of in the authority section.
-	    ok;
-	{AnList,_} when NsList =:= undefined ->
-	    ok;
-	{_,NsList} when AnList =:= undefined ->
-	    ok
+    NormAnList =
+        normalize_rrs(
+          [begin
+               Class = inet_dns:rr(RR, class),
+               {inet_dns:rr(RR, type),inet_dns:rr(RR, data)}
+           end || RR <- inet_dns:msg(Msg, anlist)]),
+    NormNsList =
+           normalize_rrs(
+             [begin
+                  Class = inet_dns:rr(RR, class),
+                  {inet_dns:rr(RR, type),inet_dns:rr(RR, data)}
+              end || RR <- inet_dns:msg(Msg, nslist)]),
+    case
+        check_msg(ExpectedAnswers, NormAnList) andalso
+        check_msg(ExpectedNSs, NormNsList)
+    of
+        true ->
+            ok;
+        false
+          when Type =:= ns;
+               Type =:= soa ->
+            %% Some resolvers return the answer to a NS query
+            %% in the answer section instead of in the authority section,
+            %% and some do the same for a SOA query
+            case
+                check_msg(ExpectedAnswers, NormNsList) andalso
+                check_msg(ExpectedNSs, NormAnList)
+            of
+                true ->
+                    ok;
+                false ->
+                    error({Type,
+                           {expected,ExpectedAnswers,ExpectedNSs},
+                           {got,NormAnList,NormNsList}})
+            end;
+        false ->
+            error({Type,
+                   {expected,ExpectedAnswers,ExpectedNSs},
+                   {got,NormAnList,NormNsList}})
     end,
+    %% Test the encoder against the decoder; the least we can do
     Buf = inet_dns:encode(Msg),
     {ok,Msg} = inet_dns:decode(Buf),
     ok.
 
+check_msg(undefined, RRs) when is_list(RRs)-> true;
+check_msg(RRs1, RRs2) when is_list(RRs1), is_list(RRs2) ->
+    RRs1 =:= RRs2;
+check_msg(Expected, [RR|RRs]) when is_map(Expected) ->
+    case Expected of
+        #{RR := _} ->
+            case RRs of
+                []    -> true;
+                [_|_] -> check_msg(Expected, RRs)
+            end;
+        #{} -> false
+    end;
+check_msg(#{}, []) -> false. % At least one has to be ok
 
 %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
@@ -898,6 +1224,26 @@ do_files_monitor(Config) ->
 	inet:gethostbyname("resolve.otptest"),
     ok.
 
+%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%% Get full DNS answer on nxdomain (when option set)
+%% Check that we get the error code from the first server.
+
+nxdomain_reply(Config) when is_list(Config) ->
+    NS    = ns(Config),
+    Name  = "nxdomain.otptest",
+    Class = in,
+    Type  = a,
+    Opts  =
+        [{nameservers,[NS]}, {servfail_retry_timeout, 1000}, verbose],
+    ?P("try resolve"),
+    {error, nxdomain} = inet_res:resolve(Name, Class, Type, Opts),
+    {error, {nxdomain, Rec}} =
+        inet_res:resolve(Name, Class, Type, [nxdomain_reply|Opts]),
+    ?P("resolved: "
+       "~n      ~p", [Rec]),
+    ok.
+
+
 %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 %% Answer just when timeout is triggered (OTP-9221).
@@ -909,7 +1255,7 @@ last_ms_answer(Config) when is_list(Config) ->
     PSpec = proxy_start(last_ms_answer, NS),
     ProxyNS = proxy_ns(PSpec),
     %%
-    %% resolve; whith short timeout to trigger Timeout =:= 0 in inet_res
+    %% resolve; with short timeout to trigger Timeout =:= 0 in inet_res
     {error,timeout} =
 	inet_res:resolve(
 	  Name, in, a, [{nameservers,[ProxyNS]},verbose], Time + 10),
@@ -973,6 +1319,79 @@ servfail_retry_timeout_1000(Config) when is_list(Config) ->
     ok.
 
 
+%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%% Test that label encoding compression limits at 14 bits pointer size
+
+label_compression_limit(Config) when is_list(Config) ->
+    FirstSz = 4,
+    Count = 512,
+    Sz = 16,
+    %% We create a long DNS message with an answer list containing
+    %% 1+512+1 RR:s.  The first label is 4 chars that with message
+    %% and RR overhead places the second label on offset 32.
+    %% All other labels are 16 chars that with RR overhead
+    %% places them on offsets of N * 32.
+    %%
+    %% The labels are: "ZZZZ", then; "AAAAAAAAAAAAAAAA",
+    %% "AAAAAAAAAAAAAAAB", incrementing, so no one is
+    %% equal and can not be compressed, until the last one
+    %% that refers to the second to last one, so it could be compressed.
+    %%
+    %% However, the second to last label lands on offset 512 * 32 = 16384
+    %% which is out of reach for compression since compression uses
+    %% a 14 bit reference from the start of the message.
+    %%
+    %% The last label can only be compressed when we instead
+    %% generate a message with *one less* char in the first label,
+    %% placing the second to last label on offset 16383.
+    %%
+    %% So, MsgShort can use compression for the last RR
+    %% by referring to the second to last RR, but MsgLong can not.
+    %%
+    %% Disclaimer:
+    %%    All offsets and overheads are deduced
+    %%    through trial and observation
+    %%
+    [D | Domains] = gen_domains(Count, lists:duplicate(Sz, $A), []),
+    LastD = "Y." ++ D,
+    DomainsCommon = lists:reverse(Domains, [D, LastD]),
+    DomainsShort =  [lists:duplicate(FirstSz-1, $Z) | DomainsCommon],
+    DomainsLong =   [lists:duplicate(FirstSz, $Z) | DomainsCommon],
+    MsgShort = gen_msg(DomainsShort),
+    MsgLong = gen_msg(DomainsLong),
+    DataShort = inet_dns:encode(MsgShort),
+    DataShortSz = byte_size(DataShort),
+    ?P("DataShort[~w]:~n    ~p~n", [DataShortSz, DataShort]),
+    DataLong = inet_dns:encode(MsgLong),
+    DataLongSz = byte_size(DataLong),
+    ?P("DataLong[~w]:~n    ~p~n", [DataLongSz, DataLong]),
+    %% When the first (long) RR size pushes the last compressed label out,
+    %% that occupied a 2 bytes reference, it instead becomes a label
+    %% with 1 byte size and a final empty label size 1
+    0 = DataLongSz - (DataShortSz+1 - 2 + 1+Sz+1),
+    ok.
+
+gen_msg(Domains) ->
+    inet_dns:make_msg(
+      [{header, inet_dns:make_header()},
+       {anlist, gen_rrs(Domains)}]).
+
+gen_rrs(Domains) ->
+    [inet_dns:make_rr([{class,in},{type,a},{domain,D},{data,{17,18,19,20}}]) ||
+        D <- Domains].
+
+gen_domains(0, _Domain, Acc) ->
+    Acc;
+gen_domains(N, Domain, Acc) ->
+    gen_domains(
+      N - 1, incr_domain(Domain), [lists:reverse(Domain) | Acc]).
+
+incr_domain([$Z | Domain]) ->
+    [$A | incr_domain(Domain)];
+incr_domain([Char | Domain]) ->
+    [Char+1 | Domain].
+
+
 %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% Compatibility tests. Call the inet_SUITE tests, but with
 %% lookup = [file,dns] instead of [native]
@@ -999,42 +1418,53 @@ timestamp() ->
     erlang:monotonic_time(milli_seconds).
 
 
-%% Case flip helper
+%% Case flip helper, randomly flips the case of about every second [a-zA-Z]
 
-caseflip([C|Cs]) when is_integer(C), $a =< C, C =< $z ->
-    [(C - $a + $A)|caseflip_skip(Cs)];
-caseflip([C|Cs]) when is_integer(C), $A =< C, C =< $Z ->
-    [(C - $A + $a)|caseflip_skip(Cs)];
-caseflip([C|Cs]) ->
-    [C|caseflip(Cs)];
+-compile({inline, [caseflip/3]}).
+
+caseflip([C | Cs]) when is_integer(C), $a =< C, C =< $z ->
+    caseflip(Cs, C, $a - $A);
+caseflip([C | Cs]) when is_integer(C), $A =< C, C =< $Z ->
+    caseflip(Cs, C, $A - $a);
+caseflip([C | Cs]) ->
+    [C | caseflip(Cs)];
 caseflip([]) ->
     [].
-
-caseflip_skip([C|Cs]) when is_integer(C), $a =< C, C =< $z ->
-    [C|caseflip(Cs)];
-caseflip_skip([C|Cs]) when is_integer(C), $A =< C, C =< $Z ->
-    [C|caseflip(Cs)];
-caseflip_skip([C|Cs]) ->
-    [C|caseflip_skip(Cs)];
-caseflip_skip([]) ->
-    [].
-
-tolower_email([$.|Cs]) ->
-    [$.|tolower(Cs)];
-tolower_email([C|Cs]) ->
-    [C|tolower_email(Cs)].
-
-%% Case fold to lower case according to RFC 4343
 %%
-tolower([C|Cs]) when is_integer(C) ->
+caseflip(Cs, C, Diff) ->
+    [case 0.5 =< rand:uniform() of
+         true ->
+             C - Diff;
+         false ->
+             C
+     end | caseflip(Cs)].
+
+
+tolower_email([$. | Cs]) ->
+    [$. | tolower(Cs)];
+tolower_email([C | Cs]) ->
+    [C | tolower_email(Cs)].
+
+%% Case fold according to RFC 4343
+%%
+tolower([C | Cs]) when is_integer(C) ->
     if  $A =< C, C =< $Z ->
-	    [(C - $A + $a)|tolower(Cs)];
+	    [(C - $A + $a) | tolower(Cs)];
 	true ->
-	    [C|tolower(Cs)]
+	    [C | tolower(Cs)]
     end;
 tolower([]) ->
     [].
 
+toupper([C | Cs]) when is_integer(C) ->
+    if  $a =< C, C =< $z ->
+	    [(C - $a + $A) | toupper(Cs)];
+	true ->
+	    [C | toupper(Cs)]
+    end;
+toupper([]) ->
+    [].
+
 -compile({inline,[ok/1]}).
 ok(ok) -> ok;
 ok({ok,X}) -> X;
