@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2018. All Rights Reserved.
+%% Copyright Ericsson AB 2018-2020. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -247,68 +247,78 @@ template(_Config) ->
     ct:log(String5),
     "" = String5,
 
-    Ref6 = erlang:make_ref(),
-    Meta6 = #{atom=>some_atom,
+    Template6 = [<<"binary">>],
+    String6 = format(info,{"~p",[term]},#{time=>Time},#{template=>Template6}),
+    ct:log(String6),
+    "binary" = String6,
+
+    Template7 = [{metakey,[[$e,<<"x">>,$i,[<<"s">>],":"],metakey],[[[[[[[[["does ",<<"not">>," exist"]]]]]]]]]},[[[<<" more ">>]],"deep"]],
+    String7 = format(info,{"~p",[term]},#{time=>Time},#{template=>Template7}),
+    ct:log(String7),
+    "does not exist more deep" = String7,
+
+    Ref8 = erlang:make_ref(),
+    Meta8 = #{atom=>some_atom,
               integer=>632,
               list=>[list,"string",4321,#{},{tuple}],
               mfa=>{mod,func,0},
               pid=>self(),
-              ref=>Ref6,
+              ref=>Ref8,
               string=>"some string",
               time=>Time,
               tuple=>{1,atom,"list"},
               nested=>#{subkey=>subvalue}},
-    Template6 = lists:join(";",lists:sort(maps:keys(maps:remove(nested,Meta6))) ++
+    Template8 = lists:join(";",lists:sort(maps:keys(maps:remove(nested,Meta8))) ++
                                [[nested,subkey]]),
-    String6 = format(info,{"~p",[term]},Meta6,#{template=>Template6,
+    String8 = format(info,{"~p",[term]},Meta8,#{template=>Template8,
                                                 single_line=>true}),
     ct:log(String6),
     SelfStr = pid_to_list(self()),
-    RefStr6 = ref_to_list(Ref6),
+    RefStr8 = ref_to_list(Ref8),
     ListStr = "[list,\"string\",4321,#{},{tuple}]",
-    ExpectedTime6 = default_time_format(Time),
+    ExpectedTime8 = default_time_format(Time),
     ["some_atom",
      "632",
      ListStr,
      "mod:func/0",
      SelfStr,
-     RefStr6,
+     RefStr8,
      "some string",
-     ExpectedTime6,
+     ExpectedTime8,
      "{1,atom,\"list\"}",
-     "subvalue"] = string:lexemes(String6,";"),
+     "subvalue"] = string:lexemes(String8,";"),
 
-    Meta7 = #{time=>Time,
+    Meta9 = #{time=>Time,
               nested=>#{key1=>#{subkey1=>value1},
                         key2=>value2}},
-    Template7 = lists:join(";",[nested,
+    Template9 = lists:join(";",[nested,
                                 [nested,key1],
                                 [nested,key1,subkey1],
                                 [nested,key2],
                                 [nested,key2,subkey2],
                                 [nested,key3],
                                 [nested,key3,subkey3]]),
-    String7 = format(info,{"~p",[term]},Meta7,#{template=>Template7,
+    String9 = format(info,{"~p",[term]},Meta9,#{template=>Template9,
                                                 single_line=>true}),
-    ct:log(String7),
-    [MultipleKeysStr7,
+    ct:log(String9),
+    [MultipleKeysStr9,
      "#{subkey1 => value1}",
      "value1",
      "value2",
      "",
      "",
-     ""] = string:split(String7,";",all),
+     ""] = string:split(String9,";",all),
     %% Order of keys is not fixed
-    case MultipleKeysStr7 of
+    case MultipleKeysStr9 of
         "#{key2 => value2,key1 => #{subkey1 => value1}}" -> ok;
         "#{key1 => #{subkey1 => value1},key2 => value2}" -> ok;
-        _ -> ct:fail({full_nested_map_unexpected,MultipleKeysStr7})
+        _ -> ct:fail({full_nested_map_unexpected,MultipleKeysStr9})
     end,
 
-    Meta8 = #{time=>Time,
+    Meta10 = #{time=>Time,
               nested=>#{key1=>#{subkey1=>value1},
                         key2=>value2}},
-    Template8 =
+    Template10 =
         lists:join(
           ";",
           [{nested,["exist:",nested],["noexist"]},
@@ -318,21 +328,21 @@ template(_Config) ->
            {[nested,key2,subkey2],["exist:",[nested,key2,subkey2]],["noexist"]},
            {[nested,key3],["exist:",[nested,key3]],["noexist"]},
            {[nested,key3,subkey3],["exist:",[nested,key3,subkey3]],["noexist"]}]),
-    String8 = format(info,{"~p",[term]},Meta8,#{template=>Template8,
-                                                single_line=>true}),
-    ct:log(String8),
-    [MultipleKeysStr8,
+    String10 = format(info,{"~p",[term]},Meta10,#{template=>Template10,
+                                                 single_line=>true}),
+    ct:log(String10),
+    [MultipleKeysStr10,
      "exist:#{subkey1 => value1}",
      "exist:value1",
      "exist:value2",
      "noexist",
      "noexist",
-     "noexist"] = string:split(String8,";",all),
+     "noexist"] = string:split(String10,";",all),
     %% Order of keys is not fixed
-    case MultipleKeysStr8 of
+    case MultipleKeysStr10 of
         "exist:#{key2 => value2,key1 => #{subkey1 => value1}}" -> ok;
         "exist:#{key1 => #{subkey1 => value1},key2 => value2}" -> ok;
-        _ -> ct:fail({full_nested_map_unexpected,MultipleKeysStr8})
+        _ -> ct:fail({full_nested_map_unexpected,MultipleKeysStr10})
     end,
 
     ok.
@@ -347,7 +357,7 @@ format_msg(_Config) ->
     String2 = format(info,{"list",[term]},#{},#{template=>Template}),
     ct:log(String2),
     "FORMAT ERROR: \"list\" - [term]" = String2,
-    
+
     String3 = format(info,{report,term},#{},#{template=>Template}),
     ct:log(String3),
     "term" = String3,
@@ -568,6 +578,11 @@ format_mfa(_Config) ->
     ct:log(String4),
     "othermfa" = String4,
 
+    Meta5 = #{mfa=>{'m o d','a\x{281}b',['  ']}},
+    String5 = format(info,{"~p",[term]},Meta5,#{template=>Template}),
+    ct:log(String5),
+    "'m o d':'a\x{281}b'/1" = String5,
+
     ok.
     
 format_time(_Config) ->
@@ -837,7 +852,7 @@ format(Level,Msg,Meta,Config) ->
     format(#{level=>Level,msg=>Msg,meta=>add_time(Meta)},Config).
 
 format(Log,Config) ->
-    lists:flatten(logger_formatter:format(Log,Config)).
+    unicode:characters_to_list(logger_formatter:format(Log,Config)).
 
 default_time_format(Timestamp) ->
     default_time_format(Timestamp,false).
