@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1997-2016. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2021. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -38,6 +38,7 @@
 	
 	 dump_log_update_in_place/1,
 	 event_module/1,
+         backend_plugin_registration/1,
 	 inconsistent_database/1,
 	 max_wait_for_decision/1,
 	 send_compressed/1,
@@ -104,7 +105,7 @@ all() ->
     [access_module, auto_repair, backup_module, debug, dir,
      dump_log_load_regulation, {group, dump_log_thresholds},
      dump_log_update_in_place,
-     event_module,
+     event_module, backend_plugin_registration,
      inconsistent_database, max_wait_for_decision,
      send_compressed, app_test, {group, schema_config},
      unknown_config].
@@ -720,6 +721,21 @@ event_module(Config) when is_list(Config) ->
     ?cleanup(1, Config),
     ok.
 
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+backend_plugin_registration(doc) ->
+    ["Ensure that backend plugins can be registered at runtime,",
+     "that the init_backend is called exactly once when registering",
+     "a plugin, and not again when registering a new alias"];
+backend_plugin_registration(Config) when is_list(Config) ->
+    Nodes = ?acquire_schema(1, [{default_properties, []} | Config]),
+    ?match(ok, mnesia:start()),
+    ?match({atomic,ok}, mnesia:add_backend_type(ext_ets, ext_test)),
+    ?match({atomic,ok}, mnesia:add_backend_type(ext_dets, ext_test)),
+    ?verify_mnesia(Nodes, []),
+    ?cleanup(1, Config).
+
+
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 start_one_disc_full_then_one_disc_less(doc)->
@@ -921,7 +937,7 @@ start_first_one_disc_less_then_two_more_disc_less(Config) when is_list(Config) -
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 schema_location_and_extra_db_nodes_combinations(doc)->
-    ["Test schema loaction and extra_db_nodes combinations."];
+    ["Test schema location and extra_db_nodes combinations."];
 schema_location_and_extra_db_nodes_combinations(suite) -> [];
 schema_location_and_extra_db_nodes_combinations(Config) when is_list(Config) ->
     [N1, N2] = Nodes = ?init(2, Config),
