@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1997-2016. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2022. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -21,7 +21,78 @@
 %%
 -module(mnesia_consistency_test).
 -author('hakan@erix.ericsson.se').
--compile([export_all]).
+
+-export([init_per_testcase/2, end_per_testcase/2,
+         init_per_group/2, end_per_group/2,
+         all/0, groups/0]).
+
+-export([consistency_after_change_table_copy_type/1,
+         consistency_after_rename_of_node/1,
+         consistency_after_restart_1_ram/1,
+         consistency_after_restart_1_disc/1,
+         consistency_after_restart_1_disc_only/1,
+         consistency_after_restart_2_ram/1,
+         consistency_after_restart_2_disc/1,
+         consistency_after_restart_2_disc_only/1,
+         consistency_after_dump_tables_1_ram/1,
+         consistency_after_dump_tables_2_ram/1,
+         consistency_after_add_replica_2_ram/1,
+         consistency_after_add_replica_2_disc/1,
+         consistency_after_add_replica_2_disc_only/1,
+         consistency_after_add_replica_3_ram/1,
+         consistency_after_add_replica_3_disc/1,
+         consistency_after_add_replica_3_disc_only/1,
+         consistency_after_del_replica_2_ram/1,
+         consistency_after_del_replica_2_disc/1,
+         consistency_after_del_replica_2_disc_only/1,
+         consistency_after_del_replica_3_ram/1,
+         consistency_after_del_replica_3_disc/1,
+         consistency_after_del_replica_3_disc_only/1,
+         consistency_after_move_replica_2_ram/1,
+         consistency_after_move_replica_2_disc/1,
+         consistency_after_move_replica_2_disc_only/1,
+         consistency_after_move_replica_3_ram/1,
+         consistency_after_move_replica_3_disc/1,
+         consistency_after_move_replica_3_disc_only/1,
+         consistency_after_transform_table_ram/1,
+         consistency_after_transform_table_disc/1,
+         consistency_after_transform_table_disc_only/1,
+         consistency_after_fallback_2_ram/1,
+         consistency_after_fallback_2_disc/1,
+         consistency_after_fallback_2_disc_only/1,
+         consistency_after_fallback_3_ram/1,
+         consistency_after_fallback_3_disc/1,
+         consistency_after_fallback_3_disc_only/1,
+         consistency_after_restore_clear_ram/1,
+         consistency_after_restore_clear_disc/1,
+         consistency_after_restore_clear_disc_only/1,
+         consistency_after_restore_recreate_ram/1,
+         consistency_after_restore_recreate_disc/1,
+         consistency_after_restore_recreate_disc_only/1,
+         updates_during_checkpoint_activation_1_ram/1,
+         updates_during_checkpoint_activation_1_disc/1,
+         updates_during_checkpoint_activation_1_disc_only/1,
+         updates_during_checkpoint_activation_2_ram/1,
+         updates_during_checkpoint_activation_2_disc/1,
+         updates_during_checkpoint_activation_2_disc_only/1,
+         updates_during_checkpoint_activation_3_ram/1,
+         updates_during_checkpoint_activation_3_disc/1,
+         updates_during_checkpoint_activation_3_disc_only/1,
+         updates_during_checkpoint_iteration_2_ram/1,
+         updates_during_checkpoint_iteration_2_disc/1,
+         updates_during_checkpoint_iteration_2_disc_only/1,
+         load_table_with_activated_checkpoint_ram/1,
+         load_table_with_activated_checkpoint_disc/1,
+         load_table_with_activated_checkpoint_disc_only/1,
+         add_table_copy_to_table_checkpoint_ram/1,
+         add_table_copy_to_table_checkpoint_disc/1,
+         add_table_copy_to_table_checkpoint_disc_only/1,
+         inst_fallback_process_dies/1, fatal_when_inconsistency/1,
+         after_delete/1,cause_switch_before/1, cause_switch_after/1,
+         cause_abort_before/1, cause_abort_after/1,
+         change_schema_before/1, change_schema_after/1]).
+
+-export([change_tab/3]).
 
 -include("mnesia_test_lib.hrl").
 
@@ -270,7 +341,7 @@ receive_messages(ListOfMsgs) ->
                     ?verbose("I (~p) got msg ~p from ~p ~n", [self(),Msg, Pid]),
                     [{Pid, Msg} | receive_messages(ListOfMsgs -- [Msg])]
             end;
-        Else -> ?warning("Recevied unexpected Msg~n ~p ~n", [Else])
+        Else -> ?warning("Received unexpected Msg~n ~p ~n", [Else])
     after timer:minutes(3) -> 
             ?error("Timeout in receive msgs while waiting for ~p~n", 
                    [ListOfMsgs])
