@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1997-2020. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2022. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -632,8 +632,8 @@ sticky_sync(Config) when is_list(Config) ->
     {Time, ok} = timer:tc(fun() -> lists:foreach(TestFun, lists:seq(1,200)) end),
     io:format("200 trans done in ~p ~n",[Time div (1000000)]),
     case (Time div (1000000)) < 20 of
-        false -> lists:foreach(TestFun, lists:seq(201,1000));
-        true -> ignore  %% Some virtual test machines are really slow..
+        true -> lists:foreach(TestFun, lists:seq(201,1000));
+        false -> ignore  %% Some virtual test machines are really slow..
     end,
     io:format("Written, check content~n",[]),
     All = fun() -> mnesia:select(dc, [ {{dc, '_', 0}, [] ,['$_']} ]) end,
@@ -967,7 +967,7 @@ snmp_open_table(Config) when is_list(Config) ->
     A ! end_trans,             %% Kill A, locks should be released
     ?match_receive({A,{atomic,end_trans}}),     
     
-    %% Locks released! op should be exec. Can take a while (thats the timeout)
+    %% Locks released! op should be exec. Can take a while (that's the timeout)
     receive 
 	Msg -> ?match({Pid, {atomic, ok}}, Msg)
     after
@@ -999,7 +999,7 @@ snmp_close_table(Config) when is_list(Config) ->
     A ! end_trans,             %% Kill A, locks should be released
     ?match_receive({A,{atomic,end_trans}}),     
     
-    %% Locks released! op should be exec. Can take a while (thats the timeout)
+    %% Locks released! op should be exec. Can take a while (that's the timeout)
     receive 
 	Msg -> ?match({Pid, {atomic, ok}}, Msg)
     after
@@ -1118,16 +1118,22 @@ add_table_copy(Config) when is_list(Config) ->
                      end),
     receive {New,ok} -> ok end,
 
-    Add = fun Add() ->
+    Add = fun Add(N) ->
                   case mnesia:add_table_copy(Tab, Node2, disc_copies) of
                       {atomic, ok} -> ok;
-                      _R -> io:format(user, "aborted with reason ~p~n", [_R]),
-                            timer:sleep(10),
-                            Add()
+                      _R ->
+                          case N > 0 of
+                              true ->
+                                  timer:sleep(25),
+                                  Add(N-1);
+                              false ->
+                                  io:format(user, "aborted with reason ~p~n", [_R]),
+                                  fail
+                          end
                   end
           end,
-
-    ?match(ok, Add()),
+    
+    ?match(ok, Add(200)),
     ?match_receive({New,ok}),
 
     sys:get_status(whereis(mnesia_locker)), % Explicit sync, release locks is async
@@ -1723,7 +1729,7 @@ write_shadows(Config) when is_list(Config) ->
     ?match({atomic, ok}, mnesia:transaction(Fun1)), 
 
     Fun2 = fun() ->
-		  %% write shadow old write - is the confirmed value visable
+		  %% write shadow old write - is the confirmed value visible
 		  %%                          in the shadow ?
 		  ?match([RecA1], mnesia:read({Tab, a})), 
 		  ?match([RecA1], mnesia:wread({Tab, a})), 
@@ -1732,14 +1738,14 @@ write_shadows(Config) when is_list(Config) ->
 		  ?match([RecA1], mnesia:index_match_object(PatA1, ValPos)), 
 		  ?match([RecA1], mnesia:index_read(Tab, 1, ValPos)), 
 
-		  %% write shadow new write - is a new value visable instead
+		  %% write shadow new write - is a new value visible instead
 		  %%                          of the old value ?
 		  ?match(ok, mnesia:write(RecA2)), 
 
 		  ?match([RecA2], mnesia:read({Tab, a})), 
 		  ?match([RecA2], mnesia:wread({Tab, a})), 
 		   ?match([],      mnesia:match_object(PatA1)), %% delete shadow old but not new write
-		   ?match([RecA2], mnesia:match_object(PatA2)), %% is the new value visable
+		   ?match([RecA2], mnesia:match_object(PatA2)), %% is the new value visible
 
 		  ?match([a], mnesia:all_keys(Tab)), 
 		  ?match([RecA2], mnesia:index_match_object(PatA2, ValPos)), 
@@ -1792,7 +1798,7 @@ delete_shadows(Config) when is_list(Config) ->
 		  ?match([], mnesia:index_match_object(PatA1, ValPos)), 
 		  ?match([], mnesia:index_read(Tab, 1, ValPos)), 
 
-		  %% delete shadow old but not new write - is the new value visable
+		  %% delete shadow old but not new write - is the new value visible
 		  %%                           when the old one was deleted ?
 		  ?match(ok, mnesia:write(RecA2)), 
 
@@ -1877,7 +1883,7 @@ write_delete_shadows_bag(Config) when is_list(Config) ->
 		   ?match([], mnesia:index_match_object(PatA1, ValPos)), 
 		   ?match([], mnesia:index_read(Tab, 1, ValPos)), 
 
-		   %% delete shadow old but not new write - are both new value visable
+		   %% delete shadow old but not new write - are both new value visible
 		   %%                           when the old one was deleted ?
 		   ?match(ok, mnesia:write(RecA2)), 
 		   ?match(ok, mnesia:write(RecA3)), 
@@ -2137,7 +2143,7 @@ removed_resources([_N1,N2,N3], DeleteRes) ->
 nasty(suite) -> [];
 
 nasty(doc) ->
-    ["Tries to fullfill a rather nasty locking scenario, where we have had "
+    ["Tries to fulfill a rather nasty locking scenario, where we have had "
      "bugs, the testcase tries a combination of locks in locker queue"];
 
 %%  This testcase no longer works as it was intended to show errors when 
