@@ -1,7 +1,7 @@
 %% 
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1999-2019. All Rights Reserved.
+%% Copyright Ericsson AB 1999-2022. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -48,6 +48,7 @@
 -include("SNMPv2-TC.hrl").
 -include("SNMP-VIEW-BASED-ACM-MIB.hrl").
 -include("snmpa_vacm.hrl").
+-include("snmpa_internal.hrl").
 
 
 -define(VMODULE,"VACM-MIB").
@@ -284,7 +285,7 @@ delete_sec2group(Key) ->
 	    {error, delete_failed}
     end.
     
-%% NOTE: This function must be used in conjuction with
+%% NOTE: This function must be used in conjunction with
 %%       snmpa_vacm:dump_table.
 %%       That is, when all access has been added, call
 %%       snmpa_vacm:dump_table/0
@@ -356,7 +357,7 @@ init_vacm_mnesia() ->
     
     %% The 5 is intentional: It is a trick to get a tuple with the
     %% columns needed by the vacm ets-table (corresponding to the 
-    %% tuple read from the config files). Therefor, 5 since it it
+    %% tuple read from the config files). Therefore, 5 since it it
     %% is not a real foi...
     snmp_generic:table_foreach({vacmAccessTable, mnesia}, F, 5).
 
@@ -653,7 +654,7 @@ vacmAccessTable(is_set_ok, RowIndex, Cols0) ->
 		{{Col, ?'RowStatus_createAndWait'}, _} ->
 		    %% Row already exists => inconsistentValue
 		    {inconsistentValue, Col};
-		{value, {_Col, ?'RowStatus_destroy'}} ->
+		{{_Col, ?'RowStatus_destroy'}, _} ->
 		    %% always ok!
 		    {noError, 0};
 		{_, false} ->
@@ -860,10 +861,12 @@ vacmViewSpinLock(print) ->
 
 vacmViewSpinLock(new) ->
     snmp_generic:variable_func(new, volatile_db(vacmViewSpinLock)),
-    random:seed(erlang:phash2([node()]),
-                erlang:monotonic_time(),
-                erlang:unique_integer()),
-    Val = random:uniform(2147483648) - 1,
+    ?SNMP_RAND_SEED(),
+    %% rand:seed(exrop,
+    %%           {erlang:phash2([node()]),
+    %%            erlang:monotonic_time(),
+    %%            erlang:unique_integer()}),
+    Val = rand:uniform(2147483648) - 1,
     snmp_generic:variable_func(set, Val, volatile_db(vacmViewSpinLock));
 
 vacmViewSpinLock(delete) ->
@@ -1045,7 +1048,7 @@ imask2emask(IMask) ->
 imask2emask([], EMask) ->
     lists:reverse(EMask);
 imask2emask(IMask, EMask) ->
-    %% Make sure we have atleast 8 bits
+    %% Make sure we have at least 8 bits
     %% (maybe extend with 1's)
     IMask2 = 
 	case length(IMask) of
@@ -1112,9 +1115,7 @@ externalize_next(Name, Result) when is_list(Result) ->
     F = fun({[Col | _] = Idx, Val}) -> {Idx, externalize(Name, Col, Val)};
 	   (Other)                  -> Other
 	end,
-    [F(R) || R <- Result];
-externalize_next(_, Result) ->
-    Result.
+    [F(R) || R <- Result].
 
 
 externalize_get(Name, Cols, Result) when is_list(Result) ->
@@ -1124,9 +1125,7 @@ externalize_get(Name, Cols, Result) when is_list(Result) ->
 	end,
     %% Merge column numbers and return values. there must be as much
     %% return values as there are columns requested. And then patch all values
-    [F(R) || R <- lists:zip(Cols, Result)];
-externalize_get(_, _, Result) ->
-    Result. 
+    [F(R) || R <- lists:zip(Cols, Result)]. 
 
 externalize(vacmViewTreeFamilyTable, ?vacmViewTreeFamilyMask, Val) ->
     imask2emask(Val);
