@@ -1,7 +1,7 @@
 %% 
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 2003-2021. All Rights Reserved.
+%% Copyright Ericsson AB 2003-2022. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -163,7 +163,7 @@ init_per_testcase(Case, Config0) when is_list(Config0) ->
     Config1    = ?LIB:fix_data_dir(Config0),
     CaseTopDir = ?LIB:init_testcase_top_dir(Case, Config1), 
     DbDir      = join(CaseTopDir, "db_dir/"),
-    ?line ok   = file:make_dir(DbDir),
+    ok   = file:make_dir(DbDir),
     init_per_testcase2(Case, [{db_dir,       DbDir}, 
 			      {case_top_dir, CaseTopDir} | Config1]).
 
@@ -215,7 +215,6 @@ end_per_testcase1(_Case, Config) when is_list(Config) ->
 %% Test functions
 %%======================================================================
 
-start_and_stop(suite) -> [];
 start_and_stop(Config) when is_list(Config) ->
     tc_try(start_and_start,
            fun() -> do_start_and_stop(Config) end).
@@ -224,20 +223,19 @@ do_start_and_stop(_Config) ->
     Prio      = normal,
     Verbosity = trace,
 
-    ?line sym_start(Prio, Verbosity),
-    ?line MibsPid = mibs_start(Prio, Verbosity),
+    sym_start(Prio, Verbosity),
+    MibsPid = mibs_start(Prio, Verbosity),
 
-    ?line mibs_info(MibsPid),
+    mibs_info(MibsPid),
 
-    ?line mibs_stop(MibsPid),
-    ?line sym_stop(),
+    mibs_stop(MibsPid),
+    sym_stop(),
 
     ok.
 
 
 %% ---------------------------------------------------------------------
 
-load_unload(suite) -> [];
 load_unload(Config) when is_list(Config) ->
     tc_try(load_unload,
            fun() -> do_load_unload(Config) end).
@@ -250,44 +248,44 @@ do_load_unload(Config) ->
     MibDir     = ?config(data_dir, Config),
 
     ?DBG("load_unload -> start symbolic store", []),
-    ?line sym_start(Prio, Verbosity),
+    sym_start(Prio, Verbosity),
 
     ?DBG("load_unload -> start mib server", []),
-    ?line MibsPid = mibs_start(Prio, Verbosity),
+    MibsPid = mibs_start(Prio, Verbosity),
     
     ?DBG("load_unload -> load one not already loaded mib", []),
-    ?line ok = verify_loaded_mibs(MibsPid, MibDir, []),
-    ?line ok = load_mibs(MibsPid, MibDir, ["Test2"]),
-    ?line ok = verify_loaded_mibs(MibsPid, MibDir, ["Test2"]),
+    ok = verify_loaded_mibs(MibsPid, MibDir, []),
+    ok = load_mibs(MibsPid, MibDir, ["Test2"]),
+    ok = verify_loaded_mibs(MibsPid, MibDir, ["Test2"]),
     
     ?DBG("load_unload -> try load one *already loaded* mib", []),
     EMib = join(MibDir, "Test2"), 
-    ?line {error, {'load aborted at', EMib, already_loaded}} = 
+    {error, {'load aborted at', EMib, already_loaded}} =
 	load_mibs(MibsPid, MibDir, ["Test2"]),
 
     ?DBG("load_unload -> load 2 not already loaded mibs", []),
-    ?line ok = load_mibs(MibsPid, MibDir, ["TestTrap", "TestTrapv2"]),
-    ?line ok = verify_loaded_mibs(MibsPid, MibDir, 
+    ok = load_mibs(MibsPid, MibDir, ["TestTrap", "TestTrapv2"]),
+    ok = verify_loaded_mibs(MibsPid, MibDir,
 				  ["Test2", "TestTrap", "TestTrapv2"]),
     
     ?DBG("load_unload -> unload one loaded mib", []),
-    ?line ok = unload_mibs(MibsPid, ["Test2"]),
-    ?line ok = verify_loaded_mibs(MibsPid, MibDir, ["TestTrap", "TestTrapv2"]),
+    ok = unload_mibs(MibsPid, ["Test2"]),
+    ok = verify_loaded_mibs(MibsPid, MibDir, ["TestTrap", "TestTrapv2"]),
     
     ?DBG("load_unload -> try unload two loaded mibs and one not loaded", []),
-    ?line {error, {'unload aborted at', "Test2", not_loaded}} = 
+    {error, {'unload aborted at', "Test2", not_loaded}} =
 	   unload_mibs(MibsPid, ["TestTrap","Test2","TestTrapv2"]),
-    ?line ok = verify_loaded_mibs(MibsPid, MibDir, ["TestTrapv2"]),
+    ok = verify_loaded_mibs(MibsPid, MibDir, ["TestTrapv2"]),
     
     ?DBG("load_unload -> unload the remaining loaded mib", []),
-    ?line ok = unload_mibs(MibsPid, ["TestTrapv2"]),
-    ?line ok = verify_loaded_mibs(MibsPid, MibDir, []),
+    ok = unload_mibs(MibsPid, ["TestTrapv2"]),
+    ok = verify_loaded_mibs(MibsPid, MibDir, []),
     
     ?DBG("load_unload -> stop mib server", []),
-    ?line mibs_stop(MibsPid),
+    mibs_stop(MibsPid),
 
     ?DBG("load_unload -> stop symbolic store", []),
-    ?line sym_stop(),
+    sym_stop(),
 
     ?DBG("load_unload -> done", []),
     ok.
@@ -296,15 +294,11 @@ do_load_unload(Config) ->
 %% ---------------------------------------------------------------------
 
 
-size_check_ets1(suite) ->
-    [];
 size_check_ets1(Config) when is_list(Config) ->
     MibStorage = [{module, snmpa_mib_storage_ets}], 
     do_size_check(size_check_ets1,
                   [{mib_storage, MibStorage}|Config]).
 
-size_check_ets2(suite) ->
-    [];
 size_check_ets2(Config) when is_list(Config) ->
     Dir = ?config(db_dir, Config),    
     MibStorage = [{module,  snmpa_mib_storage_ets}, 
@@ -312,8 +306,6 @@ size_check_ets2(Config) when is_list(Config) ->
     do_size_check(size_check_ets2,
                   [{mib_storage, MibStorage}|Config]).
 
-size_check_ets2_bad_file1(suite) ->
-    [];
 size_check_ets2_bad_file1(Config) when is_list(Config) ->
     Dir = ?config(db_dir, Config),    
     %% Ensure that the bad file does not cause any problems (action = clear)
@@ -323,8 +315,6 @@ size_check_ets2_bad_file1(Config) when is_list(Config) ->
     do_size_check(size_check_ets2_bad_file1,
                   [{mib_storage, MibStorage}|Config]).
 
-size_check_ets3(suite) ->
-    [];
 size_check_ets3(Config) when is_list(Config) ->
     Dir = ?config(db_dir, Config),    
     MibStorage = [{module,  snmpa_mib_storage_ets}, 
@@ -333,8 +323,6 @@ size_check_ets3(Config) when is_list(Config) ->
     do_size_check(size_check_ets3,
                   [{mib_storage, MibStorage}|Config]).
 
-size_check_ets3_bad_file1(suite) ->
-    [];
 size_check_ets3_bad_file1(Config) when is_list(Config) ->
     Dir = ?config(db_dir, Config),    
     %% Ensure that the bad file does not cause any problems (action = clear)
@@ -345,8 +333,6 @@ size_check_ets3_bad_file1(Config) when is_list(Config) ->
     do_size_check(size_check_ets3_bad_file1,
                   [{mib_storage, MibStorage}|Config]).
 
-size_check_dets(suite) ->
-    [];
 size_check_dets(Config) when is_list(Config) ->
     Dir = ?config(db_dir, Config),
     MibStorage = [{module,  snmpa_mib_storage_dets}, 
@@ -354,8 +340,6 @@ size_check_dets(Config) when is_list(Config) ->
     do_size_check(size_check_dets,
                   [{mib_storage, MibStorage}|Config]).
 
-size_check_mnesia(suite) ->
-    [];
 size_check_mnesia(Config) when is_list(Config) ->
     MibStorage = [{module,  snmpa_mib_storage_mnesia}, 
                   {options, [{nodes, []}]}],
@@ -385,9 +369,9 @@ do_size_check(Config) ->
     StdMibDir  = filename:join(code:priv_dir(snmp), "mibs") ++ "/",
     
     ?IPRINT("do_size_check -> start symbolic store", []),
-    ?line sym_start(Prio, MibStorage, Verbosity),
+    sym_start(Prio, MibStorage, Verbosity),
     ?IPRINT("do_size_check -> start mib server", []),
-    ?line MibsPid = mibs_start(Prio, MibStorage, Verbosity),
+    MibsPid = mibs_start(Prio, MibStorage, Verbosity),
 
     Mibs    = ["Test2", "TestTrap", "TestTrapv2"],
     StdMibs = ["OTP-SNMPEA-MIB",
@@ -403,23 +387,23 @@ do_size_check(Config) ->
 	       "SNMPv2-TM"],
 
     ?IPRINT("do_size_check -> load std mibs", []),
-    ?line load_mibs(MibsPid, StdMibDir, StdMibs),
+    load_mibs(MibsPid, StdMibDir, StdMibs),
     ?IPRINT("do_size_check -> load (own) mibs", []),
-    ?line load_mibs(MibsPid, MibDir, Mibs),
+    load_mibs(MibsPid, MibDir, Mibs),
 
     ?SLEEP(2000),
     ?IPRINT("do_size_check -> display mem usage", []),
-    ?line display_memory_usage(MibsPid),
+    display_memory_usage(MibsPid),
     
     ?IPRINT("do_size_check -> unload (own) mibs", []),
-    ?line unload_mibs(MibsPid, Mibs),
+    unload_mibs(MibsPid, Mibs),
     ?IPRINT("do_size_check -> unload std mibs", []),
-    ?line unload_mibs(MibsPid, StdMibs),
+    unload_mibs(MibsPid, StdMibs),
 
     ?IPRINT("do_size_check -> stop mib server", []),
-    ?line mibs_stop(MibsPid),
+    mibs_stop(MibsPid),
     ?IPRINT("do_size_check -> stop symbolic store", []),
-    ?line sym_stop(),
+    sym_stop(),
 
     ?IPRINT("do_size_check -> done", []),
     ok.
@@ -427,7 +411,6 @@ do_size_check(Config) ->
 
 %% ---------------------------------------------------------------------
 
-me_lookup(suite) -> [];
 me_lookup(Config) when is_list(Config) ->
     tc_try(me_lookup,
            fun() -> do_me_lookup(Config) end).
@@ -451,40 +434,39 @@ do_me_lookup(Config) ->
 	       "SNMPv2-TM"],
 
     ?DBG("me_lookup -> start symbolic store", []),
-    ?line sym_start(Prio, Verbosity),
+    sym_start(Prio, Verbosity),
 
     ?DBG("me_lookup -> start mib server", []),
-    ?line MibsPid = mibs_start(Prio, Verbosity),
+    MibsPid = mibs_start(Prio, Verbosity),
     
     ?DBG("me_lookup -> load mibs", []),
-    ?line load_mibs(MibsPid, MibDir, Mibs),
+    load_mibs(MibsPid, MibDir, Mibs),
     ?DBG("me_lookup -> load std mibs", []),
-    ?line load_mibs(MibsPid, StdMibDir, StdMibs),
+    load_mibs(MibsPid, StdMibDir, StdMibs),
 
     ?DBG("me_lookup -> find ~w from SNMP-COMMUNITY-MIB", 
 	 [?snmpTrapCommunity_instance]),
-    ?line ok = me_lookup(MibsPid, ?snmpTrapCommunity_instance),
+    ok = me_lookup(MibsPid, ?snmpTrapCommunity_instance),
     
     ?DBG("me_lookup -> find ~w from SNMP-VIEW-BASED-ACM-MIB", 
 	 [?vacmViewSpinLock_instance]),
-    ?line ok = me_lookup(MibsPid, ?vacmViewSpinLock_instance),
+    ok = me_lookup(MibsPid, ?vacmViewSpinLock_instance),
     
     ?DBG("me_lookup -> find ~w from SNMP-USER-BASED-SM-MIB", 
 	 [?usmStatsNotInTimeWindows_instance]),
-    ?line {error, _} = me_lookup(MibsPid, ?usmStatsNotInTimeWindows_instance),
+    {error, _} = me_lookup(MibsPid, ?usmStatsNotInTimeWindows_instance),
     
     ?DBG("me_lookup -> stop mib server", []),
-    ?line mibs_stop(MibsPid),
+    mibs_stop(MibsPid),
 
     ?DBG("me_lookup -> stop symbolic store", []),
-    ?line sym_stop(),
+    sym_stop(),
 
     ok.
 
 
 %% ---------------------------------------------------------------------
 
-which_mib(suite) -> [];
 which_mib(Config) when is_list(Config) ->
     tc_try(which_mib,
            fun() -> do_which_mib(Config) end).
@@ -508,43 +490,42 @@ do_which_mib(Config) ->
 	       "SNMPv2-TM"],
 
     ?DBG("which_mib -> start symbolic store", []),
-    ?line sym_start(Prio, Verbosity),
+    sym_start(Prio, Verbosity),
 
     ?DBG("which_mib -> start mib server", []),
-    ?line MibsPid = mibs_start(Prio, Verbosity),
+    MibsPid = mibs_start(Prio, Verbosity),
     
     ?DBG("which_mib -> load mibs", []),
-    ?line load_mibs(MibsPid, MibDir, Mibs),
+    load_mibs(MibsPid, MibDir, Mibs),
     ?DBG("which_mib -> load std mibs", []),
-    ?line load_mibs(MibsPid, StdMibDir, StdMibs),
+    load_mibs(MibsPid, StdMibDir, StdMibs),
 
     ?DBG("which_mib -> find ~w from SNMP-COMMUNITY-MIB", 
 	 [?snmpTrapCommunity_instance]),
-    ?line ok = which_mib(MibsPid, ?snmpTrapCommunity_instance, 
+    ok = which_mib(MibsPid, ?snmpTrapCommunity_instance,
 			 "SNMP-COMMUNITY-MIB"),
     
     ?DBG("which_mib -> find ~w from SNMP-VIEW-BASED-ACM-MIB", 
 	 [?vacmViewSpinLock_instance]),
-    ?line ok = which_mib(MibsPid, ?vacmViewSpinLock_instance, 
+    ok = which_mib(MibsPid, ?vacmViewSpinLock_instance,
 			 "SNMP-VIEW-BASED-ACM-MIB"),
     
     ?DBG("which_mib -> find ~w from SNMP-USER-BASED-SM-MIB (not loaded)", 
 	 [?usmStatsNotInTimeWindows_instance]),
-    ?line {error, _} = which_mib(MibsPid, ?usmStatsNotInTimeWindows_instance,
+    {error, _} = which_mib(MibsPid, ?usmStatsNotInTimeWindows_instance,
 				"SNMP-USER-BASED-SM-MIB"),
     
     ?DBG("which_mib -> stop mib server", []),
-    ?line mibs_stop(MibsPid),
+    mibs_stop(MibsPid),
 
     ?DBG("which_mib -> stop symbolic store", []),
-    ?line sym_stop(),
+    sym_stop(),
 
     ok.
 
 
 %% ---------------------------------------------------------------------
 
-cache_test(suite) -> [];
 cache_test(Config) when is_list(Config) ->
     tc_try(cache_test,
            fun() -> do_cache_test(Config) end).
@@ -571,7 +552,7 @@ do_cache_test(Config) ->
 		  "SNMPv2-TM"],
 
     ?IPRINT("cache_test -> start symbolic store"),
-    ?line sym_start(Prio, MibStorage, silence), % Verbosity),
+    sym_start(Prio, MibStorage, silence), % Verbosity),
 
     ?IPRINT("cache_test -> start mib server"),
     GcLimit   = 3,
@@ -580,25 +561,25 @@ do_cache_test(Config) ->
                  {age,       Age},
                  {gclimit,   GcLimit},
                  {gcverbose, true}],
-    ?line MibsPid = mibs_start(Prio, MibStorage, [], Verbosity, CacheOpts), 
+    MibsPid = mibs_start(Prio, MibStorage, [], Verbosity, CacheOpts),
     
     ?NPRINT("Info before load mibs: "
             "~n      ~p", [snmpa_mib:info(MibsPid)]),
 
     ?IPRINT("cache_test -> load mibs"),
-    ?line load_mibs(MibsPid, MibDir, Mibs),
+    load_mibs(MibsPid, MibDir, Mibs),
 
     ?NPRINT("Info before load std mibs: "
             "~n      ~p", [snmpa_mib:info(MibsPid)]),
 
     ?IPRINT("cache_test -> load std mibs"),
-    ?line load_mibs(MibsPid, StdMibDir, StdMibs),
+    load_mibs(MibsPid, StdMibDir, StdMibs),
 
     ?NPRINT("Info (after mibs load but) before populate: "
             "~n      ~p", [snmpa_mib:info(MibsPid)]),
 
     ?IPRINT("cache_test -> populate the cache"),
-    ?line ok = populate(MibsPid),
+    ok = populate(MibsPid),
 
     ?NPRINT("Info after populate: "
             "~n      ~p", [snmpa_mib:info(MibsPid)]),
@@ -625,10 +606,10 @@ do_cache_test(Config) ->
 
 
     ?IPRINT("cache_test -> subscribe to GC events"),
-    ?line ok = snmpa_mib:subscribe_gc_events(MibsPid),
+    ok = snmpa_mib:subscribe_gc_events(MibsPid),
 
     ?IPRINT("cache_test -> enable cache autogc"),
-    ?line ok = snmpa_mib:enable_cache_autogc(MibsPid),
+    ok = snmpa_mib:enable_cache_autogc(MibsPid),
 
     ?IPRINT("cache_test -> wait 65 seconds to allow gc to happen"),
     ?SLEEP(timer:seconds(65)),
@@ -738,10 +719,10 @@ do_cache_test(Config) ->
 
 
     ?IPRINT("cache_test -> stop mib server"),
-    ?line mibs_stop(MibsPid),
+    mibs_stop(MibsPid),
 
     ?IPRINT("cache_test -> stop symbolic store"),
-    ?line sym_stop(),
+    sym_stop(),
 
     ?IPRINT("cache_test -> end"),
     ok.
@@ -873,13 +854,13 @@ mnesia_start(Opts) ->
     mnesia_start(Opts, [node()]).
 
 mnesia_start(Opts, Nodes) ->
-    %% We can accept mnesia beeing loaded but *not* started.
+    %% We can accept mnesia being loaded but *not* started.
     %% If its started it *may* contain data that will invalidate
     %% this test case.
     ?IPRINT("mnesia_start -> try load mnesia when:"
             "~n   Loaded:  ~p"
             "~n   Running: ~p", [apps_loaded(), apps_running()]),
-    ?line ok = case application:load(mnesia) of
+    ok = case application:load(mnesia) of
                    ok ->
                        ok;
                    {error, {already_loaded, mnesia}} ->
@@ -890,11 +871,11 @@ mnesia_start(Opts, Nodes) ->
     F = fun({Key, Val}) ->
 		?IPRINT("mnesia_start -> try set mnesia env: "
                         "~n   ~p -> ~p", [Key, Val]),
-		?line application_controller:set_env(mnesia, Key, Val)
+		application_controller:set_env(mnesia, Key, Val)
 	end,
     lists:foreach(F, Opts),
     ?IPRINT("mnesia_start -> create mnesia schema on ~p", [Nodes]),
-    ?line case mnesia:create_schema(Nodes) of
+    case mnesia:create_schema(Nodes) of
               ok ->
                   ok;
               {error, {_, {already_exist, _}}} ->
@@ -904,7 +885,7 @@ mnesia_start(Opts, Nodes) ->
                          ?F("Failed create mnesia schema: ~p", [SchemaReason])})
           end,
     ?IPRINT("mnesia_start -> start mnesia", []),
-    ?line case application:start(mnesia) of
+    case application:start(mnesia) of
               ok ->
                   ok;
               {error, {already_started, mnesia}} ->
@@ -988,13 +969,13 @@ mibs_info(Pid) ->
 
 load_mibs(Pid, Dir, Mibs0) ->
     Mibs = [join(Dir, Mib) || Mib <- Mibs0],
-    Res = snmpa_mib:load_mibs(Pid, Mibs),
+    Res = snmpa_mib:load_mibs(Pid, Mibs, false),
     %% ?DBG("load_mibs -> "
     %% 	 "~n   Res: ~p", [Res]),
     Res.
 
 unload_mibs(Pid, Mibs) ->
-    Res = snmpa_mib:unload_mibs(Pid, Mibs),
+    Res = snmpa_mib:unload_mibs(Pid, Mibs, false),
     %% ?DBG("unload_mibs -> "
     %% 	 "~n   Res: ~p", [Res]),
     Res.
@@ -1093,11 +1074,11 @@ tc_try(Name, TC) ->
 tc_try(Name, Init, TC)
   when is_atom(Name) andalso is_function(Init, 0) andalso is_function(TC, 0) ->
     Pre = fun() ->
-                  {ok, Node} = ?ALIB:start_node(unique(Name)),
+                  {ok, Peer, Node} = ?START_PEER(atom_to_list(Name)),
                   ok = run_on(Node, Init),
-                  Node
+                  {Peer, Node}
           end,
-    Case = fun(Node) ->
+    Case = fun({_Peer, Node}) ->
                    monitor_node(Node, true),
                    Pid = spawn_link(Node, TC),
                    receive
@@ -1125,7 +1106,7 @@ tc_try(Name, Init, TC)
                            exit(Reason)
                    end
            end,
-    Post = fun(Node) ->
+    Post = fun({Peer, Node}) ->
                    receive
                        {nodedown, Node} ->
                            ?NPRINT("node ~p (already) stopped", [Node]),
@@ -1133,7 +1114,7 @@ tc_try(Name, Init, TC)
                    after 0 ->
                            monitor_node(Node, true),
                            ?NPRINT("try stop node ~p", [Node]),
-                           ?STOP_NODE(Node),
+                           peer:stop(Peer),
                            receive
                                {nodedown, Node} ->
                                    ?NPRINT("node ~p stopped", [Node]),
@@ -1156,9 +1137,6 @@ run_on(Node, F) when is_atom(Node) andalso is_function(F, 0) ->
             monitor_node(Node, false),                           
             Reason
     end.
-    
-unique(PreName) ->
-    list_to_atom(?F("~w_~w", [PreName, erlang:system_time(millisecond)])).
 
 
 %% -- 
