@@ -1,7 +1,7 @@
 %% 
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1996-2021. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2022. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -147,13 +147,19 @@ receive_response() ->
     receive_response(get_timeout()).
 
 receive_response(Timeout) ->
-    d("await response within ~w ms",[Timeout]),
+    d("await response within ~w ms", [Timeout]),
     receive
 	{snmp_pdu, PDU} when is_record(PDU, pdu) ->
-	    d("received PDU: ~n\t~p",[PDU]),
-	    PDU
+	    d("[await response] received PDU: "
+              "~n      ~p", [PDU]),
+	    PDU;
+        {error, Reason} = ERROR ->
+	    ?EPRINT("[await response] received unexpected error: "
+                    "~n      ~p", [Reason]),
+            ERROR
     after Timeout ->
-	    d("response timeout",[]),
+	    ?EPRINT("[await response] unexpected timeout: "
+                    "~n      ~p", [process_info(self(), messages)]),
 	    {error, timeout}
     end.
 
