@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2008-2021. All Rights Reserved.
+%% Copyright Ericsson AB 2008-2022. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -177,7 +177,7 @@
                         Command :: string()
                        } .
 
-%%% This function is soley to convince all
+%%% This function is solely to convince all
 %%% checks that the type event() exists...
 -export([dummy/1]).
 -spec dummy(event()) -> false.
@@ -404,7 +404,7 @@ ptty_alloc(ConnectionHandler, Channel, Options0, TimeOut) ->
 	   ).
 
 %%--------------------------------------------------------------------
-%% Not yet officialy supported! The following functions are part of the
+%% Not yet officially supported! The following functions are part of the
 %% initial contributed ssh application. They are untested. Do we want them?
 %% Should they be documented and tested?
 %%--------------------------------------------------------------------
@@ -811,26 +811,20 @@ handle_msg(#ssh_msg_channel_request{recipient_channel = ChannelId,
 				    request_type = "pty-req",
 				    want_reply = WantReply,
 				    data = Data},
-	   Connection, server, SSH) ->
+	   Connection, server, _SSH) ->
     <<?DEC_BIN(BTermName,_TermLen),
       ?UINT32(Width),?UINT32(Height),
       ?UINT32(PixWidth), ?UINT32(PixHeight),
       Modes/binary>> = Data,
     TermName = binary_to_list(BTermName),
     PtyOpts0 = decode_pty_opts(Modes),
-    PtyOpts = case SSH#ssh.c_version of
-                  "SSH-2.0-PuTTY"++_ ->
-                      %% If - peer client is PuTTY
-                      %%    - it asked for pty
+    PtyOpts = case proplists:get_value(onlcr, PtyOpts0, undefined) of
+                  undefined ->
+                      %% If - peer client asked for pty
                       %%    - did not tell if LF->CRLF expansion is wanted
                       %% then
                       %%    - do LF->CRLF expansion
-                      case  proplists:get_value(onlcr, PtyOpts0, undefined) of
-                          undefined ->
-                              [{onlcr,1} | PtyOpts0];
-                          _ ->
-                              PtyOpts0
-                      end;
+                      [{onlcr,1} | PtyOpts0];
                   _ ->
                       PtyOpts0
               end,
