@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2007-2021. All Rights Reserved.
+%% Copyright Ericsson AB 2007-2022. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -21,12 +21,14 @@
 
 -include("tls_record.hrl").
 -include("tls_record_1_3.hrl").
+-include("tls_handshake_1_3.hrl").
 -include("ssl_internal.hrl").
 -include("ssl_alert.hrl").
 -include("ssl_cipher.hrl").
 
 %% Encoding 
--export([encode_handshake/2, encode_alert_record/2,
+-export([encode_handshake/2,
+         encode_alert_record/2,
 	 encode_data/2]).
 -export([encode_plain_text/3]).
 
@@ -43,7 +45,8 @@
 %
 %% Description: Encodes a handshake message to send on the tls-1.3-socket.
 %%--------------------------------------------------------------------
-encode_handshake(Frag, #{current_write := #{max_fragment_length := MaxFragmentLength}} =
+encode_handshake(Frag, #{current_write :=
+                             #{max_fragment_length := MaxFragmentLength}} =
                      ConnectionStates) ->
     MaxLength = if is_integer(MaxFragmentLength) ->
                         MaxFragmentLength;
@@ -75,7 +78,8 @@ encode_alert_record(#alert{level = Level, description = Description},
 %%
 %% Description: Encodes data to send on the ssl-socket.
 %%--------------------------------------------------------------------
-encode_data(Frag, #{current_write := #{max_fragment_length := MaxFragmentLength}} =
+encode_data(Frag, #{current_write :=
+                        #{max_fragment_length := MaxFragmentLength}} =
                      ConnectionStates) ->
     MaxLength = if is_integer(MaxFragmentLength) ->
                         MaxFragmentLength;
@@ -85,7 +89,8 @@ encode_data(Frag, #{current_write := #{max_fragment_length := MaxFragmentLength}
     Data = tls_record:split_iovec(Frag, MaxLength),
     encode_iolist(?APPLICATION_DATA, Data, ConnectionStates).
 
-encode_plain_text(Type, Data0, #{current_write := Write0} = ConnectionStates) ->
+encode_plain_text(Type, Data0, #{current_write := Write0} =
+                      ConnectionStates) ->
     PadLen = 0, %% TODO where to specify PadLen?
     Data = inner_plaintext(Type, Data0, PadLen),
     CipherFragment = encode_plain_text(Data, Write0),
@@ -107,12 +112,13 @@ encode_iolist(Type, Data, ConnectionStates0) ->
 
 %%--------------------------------------------------------------------
 -spec decode_cipher_text(#ssl_tls{}, ssl_record:connection_states()) ->
-				{#ssl_tls{} | trial_decryption_failed,
+				{#ssl_tls{} | no_record,
                                  ssl_record:connection_states()}| #alert{}.
 %%
-%% Description: Decode cipher text, use legacy type ssl_tls instead of tls_cipher_text
-%% in decoding context so that we can reuse the code from erlier versions. 
-%%--------------------------------------------------------------------
+%% Description: Decode cipher text, use legacy type ssl_tls instead of
+%% tls_cipher_text in decoding context so that we can reuse the code
+%% from earlier versions.
+%% --------------------------------------------------------------------
 decode_cipher_text(#ssl_tls{type = ?OPAQUE_TYPE,
                             version = ?LEGACY_VERSION,
                             fragment = CipherFragment},
@@ -126,22 +132,25 @@ decode_cipher_text(#ssl_tls{type = ?OPAQUE_TYPE,
 				  cipher_type = ?AEAD,
                                   bulk_cipher_algorithm =
                                       BulkCipherAlgo},
-                           max_early_data_size := MaxEarlyDataSize0,
+                           pending_early_data_size := PendingMaxEarlyDataSize0,
                            trial_decryption := TrialDecryption,
-                           early_data_limit := EarlyDataLimit
+                           early_data_accepted := EarlyDataAccepted
 			  } = ReadState0} = ConnectionStates0) ->
     case decipher_aead(CipherFragment, BulkCipherAlgo, Key, Seq, IV, TagLen) of
 	#alert{} when TrialDecryption =:= true andalso
-                      MaxEarlyDataSize0 > 0 -> %% Trial decryption
-            trial_decrypt(ConnectionStates0, ReadState0, MaxEarlyDataSize0,
-                          BulkCipherAlgo, CipherFragment);
+                      EarlyDataAccepted =:= false andalso
+                      PendingMaxEarlyDataSize0 > 0 -> %% Trial decryption
+            ignore_early_data(ConnectionStates0, ReadState0,
+                              PendingMaxEarlyDataSize0,
+                              BulkCipherAlgo, CipherFragment);
 	#alert{} = Alert ->
 	    Alert;
-        PlainFragment0 when EarlyDataLimit =:= true andalso
-                            MaxEarlyDataSize0 > 0 ->
+        PlainFragment0 when EarlyDataAccepted =:= true andalso
+                            PendingMaxEarlyDataSize0 > 0 ->
             PlainFragment = remove_padding(PlainFragment0),
-            process_early_data(ConnectionStates0, ReadState0, MaxEarlyDataSize0, Seq,
-                               BulkCipherAlgo, CipherFragment, PlainFragment);
+            process_early_data(ConnectionStates0, ReadState0,
+                               PendingMaxEarlyDataSize0, Seq,
+                               PlainFragment);
 	PlainFragment0 ->
             PlainFragment = remove_padding(PlainFragment0),
 	    ConnectionStates =
@@ -159,20 +168,20 @@ decode_cipher_text(#ssl_tls{type = ?OPAQUE_TYPE,
 %% the signature algorithm of the client's certificate.)
 decode_cipher_text(#ssl_tls{type = ?ALERT,
                             version = ?LEGACY_VERSION,
-                            fragment = <<2,47>>},
+                            fragment = <<?FATAL,?ILLEGAL_PARAMETER>>},
 		   ConnectionStates0) ->
     {#ssl_tls{type = ?ALERT,
               version = {3,4}, %% Internally use real version
-              fragment = <<2,47>>}, ConnectionStates0};
+              fragment = <<?FATAL,?ILLEGAL_PARAMETER>>}, ConnectionStates0};
 %% TLS 1.3 server can receive a User Cancelled Alert when handshake is
 %% paused and then cancelled on the client side.
 decode_cipher_text(#ssl_tls{type = ?ALERT,
                             version = ?LEGACY_VERSION,
-                            fragment = <<2,90>>},
+                            fragment = <<?FATAL,?USER_CANCELED>>},
 		   ConnectionStates0) ->
     {#ssl_tls{type = ?ALERT,
               version = {3,4}, %% Internally use real version
-              fragment = <<2,90>>}, ConnectionStates0};
+              fragment = <<?FATAL,?USER_CANCELED>>}, ConnectionStates0};
 %% RFC8446 - TLS 1.3
 %% D.4.  Middlebox Compatibility Mode
 %%    -  If not offering early data, the client sends a dummy
@@ -208,48 +217,49 @@ decode_cipher_text(#ssl_tls{type = Type}, _) ->
 %%--------------------------------------------------------------------
 %%% Internal functions
 %%--------------------------------------------------------------------
-trial_decrypt(ConnectionStates0, ReadState0, MaxEarlyDataSize0,
+ignore_early_data(ConnectionStates0, ReadState0, PendingMaxEarlyDataSize0,
               BulkCipherAlgo, CipherFragment) ->
-    MaxEarlyDataSize = update_max_early_date_size(MaxEarlyDataSize0, BulkCipherAlgo, CipherFragment),
+    PendingMaxEarlyDataSize =
+        approximate_pending_early_data_size(PendingMaxEarlyDataSize0,
+                                            BulkCipherAlgo, CipherFragment),
     ConnectionStates =
-        ConnectionStates0#{current_read =>
-                               ReadState0#{max_early_data_size => MaxEarlyDataSize}},
-    if MaxEarlyDataSize < 0 ->
-            %% More early data is trial decrypted as the configured limit
-            ?ALERT_REC(?FATAL, ?BAD_RECORD_MAC, decryption_failed);
-       true ->
-            {trial_decryption_failed, ConnectionStates}
-    end.
-
-process_early_data(ConnectionStates0, ReadState0, _MaxEarlyDataSize0, Seq,
-                   _BulkCipherAlgo, _CipherFragment, PlainFragment)
-  when PlainFragment =:= <<5,0,0,0,22>> ->
-    %% struct {
-    %%     opaque content[TLSPlaintext.length];    <<5,0,0,0>> - 5 = EndOfEarlyData
-    %%                                                           0 = (uint24) size
-    %%     ContentType type;                       <<22>> - Handshake
-    %%     uint8 zeros[length_of_padding];         <<>> - no padding
-    %% } TLSInnerPlaintext;
-    %% EndOfEarlyData should not be counted into early data
-    ConnectionStates =
-        ConnectionStates0#{current_read =>
-                               ReadState0#{sequence_number => Seq + 1}},
-    {decode_inner_plaintext(PlainFragment), ConnectionStates};
-process_early_data(ConnectionStates0, ReadState0, MaxEarlyDataSize0, Seq,
-                   BulkCipherAlgo, CipherFragment, PlainFragment) ->
+         ConnectionStates0#{current_read =>
+                                ReadState0#{pending_early_data_size => PendingMaxEarlyDataSize}},
+     if PendingMaxEarlyDataSize < 0 ->
+             %% More early data is trial decrypted as the configured limit
+             ?ALERT_REC(?FATAL, ?BAD_RECORD_MAC, {decryption_failed,
+                                                  {max_early_data_threshold_exceeded,
+                                                   PendingMaxEarlyDataSize}});
+        true ->
+             {no_record, ConnectionStates}
+     end.
+process_early_data(ConnectionStates0, ReadState0, PendingMaxEarlyDataSize0, Seq,
+                   PlainFragment) ->
     %% First packet is deciphered anyway so we must check if more early data is received
     %% than the configured limit (max_early_data_size).
-    MaxEarlyDataSize =
-        update_max_early_date_size(MaxEarlyDataSize0, BulkCipherAlgo, CipherFragment),
-    if MaxEarlyDataSize < 0 ->
-            %% Too much early data received, send alert unexpected_message
-            ?ALERT_REC(?FATAL, ?UNEXPECTED_MESSAGE, too_much_early_data);
-       true ->
+    Record = decode_inner_plaintext(PlainFragment),
+    case {Record#ssl_tls.type, remove_padding(Record#ssl_tls.fragment)} of
+        {?HANDSHAKE, <<?END_OF_EARLY_DATA>>} ->
             ConnectionStates =
                 ConnectionStates0#{current_read =>
-                                       ReadState0#{sequence_number => Seq + 1,
-                                                   max_early_data_size => MaxEarlyDataSize}},
-            {decode_inner_plaintext(PlainFragment), ConnectionStates}
+                               ReadState0#{sequence_number => Seq + 1}},
+            {Record, ConnectionStates};
+        {?APPLICATION_DATA, Data} ->
+            PendingMaxEarlyDataSize =
+                pending_early_data_size(PendingMaxEarlyDataSize0, Data),
+            if PendingMaxEarlyDataSize < 0 ->
+                    %% Too much early data received, send alert unexpected_message
+                    ?ALERT_REC(?FATAL, ?UNEXPECTED_MESSAGE,
+                               {too_much_early_data,
+                                {max_early_data_threshold_exceeded,
+                                 PendingMaxEarlyDataSize}});
+               true ->
+                    ConnectionStates =
+                        ConnectionStates0#{current_read =>
+                                               ReadState0#{sequence_number => Seq + 1,
+                                                           pending_early_data_size => PendingMaxEarlyDataSize}},
+                    {Record#ssl_tls{early_data = true}, ConnectionStates}
+            end
     end.
 
 inner_plaintext(Type, Data, Length) ->
@@ -276,7 +286,8 @@ encode_plain_text(#inner_plaintext{
                         }) ->
     PlainText = [Data, Type, Zeros],
     Encoded = cipher_aead(PlainText, BulkCipherAlgo, Key, Seq, IV, TagLen),
-    #tls_cipher_text{opaque_type = 23,  %% 23 (application_data) for outward compatibility
+    %% 23 (application_data) for outward compatibility
+    #tls_cipher_text{opaque_type = ?OPAQUE_TYPE,
                      legacy_version = {3,3},
                      encoded_record = Encoded};
 encode_plain_text(#inner_plaintext{
@@ -320,7 +331,8 @@ cipher_aead(Fragment, BulkCipherAlgo, Key, Seq, IV, TagLen) ->
 
 encode_tls_cipher_text(#tls_cipher_text{opaque_type = Type,
                                         legacy_version = {MajVer, MinVer},
-                                        encoded_record = Encoded}, #{sequence_number := Seq} = Write) ->
+                                        encoded_record = Encoded},
+                       #{sequence_number := Seq} = Write) ->
     Length = erlang:iolist_size(Encoded),
     {[<<?BYTE(Type), ?BYTE(MajVer), ?BYTE(MinVer), ?UINT16(Length)>>, Encoded],
      Write#{sequence_number => Seq +1}}.
@@ -333,11 +345,14 @@ decipher_aead(CipherFragment, BulkCipherAlgo, Key, Seq, IV, TagLen) ->
 	case ssl_cipher:aead_decrypt(BulkCipherAlgo, Key, Nonce, CipherText, CipherTag, AAD) of
 	    Content when is_binary(Content) ->
 		Content;
-	    _ ->
+	    Reason ->
+                ?SSL_LOG(debug, decrypt_error, [{reason,Reason},
+                                                {stacktrace, process_info(self(), current_stacktrace)}]),
                 ?ALERT_REC(?FATAL, ?BAD_RECORD_MAC, decryption_failed)
 	end
     catch
-	_:_ ->
+	_:Reason2:ST ->
+            ?SSL_LOG(debug, decrypt_error, [{reason,Reason2}, {stacktrace, ST}]),
             ?ALERT_REC(?FATAL, ?BAD_RECORD_MAC, decryption_failed)
     end.
 
@@ -379,19 +394,19 @@ remove_padding(InnerPlainText) ->
             InnerPlainText
     end.
 
-update_max_early_date_size(MaxEarlyDataSize, BulkCipherAlgo, CipherFragment) ->
-    %% CipherFragment is the binary encoded form of a TLSInnerPlaintext:
-    %%
-    %% struct {
-    %%     opaque content[TLSPlaintext.length];
-    %%     ContentType type;
-    %%     uint8 zeros[length_of_padding];
-    %% } TLSInnerPlaintext;
-    %%
-    TypeLen = 1,
-    PaddingLen = 0, %% TODO Update formula when padding is implemented!
-    MaxEarlyDataSize - (byte_size(CipherFragment) - TypeLen - PaddingLen -
-                            bca_tag_len(BulkCipherAlgo)).
+pending_early_data_size(PendingMaxEarlyDataSize, PlainFragment) ->
+    %% The maximum amount of 0-RTT data that the client is allowed to
+    %% send when using this ticket, in bytes.  Only Application Data
+    %% payload (i.e., plaintext but not padding or the inner content
+    %% type byte) is counted.
+    PendingMaxEarlyDataSize - (byte_size(PlainFragment)).
+
+approximate_pending_early_data_size(PendingMaxEarlyDataSize,
+                                    BulkCipherAlgo, CipherFragment) ->
+    %% We can not know how much is padding!
+    InnerContTypeLen = 1,
+    PendingMaxEarlyDataSize - (byte_size(CipherFragment) -
+                                   InnerContTypeLen - bca_tag_len(BulkCipherAlgo)).
 
 bca_tag_len(?AES_CCM_8) ->
     8;
