@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2019-2021. All Rights Reserved.
+%% Copyright Ericsson AB 2019-2022. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -35,7 +35,7 @@
          end_per_testcase/2]).
 
 %% Testcases
--export([dhe_psk_des_cbc/1,
+-export([%%dhe_psk_des_cbc/1,
          dhe_rsa_3des_ede_cbc/1,
          dhe_rsa_aes_128_cbc/1,
          dhe_rsa_aes_128_gcm/1,
@@ -89,19 +89,14 @@
          ecdhe_ecdsa_with_aes_256_ccm_8/1
         ]).
 
--define(DEFAULT_TIMEOUT, {seconds, 10}).
+-define(DEFAULT_TIMEOUT, {seconds, 15}).
 
 %%--------------------------------------------------------------------
 %% Common Test interface functions -----------------------------------
 %%--------------------------------------------------------------------
-all() -> 
-     case ssl_test_lib:working_openssl_client() of
-         true ->
-             [{group,  openssl_server},
-              {group,  openssl_client}];
-         false ->
-             [{group,  openssl_server}]
-     end.
+all() ->
+    [{group,  openssl_server},
+     {group,  openssl_client}].
 
 all_protocol_groups() ->
     [
@@ -125,14 +120,14 @@ groups() ->
      {'tlsv1', [], kex()},
      {'dtlsv1.2', [], dtls_kex()},
      {'dtlsv1', [], dtls_kex()},
-     {dhe_rsa, [],[dhe_rsa_3des_ede_cbc, 
+     {dhe_rsa, [],[dhe_rsa_3des_ede_cbc,
                    dhe_rsa_aes_128_cbc,
                    dhe_rsa_aes_128_gcm,
                    dhe_rsa_aes_256_cbc,
                    dhe_rsa_aes_256_gcm,
                    dhe_rsa_chacha20_poly1305
                   ]},
-     {ecdhe_rsa, [], [ecdhe_rsa_3des_ede_cbc, 
+     {ecdhe_rsa, [], [ecdhe_rsa_3des_ede_cbc,
                       ecdhe_rsa_rc4_128,
                       ecdhe_rsa_aes_128_cbc,
                       ecdhe_rsa_aes_128_gcm,
@@ -141,8 +136,8 @@ groups() ->
                       ecdhe_rsa_chacha20_poly1305
                     ]},
      {ecdhe_1_3_rsa_cert, [], tls_1_3_cipher_suites()},
-     {ecdhe_ecdsa, [],[ecdhe_ecdsa_rc4_128, 
-                       ecdhe_ecdsa_3des_ede_cbc, 
+     {ecdhe_ecdsa, [],[ecdhe_ecdsa_rc4_128,
+                       ecdhe_ecdsa_3des_ede_cbc,
                        ecdhe_ecdsa_aes_128_cbc,
                        ecdhe_ecdsa_aes_128_gcm,
                        ecdhe_ecdsa_aes_256_cbc,
@@ -159,34 +154,34 @@ groups() ->
                 rsa_aes_256_cbc,
                 rsa_rc4_128
                ]},
-     {dhe_dss, [], [dhe_dss_3des_ede_cbc, 
+     {dhe_dss, [], [dhe_dss_3des_ede_cbc,
                     dhe_dss_aes_128_cbc,
                     dhe_dss_aes_256_cbc]},
-     %% {srp_rsa, [], [srp_rsa_3des_ede_cbc, 
+     %% {srp_rsa, [], [srp_rsa_3des_ede_cbc,
      %%                srp_rsa_aes_128_cbc,
      %%                srp_rsa_aes_256_cbc]},
-     %% {srp_dss, [], [srp_dss_3des_ede_cbc, 
+     %% {srp_dss, [], [srp_dss_3des_ede_cbc,
      %%                srp_dss_aes_128_cbc,
      %%                srp_dss_aes_256_cbc]},
-     %% {rsa_psk, [], [rsa_psk_3des_ede_cbc,                    
+     %% {rsa_psk, [], [rsa_psk_3des_ede_cbc,
      %%                rsa_psk_rc4_128,
      %%                rsa_psk_aes_128_cbc,
      %%                rsa_psk_aes_256_cbc
      %%               ]},
      {dh_anon, [], [dh_anon_rc4_128,
-                    dh_anon_3des_ede_cbc, 
+                    dh_anon_3des_ede_cbc,
                     dh_anon_aes_128_cbc,
                     dh_anon_aes_128_gcm,
                     dh_anon_aes_256_cbc,
                     dh_anon_aes_256_gcm]},
-     {ecdh_anon, [], [ecdh_anon_3des_ede_cbc, 
+     {ecdh_anon, [], [ecdh_anon_3des_ede_cbc,
                       ecdh_anon_aes_128_cbc,
                       ecdh_anon_aes_256_cbc
-                     ]}    
-     %% {srp_anon, [], [srp_anon_3des_ede_cbc, 
+                     ]}
+     %% {srp_anon, [], [srp_anon_3des_ede_cbc,
      %%                 srp_anon_aes_128_cbc,
      %%                 srp_anon_aes_256_cbc]},
-     %% {psk, [], [psk_3des_ede_cbc,                    
+     %% {psk, [], [psk_3des_ede_cbc,
      %%            psk_rc4_128,
      %%            psk_aes_128_cbc,
      %%            psk_aes_128_ccm,
@@ -195,7 +190,7 @@ groups() ->
      %%            psk_aes_256_ccm,
      %%            psk_aes_256_ccm_8
      %%           ]},
-     %% {dhe_psk, [], [dhe_psk_3des_ede_cbc,                    
+     %% {dhe_psk, [], [dhe_psk_3des_ede_cbc,
      %%                dhe_psk_rc4_128,
      %%                dhe_psk_aes_128_cbc,
      %%                dhe_psk_aes_128_ccm,
@@ -204,7 +199,7 @@ groups() ->
      %%                dhe_psk_aes_256_ccm,
      %%                dhe_psk_aes_256_ccm_8
      %%           ]},
-     %% {ecdhe_psk, [], [ecdhe_psk_3des_ede_cbc,                    
+     %% {ecdhe_psk, [], [ecdhe_psk_3des_ede_cbc,
      %%                 ecdhe_psk_rc4_128,
      %%                 ecdhe_psk_aes_128_cbc,
      %%                 ecdhe_psk_aes_128_ccm,
@@ -238,13 +233,13 @@ rsa() ->
 
 dtls_rsa() ->
     [
-     {group, rsa} 
+     {group, rsa}
      %%,{group, rsa_psk}
     ].
 
 ecdsa() ->
     [{group, ecdhe_ecdsa}].
-    
+
 dss() ->
     [{group, dhe_dss}
      %%{group, srp_dss}
@@ -260,35 +255,31 @@ anonymous() ->
     ].
 
 init_per_suite(Config) ->
-    catch crypto:stop(),
-    try crypto:start() of
-	ok ->
-	    ssl_test_lib:clean_start(),
-            Config
-    catch _:_ ->
-	    {skip, "Crypto did not start"}
-    end.
+    ssl_test_lib:init_per_suite(Config, openssl).
 
-end_per_suite(_Config) ->
-    ssl:stop(),
-    application:stop(crypto),
-    ssl_test_lib:kill_openssl().
+end_per_suite(Config) ->
+    ssl_test_lib:end_per_suite(Config).
 
 %%--------------------------------------------------------------------
 init_per_group(GroupName, Config) ->
+    case ssl_test_lib:working_openssl_client(Config) of
+        false when GroupName =:= openssl_client ->
+            throw({skip, "Ignore non-working openssl_client"});
+        _ -> ok
+    end,
     case ssl_test_lib:is_protocol_version(GroupName) of
         true ->
             ssl_test_lib:init_per_group_openssl(GroupName, Config);
         false ->
             do_init_per_group(GroupName, Config)
     end.
-  
+
 do_init_per_group(openssl_client, Config0) ->
     Config = proplists:delete(server_type, proplists:delete(client_type, Config0)),
     [{client_type, openssl}, {server_type, erlang} | Config];
 do_init_per_group(openssl_server, Config0) ->
     Config = proplists:delete(server_type, proplists:delete(client_type, Config0)),
-    [{client_type, erlang}, {server_type, openssl} | Config];    
+    [{client_type, erlang}, {server_type, openssl} | Config];
 do_init_per_group(GroupName, Config) when GroupName == ecdh_anon;
                                        GroupName == ecdhe_rsa;
                                        GroupName == ecdhe_psk;
@@ -318,7 +309,7 @@ do_init_per_group(dhe_dss = GroupName, Config) ->
     end;
 do_init_per_group(srp_dss = GroupName, Config) ->
     PKAlg = proplists:get_value(public_keys, crypto:supports()),
-    case lists:member(dss, PKAlg) andalso lists:member(srp, PKAlg) 
+    case lists:member(dss, PKAlg) andalso lists:member(srp, PKAlg)
         andalso (ssl_test_lib:openssl_dsa_suites() =/= []) of
         true ->
             init_certs(GroupName, Config);
@@ -386,9 +377,9 @@ init_per_testcase(TestCase, Config) when TestCase == psk_3des_ede_cbc;
                                          TestCase == ecdh_anon_3des_ede_cbc;
                                          TestCase == ecdhe_ecdsa_3des_ede_cbc ->
     SupCiphers = proplists:get_value(ciphers, crypto:supports()),
-    case lists:member(des_ede3, SupCiphers) of
+    case lists:member(des_ede3_cbc, SupCiphers) of
         true ->
-            ct:timetrap({seconds, ?DEFAULT_TIMEOUT}),
+            ct:timetrap(?DEFAULT_TIMEOUT),
             Config;
         _ ->
             {skip, "Missing 3DES crypto support"}
@@ -525,10 +516,10 @@ init_certs(srp_anon, Config) ->
      proplists:delete(tls_config, Config)];
 init_certs(rsa_psk, Config) ->
     Ext = x509_test:extensions([{key_usage, [digitalSignature, keyEncipherment]}]),
-    {ClientOpts, ServerOpts} = ssl_test_lib:make_rsa_cert_chains([{server_chain, 
+    {ClientOpts, ServerOpts} = ssl_test_lib:make_rsa_cert_chains([{server_chain,
                                                                    [[ssl_test_lib:digest()],[ssl_test_lib:digest()],
                                                                     [ssl_test_lib:digest(), {extensions, Ext}]]},
-                                                                  {client_chain, ssl_test_lib:default_cert_chain_conf()}], 
+                                                                  {client_chain, ssl_test_lib:default_cert_chain_conf()}],
                                                                  Config, "_peer_keyEncipherment"),
     PskSharedSecret = <<1,2,3,4,5,6,7,8,9,10,11,12,13,14,15>>,
     [{tls_config, #{server_config => [{user_lookup_fun, {fun ssl_test_lib:user_lookup/3, PskSharedSecret}} | ServerOpts],
@@ -537,24 +528,24 @@ init_certs(rsa_psk, Config) ->
      proplists:delete(tls_config, Config)];
 init_certs(rsa, Config) ->
     Ext = x509_test:extensions([{key_usage, [digitalSignature, keyEncipherment]}]),
-    {ClientOpts, ServerOpts} = ssl_test_lib:make_rsa_cert_chains([{server_chain, 
+    {ClientOpts, ServerOpts} = ssl_test_lib:make_rsa_cert_chains([{server_chain,
                                                                    [[ssl_test_lib:digest()],[ssl_test_lib:digest()],
-                                                                    [ssl_test_lib:digest(), {extensions, Ext}]]} 
-                                                                 ], 
+                                                                    [ssl_test_lib:digest(), {extensions, Ext}]]}
+                                                                 ],
                                                                  Config, "_peer_keyEncipherment"),
     [{tls_config, #{server_config => ServerOpts,
                     client_config => ClientOpts}} |
      proplists:delete(tls_config, Config)];
 init_certs(dhe_dss, Config) ->
      {ClientOpts, ServerOpts} = ssl_test_lib:make_dsa_cert_chains([{server_chain, ssl_test_lib:default_cert_chain_conf()},
-                                                                  {client_chain, ssl_test_lib:default_cert_chain_conf()}], 
+                                                                  {client_chain, ssl_test_lib:default_cert_chain_conf()}],
                                                                   Config, ""),
     [{tls_config, #{server_config => ServerOpts,
                     client_config => ClientOpts}} |
      proplists:delete(tls_config, Config)];
 init_certs(srp_dss, Config) ->
     {ClientOpts, ServerOpts} = ssl_test_lib:make_dsa_cert_chains([{server_chain, ssl_test_lib:default_cert_chain_conf()},
-                                                                  {client_chain, ssl_test_lib:default_cert_chain_conf()}], 
+                                                                  {client_chain, ssl_test_lib:default_cert_chain_conf()}],
                                                                  Config, ""),
     [{tls_config, #{server_config => [{user_lookup_fun, {fun ssl_test_lib:user_lookup/3, undefined}} | ServerOpts],
                     client_config => [{srp_identity, {"Test-User", "secret"}} | ClientOpts]}} |
@@ -562,7 +553,7 @@ init_certs(srp_dss, Config) ->
 init_certs(GroupName, Config) when GroupName == dhe_rsa;
                                    GroupName == ecdhe_rsa ->
     {ClientOpts, ServerOpts} = ssl_test_lib:make_rsa_cert_chains([{server_chain, ssl_test_lib:default_cert_chain_conf()},
-                                                                  {client_chain, ssl_test_lib:default_cert_chain_conf()}], 
+                                                                  {client_chain, ssl_test_lib:default_cert_chain_conf()}],
                                                                  Config, ""),
     [{tls_config, #{server_config => ServerOpts,
                     client_config => ClientOpts}} |
@@ -579,7 +570,7 @@ init_certs(ecdhe_1_3_rsa_cert, Config) ->
 init_certs(GroupName, Config) when GroupName == dhe_ecdsa;
                                    GroupName == ecdhe_ecdsa ->
     {ClientOpts, ServerOpts} = ssl_test_lib:make_ecc_cert_chains([{server_chain, ssl_test_lib:default_cert_chain_conf()},
-                                                                 {client_chain, ssl_test_lib:default_cert_chain_conf()}], 
+                                                                 {client_chain, ssl_test_lib:default_cert_chain_conf()}],
                                                                  Config, ""),
     [{tls_config, #{server_config => ServerOpts,
                     client_config => ClientOpts}} |
@@ -591,12 +582,12 @@ init_certs(GroupName, Config) when GroupName == psk;
     [{tls_config, #{server_config => [{user_lookup_fun, {fun ssl_test_lib:user_lookup/3, PskSharedSecret}}],
                     client_config => [{psk_identity, "Test-User"},
                                       {user_lookup_fun, {fun ssl_test_lib:user_lookup/3, PskSharedSecret}}]}} |
-     proplists:delete(tls_config, Config)]; 
-init_certs(srp, Config) -> 
+     proplists:delete(tls_config, Config)];
+init_certs(srp, Config) ->
       [{tls_config, #{server_config => [{user_lookup_fun, {fun ssl_test_lib:user_lookup/3, undefined}}],
                       client_config => [{srp_identity, {"Test-User", "secret"}}]}} |
        proplists:delete(tls_config, Config)];
-init_certs(_GroupName, Config) -> 
+init_certs(_GroupName, Config) ->
     %% Anonymous does not need certs
      [{tls_config, #{server_config => [],
                      client_config => []}} |
@@ -622,76 +613,76 @@ aes_128_ccm_8_sha256(Config) when is_list(Config) ->
 %%--------------------------------------------------------------------
 %% SRP --------------------------------------------------------
 %%--------------------------------------------------------------------
-srp_rsa_3des_ede_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(srp_rsa, '3des_ede_cbc', Config).                 
-    
-srp_rsa_aes_128_cbc(Config) when is_list(Config) ->
-   run_ciphers_test(srp_rsa, 'aes_128_cbc', Config).             
+%% srp_rsa_3des_ede_cbc(Config) when is_list(Config) ->
+%%     run_ciphers_test(srp_rsa, '3des_ede_cbc', Config).
 
-srp_rsa_aes_256_cbc(Config) when is_list(Config) ->
-   run_ciphers_test(srp_rsa, 'aes_256_cbc', Config).             
+%% srp_rsa_aes_128_cbc(Config) when is_list(Config) ->
+%%    run_ciphers_test(srp_rsa, 'aes_128_cbc', Config).
 
-srp_dss_3des_ede_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(srp_dss, '3des_ede_cbc', Config).                 
-    
-srp_dss_aes_128_cbc(Config) when is_list(Config) ->
-   run_ciphers_test(srp_dss, 'aes_128_cbc', Config).             
+%% srp_rsa_aes_256_cbc(Config) when is_list(Config) ->
+%%    run_ciphers_test(srp_rsa, 'aes_256_cbc', Config).
 
-srp_dss_aes_256_cbc(Config) when is_list(Config) ->
-   run_ciphers_test(srp_dss, 'aes_256_cbc', Config).     
+%% srp_dss_3des_ede_cbc(Config) when is_list(Config) ->
+%%     run_ciphers_test(srp_dss, '3des_ede_cbc', Config).
+
+%% srp_dss_aes_128_cbc(Config) when is_list(Config) ->
+%%    run_ciphers_test(srp_dss, 'aes_128_cbc', Config).
+
+%% srp_dss_aes_256_cbc(Config) when is_list(Config) ->
+%%    run_ciphers_test(srp_dss, 'aes_256_cbc', Config).
 
 %%--------------------------------------------------------------------
 %% PSK --------------------------------------------------------
 %%--------------------------------------------------------------------
-rsa_psk_3des_ede_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(rsa_psk, '3des_ede_cbc', Config).            
+%% rsa_psk_3des_ede_cbc(Config) when is_list(Config) ->
+%%     run_ciphers_test(rsa_psk, '3des_ede_cbc', Config).
 
-rsa_psk_aes_128_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(rsa_psk, 'aes_128_cbc', Config).             
+%% rsa_psk_aes_128_cbc(Config) when is_list(Config) ->
+%%     run_ciphers_test(rsa_psk, 'aes_128_cbc', Config).
+
+%% rsa_psk_aes_256_cbc(Config) when is_list(Config) ->
+%%     run_ciphers_test(rsa_psk, 'aes_256_cbc', Config).
+
+%% rsa_psk_rc4_128(Config) when is_list(Config) ->
+%%     run_ciphers_test(rsa_psk, 'rc4_128', Config).
 
-rsa_psk_aes_256_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(rsa_psk, 'aes_256_cbc', Config). 
-     
-rsa_psk_rc4_128(Config) when is_list(Config) ->
-    run_ciphers_test(rsa_psk, 'rc4_128', Config).    
-         
 %%--------------------------------------------------------------------
 %% RSA --------------------------------------------------------
 %%--------------------------------------------------------------------
 rsa_des_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(rsa, 'des_cbc', Config).            
+    run_ciphers_test(rsa, 'des_cbc', Config).
 
 rsa_3des_ede_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(rsa, '3des_ede_cbc', Config).            
+    run_ciphers_test(rsa, '3des_ede_cbc', Config).
 
 rsa_aes_128_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(rsa, 'aes_128_cbc', Config).             
+    run_ciphers_test(rsa, 'aes_128_cbc', Config).
 
 rsa_aes_256_cbc(Config) when is_list(Config) ->
     run_ciphers_test(rsa, 'aes_256_cbc', Config).
 
 rsa_aes_128_gcm(Config) when is_list(Config) ->
-    run_ciphers_test(rsa, 'aes_128_gcm', Config).             
+    run_ciphers_test(rsa, 'aes_128_gcm', Config).
 
 rsa_aes_256_gcm(Config) when is_list(Config) ->
     run_ciphers_test(rsa, 'aes_256_gcm', Config).
 
 rsa_rc4_128(Config) when is_list(Config) ->
-    run_ciphers_test(rsa, 'rc4_128', Config).    
+    run_ciphers_test(rsa, 'rc4_128', Config).
 %%--------------------------------------------------------------------
 %% DHE_RSA --------------------------------------------------------
 %%--------------------------------------------------------------------
 dhe_rsa_3des_ede_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_rsa, '3des_ede_cbc', Config).         
+    run_ciphers_test(dhe_rsa, '3des_ede_cbc', Config).
 
 dhe_rsa_aes_128_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_rsa, 'aes_128_cbc', Config).   
+    run_ciphers_test(dhe_rsa, 'aes_128_cbc', Config).
 
 dhe_rsa_aes_128_gcm(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_rsa, 'aes_128_gcm', Config).   
+    run_ciphers_test(dhe_rsa, 'aes_128_gcm', Config).
 
 dhe_rsa_aes_256_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_rsa, 'aes_256_cbc', Config).   
+    run_ciphers_test(dhe_rsa, 'aes_256_cbc', Config).
 
 dhe_rsa_aes_256_gcm(Config) when is_list(Config) ->
     run_ciphers_test(dhe_rsa, 'aes_256_gcm', Config).
@@ -702,22 +693,22 @@ dhe_rsa_chacha20_poly1305(Config) when is_list(Config) ->
 %% ECDHE_RSA --------------------------------------------------------
 %%--------------------------------------------------------------------
 ecdhe_rsa_3des_ede_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_rsa, '3des_ede_cbc', Config).         
+    run_ciphers_test(ecdhe_rsa, '3des_ede_cbc', Config).
 
 ecdhe_rsa_aes_128_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_rsa, 'aes_128_cbc', Config).         
+    run_ciphers_test(ecdhe_rsa, 'aes_128_cbc', Config).
 
 ecdhe_rsa_aes_128_gcm(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_rsa, 'aes_128_gcm', Config).         
+    run_ciphers_test(ecdhe_rsa, 'aes_128_gcm', Config).
 
 ecdhe_rsa_aes_256_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_rsa, 'aes_256_cbc', Config).   
+    run_ciphers_test(ecdhe_rsa, 'aes_256_cbc', Config).
 
 ecdhe_rsa_aes_256_gcm(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_rsa, 'aes_256_gcm', Config).   
+    run_ciphers_test(ecdhe_rsa, 'aes_256_gcm', Config).
 
 ecdhe_rsa_rc4_128(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_rsa, 'rc4_128', Config).      
+    run_ciphers_test(ecdhe_rsa, 'rc4_128', Config).
 
 ecdhe_rsa_chacha20_poly1305(Config) when is_list(Config) ->
     run_ciphers_test(ecdhe_rsa, 'chacha20_poly1305', Config).
@@ -726,54 +717,54 @@ ecdhe_rsa_chacha20_poly1305(Config) when is_list(Config) ->
 %% ECDHE_ECDSA --------------------------------------------------------
 %%--------------------------------------------------------------------
 ecdhe_ecdsa_rc4_128(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_ecdsa, 'rc4_128', Config).         
+    run_ciphers_test(ecdhe_ecdsa, 'rc4_128', Config).
 
 ecdhe_ecdsa_3des_ede_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_ecdsa, '3des_ede_cbc', Config).         
+    run_ciphers_test(ecdhe_ecdsa, '3des_ede_cbc', Config).
 
 ecdhe_ecdsa_aes_128_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_ecdsa, 'aes_128_cbc', Config).         
+    run_ciphers_test(ecdhe_ecdsa, 'aes_128_cbc', Config).
 
 ecdhe_ecdsa_aes_128_gcm(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_ecdsa, 'aes_128_gcm', Config).         
+    run_ciphers_test(ecdhe_ecdsa, 'aes_128_gcm', Config).
 
 ecdhe_ecdsa_aes_256_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_ecdsa, 'aes_256_cbc', Config).   
+    run_ciphers_test(ecdhe_ecdsa, 'aes_256_cbc', Config).
 
 ecdhe_ecdsa_aes_256_gcm(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_ecdsa, 'aes_256_gcm', Config).   
+    run_ciphers_test(ecdhe_ecdsa, 'aes_256_gcm', Config).
 
 ecdhe_ecdsa_chacha20_poly1305(Config) when is_list(Config) ->
     run_ciphers_test(ecdhe_ecdsa, 'chacha20_poly1305', Config).
 
 ecdhe_ecdsa_with_aes_128_ccm(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_ecdsa, 'aes_128_ccm', Config). 
+    run_ciphers_test(ecdhe_ecdsa, 'aes_128_ccm', Config).
 
 ecdhe_ecdsa_with_aes_256_ccm(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_ecdsa, 'aes_256_ccm', Config). 
+    run_ciphers_test(ecdhe_ecdsa, 'aes_256_ccm', Config).
 
 ecdhe_ecdsa_with_aes_128_ccm_8(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_ecdsa, 'aes_128_ccm_8', Config). 
+    run_ciphers_test(ecdhe_ecdsa, 'aes_128_ccm_8', Config).
 
 ecdhe_ecdsa_with_aes_256_ccm_8(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_ecdsa, 'aes_256_ccm_8', Config). 
+    run_ciphers_test(ecdhe_ecdsa, 'aes_256_ccm_8', Config).
 %%--------------------------------------------------------------------
 %% DHE_DSS --------------------------------------------------------
 %%--------------------------------------------------------------------
 dhe_dss_des_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_dss, 'des_cbc', Config).            
+    run_ciphers_test(dhe_dss, 'des_cbc', Config).
 
 dhe_dss_3des_ede_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_dss, '3des_ede_cbc', Config).            
+    run_ciphers_test(dhe_dss, '3des_ede_cbc', Config).
 
 dhe_dss_aes_128_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_dss, 'aes_128_cbc', Config).             
+    run_ciphers_test(dhe_dss, 'aes_128_cbc', Config).
 
 dhe_dss_aes_256_cbc(Config) when is_list(Config) ->
     run_ciphers_test(dhe_dss, 'aes_256_cbc', Config).
 
 dhe_dss_aes_128_gcm(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_dss, 'aes_128_gcm', Config).             
+    run_ciphers_test(dhe_dss, 'aes_128_gcm', Config).
 
 dhe_dss_aes_256_gcm(Config) when is_list(Config) ->
     run_ciphers_test(dhe_dss, 'aes_256_gcm', Config).
@@ -782,144 +773,144 @@ dhe_dss_aes_256_gcm(Config) when is_list(Config) ->
 %% Anonymous --------------------------------------------------------
 %%--------------------------------------------------------------------
 dh_anon_3des_ede_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(dh_anon, '3des_ede_cbc', Config).         
+    run_ciphers_test(dh_anon, '3des_ede_cbc', Config).
 
 dh_anon_aes_128_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(dh_anon, 'aes_128_cbc', Config).         
+    run_ciphers_test(dh_anon, 'aes_128_cbc', Config).
 
 dh_anon_aes_128_gcm(Config) when is_list(Config) ->
-    run_ciphers_test(dh_anon, 'aes_128_gcm', Config).         
+    run_ciphers_test(dh_anon, 'aes_128_gcm', Config).
 
 dh_anon_aes_256_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(dh_anon, 'aes_256_cbc', Config).   
+    run_ciphers_test(dh_anon, 'aes_256_cbc', Config).
 
 dh_anon_aes_256_gcm(Config) when is_list(Config) ->
-    run_ciphers_test(dh_anon, 'aes_256_gcm', Config).   
+    run_ciphers_test(dh_anon, 'aes_256_gcm', Config).
 
 dh_anon_rc4_128(Config) when is_list(Config) ->
-    run_ciphers_test(dh_anon, 'rc4_128', Config).      
+    run_ciphers_test(dh_anon, 'rc4_128', Config).
 
 ecdh_anon_3des_ede_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(ecdh_anon, '3des_ede_cbc', Config).         
+    run_ciphers_test(ecdh_anon, '3des_ede_cbc', Config).
 
 ecdh_anon_aes_128_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(ecdh_anon, 'aes_128_cbc', Config).   
+    run_ciphers_test(ecdh_anon, 'aes_128_cbc', Config).
 
 ecdh_anon_aes_256_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(ecdh_anon, 'aes_256_cbc', Config).   
+    run_ciphers_test(ecdh_anon, 'aes_256_cbc', Config).
 
-srp_anon_3des_ede_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(srp_anon, '3des_ede_cbc', Config).                 
-    
-srp_anon_aes_128_cbc(Config) when is_list(Config) ->
-   run_ciphers_test(srp_anon, 'aes_128_cbc', Config).             
+%% srp_anon_3des_ede_cbc(Config) when is_list(Config) ->
+%%     run_ciphers_test(srp_anon, '3des_ede_cbc', Config).
 
-srp_anon_aes_256_cbc(Config) when is_list(Config) ->
-   run_ciphers_test(srp_anon, 'aes_256_cbc', Config).     
+%% srp_anon_aes_128_cbc(Config) when is_list(Config) ->
+%%    run_ciphers_test(srp_anon, 'aes_128_cbc', Config).
 
-dhe_psk_des_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_psk, 'des_cbc', Config).            
+%% srp_anon_aes_256_cbc(Config) when is_list(Config) ->
+%%    run_ciphers_test(srp_anon, 'aes_256_cbc', Config).
 
-dhe_psk_rc4_128(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_psk, 'rc4_128', Config).            
+%% dhe_psk_des_cbc(Config) when is_list(Config) ->
+%%     run_ciphers_test(dhe_psk, 'des_cbc', Config).
 
-dhe_psk_3des_ede_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_psk, '3des_ede_cbc', Config).            
+%% dhe_psk_rc4_128(Config) when is_list(Config) ->
+%%     run_ciphers_test(dhe_psk, 'rc4_128', Config).
 
-dhe_psk_aes_128_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_psk, 'aes_128_cbc', Config).             
+%% dhe_psk_3des_ede_cbc(Config) when is_list(Config) ->
+%%     run_ciphers_test(dhe_psk, '3des_ede_cbc', Config).
 
-dhe_psk_aes_256_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_psk, 'aes_256_cbc', Config).
+%% dhe_psk_aes_128_cbc(Config) when is_list(Config) ->
+%%     run_ciphers_test(dhe_psk, 'aes_128_cbc', Config).
 
-dhe_psk_aes_128_gcm(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_psk, 'aes_128_gcm', Config).             
+%% dhe_psk_aes_256_cbc(Config) when is_list(Config) ->
+%%     run_ciphers_test(dhe_psk, 'aes_256_cbc', Config).
 
-dhe_psk_aes_256_gcm(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_psk, 'aes_256_gcm', Config).
+%% dhe_psk_aes_128_gcm(Config) when is_list(Config) ->
+%%     run_ciphers_test(dhe_psk, 'aes_128_gcm', Config).
 
-dhe_psk_aes_128_ccm(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_psk, 'aes_128_ccm', Config).             
+%% dhe_psk_aes_256_gcm(Config) when is_list(Config) ->
+%%     run_ciphers_test(dhe_psk, 'aes_256_gcm', Config).
 
-dhe_psk_aes_256_ccm(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_psk, 'aes_256_ccm', Config).
+%% dhe_psk_aes_128_ccm(Config) when is_list(Config) ->
+%%     run_ciphers_test(dhe_psk, 'aes_128_ccm', Config).
 
-dhe_psk_aes_128_ccm_8(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_psk, 'aes_128_ccm_8', Config).
+%% dhe_psk_aes_256_ccm(Config) when is_list(Config) ->
+%%     run_ciphers_test(dhe_psk, 'aes_256_ccm', Config).
 
-dhe_psk_aes_256_ccm_8(Config) when is_list(Config) ->
-    run_ciphers_test(dhe_psk, 'aes_256_ccm_8', Config).
+%% dhe_psk_aes_128_ccm_8(Config) when is_list(Config) ->
+%%     run_ciphers_test(dhe_psk, 'aes_128_ccm_8', Config).
 
-ecdhe_psk_des_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_psk, 'des_cbc', Config).            
+%% dhe_psk_aes_256_ccm_8(Config) when is_list(Config) ->
+%%     run_ciphers_test(dhe_psk, 'aes_256_ccm_8', Config).
 
-ecdhe_psk_rc4_128(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_psk, 'rc4_128', Config).            
+%% ecdhe_psk_des_cbc(Config) when is_list(Config) ->
+%%     run_ciphers_test(ecdhe_psk, 'des_cbc', Config).
 
-ecdhe_psk_3des_ede_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_psk, '3des_ede_cbc', Config).            
+%% ecdhe_psk_rc4_128(Config) when is_list(Config) ->
+%%     run_ciphers_test(ecdhe_psk, 'rc4_128', Config).
 
-ecdhe_psk_aes_128_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_psk, 'aes_128_cbc', Config).             
+%% ecdhe_psk_3des_ede_cbc(Config) when is_list(Config) ->
+%%     run_ciphers_test(ecdhe_psk, '3des_ede_cbc', Config).
 
-ecdhe_psk_aes_256_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_psk, 'aes_256_cbc', Config).
+%% ecdhe_psk_aes_128_cbc(Config) when is_list(Config) ->
+%%     run_ciphers_test(ecdhe_psk, 'aes_128_cbc', Config).
 
-ecdhe_psk_aes_128_gcm(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_psk, 'aes_128_gcm', Config).             
+%% ecdhe_psk_aes_256_cbc(Config) when is_list(Config) ->
+%%     run_ciphers_test(ecdhe_psk, 'aes_256_cbc', Config).
 
-ecdhe_psk_aes_256_gcm(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_psk, 'aes_256_gcm', Config).
+%% ecdhe_psk_aes_128_gcm(Config) when is_list(Config) ->
+%%     run_ciphers_test(ecdhe_psk, 'aes_128_gcm', Config).
 
-ecdhe_psk_aes_128_ccm(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_psk, 'aes_128_ccm', Config).             
+%% ecdhe_psk_aes_256_gcm(Config) when is_list(Config) ->
+%%     run_ciphers_test(ecdhe_psk, 'aes_256_gcm', Config).
 
-ecdhe_psk_aes_128_ccm_8(Config) when is_list(Config) ->
-    run_ciphers_test(ecdhe_psk, 'aes_128_ccm_8', Config).
+%% ecdhe_psk_aes_128_ccm(Config) when is_list(Config) ->
+%%     run_ciphers_test(ecdhe_psk, 'aes_128_ccm', Config).
 
-psk_des_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(psk, 'des_cbc', Config).            
+%% ecdhe_psk_aes_128_ccm_8(Config) when is_list(Config) ->
+%%     run_ciphers_test(ecdhe_psk, 'aes_128_ccm_8', Config).
 
-psk_rc4_128(Config) when is_list(Config) ->
-    run_ciphers_test(psk, 'rc4_128', Config).            
+%% psk_des_cbc(Config) when is_list(Config) ->
+%%     run_ciphers_test(psk, 'des_cbc', Config).
 
-psk_3des_ede_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(psk, '3des_ede_cbc', Config).            
+%% psk_rc4_128(Config) when is_list(Config) ->
+%%     run_ciphers_test(psk, 'rc4_128', Config).
 
-psk_aes_128_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(psk, 'aes_128_cbc', Config).             
+%% psk_3des_ede_cbc(Config) when is_list(Config) ->
+%%     run_ciphers_test(psk, '3des_ede_cbc', Config).
 
-psk_aes_256_cbc(Config) when is_list(Config) ->
-    run_ciphers_test(psk, 'aes_256_cbc', Config).
+%% psk_aes_128_cbc(Config) when is_list(Config) ->
+%%     run_ciphers_test(psk, 'aes_128_cbc', Config).
 
-psk_aes_128_gcm(Config) when is_list(Config) ->
-    run_ciphers_test(psk, 'aes_128_gcm', Config).             
+%% psk_aes_256_cbc(Config) when is_list(Config) ->
+%%     run_ciphers_test(psk, 'aes_256_cbc', Config).
 
-psk_aes_256_gcm(Config) when is_list(Config) ->
-    run_ciphers_test(psk, 'aes_256_gcm', Config).
+%% psk_aes_128_gcm(Config) when is_list(Config) ->
+%%     run_ciphers_test(psk, 'aes_128_gcm', Config).
 
-psk_aes_128_ccm(Config) when is_list(Config) ->
-    run_ciphers_test(psk, 'aes_128_ccm', Config).             
+%% psk_aes_256_gcm(Config) when is_list(Config) ->
+%%     run_ciphers_test(psk, 'aes_256_gcm', Config).
 
-psk_aes_256_ccm(Config) when is_list(Config) ->
-    run_ciphers_test(psk, 'aes_256_ccm', Config).
+%% psk_aes_128_ccm(Config) when is_list(Config) ->
+%%     run_ciphers_test(psk, 'aes_128_ccm', Config).
 
-psk_aes_128_ccm_8(Config) when is_list(Config) ->
-    run_ciphers_test(psk, 'aes_128_ccm_8', Config).             
+%% psk_aes_256_ccm(Config) when is_list(Config) ->
+%%     run_ciphers_test(psk, 'aes_256_ccm', Config).
 
-psk_aes_256_ccm_8(Config) when is_list(Config) ->
-    run_ciphers_test(psk, 'aes_256_ccm_8', Config).
+%% psk_aes_128_ccm_8(Config) when is_list(Config) ->
+%%     run_ciphers_test(psk, 'aes_128_ccm_8', Config).
+
+%% psk_aes_256_ccm_8(Config) when is_list(Config) ->
+%%     run_ciphers_test(psk, 'aes_256_ccm_8', Config).
 
 %%--------------------------------------------------------------------
 %% Internal functions  ----------------------------------------------
 %%--------------------------------------------------------------------
 run_ciphers_test(Kex, Cipher, Config) ->
     Version = ssl_test_lib:protocol_version(Config),
-    TestCiphers = test_ciphers(Kex, Cipher, Version),                  
-    
+    TestCiphers = test_ciphers(Kex, Cipher, Version),
+
     case TestCiphers of
-        [_|_] -> 
-            lists:foreach(fun(TestCipher) -> 
+        [_|_] ->
+            lists:foreach(fun(TestCipher) ->
                                   cipher_suite_test(TestCipher, Version, Config)
                           end, TestCiphers);
         []  ->
@@ -936,15 +927,15 @@ cipher_suite_test(CipherSuite, Version, Config) ->
     ct:log("Client Opts ~p~n", [ClientOpts]),
     case proplists:get_value(server_type, Config) of
         erlang ->
-            ssl_test_lib:basic_test([{ciphers, ssl:cipher_suites(all, Version)} | COpts], 
+            ssl_test_lib:basic_test([{ciphers, ssl:cipher_suites(all, Version)} | COpts],
                                     [{ciphers, [CipherSuite]} | SOpts], Config);
         _ ->
-            ssl_test_lib:basic_test([{versions, [Version]}, {ciphers, [CipherSuite]} | COpts], 
+            ssl_test_lib:basic_test([{versions, [Version]}, {ciphers, [CipherSuite]} | COpts],
                                     [{ciphers,  ssl_test_lib:openssl_ciphers()} | SOpts], Config)
     end.
 
 test_ciphers(Kex, Cipher, Version) ->
-    Ciphers = ssl:filter_cipher_suites(ssl:cipher_suites(all, Version) ++ ssl:cipher_suites(anonymous, Version), 
+    Ciphers = ssl:filter_cipher_suites(ssl:cipher_suites(all, Version) ++ ssl:cipher_suites(anonymous, Version),
                                        [{key_exchange,
                                          fun(Kex0) when (Kex0 == Kex) andalso (Version =/= 'tlsv1.3') -> true;
                                             (Kex0) when (Kex0 == any) andalso (Version == 'tlsv1.3') -> true;
