@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2019-2021. All Rights Reserved.
+%% Copyright Ericsson AB 2019-2022. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -21,8 +21,6 @@
 %%
 -module(ssl_cert_tests).
 
--behaviour(ct_suite).
-
 -include_lib("public_key/include/public_key.hrl").
 
 %% Test cases
@@ -34,12 +32,12 @@
          client_auth_empty_cert_accepted/1,
          client_auth_empty_cert_rejected/0,
          client_auth_empty_cert_rejected/1,
-         client_auth_partial_chain/0,
-         client_auth_partial_chain/1,
-         client_auth_allow_partial_chain/0,
-         client_auth_allow_partial_chain/1,
-         client_auth_do_not_allow_partial_chain/0,
-         client_auth_do_not_allow_partial_chain/1,
+         client_auth_no_suitable_chain/0,
+         client_auth_no_suitable_chain/1,
+         client_auth_use_partial_chain/0,
+         client_auth_use_partial_chain/1,
+         client_auth_do_not_use_partial_chain/0,
+         client_auth_do_not_use_partial_chain/1,
          client_auth_partial_chain_fun_fail/0,
          client_auth_partial_chain_fun_fail/1,
          client_auth_sni/0,
@@ -87,15 +85,21 @@ auth() ->
      [{doc,"Test connection with mutual authentication"}].
 
 auth(Config) ->
-    ClientOpts = [{verify, verify_peer} | ssl_test_lib:ssl_options(extra_client, client_cert_opts, Config)],
+    Version = proplists:get_value(version,Config),
+    ClientOpts =  case Version of
+                      'tlsv1.3' ->
+                          [{verify, verify_peer},
+                           {certificate_authorities, true} |
+                           ssl_test_lib:ssl_options(extra_client, client_cert_opts, Config)];
+                      _ ->
+                          [{verify, verify_peer} | ssl_test_lib:ssl_options(extra_client, client_cert_opts, Config)]
+                  end,
     ServerOpts =  [{verify, verify_peer} | ssl_test_lib:ssl_options(extra_server, server_cert_opts, Config)],
-    
     ssl_test_lib:basic_test(ClientOpts, ServerOpts, Config).
 
 %%--------------------------------------------------------------------
 client_auth_empty_cert_accepted() ->
-    [{doc,"Test client authentication when client sends an empty certificate and " 
-      "fail_if_no_peer_cert is set to false."}].
+    [{doc,"Client sends empty cert chain as no cert is configured and server allows it"}].
 
 client_auth_empty_cert_accepted(Config) ->
     ClientOpts = proplists:delete(keyfile,
@@ -107,8 +111,7 @@ client_auth_empty_cert_accepted(Config) ->
     ssl_test_lib:basic_test(ClientOpts, ServerOpts, Config).
 %%--------------------------------------------------------------------
 client_auth_empty_cert_rejected() ->
-     [{doc,"Test client authentication when client sends an empty certificate and " 
-       "fail_if_no_peer_cert is set to true."}].
+    [{doc,"Client sends empty cert chain as no cert is configured"}].
 
 client_auth_empty_cert_rejected(Config) ->
     ServerOpts = [{verify, verify_peer}, {fail_if_no_peer_cert, true}
@@ -126,30 +129,49 @@ client_auth_empty_cert_rejected(Config) ->
             ssl_test_lib:basic_alert(ClientOpts, ServerOpts, Config, handshake_failure)
     end.
 %%--------------------------------------------------------------------
-client_auth_partial_chain() ->
-    [{doc, "Client sends an incompleate chain, by default not acceptable."}].
+client_auth_no_suitable_chain() ->
+    [{doc, "Client sends an empty cert chain as no suitable chain is found."}].
 
-client_auth_partial_chain(Config) when is_list(Config) ->
+client_auth_no_suitable_chain(Config) when is_list(Config) ->
+    CRoot = public_key:pkix_test_root_cert("OTP other client test ROOT", []),
+    #{client_config := ClientOpts0} = public_key:pkix_test_data(#{server_chain => #{root => [],
+                                                                                    intermediates => [[]],
+                                                                                    peer => []},
+                                                                  client_chain => #{root => CRoot,
+                                                                                    intermediates => [[]],
+                                                                                    peer => []}}),
+    ClientOpts = ssl_test_lib:ssl_options(extra_client, ClientOpts0, Config),
     ServerOpts = [{verify, verify_peer}, {fail_if_no_peer_cert, true}
 		  | ssl_test_lib:ssl_options(extra_server, server_cert_opts, Config)],
-    ClientOpts0 = ssl_test_lib:ssl_options(extra_client, client_cert_opts, Config),
-    {ok, ClientCAs} = file:read_file(proplists:get_value(cacertfile, ClientOpts0)),
-    [{_,RootCA,_} | _] = public_key:pem_decode(ClientCAs),
-    ClientOpts =  [{cacerts, [RootCA]} |
-                   proplists:delete(cacertfile, ClientOpts0)],
-    ssl_test_lib:basic_alert(ClientOpts, ServerOpts, Config, unknown_ca).
-    
+    Version = proplists:get_value(version, Config),
+
+    case Version of
+        'tlsv1.3' ->
+            ssl_test_lib:basic_alert(ClientOpts, ServerOpts, Config, certificate_required);
+        _ ->
+            ssl_test_lib:basic_alert(ClientOpts, ServerOpts, Config, handshake_failure)
+    end.
+
 %%--------------------------------------------------------------------
-client_auth_allow_partial_chain() ->
-    [{doc, "Server trusts intermediat CA and accepts a partial chain. (partial_chain option)"}].
-
-client_auth_allow_partial_chain(Config) when is_list(Config) ->
-    ServerOpts0 = [{verify, verify_peer}, {fail_if_no_peer_cert, true}
-		  | ssl_test_lib:ssl_options(extra_server, server_cert_opts, Config)],
-    ClientOpts = ssl_test_lib:ssl_options(extra_client, client_cert_opts, Config),
-    {ok, ClientCAs} = file:read_file(proplists:get_value(cacertfile, ClientOpts)),
-    [{_,_,_}, {_, IntermidiateCA, _} | _] = public_key:pem_decode(ClientCAs),
+client_auth_use_partial_chain() ->
+    [{doc, "Client trusts intermediat CA and verifies the shorter chain."}].
 
+client_auth_use_partial_chain(Config) when is_list(Config) ->
+    Prop = proplists:get_value(tc_group_properties, Config),
+    DefaultCertConf = ssl_test_lib:default_ecc_cert_chain_conf(proplists:get_value(name, Prop)),
+    {Year, Month, Day} = date(),
+    #{client_config := ClientOpts0,
+      server_config := ServerOpts0} = ssl_test_lib:make_cert_chains_der(proplists:get_value(cert_key_alg, Config),
+                                                                        [{server_chain,
+                                                                          [[{validity, {{Year-2, Month, Day},
+                                                                                        {Year-1, Month, Day}}}],
+                                                                           [],
+                                                                           []
+                                                                          ]},
+                                                                         {client_chain, DefaultCertConf}]),
+    ClientOpts = ssl_test_lib:ssl_options(extra_client, ClientOpts0, Config),
+    ServerOpts = ssl_test_lib:ssl_options(extra_server, ServerOpts0, Config),
+    [_, IntermidiateCA, _] = proplists:get_value(cacerts, ServerOpts),
     PartialChain =  fun(CertChain) ->
 			    case lists:member(IntermidiateCA, CertChain) of
 				true ->
@@ -158,52 +180,58 @@ client_auth_allow_partial_chain(Config) when is_list(Config) ->
 				    unknown_ca
 			    end
 		    end,
-    ServerOpts = [{cacerts, [IntermidiateCA]},
-                  {partial_chain, PartialChain} |
-                  proplists:delete(cacertfile, ServerOpts0)],
-
-    ssl_test_lib:basic_test(ClientOpts, ServerOpts, Config).
+    ssl_test_lib:basic_test([{verify, verify_peer}, {partial_chain, PartialChain} |ClientOpts], ServerOpts, Config).
 
  %%--------------------------------------------------------------------
-client_auth_do_not_allow_partial_chain() ->
-    [{doc, "Server does not accept the chain sent by the client as ROOT CA is unkown, "
-      "and we do not choose to trust the intermediate CA. (partial_chain option)"}].
-
-client_auth_do_not_allow_partial_chain(Config) when is_list(Config) ->
-    ServerOpts0 = [{verify, verify_peer}, {fail_if_no_peer_cert, true}
-		  | ssl_test_lib:ssl_options(extra_server, server_cert_opts, Config)],
-    ClientOpts = ssl_test_lib:ssl_options(extra_client, client_cert_opts, Config),
-    {ok, ServerCAs} = file:read_file(proplists:get_value(cacertfile, ServerOpts0)),
-    [{_,_,_}, {_, IntermidiateCA, _} | _] = public_key:pem_decode(ServerCAs),
+client_auth_do_not_use_partial_chain() ->
+    [{doc, "Client does not trust an intermediat CA and fails the connetion as ROOT has expired"}].
 
+client_auth_do_not_use_partial_chain(Config) when is_list(Config) ->
+    Prop = proplists:get_value(tc_group_properties, Config),
+    DefaultCertConf = ssl_test_lib:default_ecc_cert_chain_conf(proplists:get_value(name, Prop)),
+    {Year, Month, Day} = date(),
+    #{client_config := ClientOpts0,
+      server_config := ServerOpts0} = ssl_test_lib:make_cert_chains_der(proplists:get_value(cert_key_alg, Config),
+                                                                        [{server_chain,
+                                                                          [[{validity, {{Year-2, Month, Day},
+                                                                                        {Year-1, Month, Day}}}],
+                                                                           [],
+                                                                           []
+                                                                          ]},
+                                                                         {client_chain, DefaultCertConf}]),
     PartialChain =  fun(_CertChain) ->
 			    unknown_ca
 		    end,
-    ServerOpts = [{cacerts, [IntermidiateCA]},
-                  {partial_chain, PartialChain} |
-                  proplists:delete(cacertfile, ServerOpts0)],
-    ssl_test_lib:basic_alert(ClientOpts, ServerOpts, Config, unknown_ca).
+    ClientOpts = [{verify, verify_peer}, {partial_chain, PartialChain} | ssl_test_lib:ssl_options(extra_client, ClientOpts0, Config)],
+    ServerOpts = ssl_test_lib:ssl_options(extra_server, ServerOpts0, Config),
+    ssl_test_lib:basic_alert(ClientOpts, ServerOpts, Config, certificate_expired).
     
  %%--------------------------------------------------------------------
 client_auth_partial_chain_fun_fail() ->
-    [{doc, "If parial_chain fun crashes, treat it as if it returned unkown_ca"}].
+    [{doc, "If partial_chain fun crashes, treat it as if it returned unkown_ca"}].
 
 client_auth_partial_chain_fun_fail(Config) when is_list(Config) ->
-    ServerOpts0 = [{verify, verify_peer}, {fail_if_no_peer_cert, true}
-                   | ssl_test_lib:ssl_options(extra_server, server_cert_opts, Config)],
-    ClientOpts = ssl_test_lib:ssl_options(extra_client, client_cert_opts, Config),
+    Prop = proplists:get_value(tc_group_properties, Config),
+    DefaultCertConf = ssl_test_lib:default_ecc_cert_chain_conf(proplists:get_value(name, Prop)),
+    {Year, Month, Day} = date(),
+    #{client_config := ClientOpts0,
+      server_config := ServerOpts0} = ssl_test_lib:make_cert_chains_der(proplists:get_value(cert_key_alg, Config),
+                                                                        [{server_chain,
+                                                                          [[{validity, {{Year-2, Month, Day},
+                                                                                        {Year-1, Month, Day}}}],
+                                                                           [],
+                                                                           []
+                                                                          ]},
+                                                                         {client_chain, DefaultCertConf}]),
 
-    {ok, ServerCAs} = file:read_file(proplists:get_value(cacertfile, ServerOpts0)),
-    [{_,_,_}, {_, IntermidiateCA, _} | _] = public_key:pem_decode(ServerCAs),
 
-    PartialChain =  fun(_CertChain) ->
-                            true = false %% crash on purpose
-		    end,
-    ServerOpts = [{cacerts, [IntermidiateCA]},
-                  {partial_chain, PartialChain} |
-                  proplists:delete(cacertfile, ServerOpts0)],
+    PartialChain = fun(_CertChain) ->
+                           error(crash_on_purpose)
+                   end,
+    ClientOpts = [{verify, verify_peer}, {partial_chain, PartialChain} | ssl_test_lib:ssl_options(extra_client, ClientOpts0, Config)],
+    ServerOpts = [ssl_test_lib:ssl_options(extra_server, ServerOpts0, Config)],
+    ssl_test_lib:basic_alert(ClientOpts, ServerOpts, Config, certificate_expired).
     
-    ssl_test_lib:basic_alert(ClientOpts, ServerOpts, Config, unknown_ca).
 
 %%--------------------------------------------------------------------
 client_auth_sni() ->
@@ -252,7 +280,7 @@ client_auth_seelfsigned_peer(Config) when is_list(Config) ->
                                                                      {key, {'RSAPrivateKey', DerKey}}], Config), Config, bad_certificate).
 %%--------------------------------------------------------------------
 missing_root_cert_no_auth() ->
-     [{doc,"Test that the client succeds if the ROOT CA is unknown in verify_none mode"}].
+     [{doc,"Test that the client succeeds if the ROOT CA is unknown in verify_none mode"}].
 
 missing_root_cert_no_auth(Config) ->
     ClientOpts = [{verify, verify_none} | ssl_test_lib:ssl_options(extra_client, client_cert_opts, Config)],
@@ -281,7 +309,7 @@ invalid_signature_client(Config) when is_list(Config) ->
     NewClientDerCert = public_key:pkix_sign(ClientOTPTbsCert, Key),
     ssl_test_lib:der_to_pem(NewClientCertFile, [{'Certificate', NewClientDerCert, not_encrypted}]),
     ClientOpts = [{certfile, NewClientCertFile} | proplists:delete(certfile, ClientOpts0)],
-    ServerOpts = [{verify, verify_peer} | ServerOpts0],
+    ServerOpts = [{verify, verify_peer}, {fail_if_no_peer_cert, true} | ServerOpts0],
     ssl_test_lib:basic_alert(ClientOpts, ServerOpts, Config, unknown_ca).
 
 %%--------------------------------------------------------------------
