@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2007-2021. All Rights Reserved.
+%% Copyright Ericsson AB 2007-2022. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -35,8 +35,16 @@
 %% Testcases
 -export([basic/0,
          basic/1,
-         basic_anti_replay/0,
-         basic_anti_replay/1,
+         ticketage_smaller_than_windowsize_anti_replay/0,
+         ticketage_smaller_than_windowsize_anti_replay/1,
+         ticketage_bigger_than_windowsize_anti_replay/0,
+         ticketage_bigger_than_windowsize_anti_replay/1,
+         ticketage_out_of_lifetime_anti_replay/0,
+         ticketage_out_of_lifetime_anti_replay/1,
+         ticket_reuse_anti_replay/0,
+         ticket_reuse_anti_replay/1,
+         ticket_reuse_anti_replay_server_restart/0,
+         ticket_reuse_anti_replay_server_restart/1,
          basic_stateful_stateless/0,
          basic_stateful_stateless/1,
          basic_stateless_stateful/0,
@@ -92,7 +100,11 @@ groups() ->
                       {group, stateless},
                       {group, mixed}]},
      {stateful, [], session_tests()},
-     {stateless, [], session_tests() ++ [basic_anti_replay]},
+     {stateless, [], session_tests() ++
+          [ticketage_smaller_than_windowsize_anti_replay,
+           ticketage_bigger_than_windowsize_anti_replay,
+           ticketage_out_of_lifetime_anti_replay, ticket_reuse_anti_replay,
+           ticket_reuse_anti_replay_server_restart]},
      {mixed, [], mixed_tests()}].
 
 session_tests() ->
@@ -167,9 +179,9 @@ basic(Config) when is_list(Config) ->
     ServerTicketMode = proplists:get_value(server_ticket_mode, Config),
 
     %% Configure session tickets
-    ClientOpts = [{session_tickets, auto}, {log_level, debug},
+    ClientOpts = [{session_tickets, auto},
                   {versions, ['tlsv1.2','tlsv1.3']}|ClientOpts0],
-    ServerOpts = [{session_tickets, ServerTicketMode}, {log_level, debug},
+    ServerOpts = [{session_tickets, ServerTicketMode},
                   {versions, ['tlsv1.2','tlsv1.3']}|ServerOpts0],
 
     Server0 =
@@ -212,73 +224,195 @@ basic(Config) when is_list(Config) ->
     ssl_test_lib:close(Server0),
     ssl_test_lib:close(Client1).
 
-basic_anti_replay() ->
-    [{doc,"Test session resumption with stateless session tickets and anti_replay (erlang client - erlang server)"}].
-basic_anti_replay(Config) when is_list(Config) ->
+ticketage_smaller_than_windowsize_anti_replay() ->
+    [{doc, "Session resumption with stateless tickets and anti_replay enabled."
+      "ClientHello with DeltaAge smaller than Bloom filter window size - fresh ClientHello."
+      "DeltaAge treated as particular ClientHello adjective, calculated "
+      "as a difference between RealAge and ReportedAge of a ticket used in ClientHello."
+      "Ticket age smaller than windowsize."
+      "(Erlang client - Erlang server)"}].
+ticketage_smaller_than_windowsize_anti_replay(Config) when is_list(Config) ->
+    {Server0, Client0, Port0, ClientNode, Hostname, ClientOpts} =
+        anti_replay_helper_init(Config, auto, 10),
+    ssl_test_lib:check_result(Server0, ok, Client0, ok),
+    Client1 = anti_replay_helper_connect(Server0, Client0, Port0, ClientNode,
+                                         Hostname, ClientOpts, 0, true),
+    process_flag(trap_exit, false),
+    [ssl_test_lib:close(A) || A <- [Server0, Client0, Client1]].
+
+ticketage_bigger_than_windowsize_anti_replay() ->
+    [{doc, "Session resumption with stateless tickets and anti_replay enabled."
+      "Fresh ClientHellos."
+      "Ticket age bigger than windowsize. 0-RTT is expected to fail."
+      "(Erlang client - Erlang server)"}].
+ticketage_bigger_than_windowsize_anti_replay(Config) when is_list(Config) ->
+    WindowSize = 3,
+    {Server0, Client0, Port0, ClientNode, Hostname, ClientOpts} =
+        anti_replay_helper_init(Config, auto, WindowSize),
+    ssl_test_lib:check_result(Server0, ok, Client0, ok),
+    Client1 = anti_replay_helper_connect(Server0, Client0, Port0, ClientNode,
+                                         Hostname, ClientOpts,
+                                         {seconds, WindowSize + 2}, false),
+    Client2 = anti_replay_helper_connect(Server0, Client0, Port0, ClientNode,
+                                         Hostname, ClientOpts,
+                                         {seconds, 2*WindowSize + 2}, false),
+    process_flag(trap_exit, false),
+    [ssl_test_lib:close(A) || A <- [Server0, Client0, Client1, Client2]].
+
+ticketage_out_of_lifetime_anti_replay() ->
+    [{doc, "Session resumption with stateless tickets and anti_replay enabled."
+      "Fresh ClientHello."
+      "Ticket age beyond its lifetime. 0-RTT is expected to fail."
+      "(Erlang client - Erlang server)"}].
+ticketage_out_of_lifetime_anti_replay(Config) when is_list(Config) ->
+    Lifetime = 4,
+    WindowSize = 2,
+    {Server0, Client0, Port0, ClientNode, Hostname, ClientOpts} =
+        anti_replay_helper_init(Config, auto, WindowSize, Lifetime),
+    ssl_test_lib:check_result(Server0, ok, Client0, ok),
+    Client1 = anti_replay_helper_connect(Server0, Client0, Port0, ClientNode,
+                                         Hostname, ClientOpts,
+                                         {seconds, Lifetime + 2}, false),
+    process_flag(trap_exit, false),
+    [ssl_test_lib:close(A) || A <- [Server0, Client0, Client1]].
+
+ticket_reuse_anti_replay() ->
+    [{doc, "Verify that 2 connection attempts with same stateless tickets "
+      "are successful."
+      "Fresh ClientHellos."
+      "Ticket age smaller than windowsize."
+      "Anti_replay allows it because both "
+      "ClientHellos are unique, look fresh(small enough DeltaAge) "
+      "and do not look like a replay attempt."
+      "(Erlang client - Erlang server)"}].
+ticket_reuse_anti_replay(Config) when is_list(Config) ->
+    {Server0, Client0, Port0, ClientNode, Hostname, ClientOpts0} =
+        anti_replay_helper_init(Config, manual, 10),
+    [Ticket] = ssl_test_lib:check_tickets(Client0),
+    ssl_test_lib:check_result(Server0, ok),
+    ClientOpts1 = [{use_ticket, [Ticket]} | ClientOpts0],
+    Client1 = anti_replay_helper_connect(Server0, Client0, Port0, ClientNode,
+                                         Hostname, ClientOpts1, 0, true),
+    Client2 = anti_replay_helper_connect(Server0, Client1, Port0, ClientNode,
+                                         Hostname, ClientOpts1, 0, true),
+    process_flag(trap_exit, false),
+    [ssl_test_lib:close(A) || A <- [Server0, Client0, Client2]].
+
+ticket_reuse_anti_replay_server_restart() ->
+    [{doc, "Verify 2 connection attempts with same stateless tickets "
+      "and server restart between. Second attempt is expected to fail as long as "
+      "Bloom filter window overlaps with startup time."
+      "Fresh ClientHellos."
+      "(Erlang client - Erlang server)"
+
+      "RFC8446 8.2: When implementations are freshly started, they SHOULD reject "
+      "0-RTT as long as any portion of their recording window overlaps the startup "
+      "time. Otherwise, they run the risk of accepting replays which were "
+      "originally sent during that period."
+     }].
+ticket_reuse_anti_replay_server_restart(Config) when is_list(Config) ->
+    WindowSize = 10,
+    {Server0, Client0, Port0, ClientNode, Hostname, ClientOpts0} =
+        anti_replay_helper_init(Config, manual, WindowSize),
+    [Ticket] = ssl_test_lib:check_tickets(Client0),
+    ssl_test_lib:check_result(Server0, ok),
+    ClientOpts1 = [{use_ticket, [Ticket]} | ClientOpts0],
+    Client1 = anti_replay_helper_connect(Server0, Client0, Port0, ClientNode,
+                                         Hostname, ClientOpts1, 0, true),
+    {Server1, Port1} = anti_replay_helper_start_server(Config, WindowSize),
+    Client2 = anti_replay_helper_connect(Server1, Client1, Port1, ClientNode,
+                                         Hostname, ClientOpts1, 0, false, false),
+    process_flag(trap_exit, false),
+    [ssl_test_lib:close(A) || A <- [Server0, Client2, Server1]].
+
+anti_replay_helper_init(Config, Mode, WindowSize) ->
+    DefaultLifetime = ssl_config:get_ticket_lifetime(),
+    anti_replay_helper_init(Config, Mode, WindowSize, DefaultLifetime).
+
+anti_replay_helper_init(Config, Mode, WindowSize, Lifetime) ->
+    application:set_env(ssl, server_session_ticket_lifetime, Lifetime),
     ClientOpts0 = ssl_test_lib:ssl_options(client_rsa_verify_opts, Config),
-    ServerOpts0 = ssl_test_lib:ssl_options(server_rsa_verify_opts, Config),
-    {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
-    ServerTicketMode = proplists:get_value(server_ticket_mode, Config),
-
+    {ClientNode, _ServerNode, Hostname} = ssl_test_lib:run_where(Config),
     %% Configure session tickets
-    ClientOpts = [{session_tickets, auto}, {log_level, debug},
-                  {versions, ['tlsv1.2','tlsv1.3']}|ClientOpts0],
-    ServerOpts = [{session_tickets, ServerTicketMode}, {log_level, debug},
-                  {anti_replay, '10k'},
+    ClientOpts = [{session_tickets, Mode},
+                  {versions, ['tlsv1.2','tlsv1.3']} | ClientOpts0],
+
+    {Server0 , Port0} = anti_replay_helper_start_server(Config, WindowSize),
+    MFA = case Mode of
+              auto ->
+                  {ssl_test_lib,
+                   verify_active_session_resumption,
+                   [false]};
+              manual ->
+                  {ssl_test_lib,
+                   verify_active_session_resumption,
+                   [false, wait_reply, {tickets, 1}]}
+          end,
+
+    %% Store ticket from first connection
+    Client0 = ssl_test_lib:start_client([{node, ClientNode},
+                                         {port, Port0}, {host, Hostname},
+                                         {mfa, MFA},
+                                         {from, self()}, {options, ClientOpts}]),
+    {Server0, Client0, Port0, ClientNode, Hostname, ClientOpts}.
+
+anti_replay_helper_start_server(Config, WindowSize) ->
+    {_ClientNode, ServerNode, _Hostname} = ssl_test_lib:run_where(Config),
+    ServerOpts0 = ssl_test_lib:ssl_options(server_rsa_verify_opts, Config),
+    ServerTicketMode = proplists:get_value(server_ticket_mode, Config),
+    ServerOpts = [{session_tickets, ServerTicketMode},
+                  {anti_replay, {WindowSize, 5, 72985}},
                   {versions, ['tlsv1.2','tlsv1.3']}|ServerOpts0],
 
-    Server0 =
+    Server =
 	ssl_test_lib:start_server([{node, ServerNode}, {port, 0},
 				   {from, self()},
 				   {mfa, {ssl_test_lib,
                                           verify_active_session_resumption,
                                           [false]}},
 				   {options, ServerOpts}]),
-    Port0 = ssl_test_lib:inet_port(Server0),
+    Port0 = ssl_test_lib:inet_port(Server),
+    {Server, Port0}.
 
-    %% Store ticket from first connection
-    Client0 = ssl_test_lib:start_client([{node, ClientNode},
-                                         {port, Port0}, {host, Hostname},
-                                         {mfa, {ssl_test_lib,  %% Full handshake
-                                                verify_active_session_resumption,
-                                                [false]}},
-                                         {from, self()}, {options, ClientOpts}]),
-    ssl_test_lib:check_result(Server0, ok, Client0, ok),
+anti_replay_helper_connect(Server, Client0, Port0, ClientNode, Hostname,
+                           ClientOpts, Delay, ExpectedResumption) ->
+    anti_replay_helper_connect(Server, Client0, Port0, ClientNode, Hostname,
+                               ClientOpts, Delay, ExpectedResumption, true).
 
-    Server0 ! {listen, {mfa, {ssl_test_lib,
-                              verify_active_session_resumption,
-                              [true]}}},
+anti_replay_helper_connect(Server, Client0, Port0, ClientNode, Hostname,
+                           ClientOpts, Delay, ExpectedResumption, SendListen) ->
+    case SendListen of
+        true ->
+            Server ! {listen, {mfa, {ssl_test_lib,
+                                     verify_active_session_resumption,
+                                     [ExpectedResumption]}}};
+        _ ->
+            ok
+    end,
 
     %% Wait for session ticket
     ct:sleep(100),
-
     ssl_test_lib:close(Client0),
-
+    ct:sleep(Delay),
     %% Use ticket
     Client1 = ssl_test_lib:start_client([{node, ClientNode},
                                          {port, Port0}, {host, Hostname},
                                          {mfa, {ssl_test_lib,  %% Short handshake
                                                 verify_active_session_resumption,
-                                                [true]}},
+                                                [ExpectedResumption]}},
                                          {from, self()}, {options, ClientOpts}]),
-    ssl_test_lib:check_result(Server0, ok, Client1, ok),
-
-    process_flag(trap_exit, false),
-    ssl_test_lib:close(Server0),
-    ssl_test_lib:close(Client1).
+    ssl_test_lib:check_result(Server, ok, Client1, ok),
+    Client1.
 
 basic_stateful_stateless() ->
     [{doc,"Test session resumption with session tickets (erlang client - erlang server)"}].
 basic_stateful_stateless(Config) when is_list(Config) ->
     do_test_mixed(Config,
                   [{session_tickets, auto},
-                   {log_level, debug},
                    {versions, ['tlsv1.2','tlsv1.3']}],
                   [{session_tickets, stateful},
-                   {log_level, debug},
                    {versions, ['tlsv1.2','tlsv1.3']}],
                   [{session_tickets, stateless},
-                   {log_level, debug},
                    {versions, ['tlsv1.2','tlsv1.3']}]).
 
 basic_stateless_stateful() ->
@@ -286,13 +420,10 @@ basic_stateless_stateful() ->
 basic_stateless_stateful(Config) when is_list(Config) ->
     do_test_mixed(Config,
                   [{session_tickets, auto},
-                   {log_level, debug},
                    {versions, ['tlsv1.2','tlsv1.3']}],
                   [{session_tickets, stateless},
-                   {log_level, debug},
                    {versions, ['tlsv1.2','tlsv1.3']}],
                   [{session_tickets, stateful},
-                   {log_level, debug},
                    {versions, ['tlsv1.2','tlsv1.3']}]).
 
 basic_stateful_stateless_anti_replay() ->
@@ -300,13 +431,10 @@ basic_stateful_stateless_anti_replay() ->
 basic_stateful_stateless_anti_replay(Config) when is_list(Config) ->
     do_test_mixed(Config,
                   [{session_tickets, auto},
-                   {log_level, debug},
                    {versions, ['tlsv1.2','tlsv1.3']}],
                   [{session_tickets, stateful},
-                   {log_level, debug},
                    {versions, ['tlsv1.2','tlsv1.3']}],
                   [{session_tickets, stateless},
-                   {log_level, debug},
                    {anti_replay, '10k'},
                    {versions, ['tlsv1.2','tlsv1.3']}]).
 
@@ -315,14 +443,11 @@ basic_stateless_stateful_anti_replay() ->
 basic_stateless_stateful_anti_replay(Config) when is_list(Config) ->
     do_test_mixed(Config,
                   [{session_tickets, auto},
-                   {log_level, debug},
                    {versions, ['tlsv1.2','tlsv1.3']}],
                   [{session_tickets, stateless},
-                   {log_level, debug},
                    {anti_replay, '10k'},
                    {versions, ['tlsv1.2','tlsv1.3']}],
                   [{session_tickets, stateful},
-                   {log_level, debug},
                    {versions, ['tlsv1.2','tlsv1.3']}]).
 
 basic_stateful_stateless_faulty_ticket() ->
@@ -330,19 +455,15 @@ basic_stateful_stateless_faulty_ticket() ->
 basic_stateful_stateless_faulty_ticket(Config) when is_list(Config) ->
     do_test_mixed(Config,
                   [{session_tickets, auto},
-                   {log_level, debug},
                    {versions, ['tlsv1.2','tlsv1.3']}],
                   [{session_tickets, manual},
                    {use_ticket, [<<131,100,0,12,"faultyticket">>,
                                  <<"faulty ticket">>]},
-                   {log_level, debug},
                    {versions, ['tlsv1.2','tlsv1.3']}],
                   [{session_tickets, stateless},
-                   {log_level, debug},
                    {anti_replay, '10k'},
                    {versions, ['tlsv1.2','tlsv1.3']}],
                   [{session_tickets, stateful},
-                   {log_level, debug},
                    {versions, ['tlsv1.2','tlsv1.3']}]).
 
 basic_stateless_stateful_faulty_ticket() ->
@@ -350,19 +471,15 @@ basic_stateless_stateful_faulty_ticket() ->
 basic_stateless_stateful_faulty_ticket(Config) when is_list(Config) ->
     do_test_mixed(Config,
                   [{session_tickets, auto},
-                   {log_level, debug},
                    {versions, ['tlsv1.2','tlsv1.3']}],
                   [{session_tickets, manual},
                    {use_ticket, [<<"faulty ticket">>,
                                  <<131,100,0,12,"faultyticket">>]},
-                   {log_level, debug},
                    {versions, ['tlsv1.2','tlsv1.3']}],
                   [{session_tickets, stateless},
-                   {log_level, debug},
                    {anti_replay, '10k'},
                    {versions, ['tlsv1.2','tlsv1.3']}],
                   [{session_tickets, stateful},
-                   {log_level, debug},
                    {versions, ['tlsv1.2','tlsv1.3']}]).
 
 hello_retry_request() ->
@@ -374,10 +491,10 @@ hello_retry_request(Config) when is_list(Config) ->
     ServerTicketMode = proplists:get_value(server_ticket_mode, Config),
     
     %% Configure session tickets
-    ClientOpts = [{session_tickets, auto}, {log_level, debug},
+    ClientOpts = [{session_tickets, auto},
                   {versions, ['tlsv1.2','tlsv1.3']},
                   {supported_groups,[secp256r1, x25519]}|ClientOpts0],
-    ServerOpts = [{session_tickets, ServerTicketMode}, {log_level, debug},
+    ServerOpts = [{session_tickets, ServerTicketMode},
                   {versions, ['tlsv1.2','tlsv1.3']},
                   {supported_groups, [x448, x25519]}|ServerOpts0],
 
@@ -430,9 +547,9 @@ multiple_tickets(Config) when is_list(Config) ->
     ServerTicketMode = proplists:get_value(server_ticket_mode, Config),
 
     %% Configure session tickets
-    ClientOpts = [{session_tickets, manual}, {log_level, debug},
+    ClientOpts = [{session_tickets, manual},
                   {versions, ['tlsv1.2','tlsv1.3']}|ClientOpts0],
-    ServerOpts = [{session_tickets, ServerTicketMode}, {log_level, debug},
+    ServerOpts = [{session_tickets, ServerTicketMode},
                   {versions, ['tlsv1.2','tlsv1.3']}|ServerOpts0],
 
     Server0 =
@@ -488,9 +605,9 @@ multiple_tickets_2hash(Config) when is_list(Config) ->
     ServerTicketMode = proplists:get_value(server_ticket_mode, Config),
     
     %% Configure session tickets
-    ClientOpts = [{session_tickets, manual}, {log_level, debug},
+    ClientOpts = [{session_tickets, manual},
                   {versions, ['tlsv1.2','tlsv1.3']}|ClientOpts0],
-    ServerOpts = [{session_tickets, ServerTicketMode}, {log_level, debug},
+    ServerOpts = [{session_tickets, ServerTicketMode},
                   {versions, ['tlsv1.2','tlsv1.3']}|ServerOpts0],
 
     Server0 =
@@ -617,7 +734,7 @@ early_data_trial_decryption(Config) when is_list(Config) ->
     ServerTicketMode = proplists:get_value(server_ticket_mode, Config),
 
     %% Configure session tickets
-    ClientOpts1 = [{session_tickets, auto}, {log_level, debug},
+    ClientOpts1 = [{session_tickets, auto},
                   {versions, ['tlsv1.2','tlsv1.3']}|ClientOpts0],
     %% Send maximum sized early data to verify calculation of plain text size
     %% in the server.
@@ -625,7 +742,6 @@ early_data_trial_decryption(Config) when is_list(Config) ->
 
     %% Disabled early data triggers trial decryption upon receiving early data
     ServerOpts = [{session_tickets, ServerTicketMode}, {early_data, disabled},
-                  {log_level, debug},
                   {versions, ['tlsv1.2','tlsv1.3']}|ServerOpts0],
 
     Server0 =
@@ -677,7 +793,7 @@ early_data_client_too_much_data(Config) when is_list(Config) ->
     ServerTicketMode = proplists:get_value(server_ticket_mode, Config),
 
     %% Configure session tickets
-    ClientOpts1 = [{session_tickets, manual}, {log_level, debug},
+    ClientOpts1 = [{session_tickets, manual},
                   {versions, ['tlsv1.2','tlsv1.3']}|ClientOpts0],
     %% Send more early data than max_early_data_size to verify calculation
     %% of plain text size in the server.
@@ -685,7 +801,6 @@ early_data_client_too_much_data(Config) when is_list(Config) ->
     ClientOpts2 = [{early_data, binary:copy(<<"F">>, 16384)}|ClientOpts1],
 
     ServerOpts = [{session_tickets, ServerTicketMode}, {early_data, disabled},
-                  {log_level, debug},
                   {versions, ['tlsv1.2','tlsv1.3']}|ServerOpts0],
 
     application:set_env(ssl, server_session_ticket_max_early_data, MaxEarlyDataSize),
@@ -739,7 +854,7 @@ early_data_trial_decryption_failure(Config) when is_list(Config) ->
     ServerTicketMode = proplists:get_value(server_ticket_mode, Config),
 
     %% Configure session tickets
-    ClientOpts1 = [{session_tickets, manual}, {log_level, debug},
+    ClientOpts1 = [{session_tickets, manual},
                   {versions, ['tlsv1.2','tlsv1.3']}|ClientOpts0],
     %% Send more early data than max_early_data_size to verify calculation
     %% of plain text size in the server.
@@ -756,7 +871,6 @@ early_data_trial_decryption_failure(Config) when is_list(Config) ->
     %%    in the ticket used for the 0-RTT handshake. If more data is sent the
     %%    client will trigger an illegal_parameter alert (too_much_early_data).
     ServerOpts = [{session_tickets, ServerTicketMode}, {early_data, disabled},
-                  {log_level, debug},
                   {versions, ['tlsv1.2','tlsv1.3']}|ServerOpts0],
 
     application:set_env(ssl, server_session_ticket_max_early_data, MaxEarlyDataSize),
@@ -792,7 +906,7 @@ early_data_trial_decryption_failure(Config) when is_list(Config) ->
     ssl_test_lib:close(Client0),
 
     %% Use ticket
-    Client1 = ssl_test_lib:start_client_error([{node, ClientNode},
+    _Client1 = ssl_test_lib:start_client_error([{node, ClientNode},
                                                {port, Port0}, {host, Hostname},
                                                {mfa, {ssl_test_lib,  %% Short handshake
                                                       verify_active_session_resumption,
@@ -812,7 +926,7 @@ early_data_decryption_failure(Config) when is_list(Config) ->
     ServerTicketMode = proplists:get_value(server_ticket_mode, Config),
 
     %% Configure session tickets
-    ClientOpts1 = [{session_tickets, manual}, {log_level, debug},
+    ClientOpts1 = [{session_tickets, manual},
                   {versions, ['tlsv1.2','tlsv1.3']}|ClientOpts0],
     %% Send more early data than max_early_data_size to verify calculation
     %% of plain text size in the server.
@@ -820,7 +934,6 @@ early_data_decryption_failure(Config) when is_list(Config) ->
     ClientOpts2 = [{early_data, binary:copy(<<"F">>, 16385)}|ClientOpts1],
 
     ServerOpts = [{session_tickets, ServerTicketMode}, {early_data, enabled},
-                  {log_level, debug},
                   {versions, ['tlsv1.2','tlsv1.3']}|ServerOpts0],
 
     application:set_env(ssl, server_session_ticket_max_early_data, MaxEarlyDataSize),
@@ -856,7 +969,7 @@ early_data_decryption_failure(Config) when is_list(Config) ->
     ssl_test_lib:close(Client0),
 
     %% Use ticket
-    Client1 = ssl_test_lib:start_client_error([{node, ClientNode},
+    _Client1 = ssl_test_lib:start_client_error([{node, ClientNode},
                                                {port, Port0}, {host, Hostname},
                                                {mfa, {ssl_test_lib,  %% Short handshake
                                                       verify_active_session_resumption,
@@ -876,7 +989,7 @@ early_data_disabled_small_limit(Config) when is_list(Config) ->
     ServerTicketMode = proplists:get_value(server_ticket_mode, Config),
 
     %% Configure session tickets
-    ClientOpts1 = [{session_tickets, auto}, {log_level, debug},
+    ClientOpts1 = [{session_tickets, auto},
                   {versions, ['tlsv1.2','tlsv1.3']}|ClientOpts0],
     %% Send maximum sized early data to verify calculation of plain text size
     %% in the server.
@@ -885,7 +998,6 @@ early_data_disabled_small_limit(Config) when is_list(Config) ->
 
     %% Disabled early data triggers trial decryption upon receiving early data
     ServerOpts = [{session_tickets, ServerTicketMode}, {early_data, disabled},
-                  {log_level, debug},
                   {versions, ['tlsv1.2','tlsv1.3']}|ServerOpts0],
     application:set_env(ssl, server_session_ticket_max_early_data, MaxEarlyDataSize),
     Server0 =
@@ -938,7 +1050,7 @@ early_data_enabled_small_limit(Config) when is_list(Config) ->
     ServerTicketMode = proplists:get_value(server_ticket_mode, Config),
 
     %% Configure session tickets
-    ClientOpts1 = [{session_tickets, auto}, {log_level, debug},
+    ClientOpts1 = [{session_tickets, auto},
                   {versions, ['tlsv1.2','tlsv1.3']}|ClientOpts0],
     %% Send maximum sized early data to verify calculation of plain text size
     %% in the server.
@@ -947,7 +1059,6 @@ early_data_enabled_small_limit(Config) when is_list(Config) ->
 
     %% Disabled early data triggers trial decryption upon receiving early data
     ServerOpts = [{session_tickets, ServerTicketMode}, {early_data, enabled},
-                  {log_level, debug},
                   {versions, ['tlsv1.2','tlsv1.3']}|ServerOpts0],
     application:set_env(ssl, server_session_ticket_max_early_data, MaxEarlyDataSize),
     Server0 =
@@ -1000,7 +1111,7 @@ early_data_basic(Config) when is_list(Config) ->
     ServerTicketMode = proplists:get_value(server_ticket_mode, Config),
 
     %% Configure session tickets
-    ClientOpts1 = [{session_tickets, auto}, {log_level, debug},
+    ClientOpts1 = [{session_tickets, auto},
                   {versions, ['tlsv1.2','tlsv1.3']}|ClientOpts0],
     %% Send maximum sized early data to verify calculation of plain text size
     %% in the server.
@@ -1009,7 +1120,6 @@ early_data_basic(Config) when is_list(Config) ->
 
     %% Disabled early data triggers trial decryption upon receiving early data
     ServerOpts = [{session_tickets, ServerTicketMode}, {early_data, enabled},
-                  {log_level, debug},
                   {versions, ['tlsv1.2','tlsv1.3']}|ServerOpts0],
 
     Server0 =
@@ -1061,7 +1171,7 @@ early_data_basic_auth(Config) when is_list(Config) ->
     ServerTicketMode = proplists:get_value(server_ticket_mode, Config),
 
     %% Configure session tickets
-    ClientOpts1 = [{session_tickets, auto}, {log_level, debug},
+    ClientOpts1 = [{session_tickets, auto},
                   {versions, ['tlsv1.2','tlsv1.3']}|ClientOpts0],
     %% Send maximum sized early data to verify calculation of plain text size
     %% in the server.
@@ -1070,7 +1180,6 @@ early_data_basic_auth(Config) when is_list(Config) ->
 
     %% Disabled early data triggers trial decryption upon receiving early data
     ServerOpts = [{session_tickets, ServerTicketMode}, {early_data, enabled},
-                  {log_level, debug},
                   {versions, ['tlsv1.2','tlsv1.3']}|ServerOpts0],
 
     Server0 =
