@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1997-2021. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2022. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -58,7 +58,16 @@ init_per_suite(Config) ->
     Config.
 
 end_per_suite(_Config) ->
-    ok.
+    %% Cleanup after strip and strip_add_chunks
+    case code:is_sticky(sofs) of
+        false ->
+            false = code:purge(sofs),
+            {module, sofs} = code:load_file(sofs),
+            code:stick_mod(sofs),
+            ok;
+        true ->
+            ok
+    end.
 
 init_per_group(_GroupName, Config) ->
     Config.
@@ -85,7 +94,6 @@ normal(Conf) when is_list(Conf) ->
     P0 = pps(),
 
     do_normal(Source, PrivDir, BeamFile, []),
-    do_normal(Source, PrivDir, BeamFile, [no_utf8_atoms]),
 
     {ok,_} = compile:file(Source, [{outdir,PrivDir}, no_debug_info]),
     {ok, {simple, [{debug_info, {debug_info_v1, erl_abstract_code, {none, _}}}]}} =
@@ -384,22 +392,29 @@ strip(Conf) when is_list(Conf) ->
     {ok, [{simple,_},{simple2,_},{make_fun,_},{constant,_}]} =
 	beam_lib:strip_files([BeamFileD1, BeamFile2D1, BeamFile3D1, BeamFile4D1]),
 
+    %% strip a complex module
+    OrigSofsPath = code:where_is_file("sofs.beam"),
+    BeamFileSofs = filename:join(PrivDir,"sofs.beam"),
+    file:copy(OrigSofsPath, BeamFileSofs),
+    {ok, {sofs,_}} = beam_lib:strip(BeamFileSofs),
+    code:unstick_mod(sofs),
+    false = code:purge(sofs),
+
     %% check that each module can be loaded.
     {module, simple} = code:load_abs(filename:rootname(BeamFileD1)),
     {module, simple2} = code:load_abs(filename:rootname(BeamFile2D1)),
     {module, make_fun} = code:load_abs(filename:rootname(BeamFile3D1)),
     {module, constant} = code:load_abs(filename:rootname(BeamFile4D1)),
+    {module, sofs} = code:load_abs(filename:rootname(BeamFileSofs)),
 
     %% check that line number information is still present after stripping
     {module, lines} = code:load_abs(filename:rootname(BeamFile5D1)),
-    {'EXIT',{badarith,[{lines,t,1,Info}|_]}} =
-	(catch lines:t(atom)),
+    Info = get_line_number_info(),
     true = code:delete(lines),
     false = code:purge(lines),
     {ok, {lines,BeamFile5D1}} = beam_lib:strip(BeamFile5D1),
     {module, lines} = code:load_abs(filename:rootname(BeamFile5D1)),
-    {'EXIT',{badarith,[{lines,t,1,Info}|_]}} =
-	(catch lines:t(atom)),
+    Info = get_line_number_info(),
 
     true = (P0 == pps()),
     NoOfTables = erlang:system_info(ets_count),
@@ -408,7 +423,12 @@ strip(Conf) when is_list(Conf) ->
 		  Source2D1, BeamFile2D1,
 		  Source3D1, BeamFile3D1,
 		  Source4D1, BeamFile4D1,
-		  Source5D1, BeamFile5D1]),
+		  Source5D1, BeamFile5D1,
+                  BeamFileSofs]),
+
+    false = code:purge(sofs),
+    {module, sofs} = code:load_file(sofs),
+    code:stick_mod(sofs),
     ok.
 
 strip_add_chunks(Conf) when is_list(Conf) ->
@@ -433,7 +453,7 @@ strip_add_chunks(Conf) when is_list(Conf) ->
     compare_chunks(B1, NB1, NBId1),
 
     %% Keep all the extra chunks
-    ExtraChunks = ["Abst" , "Dbgi" , "Attr" , "CInf" , "LocT" , "Atom" ],
+    ExtraChunks = ["Abst", "Dbgi", "Attr", "CInf", "LocT", "Atom"],
     {ok, {simple, AB1}} = beam_lib:strip(B1, ExtraChunks),
     ABId1 = chunk_ids(AB1),
     true = length(BId1) == length(ABId1),
@@ -450,20 +470,29 @@ strip_add_chunks(Conf) when is_list(Conf) ->
     {ok, [{simple,_},{simple2,_},{make_fun,_},{constant,_}]} =
 	beam_lib:strip_files([BeamFileD1, BeamFile2D1, BeamFile3D1, BeamFile4D1], ExtraChunks),
 
+    %% strip a complex module
+    OrigSofsPath = code:where_is_file("sofs.beam"),
+    BeamFileSofs = filename:join(PrivDir,"sofs.beam"),
+    file:copy(OrigSofsPath, BeamFileSofs),
+    {ok, {sofs,_}} = beam_lib:strip(BeamFileSofs, ExtraChunks),
+    code:unstick_mod(sofs),
+    false = code:purge(sofs),
+
     %% check that each module can be loaded.
     {module, simple} = code:load_abs(filename:rootname(BeamFileD1)),
     {module, simple2} = code:load_abs(filename:rootname(BeamFile2D1)),
     {module, make_fun} = code:load_abs(filename:rootname(BeamFile3D1)),
     {module, constant} = code:load_abs(filename:rootname(BeamFile4D1)),
+    {module, sofs} = code:load_abs(filename:rootname(BeamFileSofs)),
 
     %% check that line number information is still present after stripping
     {module, lines} = code:load_abs(filename:rootname(BeamFile5D1)),
-    {'EXIT',{badarith,[{lines,t,1,Info}|_]}} = (catch lines:t(atom)),
+    Info = get_line_number_info(),
     false = code:purge(lines),
     true = code:delete(lines),
     {ok, {lines,BeamFile5D1}} = beam_lib:strip(BeamFile5D1),
     {module, lines} = code:load_abs(filename:rootname(BeamFile5D1)),
-    {'EXIT',{badarith,[{lines,t,1,Info}|_]}} = (catch lines:t(atom)),
+    Info = get_line_number_info(),
 
     true = (P0 == pps()),
     NoOfTables = erlang:system_info(ets_count),
@@ -472,7 +501,13 @@ strip_add_chunks(Conf) when is_list(Conf) ->
 		  Source2D1, BeamFile2D1,
 		  Source3D1, BeamFile3D1,
 		  Source4D1, BeamFile4D1,
-		  Source5D1, BeamFile5D1]),
+		  Source5D1, BeamFile5D1,
+                  BeamFileSofs]),
+
+    false = code:purge(sofs),
+    {module, sofs} = code:load_file(sofs),
+    code:stick_mod(sofs),
+
     ok.
 
 otp_6711(Conf) when is_list(Conf) ->
@@ -964,3 +999,13 @@ run_if_crypto_works(Test) ->
 	    {skip,"The crypto application is missing or broken"}
     end.
 
+get_line_number_info() ->
+    %% The stacktrace for operators such a '+' can vary depending on
+    %% whether the JIT is used or not.
+    case catch lines:t(atom) of
+        {'EXIT',{badarith,[{erlang,'+',[atom,1],_},
+                           {lines,t,1,Info}|_]}} ->
+            Info;
+        {'EXIT',{badarith,[{lines,t,1,Info}|_]}} ->
+            Info
+    end.
