@@ -26,14 +26,19 @@
 
 -export([all/0, suite/0,groups/0,init_per_suite/1, end_per_suite/1, 
 	 init_per_group/2,end_per_group/2]).
--export([start/1, crash/1, call/1, send_request/1, cast/1, cast_fast/1,
+-export([start/1, crash/1, call/1, send_request/1,
+         send_request_receive_reqid_collection/1,
+         send_request_wait_reqid_collection/1,
+         send_request_check_reqid_collection/1,
+         cast/1, cast_fast/1,
 	 continue/1, info/1, abcast/1, multicall/1, multicall_down/1,
-	 call_remote1/1, call_remote2/1, call_remote3/1,
+	 call_remote1/1, call_remote2/1, call_remote3/1, calling_self/1,
 	 call_remote_n1/1, call_remote_n2/1, call_remote_n3/1, spec_init/1,
 	 spec_init_local_registered_parent/1, 
 	 spec_init_global_registered_parent/1,
 	 otp_5854/1, hibernate/1, auto_hibernate/1, otp_7669/1, call_format_status/1,
-	 error_format_status/1, terminate_crash_format/1,
+	 error_format_status/1, terminate_crash_format/1, crash_in_format_status/1,
+         throw_in_format_status/1, format_all_status/1,
 	 get_state/1, replace_state/1, call_with_huge_message_queue/1,
 	 undef_handle_call/1, undef_handle_cast/1, undef_handle_info/1,
 	 undef_init/1, undef_code_change/1, undef_terminate1/1,
@@ -64,14 +69,16 @@ suite() ->
      {timetrap,{minutes,1}}].
 
 all() -> 
-    [start, {group,stop}, crash, call, send_request, cast, cast_fast, info, abcast,
-     continue, multicall, multicall_down, call_remote1, call_remote2,
+    [start, {group,stop}, crash, call, send_request,
+     send_request_receive_reqid_collection, send_request_wait_reqid_collection,
+     send_request_check_reqid_collection, cast, cast_fast, info, abcast,
+     continue, multicall, multicall_down, call_remote1, call_remote2, calling_self,
      call_remote3, call_remote_n1, call_remote_n2,
      call_remote_n3, spec_init,
      spec_init_local_registered_parent,
      spec_init_global_registered_parent, otp_5854, hibernate, auto_hibernate,
      otp_7669,
-     call_format_status, error_format_status, terminate_crash_format,
+     {group, format_status},
      get_state, replace_state,
      call_with_huge_message_queue, {group, undef_callbacks},
      undef_in_terminate, undef_in_handle_info,
@@ -80,12 +87,18 @@ all() ->
 groups() -> 
     [{stop, [],
       [stop1, stop2, stop3, stop4, stop5, stop6, stop7, stop8, stop9, stop10]},
+     {format_status, [],
+      [call_format_status, error_format_status, terminate_crash_format,
+       crash_in_format_status, throw_in_format_status, format_all_status]},
      {undef_callbacks, [],
       [undef_handle_call, undef_handle_cast, undef_handle_info, undef_handle_continue,
        undef_init, undef_code_change, undef_terminate1, undef_terminate2]}].
 
 
 init_per_suite(Config) ->
+    DataDir = ?config(data_dir, Config),
+    Server = filename:join(DataDir, "format_status_server.erl"),
+    {ok, format_status_server} = compile:file(Server),
     Config.
 
 end_per_suite(_Config) ->
@@ -110,18 +123,19 @@ init_per_testcase(Case, Config) when Case == call_remote1;
 				     Case == call_remote_n2;
 				     Case == call_remote_n3;
                                      Case == send_request ->
-    {ok,N} = start_node(hubba),
-    [{node,N} | Config];
+    {ok,Peer,Node} = ?CT_PEER(),
+    ok = global:sync(),
+    [{node,Node}, {peer, Peer} | Config];
 
 init_per_testcase(_Case, Config) ->
     Config.
 
 end_per_testcase(_Case, Config) ->
-    case proplists:get_value(node, Config) of
+    case proplists:get_value(peer, Config) of
 	undefined ->
 	    ok;
-	N ->
-	    test_server:stop_node(N)
+	Peer ->
+	    peer:stop(Peer)
     end,
     ok.
 
@@ -365,20 +379,20 @@ stop7(_Config) ->
 
 %% Anonymous on remote node
 stop8(_Config) ->
-    {ok,Node} = test_server:start_node(gen_server_SUITE_stop8,slave,[]),
+    {ok,Peer,Node} = ?CT_PEER(),
     Dir = filename:dirname(code:which(?MODULE)),
     rpc:call(Node,code,add_path,[Dir]),
     {ok, Pid} = rpc:call(Node,gen_server,start,[?MODULE,[],[]]),
     ok = gen_server:stop(Pid),
     false = rpc:call(Node,erlang,is_process_alive,[Pid]),
     {'EXIT',noproc} = (catch gen_server:stop(Pid)),
-    true = test_server:stop_node(Node),
+    peer:stop(Peer),
     {'EXIT',{{nodedown,Node},_}} = (catch gen_server:stop(Pid)),
     ok.
 
 %% Registered name on remote node
 stop9(_Config) ->
-    {ok,Node} = test_server:start_node(gen_server_SUITE_stop9,slave,[]),
+    {ok,Peer,Node} = ?CT_PEER(),
     Dir = filename:dirname(code:which(?MODULE)),
     rpc:call(Node,code,add_path,[Dir]),
     {ok, Pid} = rpc:call(Node,gen_server,start,[{local,to_stop},?MODULE,[],[]]),
@@ -386,13 +400,13 @@ stop9(_Config) ->
     undefined = rpc:call(Node,erlang,whereis,[to_stop]),
     false = rpc:call(Node,erlang,is_process_alive,[Pid]),
     {'EXIT',noproc} = (catch gen_server:stop({to_stop,Node})),
-    true = test_server:stop_node(Node),
+    peer:stop(Peer),
     {'EXIT',{{nodedown,Node},_}} = (catch gen_server:stop({to_stop,Node})),
     ok.
 
 %% Globally registered name on remote node
 stop10(_Config) ->
-    {ok,Node} = test_server:start_node(gen_server_SUITE_stop10,slave,[]),
+    {ok,Peer,Node} = ?CT_PEER(),
     Dir = filename:dirname(code:which(?MODULE)),
     rpc:call(Node,code,add_path,[Dir]),
     {ok, Pid} = rpc:call(Node,gen_server,start,[{global,to_stop},?MODULE,[],[]]),
@@ -400,7 +414,7 @@ stop10(_Config) ->
     ok = gen_server:stop({global,to_stop}),
     false = rpc:call(Node,erlang,is_process_alive,[Pid]),
     {'EXIT',noproc} = (catch gen_server:stop({global,to_stop})),
-    true = test_server:stop_node(Node),
+    peer:stop(Peer),
     {'EXIT',noproc} = (catch gen_server:stop({global,to_stop})),
     ok.
 
@@ -590,6 +604,323 @@ send_request(Config) when is_list(Config) ->
     process_flag(trap_exit, OldFl),
     ok.
 
+send_request_receive_reqid_collection(Config) when is_list(Config) ->
+    {ok, Pid1} = gen_server:start_link({local, my_test_name1},
+                                       gen_server_SUITE, [], []),
+    {ok, Pid2} = gen_server:start_link({local, my_test_name2},
+                                       gen_server_SUITE, [], []),
+    {ok, Pid3} = gen_server:start_link({local, my_test_name3},
+                                       gen_server_SUITE, [], []),
+    send_request_receive_reqid_collection(Pid1, Pid2, Pid3),
+    send_request_receive_reqid_collection_timeout(Pid1, Pid2, Pid3),
+    send_request_receive_reqid_collection_error(Pid1, Pid2, Pid3),
+    unlink(Pid1),
+    exit(Pid1, kill),
+    unlink(Pid2),
+    exit(Pid2, kill),
+    unlink(Pid3),
+    exit(Pid3, kill),
+    false = is_process_alive(Pid1),
+    false = is_process_alive(Pid2),
+    false = is_process_alive(Pid3),
+    ok.
+
+send_request_receive_reqid_collection(Pid1, Pid2, Pid3) ->
+
+    ReqId0 = gen_server:send_request(Pid1, started_p),
+
+    ReqIdC0 = gen_server:reqids_new(),
+
+    ReqId1 = gen_server:send_request(Pid1, {delayed_answer,400}),
+    ReqIdC1 = gen_server:reqids_add(ReqId1, req1, ReqIdC0),
+    1 = gen_server:reqids_size(ReqIdC1),
+
+    ReqIdC2 = gen_server:send_request(Pid2, {delayed_answer,1}, req2, ReqIdC1),
+    2 = gen_server:reqids_size(ReqIdC2),
+
+    ReqIdC3 = gen_server:send_request(Pid3, {delayed_answer,200}, req3, ReqIdC2),
+    3 = gen_server:reqids_size(ReqIdC3),
+    
+    {{reply, delayed}, req2, ReqIdC4} = gen_server:receive_response(ReqIdC3, infinity, true),
+    2 = gen_server:reqids_size(ReqIdC4),
+
+    {{reply, delayed}, req3, ReqIdC5} = gen_server:receive_response(ReqIdC4, 5678, true),
+    1 = gen_server:reqids_size(ReqIdC5),
+
+    {{reply, delayed}, req1, ReqIdC6} = gen_server:receive_response(ReqIdC5, 5000, true),
+    0 = gen_server:reqids_size(ReqIdC6),
+
+    no_request = gen_server:receive_response(ReqIdC6, 5000, true),
+
+    {reply, ok} = gen_server:receive_response(ReqId0, infinity),
+
+    ok.
+
+send_request_receive_reqid_collection_timeout(Pid1, Pid2, Pid3) ->
+
+    ReqId0 = gen_server:send_request(Pid1, started_p),
+
+    ReqIdC0 = gen_server:reqids_new(),
+
+    ReqId1 = gen_server:send_request(Pid1, {delayed_answer,1000}),
+    ReqIdC1 = gen_server:reqids_add(ReqId1, req1, ReqIdC0),
+
+    ReqIdC2 = gen_server:send_request(Pid2, {delayed_answer,1}, req2, ReqIdC1),
+
+    ReqId3 = gen_server:send_request(Pid3, {delayed_answer,500}),
+    ReqIdC3 = gen_server:reqids_add(ReqId3, req3, ReqIdC2),
+
+    Deadline = erlang:monotonic_time(millisecond) + 100,
+    
+    {{reply, delayed}, req2, ReqIdC4} = gen_server:receive_response(ReqIdC3, {abs, Deadline}, true),
+    2 = gen_server:reqids_size(ReqIdC4),
+
+    timeout = gen_server:receive_response(ReqIdC4, {abs, Deadline}, true),
+
+    Abandoned = lists:sort([{ReqId1, req1}, {ReqId3, req3}]),
+    Abandoned = lists:sort(gen_server:reqids_to_list(ReqIdC4)),
+
+    %% Make sure requests were abandoned...
+    timeout = gen_server:receive_response(ReqIdC4, {abs, Deadline+1000}, true),
+
+    {reply, ok} = gen_server:receive_response(ReqId0, infinity),
+
+    ok.
+    
+send_request_receive_reqid_collection_error(Pid1, Pid2, Pid3) ->
+
+    ReqId0 = gen_server:send_request(Pid1, started_p),
+
+    ReqIdC0 = gen_server:reqids_new(),
+
+    ReqId1 = gen_server:send_request(Pid1, {delayed_answer,400}),
+    ReqIdC1 = gen_server:reqids_add(ReqId1, req1, ReqIdC0),
+    try
+        nope = gen_server:reqids_add(ReqId1, req2, ReqIdC1)
+    catch
+        error:badarg -> ok
+    end,
+ 
+    unlink(Pid2),
+    ReqIdC2 = gen_server:send_request(Pid2, stop_shutdown, req2, ReqIdC1),
+    ReqIdC3 = gen_server:send_request(Pid3, {delayed_answer,200}, req3, ReqIdC2),
+    3 = gen_server:reqids_size(ReqIdC3),
+    
+    {{error, {shutdown, _}}, req2, ReqIdC4} = gen_server:receive_response(ReqIdC3, 2000, true),
+    2 = gen_server:reqids_size(ReqIdC4),
+
+    {{reply, delayed}, req3, ReqIdC4} = gen_server:receive_response(ReqIdC4, infinity, false),
+
+    {{reply, delayed}, req1, ReqIdC4} = gen_server:receive_response(ReqIdC4, infinity, false),
+
+    {reply, ok} = gen_server:receive_response(ReqId0, infinity),
+
+    ok.
+
+send_request_wait_reqid_collection(Config) when is_list(Config) ->
+    {ok, Pid1} = gen_server:start_link({local, my_test_name1},
+                                       gen_server_SUITE, [], []),
+    {ok, Pid2} = gen_server:start_link({local, my_test_name2},
+                                       gen_server_SUITE, [], []),
+    {ok, Pid3} = gen_server:start_link({local, my_test_name3},
+                                       gen_server_SUITE, [], []),
+    send_request_wait_reqid_collection(Pid1, Pid2, Pid3),
+    send_request_wait_reqid_collection_timeout(Pid1, Pid2, Pid3),
+    send_request_wait_reqid_collection_error(Pid1, Pid2, Pid3),
+    unlink(Pid1),
+    exit(Pid1, kill),
+    unlink(Pid2),
+    exit(Pid2, kill),
+    unlink(Pid3),
+    exit(Pid3, kill),
+    false = is_process_alive(Pid1),
+    false = is_process_alive(Pid2),
+    false = is_process_alive(Pid3),
+    ok.
+
+send_request_wait_reqid_collection(Pid1, Pid2, Pid3) ->
+
+    ReqId0 = gen_server:send_request(Pid1, started_p),
+
+    ReqIdC0 = gen_server:reqids_new(),
+
+    ReqId1 = gen_server:send_request(Pid1, {delayed_answer,400}),
+    ReqIdC1 = gen_server:reqids_add(ReqId1, req1, ReqIdC0),
+    1 = gen_server:reqids_size(ReqIdC1),
+
+    ReqIdC2 = gen_server:send_request(Pid2, {delayed_answer,1}, req2, ReqIdC1),
+    2 = gen_server:reqids_size(ReqIdC2),
+
+    ReqIdC3 = gen_server:send_request(Pid3, {delayed_answer,200}, req3, ReqIdC2),
+    3 = gen_server:reqids_size(ReqIdC3),
+    
+    {{reply, delayed}, req2, ReqIdC4} = gen_server:wait_response(ReqIdC3, infinity, true),
+    2 = gen_server:reqids_size(ReqIdC4),
+
+    {{reply, delayed}, req3, ReqIdC5} = gen_server:wait_response(ReqIdC4, 5678, true),
+    1 = gen_server:reqids_size(ReqIdC5),
+
+    {{reply, delayed}, req1, ReqIdC6} = gen_server:wait_response(ReqIdC5, 5000, true),
+    0 = gen_server:reqids_size(ReqIdC6),
+
+    no_request = gen_server:wait_response(ReqIdC6, 5000, true),
+
+    {reply, ok} = gen_server:wait_response(ReqId0, infinity),
+
+    ok.
+
+send_request_wait_reqid_collection_timeout(Pid1, Pid2, Pid3) ->
+
+    ReqId0 = gen_server:send_request(Pid1, started_p),
+
+    ReqIdC0 = gen_server:reqids_new(),
+
+    ReqId1 = gen_server:send_request(Pid1, {delayed_answer,1000}),
+    ReqIdC1 = gen_server:reqids_add(ReqId1, req1, ReqIdC0),
+
+    ReqIdC2 = gen_server:send_request(Pid2, {delayed_answer,1}, req2, ReqIdC1),
+
+    ReqId3 = gen_server:send_request(Pid3, {delayed_answer,500}),
+    ReqIdC3 = gen_server:reqids_add(ReqId3, req3, ReqIdC2),
+
+    Deadline = erlang:monotonic_time(millisecond) + 100,
+    
+    {{reply, delayed}, req2, ReqIdC4} = gen_server:wait_response(ReqIdC3, {abs, Deadline}, true),
+    2 = gen_server:reqids_size(ReqIdC4),
+
+    timeout = gen_server:wait_response(ReqIdC4, {abs, Deadline}, true),
+
+    Unhandled = lists:sort([{ReqId1, req1}, {ReqId3, req3}]),
+    Unhandled = lists:sort(gen_server:reqids_to_list(ReqIdC4)),
+
+    %% Make sure requests were not abandoned...
+    {{reply, delayed}, req3, ReqIdC4} = gen_server:wait_response(ReqIdC4, {abs, Deadline+1500}, false),
+    {{reply, delayed}, req1, ReqIdC4} = gen_server:wait_response(ReqIdC4, {abs, Deadline+1500}, false),
+
+    {reply, ok} = gen_server:receive_response(ReqId0, infinity),
+
+    ok.
+    
+send_request_wait_reqid_collection_error(Pid1, Pid2, Pid3) ->
+
+    ReqId0 = gen_server:send_request(Pid1, started_p),
+
+    ReqIdC0 = gen_server:reqids_new(),
+
+    ReqId1 = gen_server:send_request(Pid1, {delayed_answer,400}),
+    ReqIdC1 = gen_server:reqids_add(ReqId1, req1, ReqIdC0),
+    try
+        nope = gen_server:reqids_add(ReqId1, req2, ReqIdC1)
+    catch
+        error:badarg -> ok
+    end,
+ 
+    unlink(Pid2),
+    ReqIdC2 = gen_server:send_request(Pid2, stop_shutdown, req2, ReqIdC1),
+    ReqIdC3 = gen_server:send_request(Pid3, {delayed_answer,200}, req3, ReqIdC2),
+    3 = gen_server:reqids_size(ReqIdC3),
+    
+    {{error, {shutdown, _}}, req2, ReqIdC4} = gen_server:wait_response(ReqIdC3, 2000, true),
+    2 = gen_server:reqids_size(ReqIdC4),
+
+    {{reply, delayed}, req3, ReqIdC4} = gen_server:wait_response(ReqIdC4, infinity, false),
+
+    {{reply, delayed}, req1, ReqIdC4} = gen_server:wait_response(ReqIdC4, infinity, false),
+
+    {reply, ok} = gen_server:wait_response(ReqId0, infinity),
+
+    ok.
+
+send_request_check_reqid_collection(Config) when is_list(Config) ->
+    {ok, Pid1} = gen_server:start_link({local, my_test_name1},
+                                       gen_server_SUITE, [], []),
+    {ok, Pid2} = gen_server:start_link({local, my_test_name2},
+                                       gen_server_SUITE, [], []),
+    {ok, Pid3} = gen_server:start_link({local, my_test_name3},
+                                       gen_server_SUITE, [], []),
+    send_request_check_reqid_collection(Pid1, Pid2, Pid3),
+    send_request_check_reqid_collection_error(Pid1, Pid2, Pid3),
+    unlink(Pid1),
+    exit(Pid1, kill),
+    unlink(Pid2),
+    exit(Pid2, kill),
+    unlink(Pid3),
+    exit(Pid3, kill),
+    false = is_process_alive(Pid1),
+    false = is_process_alive(Pid2),
+    false = is_process_alive(Pid3),
+    ok.
+
+send_request_check_reqid_collection(Pid1, Pid2, Pid3) ->
+
+    ReqId0 = gen_server:send_request(Pid1, started_p),
+
+    receive after 100 -> ok end,
+
+    ReqIdC0 = gen_server:reqids_new(),
+
+    ReqIdC1 = gen_server:send_request(Pid1, {delayed_answer,400}, req1, ReqIdC0),
+    1 = gen_server:reqids_size(ReqIdC1),
+
+    ReqId2 = gen_server:send_request(Pid2, {delayed_answer,1}),
+    ReqIdC2 = gen_server:reqids_add(ReqId2, req2, ReqIdC1),
+    2 = gen_server:reqids_size(ReqIdC2),
+
+    ReqIdC3 = gen_server:send_request(Pid3, {delayed_answer,200}, req3, ReqIdC2),
+    3 = gen_server:reqids_size(ReqIdC3),
+
+    Msg0 = next_msg(),
+    no_reply = gen_server:check_response(Msg0, ReqIdC3, true),
+    
+    {{reply, delayed}, req2, ReqIdC4} = gen_server:check_response(next_msg(), ReqIdC3, true),
+    2 = gen_server:reqids_size(ReqIdC4),
+
+    {{reply, delayed}, req3, ReqIdC5} = gen_server:check_response(next_msg(), ReqIdC4, true),
+    1 = gen_server:reqids_size(ReqIdC5),
+
+    {{reply, delayed}, req1, ReqIdC6} = gen_server:check_response(next_msg(), ReqIdC5, true),
+    0 = gen_server:reqids_size(ReqIdC6),
+
+    no_request = gen_server:check_response(Msg0, ReqIdC6, true),
+
+    {reply, ok} = gen_server:check_response(Msg0, ReqId0),
+
+    ok.
+    
+send_request_check_reqid_collection_error(Pid1, Pid2, Pid3) ->
+
+    ReqId0 = gen_server:send_request(Pid1, started_p),
+
+    receive after 100 -> ok end,
+
+    ReqIdC0 = gen_server:reqids_new(),
+
+    ReqId1 = gen_server:send_request(Pid1, {delayed_answer,400}),
+    ReqIdC1 = gen_server:reqids_add(ReqId1, req1, ReqIdC0),
+
+    unlink(Pid2),
+    ReqIdC2 = gen_server:send_request(Pid2, stop_shutdown, req2, ReqIdC1),
+
+    ReqIdC3 = gen_server:send_request(Pid3, {delayed_answer,200}, req3, ReqIdC2),
+    3 = gen_server:reqids_size(ReqIdC3),
+
+    Msg0 = next_msg(),
+
+    no_reply = gen_server:check_response(Msg0, ReqIdC3, true),
+    
+    {{error, {shutdown, _}}, req2, ReqIdC4} = gen_server:check_response(next_msg(), ReqIdC3, true),
+    2 = gen_server:reqids_size(ReqIdC4),
+
+    {{reply, delayed}, req3, ReqIdC4} = gen_server:check_response(next_msg(), ReqIdC4, false),
+
+    {{reply, delayed}, req1, ReqIdC4} = gen_server:check_response(next_msg(), ReqIdC4, false),
+
+    {reply, ok} = gen_server:check_response(Msg0, ReqId0),
+
+    ok.
+
+next_msg() ->
+    receive M -> M end.
 
 %% --------------------------------------
 %% Test handle_continue.
@@ -636,15 +967,6 @@ read_replies() ->
 %% Test call to nonexisting processes on remote nodes
 %% --------------------------------------
 
-start_node(Name) ->
-    Pa = filename:dirname(code:which(?MODULE)),
-    N = test_server:start_node(Name, slave, [{args, " -pa " ++ Pa}]),
-    %% After starting a slave, it takes a little while until global knows
-    %% about it, even if nodes() includes it, so we make sure that global
-    %% knows about it before registering something on all nodes.
-    ok = global:sync(),
-    N.
-
 call_remote1(Config) when is_list(Config) ->
     N = hubba,
     Node = proplists:get_value(node,Config),
@@ -691,7 +1013,7 @@ call_remote_n1(Config) when is_list(Config) ->
     Node = proplists:get_value(node,Config),
     {ok, _Pid} = rpc:call(Node, gen_server, start,
 			  [{global, N}, ?MODULE, [], []]),
-    _ = test_server:stop_node(Node),
+    peer:stop(proplists:get_value(peer,Config)),
     {'EXIT', {noproc, _}} =
 	(catch gen_server:call({global, N}, started_p, infinity)),
 
@@ -703,7 +1025,7 @@ call_remote_n2(Config) when is_list(Config) ->
 
     {ok, Pid} = rpc:call(Node, gen_server, start,
 			 [{global, N}, ?MODULE, [], []]),
-    _ = test_server:stop_node(Node),
+    peer:stop(proplists:get_value(peer,Config)),
     {'EXIT', {{nodedown, Node}, _}} = (catch gen_server:call(Pid,
 							     started_p, infinity)),
 
@@ -714,12 +1036,21 @@ call_remote_n3(Config) when is_list(Config) ->
 
     {ok, _Pid} = rpc:call(Node, gen_server, start,
 			  [{local, piller}, ?MODULE, [], []]),
-    _ = test_server:stop_node(Node),
+    peer:stop(proplists:get_value(peer,Config)),
     {'EXIT', {{nodedown, Node}, _}} = (catch gen_server:call({piller, Node},
 							     started_p, infinity)),
 
     ok.
 
+%% --------------------------------------
+%% Other bad calls
+%% --------------------------------------
+
+calling_self(Config) when is_list(Config) ->
+    {'EXIT', {calling_self, _}} = (catch gen_server:call(self(), oops)),
+    {'EXIT', {calling_self, _}} = (catch gen_server:call(self(), oops, infinity)),
+    ok.
+
 %% --------------------------------------
 %% Test gen_server:cast and handle_cast.
 %% Test all different return values from
@@ -760,7 +1091,11 @@ cast(Config) when is_list(Config) ->
 
 %% Test that cast really return immediately.
 cast_fast(Config) when is_list(Config) ->
-    {ok,Node} = start_node(hubba),
+    {ok,Peer,Node} = ?CT_PEER(),
+    %% After starting a slave, it takes a little while until global knows
+    %% about it, even if nodes() includes it, so we make sure that global
+    %% knows about it before registering something on all nodes.
+    ok = global:sync(),
     {_,"@"++Host} = lists:splitwith(fun ($@) -> false; (_) -> true end,
 				    atom_to_list(Node)),
     FalseNode = list_to_atom("hopp@"++Host),
@@ -770,7 +1105,7 @@ cast_fast(Config) when is_list(Config) ->
     {Time,ok} = timer:tc(fun() ->
 				 gen_server:cast({hopp,FalseNode}, hopp)
 			 end),
-    true = test_server:stop_node(Node),
+    peer:stop(Peer),
     if Time > 1000000 ->       % Default listen timeout is about 7.0 s
 	    ct:fail(hanging_cast);
        true ->
@@ -1330,48 +1665,65 @@ do_otp_7669_stop() ->
 				       ?MODULE, stop, []),
     undefined = global:whereis_name(?MODULE).
 
-%% Verify that sys:get_status correctly calls our format_status/2 fun.
+%% Verify that sys:get_status correctly calls our format_status/1,2 fun.
 call_format_status(Config) when is_list(Config) ->
+    OldFl = process_flag(trap_exit, true),
+    call_format_status(?MODULE, format_status_called),
+    call_format_status(format_status_server,{data,[{"State",format_status_called}]}),
+    process_flag(trap_exit, OldFl).
+call_format_status(Module, Match) when is_atom(Module) ->
+
     Parent = self(),
 
     {ok, Pid} = gen_server:start_link({local, call_format_status},
-				      ?MODULE, [], []),
+				      Module, [], []),
     Status1 = sys:get_status(call_format_status),
     {status, Pid, Mod, [_Pdict1, running, Parent, _, Data1]} = Status1,
-    [format_status_called | _] = lists:reverse(Data1),
+    [Match | _] = lists:reverse(Data1),
     Status2 = sys:get_status(call_format_status, 5000),
     {status, Pid, Mod, [_Pdict2, running, Parent, _, Data2]} = Status2,
-    [format_status_called | _] = lists:reverse(Data2),
+    [Match | _] = lists:reverse(Data2),
+    gen_server:call(Pid, stop),
+    receive {'EXIT',_,_} -> ok end,
 
     %% check that format_status can handle a name being a pid (atom is
     %% already checked by the previous test)
-    {ok, Pid3} = gen_server:start_link(gen_server_SUITE, [], []),
+    {ok, Pid3} = gen_server:start_link(Module, [], []),
     Status3 = sys:get_status(Pid3),
     {status, Pid3, Mod, [_PDict3, running, Parent, _, Data3]} = Status3,
-    [format_status_called | _] = lists:reverse(Data3),
+    [Match | _] = lists:reverse(Data3),
+    gen_server:call(Pid3, stop),
+    receive {'EXIT',_,_} -> ok end,
 
     %% check that format_status can handle a name being a term other than a
     %% pid or atom
     GlobalName1 = {global, "CallFormatStatus"},
-    {ok, Pid4} = gen_server:start_link(GlobalName1,
-				       gen_server_SUITE, [], []),
+    {ok, Pid4} = gen_server:start_link(GlobalName1, Module, [], []),
     Status4 = sys:get_status(Pid4),
     {status, Pid4, Mod, [_PDict4, running, Parent, _, Data4]} = Status4,
-    [format_status_called | _] = lists:reverse(Data4),
+    [Match | _] = lists:reverse(Data4),
+    gen_server:call(Pid4, stop),
+    receive {'EXIT',_,_} -> ok end,
     GlobalName2 = {global, {name, "term"}},
-    {ok, Pid5} = gen_server:start_link(GlobalName2,
-				       gen_server_SUITE, [], []),
+    {ok, Pid5} = gen_server:start_link(GlobalName2, Module, [], []),
     Status5 = sys:get_status(GlobalName2),
     {status, Pid5, Mod, [_PDict5, running, Parent, _, Data5]} = Status5,
-    [format_status_called | _] = lists:reverse(Data5),
+    [Match | _] = lists:reverse(Data5),
+    gen_server:call(Pid5, stop),
+    receive {'EXIT',_,_} -> ok end,
     ok.
 
-%% Verify that error termination correctly calls our format_status/2 fun.
+%% Verify that error termination correctly calls our format_status/1,2 fun.
 error_format_status(Config) when is_list(Config) ->
     error_logger_forwarder:register(),
     OldFl = process_flag(trap_exit, true),
+    error_format_status(?MODULE),
+    error_format_status(format_status_server),
+    process_flag(trap_exit, OldFl);
+error_format_status(Module) when is_atom(Module) ->
+
     State = "called format_status",
-    {ok, Pid} = gen_server:start_link(?MODULE, {state, State}, []),
+    {ok, Pid} = gen_server:start_link(Module, {state, State}, []),
     {'EXIT',{crashed,_}} = (catch gen_server:call(Pid, crash)),
     receive
 	{'EXIT', Pid, crashed} ->
@@ -1387,19 +1739,24 @@ error_format_status(Config) when is_list(Config) ->
 			       ClientPid, [_|_] = _ClientStack]}} ->
 	    ok;
 	Other ->
-	    io:format("Unexpected: ~p", [Other]),
+	    ct:pal("Unexpected: ~p", [Other]),
 	    ct:fail(failed)
     end,
-    process_flag(trap_exit, OldFl),
+    receive
+        {error_report,_,_} -> ok
+    end,
     ok.
 
-%% Verify that error when terminating correctly calls our format_status/2 fun
-%%
+%% Verify that error when terminating correctly calls our format_status/1,2 fun
 terminate_crash_format(Config) when is_list(Config) ->
     error_logger_forwarder:register(),
     OldFl = process_flag(trap_exit, true),
+    terminate_crash_format(?MODULE),
+    terminate_crash_format(format_status_server),
+    process_flag(trap_exit, OldFl);
+terminate_crash_format(Module) when is_atom(Module) ->
     State = crash_terminate,
-    {ok, Pid} = gen_server:start_link(?MODULE, {state, State}, []),
+    {ok, Pid} = gen_server:start_link(Module, {state, State}, []),
     gen_server:call(Pid, stop),
     receive {'EXIT', Pid, {crash, terminate}} -> ok end,
     ClientPid = self(),
@@ -1418,9 +1775,167 @@ terminate_crash_format(Config) when is_list(Config) ->
 	    io:format("Timeout: expected error logger msg", []),
 	    ct:fail(failed)
     end,
-    process_flag(trap_exit, OldFl),
+    receive
+        {error_report,_,_} -> ok
+    end,
     ok.
 
+crash_in_format_status(Config) when is_list(Config) ->
+    error_logger_forwarder:register(),
+    OldFl = process_flag(trap_exit, true),
+    crash_in_format_status(?MODULE, "gen_server_SUITE:format_status/2 crashed"),
+    crash_in_format_status(format_status_server, "format_status_server:format_status/1 crashed"),
+    process_flag(trap_exit, OldFl).
+crash_in_format_status(Module, Match) when is_atom(Module) ->
+    State = fun(_) -> exit({crash,format_status}) end,
+    {ok, Pid} = gen_server:start_link(Module, {state, State}, []),
+
+    {status,Pid, _, [_,_,_,_,Info]} = sys:get_status(Pid),
+    {data,[{"State",Match}]} = lists:last(Info),
+
+    gen_server:call(Pid, stop),
+    receive {'EXIT', Pid, stopped} -> ok end,
+    ClientPid = self(),
+    receive
+	{error,_GroupLeader,
+         {Pid,
+          "** Generic server"++_,
+          [Pid, stop, Match, stopped,
+           ClientPid, [_|_] = _ClientStack]}} ->
+	    ok;
+	Other ->
+	    ct:pal("Unexpected: ~p", [Other]),
+	    ct:fail(failed)
+    after 5000 ->
+	    io:format("Timeout: expected error logger msg", []),
+	    ct:fail(failed)
+    end,
+    receive
+        {error_report,_,_} -> ok
+    end,
+    ok.
+
+throw_in_format_status(Config) when is_list(Config) ->
+    error_logger_forwarder:register(),
+    OldFl = process_flag(trap_exit, true),
+    throw_in_format_status(?MODULE,{throw,format_status}),
+    throw_in_format_status(format_status_server,"format_status_server:format_status/1 crashed"),
+    process_flag(trap_exit, OldFl).
+throw_in_format_status(Module, Match) when is_atom(Module) ->
+    State = fun(_) -> throw({throw,format_status}) end,
+    {ok, Pid} = gen_server:start_link(Module, {state, State}, []),
+
+    {status,Pid, _, [_,_,_,_,Info]} = sys:get_status(Pid),
+    case lists:last(Info) of
+        {data,[{"State",Match}]} ->
+            ok;
+        Match ->
+            ok
+    end,
+
+    gen_server:call(Pid, stop),
+    receive {'EXIT', Pid, stopped} -> ok end,
+    ClientPid = self(),
+    receive
+	{error,_GroupLeader,
+         {Pid, "** Generic server"++_,
+          [Pid, stop, Match, stopped,
+           ClientPid, [_|_] = _ClientStack]}} ->
+	    ok;
+	Other ->
+	    ct:pal("Unexpected: ~p", [Other]),
+	    ct:fail(failed)
+    after 5000 ->
+	    io:format("Timeout: expected error logger msg", []),
+	    ct:fail(failed)
+    end,
+    receive
+        {error_report,_,_} -> ok
+    end,
+    ok.
+
+%% Test that the state, reason, message format status calls works as they should
+%% The test makes sure that both sys:get_status and the crash report works as they
+%% should and can be used to strip data from both the reason, message, state and the
+%% sys logger logs.
+
+%%%% The sys logger log messages that should be matched
+-define(LOG_MESSAGES,
+        {log,{in,{_,_,started_p}}},
+        {log,{out,ok,_,State}},
+        {log,{in,{_,_,{delayed_answer,10}}}},
+        {log,{noreply,{_,_,State}}},
+        {log,{in,timeout}},
+        {log,{noreply,State}}).
+
+format_all_status(Config) when is_list(Config) ->
+    error_logger_forwarder:register(),
+    OldFl = process_flag(trap_exit, true),
+
+    State = fun(M) ->
+                    maps:map(
+                      fun(log, Values) ->
+                              [{log, Value} || Value <- Values];
+                         (Key, Value) ->
+                              {Key, Value}
+                      end, M)
+            end,
+    {ok, Pid} = gen_server:start_link(format_status_server, {state, State}, []),
+    sys:log(Pid, true),
+    ok = gen_server:call(Pid, started_p),
+    delayed = gen_server:call(Pid, {delayed_answer, 10}),
+
+    {status,Pid, _, [_,_,_,_,Info]} = sys:get_status(Pid),
+    [{header, _Hdr},
+     {data, [_Status,_Parent,{"Logged events",LoggedEvents}]},
+     {data, [{"State",{state,State}}]}] = Info,
+
+    [?LOG_MESSAGES] = LoggedEvents,
+
+    ok = gen_server:call(Pid, stop),
+    receive {'EXIT', Pid, stopped} -> ok end,
+    ClientPid = self(),
+    receive
+	{error,_GroupLeader,
+         {Pid, "** Generic server"++_,
+          [Pid, {message, stop}, {state,State}, {reason, stopped},
+           ?LOG_MESSAGES, {log,{in,{_,_,stop}}},
+           ClientPid, [_|_] = _ClientStack]}} ->
+	    ok;
+	Other ->
+	    ct:pal("Unexpected: ~p", [Other]),
+	    ct:fail(failed)
+    after 5000 ->
+	    io:format("Timeout: expected error logger msg", []),
+	    ct:fail(failed)
+    end,
+    receive
+        {error_report,_,_} -> ok
+    end,
+
+    {ok, Pid2} = gen_server:start_link(format_status_server, {state, State}, []),
+    catch gen_server:call(Pid2, crash),
+    receive {'EXIT', Pid2, crashed} -> ok end,
+    receive
+	{error,_GroupLeader2,
+         {Pid2, "** Generic server"++_,
+          [Pid2, {message, crash}, {state,State},
+           {{reason, crashed},[_|_] = _ServerStack},
+           ClientPid, [_|_] = _ClientStack2]}} ->
+	    ok;
+	Other2 ->
+	    ct:pal("Unexpected: ~p", [Other2]),
+	    ct:fail(failed)
+    after 5000 ->
+	    io:format("Timeout: expected error logger msg", []),
+	    ct:fail(failed)
+    end,
+    receive
+        {error_report,_,_} -> ok
+    end,
+
+    process_flag(trap_exit, OldFl).
+
 %% Verify that sys:get_state correctly returns gen_server state
 get_state(Config) when is_list(Config) ->
     State = self(),
@@ -1985,8 +2500,8 @@ init({state,State}) ->
 handle_call(started_p, _From, State) ->
     io:format("FROZ"),
     {reply,ok,State};
-handle_call({delayed_answer, T}, From, _State) ->
-    {noreply,{reply_to,From},T};
+handle_call({delayed_answer, T}, From, State) ->
+    {noreply,{reply_to,From,State},T};
 handle_call({call_within, T}, _From, _) ->
     {reply,ok,call_within,T};
 handle_call(next_call, _From, call_within) ->
@@ -2042,9 +2557,9 @@ handle_cast({From, stop}, State) ->
     io:format("BAZ"),
     {stop, {From,stopped}, State}.
 
-handle_info(timeout, {reply_to, From}) ->
+handle_info(timeout, {reply_to, From, State}) ->
     gen_server:reply(From, delayed),
-    {noreply, []};
+    {noreply, State};
 handle_info(timeout, hibernate_me) -> % Arrive here from 
 						% handle_info(hibernate_later,...)
     {noreply, [], hibernate};
@@ -2124,6 +2639,8 @@ terminate(_, {undef_in_terminate, {Mod, Fun}}) ->
 terminate(_Reason, _State) ->
     ok.
 
+format_status(_, [_PDict, Fun] = S) when is_function(Fun) ->
+    Fun(S);
 format_status(terminate, [_PDict, State]) ->
     {formatted, State};
 format_status(normal, [_PDict, _State]) ->
