@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2003-2020. All Rights Reserved.
+%% Copyright Ericsson AB 2003-2021. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -30,12 +30,19 @@
 -export([from_shell/1]).
 -export([records/1]).
 -export([record_index/1]).
--export([multipass/1]).
+-export([map_pattern/1]).
+-export([map_expr_in_head/1]).
+-export([map_pattern_from_shell/1]).
+-export([map_expr_in_head_from_shell/1]).
+-export([map_exprs/1]).
+-export([map_exprs_from_shell/1]).
 -export([top_match/1]).
+-export([multipass/1]).
 -export([old_guards/1]).
 -export([autoimported/1]).
 -export([semicolon/1]).
 -export([bitsyntax/1]).
+-export([binary_bifs/1]).
 -export([record_defaults/1]).
 -export([andalso_orelse/1]).
 -export([float_1_function/1]).
@@ -44,6 +51,8 @@
 -export([no_warnings/1]).
 -export([eep37/1]).
 -export([otp_14454/1]).
+-export([otp_16824/1]).
+-export([unused_record/1]).
 
 init_per_testcase(_Func, Config) ->
     Config.
@@ -57,10 +66,13 @@ suite() ->
 
 all() -> 
     [from_shell, basic_ets, basic_dbg, records,
-     record_index, multipass, bitsyntax, record_defaults,
+     record_index, multipass, bitsyntax, binary_bifs, record_defaults,
      andalso_orelse, float_1_function, action_function,
      warnings, no_warnings, top_match, old_guards, autoimported,
-     semicolon, eep37, otp_14454].
+     semicolon, eep37, otp_14454, otp_16824, unused_record,
+     map_pattern, map_expr_in_head,
+     map_pattern_from_shell, map_expr_in_head_from_shell,
+     map_exprs, map_exprs_from_shell].
 
 groups() -> 
     [].
@@ -89,14 +101,16 @@ warnings(Config) when is_list(Config) ->
 	     "            when is_integer(A) and (A+5 > B) -> "
 	     "              A andalso B "
 	     "            end)">>,
-    [{_,[{_,ms_transform,{?WARN_NUMBER_SHADOW,'A'}}]}] =
+    [{_,[{{1,22},ms_transform,{?WARN_NUMBER_SHADOW,'A'}}]}] =
 	compile_ww(Prog),
+    [{_,[{1,ms_transform,{50,'A'}}]}] =
+        compile_ww(<<>>,Prog,[{error_location,line}]),
     Prog2 = <<"C = 5,
                ets:fun2ms(fun ({A,B} =
 				   C) when is_integer(A) and (A+5 > B) ->
                                   {A andalso B,C}
                           end)">>,
-    [{_,[{3,ms_transform,{?WARN_NUMBER_SHADOW,'C'}}]}] =
+    [{_,[{{3,8},ms_transform,{?WARN_NUMBER_SHADOW,'C'}}]}] =
 	      compile_ww(Prog2),
 	      Rec3 = <<"-record(a,{a,b,c,d=foppa}).">>,
 	      Prog3 = <<"A = 3,
@@ -106,8 +120,8 @@ warnings(Config) when is_list(Config) ->
 					 when is_integer(A) and (A+5 > B) ->
 					   {A andalso B,C}
 				   end)">>,
-    [{_,[{3,ms_transform,{?WARN_NUMBER_SHADOW,'C'}},
-         {4,ms_transform,{?WARN_NUMBER_SHADOW,'A'}}]}] =
+    [{_,[{{3,20},ms_transform,{?WARN_NUMBER_SHADOW,'C'}},
+         {{4,15},ms_transform,{?WARN_NUMBER_SHADOW,'A'}}]}] =
 			compile_ww(Rec3,Prog3),
 			Rec4 = <<"-record(a,{a,b,c,d=foppa}).">>,
 			Prog4 = <<"A=3,C=5, "
@@ -253,6 +267,32 @@ bitsyntax(Config) when is_list(Config) ->
 	"            end)">>),
     ok.
 
+
+%% Test that binary BIFs byte_size/1, binary_part/2, binary_part/3 are accepted
+binary_bifs(Config) when is_list(Config) ->
+    setup(Config),
+    TestSet = [{<<"hello">>, <<"world">>}, {<<"souldn't">>, <<"match">>}],
+    RunMS = fun(MS) -> ets:match_spec_run(TestSet, ets:match_spec_compile(MS)) end,
+    % check byte_size/1
+    MS1 = compile_and_run(<<"ets:fun2ms(fun({A, B}) when byte_size(A) == 5 -> {A, byte_size(B)} end)">>),
+    [{{'$1','$2'},
+      [{'==',{byte_size,'$1'},5}],
+      [{{'$1',{byte_size,'$2'}}}]}] = MS1,
+    [{<<"hello">>, 5}] = RunMS(MS1),
+    % check binary_part/2
+    MS2 = compile_and_run(<<"ets:fun2ms(fun({A, B}) when binary_part(A, {1, 2}) == <<\"el\">> -> binary_part(B, {2, 3}) end)">>),
+    [{{'$1','$2'},
+      [{'==',{binary_part,'$1',{{1,2}}},<<"el">>}],
+      [{binary_part,'$2',{{2,3}}}]}] = MS2,
+    [<<"rld">>] = RunMS(MS2),
+    % check binary_part/3
+    MS3 = compile_and_run(<<"ets:fun2ms(fun({A, B}) when binary_part(A, 1, 2) == <<\"el\">> -> binary_part(B, 2, 3) end)">>),
+    [{{'$1','$2'},
+      [{'==',{binary_part,'$1',1,2},<<"el">>}],
+      [{binary_part,'$2',2,3}]}] = MS3,
+    [<<"rld">>] = RunMS(MS3),
+    ok.
+
 %% Test that record defaults works.
 record_defaults(Config) when is_list(Config) ->
     setup(Config),
@@ -285,7 +325,7 @@ basic_ets(Config) when is_list(Config) ->
         compile_and_run(<<"ets:fun2ms(fun({\"foo\" ++ _, X}) -> X end)">>),
     ok.
 
-%% Tests basic ets:fun2ms.
+%% Tests basic dbg:fun2ms.
 basic_dbg(Config) when is_list(Config) ->
     setup(Config),
     [{[a,b],[],[{message,banan},{return_trace}]}] =
@@ -378,6 +418,59 @@ record_index(Config) when is_list(Config) ->
 		    <<"ets:fun2ms(fun({#a.a,A}) when A > #a.a -> #a.a end)">>),
     ok.
 
+map_pattern(Config) when is_list(Config) ->
+    setup(Config),
+    MS = [{{key, #{foo => '$1'}},[],['$1']}],
+    MS = compile_and_run(<<"ets:fun2ms(fun({key, #{foo := V}}) -> V end)">>),
+    ok.
+
+map_expr_in_head(Config) when is_list(Config) ->
+    setup(Config),
+    MS = [{{key, #{foo => '$1'}},[],['$1']}],
+    %% Accidentally it is possible to use => instead of := in the fun head,
+    %% in compiled code.
+    %% Although this is not an intended behaviour it is kept to
+    %% maintain backwards compatibility.
+    MS = compile_and_run(<<"ets:fun2ms(fun({key, #{foo => V}}) -> V end)">>),
+    ok.
+
+map_pattern_from_shell(Config) when is_list(Config) ->
+    MS = [{{key, #{foo => '$1'}},[],['$1']}],
+    MS = do_eval("ets:fun2ms(fun({key, #{foo := V}}) -> V end)"),
+    ok.
+
+map_expr_in_head_from_shell(Config) when is_list(Config) ->
+    setup(Config),
+    MS = [{{key, #{foo => '$1'}},[],['$1']}],
+    %% Accidentally it is possible to use => instead of := in the fun head,
+    %% in compiled code. This behaviour is kept for backwards compatibility.
+
+    %% As a side-effect, it is also possible to do the same with
+    %% `transform_from_shell/3', if the AST of the shell fun is
+    %% created bypassing the linter. (The linter would prevent
+    %% constructing such invalid syntax, so normally this is not
+    %% possible in the Erlang shell)
+    MS = do_eval("ets:fun2ms(fun({key, #{foo => V}}) -> V end)"),
+    ok.
+
+map_exprs(Config) when is_list(Config) ->
+    setup(Config),
+    MSGuard = [{{key,'$1','$2'}, [{'=:=','$1',#{foo => '$2'}}], ['$1']}],
+    MSGuard = compile_and_run(
+                <<"ets:fun2ms(fun({key, V1, V2}) when V1 =:= #{foo => V2} -> V1 end)">>),
+    MSBody = [{{key,'$1'}, [], [#{foo => '$1'}]}],
+    MSBody = compile_and_run(
+                <<"ets:fun2ms(fun({key, V}) -> #{foo => V} end)">>),
+    ok.
+
+map_exprs_from_shell(Config) when is_list(Config) ->
+    setup(Config),
+    MSGuard = [{{key,'$1','$2'}, [{'=:=','$1',#{foo => '$2'}}], ['$1']}],
+    MSGuard = do_eval("ets:fun2ms(fun({key, V1, V2}) when V1 =:= #{foo => V2} -> V1 end)"),
+    MSBody = [{{key,'$1'}, [], [#{foo => '$1'}]}],
+    MSBody = do_eval("ets:fun2ms(fun({key, V}) -> #{foo => V} end)"),
+    ok.
+
 %% Tests matching on top level in head to give alias for object().
 top_match(Config) when is_list(Config) ->
     setup(Config),
@@ -787,6 +880,28 @@ otp_14454(Config) when is_list(Config) ->
     ok.
 
 
+otp_16824(Config) when is_list(Config) ->
+    setup(Config),
+    Prog = <<
+      "-module(tmp).\n",
+      "-include_lib(\"stdlib/include/ms_transform.hrl\").\n",
+      "-export([tmp/1]).\n\n",
+      "tmp(_) -> ets:fun2ms(fun(<<>>) -> 1 end).\n">>,
+    FN = temp_name(),
+    ok = file:write_file(FN, Prog),
+    {ok, Forms} = parse_file(FN),
+    {error,[{_, [{{5,25},ms_transform,_}]}], []} =
+        compile:forms(Forms, [return]),
+    ok.
+
+%% OTP-17186.
+unused_record(Config) when is_list(Config) ->
+    setup(Config),
+    Record = <<"-record(r, {f}).\n\n">>,
+    Expr = <<"ets:fun2ms(fun(#r{}) -> e end)">>,
+    [] = compile_ww(Record, Expr),
+    ok.
+
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% Helpers
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -823,14 +938,18 @@ compile_and_run(Records,Expr) ->
     Expr/binary,".\n">>,
     FN=temp_name(),
     file:write_file(FN,Prog),
-    {ok,Forms} = epp:parse_file(FN,"",""),
+    {ok,Forms} = parse_file(FN),
     {ok,tmp,Bin} = compile:forms(Forms),
     code:load_binary(tmp,FN,Bin),
     tmp:tmp().
 
 compile_ww(Expr) ->
     compile_ww(<<>>,Expr).
+
 compile_ww(Records,Expr) ->
+    compile_ww(Records,Expr,[]).
+
+compile_ww(Records,Expr, Opts) ->
     Prog = <<
 	"-module(tmp).\n",
     "-include_lib(\"stdlib/include/ms_transform.hrl\").\n",
@@ -841,10 +960,9 @@ compile_ww(Records,Expr) ->
     Expr/binary,".\n">>,
     FN=temp_name(),
     file:write_file(FN,Prog),
-    {ok,Forms} = epp:parse_file(FN,"",""),
+    {ok,Forms} = parse_file(FN),
     {ok,tmp,_Bin,Wlist} = compile:forms(Forms,[return_warnings,
-					       nowarn_unused_vars,
-					       nowarn_unused_record]),
+					       nowarn_unused_vars | Opts]),
     Wlist.
 
 compile_no_ww(Expr) ->
@@ -855,16 +973,18 @@ compile_no_ww(Expr) ->
     Expr/binary,".\n">>,
     FN=temp_name(),
     file:write_file(FN,Prog),
-    {ok,Forms} = epp:parse_file(FN,"",""),
+    {ok,Forms} = parse_file(FN),
     {ok,tmp,_Bin,Wlist} = compile:forms(Forms,[return_warnings,
-					       nowarn_unused_vars,
-					       nowarn_unused_record]),
+					       nowarn_unused_vars]),
     Wlist.
 
+parse_file(FN) ->
+    epp:parse_file(FN, [{location, {1,1}}]).
+
 do_eval(String) ->
     {done,{ok,T,_},[]} = erl_scan:tokens(
 			   [],
 			   String++".\n",1),
     {ok,Tree} = erl_parse:parse_exprs(T),
-    {value,Res,[]} =  erl_eval:exprs(Tree,[]),
+    {value,Res,[]} =  erl_eval:exprs(Tree,[],none,none),
     Res.
