@@ -25,16 +25,18 @@
 %% Test cases
 -export([app_test/1,appup_test/1,smoke_test/1,revert/1,revert_map/1,
          revert_map_type/1,wrapped_subtrees/1,
-	t_abstract_type/1,t_erl_parse_type/1,t_type/1, t_epp_dodger/1,
-	t_comment_scan/1,t_igor/1,t_erl_tidy/1,t_prettypr/1]).
+         t_abstract_type/1,t_erl_parse_type/1,t_type/1,
+         t_epp_dodger/1,t_epp_dodger_clever/1,
+         t_comment_scan/1,t_prettypr/1]).
 
 suite() -> [{ct_hooks,[ts_install_cth]}].
 
 all() -> 
     [app_test,appup_test,smoke_test,revert,revert_map,revert_map_type,
      wrapped_subtrees,
-    t_abstract_type,t_erl_parse_type,t_type,t_epp_dodger,
-    t_comment_scan,t_igor,t_erl_tidy,t_prettypr].
+     t_abstract_type,t_erl_parse_type,t_type,
+     t_epp_dodger,t_epp_dodger_clever,
+     t_comment_scan,t_prettypr].
 
 groups() -> 
     [].
@@ -52,22 +54,22 @@ end_per_group(_GroupName, Config) ->
     Config.
 
 app_test(Config) when is_list(Config) ->
-    ok = ?t:app_test(syntax_tools).
+    ok = test_server:app_test(syntax_tools).
 
 appup_test(Config) when is_list(Config) ->
-    ok = ?t:appup_test(syntax_tools).
+    ok = test_server:appup_test(syntax_tools).
 
 %% Read and parse all source in the OTP release.
 smoke_test(Config) when is_list(Config) ->
-    Dog = ?t:timetrap(?t:minutes(12)),
+    Dog = test_server:timetrap(test_server:minutes(12)),
     Wc = filename:join([code:lib_dir(),"*","src","*.erl"]),
     Fs = filelib:wildcard(Wc) ++ test_files(Config),
     io:format("~p files\n", [length(Fs)]),
     case p_run(fun smoke_test_file/1, Fs) of
         0 -> ok;
-        N -> ?t:fail({N,errors})
+        N -> ct:fail({N,errors})
     end,
-    ?t:timetrap_cancel(Dog).
+    test_server:timetrap_cancel(Dog).
 
 smoke_test_file(File) ->
     case epp_dodger:parse_file(File) of
@@ -91,7 +93,7 @@ print_error_markers(F, File) ->
 
 %% Read with erl_parse, wrap and revert with erl_syntax and check for equality.
 revert(Config) when is_list(Config) ->
-    Dog = ?t:timetrap(?t:minutes(12)),
+    Dog = test_server:timetrap(test_server:minutes(12)),
     Wc = filename:join([code:lib_dir("stdlib"),"src","*.erl"]),
     Fs = filelib:wildcard(Wc) ++ test_files(Config),
     Path = [filename:join(code:lib_dir(stdlib), "include"),
@@ -99,12 +101,13 @@ revert(Config) when is_list(Config) ->
     io:format("~p files\n", [length(Fs)]),
     case p_run(fun (File) -> revert_file(File, Path) end, Fs) of
         0 -> ok;
-        N -> ?t:fail({N,errors})
+        N -> ct:fail({N,errors})
         end,
-    ?t:timetrap_cancel(Dog).
+    test_server:timetrap_cancel(Dog).
 
 revert_file(File, Path) ->
-    case epp:parse_file(File, Path, []) of
+    case epp:parse_file(File, [{includes,Path},
+                               res_word_option()]) of
         {ok,Fs0} ->
             Fs1 = erl_syntax:form_list(Fs0),
             Fs2 = erl_syntax_lib:map(fun (Node) -> Node end, Fs1),
@@ -116,16 +119,16 @@ revert_file(File, Path) ->
 
 %% Testing bug fix for reverting map_field_assoc
 revert_map(Config) when is_list(Config) ->
-    Dog = ?t:timetrap(?t:minutes(1)),
+    Dog = test_server:timetrap(test_server:minutes(1)),
     [{map_field_assoc,16,{atom,17,name},{var,18,'Value'}}] =
     erl_syntax:revert_forms([{tree,map_field_assoc,
                              {attr,16,[],none},
 			     {map_field_assoc,{atom,17,name},{var,18,'Value'}}}]),
-    ?t:timetrap_cancel(Dog).
+    test_server:timetrap_cancel(Dog).
 
 %% Testing bug fix for reverting map_field_assoc in types
 revert_map_type(Config) when is_list(Config) ->
-    Dog = ?t:timetrap(?t:minutes(1)),
+    Dog = test_server:timetrap(test_server:minutes(1)),
     Form1 = {attribute,4,record,
              {state,
               [{typed_record_field,
@@ -142,12 +145,12 @@ revert_map_type(Config) when is_list(Config) ->
                  [{type,5,map_field_assoc,[{atom,5,y},{atom,5,z}]}]}}]}},
     Mapped2 = erl_syntax_lib:map(fun(X) -> X end, Form2),
     Form2 = erl_syntax:revert(Mapped2),
-    ?t:timetrap_cancel(Dog).
+    test_server:timetrap_cancel(Dog).
 
 %% Read with erl_parse, wrap each tree node with erl_syntax and check that
 %% erl_syntax:subtrees can access the wrapped node.
 wrapped_subtrees(Config) when is_list(Config) ->
-    Dog = ?t:timetrap(?t:minutes(2)),
+    Dog = test_server:timetrap(test_server:minutes(2)),
     Wc = filename:join([code:lib_dir(stdlib),"src","*.erl"]),
     Fs = filelib:wildcard(Wc) ++ test_files(Config),
     Path = [filename:join(code:lib_dir(stdlib), "include"),
@@ -156,9 +159,9 @@ wrapped_subtrees(Config) when is_list(Config) ->
     Map = fun (File) -> wrapped_subtrees_file(File, Path) end,
     case p_run(Map, Fs) of
         0 -> ok;
-        N -> ?t:fail({N,errors})
+        N -> ct:fail({N,errors})
     end,
-    ?t:timetrap_cancel(Dog).
+    test_server:timetrap_cancel(Dog).
 
 wrapped_subtrees_file(File, Path) ->
     case epp:parse_file(File, Path, []) of
@@ -308,7 +311,9 @@ t_erl_parse_type(Config) when is_list(Config) ->
 		     {"#{ a:=1, b:=2 }", map_expr,false},
 		     {"M#{ a=>1, b=>2 }", map_expr,false},
 		     {"[V||V <- Vs]", list_comp,false},
+		     {"[catch V||V <- Vs]", list_comp,false},
 		     {"<< <<B>> || <<B>> <= Bs>>", binary_comp,false},
+		     {"<< (catch <<B>>) || <<B>> <= Bs>>", binary_comp,false},
 		     {"#state{ a = A, b = B}", record_expr,false},
 		     {"#state{}", record_expr,false},
 		     {"#s{ a = #def{ a=A }, b = B}", record_expr,false},
@@ -330,6 +335,13 @@ t_epp_dodger(Config) when is_list(Config) ->
     ok = test_epp_dodger(Filenames,DataDir,PrivDir),
     ok.
 
+t_epp_dodger_clever(Config) when is_list(Config) ->
+    DataDir   = ?config(data_dir, Config),
+    PrivDir   = ?config(priv_dir, Config),
+    Filenames = ["epp_dodger_clever.erl"],
+    ok = test_epp_dodger_clever(Filenames,DataDir,PrivDir),
+    ok.
+
 t_comment_scan(Config) when is_list(Config) ->
     DataDir   = ?config(data_dir, Config),
     Filenames = test_files(),
@@ -339,8 +351,7 @@ t_comment_scan(Config) when is_list(Config) ->
 t_prettypr(Config) when is_list(Config) ->
     DataDir   = ?config(data_dir, Config),
     PrivDir   = ?config(priv_dir, Config),
-    Filenames = ["type_specs.erl",
-                 "specs_and_funs.erl"],
+    Filenames = test_files(),
     ok = test_prettypr(Filenames,DataDir,PrivDir),
     ok.
 
@@ -354,33 +365,6 @@ test_files() ->
      "type_specs.erl",
      "specs_and_funs.erl"].
 
-t_igor(Config) when is_list(Config) ->
-    DataDir   = ?config(data_dir, Config),
-    PrivDir   = ?config(priv_dir, Config),
-    FileM1  = filename:join(DataDir,"m1.erl"),
-    FileM2  = filename:join(DataDir,"m2.erl"),
-    ["m.erl",_]=R = igor:merge(m,[FileM1,FileM2],[{outdir,PrivDir}]),
-    io:format("igor:merge/3 = ~p~n", [R]),
-
-    FileTypeSpecs = filename:join(DataDir,"igor_type_specs.erl"),
-    Empty = filename:join(DataDir,"empty.erl"),
-    ["n.erl",_]=R2 = igor:merge(n,[FileTypeSpecs,Empty],[{outdir,PrivDir}]),
-    io:format("igor:merge/3 = ~p~n", [R2]),
-
-    ok.
-
-t_erl_tidy(Config) when is_list(Config) ->
-    DataDir   = ?config(data_dir, Config),
-    File  = filename:join(DataDir,"erl_tidy_tilde.erl"),
-    ok = erl_tidy:file(File, [{stdout, true}]),
-
-    %% OTP-14471.
-    Old = process_flag(trap_exit, true),
-    NonExisting  = filename:join(DataDir,"non_existing_file.erl"),
-    {'EXIT',{error,{0,file,enoent}}} = (catch erl_tidy:file(NonExisting)),
-    true = process_flag(trap_exit, Old),
-    ok.
-
 test_comment_scan([],_) -> ok;
 test_comment_scan([File|Files],DataDir) ->
     Filename  = filename:join(DataDir,File),
@@ -407,39 +391,41 @@ test_comment_scan([File|Files],DataDir) ->
 test_prettypr([],_,_) -> ok;
 test_prettypr([File|Files],DataDir,PrivDir) ->
     Filename  = filename:join(DataDir,File),
+    Options = [res_word_option()],
     io:format("Parsing ~p~n", [Filename]),
-    {ok, Fs0} = epp:parse_file(Filename, [], []),
+    {ok, Fs0} = epp:parse_file(Filename, Options),
     Fs = erl_syntax:form_list(Fs0),
     PP = erl_prettypr:format(Fs, [{paper,  120}, {ribbon, 110}]),
     io:put_chars(PP),
     OutFile = filename:join(PrivDir, File),
     ok = file:write_file(OutFile,unicode:characters_to_binary(PP)),
     io:format("Parsing OutFile: ~ts~n", [OutFile]),
-    {ok, Fs2} = epp:parse_file(OutFile, [], []),
+    {ok, Fs2} = epp:parse_file(OutFile, Options),
     case [Error || {error, _} = Error <- Fs2] of
         [] ->
             ok;
         Errors ->
-            ?t:fail(Errors)
+            ct:fail(Errors)
     end,
     test_prettypr(Files,DataDir,PrivDir).
 
-
 test_epp_dodger([], _, _) -> ok;
 test_epp_dodger([Filename|Files],DataDir,PrivDir) ->
     io:format("Parsing ~p~n", [Filename]),
+    Options  = [{feature, maybe_expr, enable}],
     InFile   = filename:join(DataDir, Filename),
-    Parsers  = [{fun epp_dodger:parse_file/1,parse_file},
-		{fun epp_dodger:quick_parse_file/1,quick_parse_file},
+    Parsers  = [{fun(File) -> epp_dodger:parse_file(File, Options) end,parse_file},
+		{fun(File) -> epp_dodger:quick_parse_file(File,
+                                                          Options) end,quick_parse_file},
 		{fun (File) ->
 			{ok,Dev} = file:open(File,[read]),
-			Res = epp_dodger:parse(Dev),
+			Res = epp_dodger:parse(Dev, Options),
 			file:close(File),
 			Res
 		 end, parse},
 		{fun (File) ->
 			{ok,Dev} = file:open(File,[read]),
-			Res = epp_dodger:quick_parse(Dev),
+			Res = epp_dodger:quick_parse(Dev, Options),
 			file:close(File),
 			Res
 		 end, quick_parse}],
@@ -447,9 +433,30 @@ test_epp_dodger([Filename|Files],DataDir,PrivDir) ->
     ok = pretty_print_parse_forms(FsForms,PrivDir,Filename),
     test_epp_dodger(Files,DataDir,PrivDir).
 
+test_epp_dodger_clever([], _, _) -> ok;
+test_epp_dodger_clever([Filename|Files],DataDir,PrivDir) ->
+    io:format("Parsing ~p~n", [Filename]),
+    InFile   = filename:join(DataDir, Filename),
+    Parsers  = [{fun(File) ->
+                         epp_dodger:parse_file(File, [clever])
+                 end, parse_file},
+		{fun(File) ->
+                         epp_dodger:quick_parse_file(File, [clever])
+                 end, quick_parse_file}],
+    FsForms  = parse_with(Parsers, InFile),
+    ok = pretty_print_parse_forms(FsForms,PrivDir,Filename),
+    test_epp_dodger_clever(Files,DataDir,PrivDir).
+
 parse_with([],_) -> [];
 parse_with([{Fun,ParserType}|Funs],File) ->
     {ok, Fs} = Fun(File),
+    ErrorMarkers = [begin
+                        print_error_markers(F, File),
+                        F
+                    end
+                    || F <- Fs,
+                       erl_syntax:type(F) =:= error_marker],
+    [] = ErrorMarkers,
     [{Fs,ParserType}|parse_with(Funs,File)].
 
 pretty_print_parse_forms([],_,_) -> ok;
@@ -612,3 +619,9 @@ p_run_loop(Test, List, N, Refs0, Errors0) ->
 	    Refs = Refs0 -- [Ref],
 	    p_run_loop(Test, List, N, Refs, Errors)
     end.
+
+res_word_option() ->
+    Options = [{feature, maybe_expr, enable}],
+    {ok, {_Ftrs, ResWordFun}} =
+        erl_features:keyword_fun(Options, fun erl_scan:f_reserved_word/1),
+    {reserved_word_fun, ResWordFun}.
